#!/bin/bash

 ##############################################################################
 # Filename: bash/release.sh
 #
 # This file is part of the JAS project.
 # Copyright 2003-2016 by Matthijs Schuurman
 #
 # The JAS project is free software under the terms of the
 # GNU Lesser General Public License version 3 or any later version.
 #
 # For information about the license please refer to:
 # LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 ##############################################################################


cd "`dirname $0`/../"
. ./bash/functions.sh

title "Release generator" "0.4"

PKGFORMATS=("ZIP")
CHANGELOGFILE="./include/changelog.txt"


action "Generating JAS changelog"
CHANGELOG=`generateChangelog "./"`
if [ $? != 0 ]; then
 status failed "$CHANGELOG"
 exit 1
fi

echo "$CHANGELOG" > "$CHANGELOGFILE"
status $? "$CHANGELOG" #error hangling
if [ $? == 1 ]; then exit 1; fi


#get version from changelog
VERSION=(`grep -m 1 "^Version \([0-9\.]\+\)" "$CHANGELOGFILE" | sed -e "s/Version \([0-9\.]\+\)[ \-]*\([A-Z0-9]*\)/\1 \2/i"`)

#generate release string
if [ "${VERSION[1]}" == "" -o "${VERSION[1]}" == "Stable" ]; then
 RELEASE="${VERSION[0]}"
else
 RELEASE="${VERSION[0]}-${VERSION[1]}"
fi
RELEASE="JAS-$RELEASE"

if [ -d "$RELEASE" ]; then
 action "Removing old release directory" "$RELEASE"
 OUTPUT=`rm -R "$RELEASE" 2>&1`
 status $? "$OUTPUT" #error hangling
 if [ $? == 1 ]; then exit 1; fi
fi


action "Creating release" "$RELEASE"

#gather core components
FILES=`getFiles jas`
FILES="${FILES//
/ }"

mkdir "$RELEASE"
OUTPUT=`cp -R --parents ${FILES} "$RELEASE" 2>&1`
if [ $? != 0 ]; then
 status failed "$OUTPUT"
 exit 1
fi

#reset jas project passwords
cat project/jas/include/data/users.data.xml \
 | sed "s/\"\(password\)\" value=\".*\"/\"\1\" value=\"\"/" \
  > "$RELEASE/project/jas/include/data/users.data.xml"

cat project/jas/include/data/config.data.xml \
 | sed "s/\"\(language\)\" value=\".*\"/\"\1\" value=\"EN\"/" \
 | sed "s/\"\(prettyurls\)\" value=\".*\"/\"\1\" value=\"false\"/" \
 | sed "s/\"\(benchmark\)\" value=\".*\"/\"\1\" value=\"false\"/" \
 | sed "s/\"\(debug\)\" value=\".*\"/\"\1\" value=\"false\"/" \
 | sed "s/\"\(logging\)\" value=\".*\"/\"\1\" value=\"false\"/" \
 | sed "s/\"\(cache\)\" value=\".*\"/\"\1\" value=\"\"/" \
  > "$RELEASE/project/jas/include/data/config.data.xml"

status done
echo

#create release archive
for PKGFORMAT in ${PKGFORMATS[@]}; do
{
 if [ "$PKGFORMAT" == "GZ" ]; then
  FILE="$RELEASE.tar.gz"
  CMD="tar czf"
 elif [ "$PKGFORMAT" == "BZ2" ]; then
  FILE="$RELEASE.tar.bz2"
  CMD="tar cjf"
 elif [ "$PKGFORMAT" == "XZ" ]; then
  FILE="$RELEASE.tar.xz"
  CMD="tar cJf"
 elif [ "$PKGFORMAT" == "ZIP" ]; then
  FILE="$RELEASE.zip"
  CMD="zip -r"
 fi

 FILE="$FILE"

 action "Creating $PKGFORMAT package" "$FILE"
 if [ -f "$FILE" ]; then
  rm "$FILE"
  if [ $? != 0 ]; then
   status failed
   exit 1
  fi
 fi

 $CMD "$FILE" "$RELEASE" > /dev/null 2>&1
 status $? #error hangling
 if [ $? == 1 ]; then exit 1; fi
}
done


#clean up release directory
rm -R "$RELEASE"

echo
