<?

 /*****************************************************************************
 * Filename: include/api/xml.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."data.php");
 require_once($_path["format"]."xml.php");
 require_once($_path["api"]."api.php");

 class API_XML extends API
 {
  function __construct($mode=false, $config=false)
  {
   global $_path;

   parent::__construct($mode, $config);

   $this->contentType="text/xml";

   $this->templates=$_path["api"]."xml/";
  }


  function error()
  {
   switch($this->version)
   {
    case "1.0":
    {
     if($this->output=$this->format_errors_1_0())
      return true;

     break;
    }
    default: break;
   }

   return parent::error();
  }


  function parseTemplate($template)
  {
   if(!file_exists($this->templates.$this->version."/".$template.".xml"))
   {
    $this->e->add($this->type." API template: \"".$template."\" version: \"".$this->version."\" not found", "error");
    return false;
   }

   $xml=new Format_XML();

   if(!$xml->read($this->templates.$this->version."/".$template.".xml"))
   {
    $this->e->add($xml);
    return false;
   }

   if($xml->getAttribute("?xml", "encoding")!=$this->encoding)
    $xml->setAttribute("?xml", "encoding", $this->encoding);

   if($xml->getAttribute("jas", "version")!=$this->version)
    $xml->setAttribute("jas", "version", $this->version);

   return $xml;
  }


  function parse($xml)
  {
   if(!$xml)
   {
    $this->e->add($this->type." API no parse ".($this->mode=="client"?"response":"request")." provided", "error");
    return false;
   }

   $tmp=new Format_XML();

   if(!$tmp->parse($xml))
   {
    $this->e->add($tmp->e);
    return false;
   }

   if($tmp->getKey("jas")===false)
   {
    $this->e->add($this->type." API \"jas\" tag not found", "error");
    return false;
   }

   if(!($childs=$tmp->getChildren("jas")))
   {
    $this->e->add($this->type." API \"jas\" tag is empty", "error");
    return false;
   }

   if($this->mode=="server")
   {
    if($version=$this->getHeaderVersion()) //header version
    {
     if($tmp->getAttribute("jas", "version")&&$version!=$tmp->getAttribute("jas", "version")) //compare header and XML version
     {
      $this->e->add($this->type." API header version: \"".$version."\" doesn't match XML version: \"".$tmp->getAttribute("jas", "version")."\"", "error");
      return false;
     }

     $this->version=$version; //use header version
    }
    else //no header version
    {
     if($version=$tmp->getAttribute("jas", "version")) //check if XML version is set
      $this->version=$version; //use XML version
    }
   }
   else //client
   {
    if(!($version=$this->getHeaderVersion())) //no header version
     $this->e->add($this->type." API header version not set, using version: \"".$this->version."\"", "warning");
    else if($this->version!=$version) //version mismatch
    {
     $this->e->add($this->type." API header version: \"".$version."\" doesn't match version: \"".$this->version."\"", "error");
     return false;
    }

    if($tmp->getAttribute("jas", "version")&&$this->version!=$tmp->getAttribute("jas", "version")) //compare set and XML version
    {
     $this->e->add($this->type." API XML version: \"".$tmp->getAttribute("jas", "version")."\" doesn't match version: \"".$this->version."\"", "error");
     return false;
    }
   }

   for($c=0;$c<sizeof($childs);$c++)
   {
    switch($this->version)
    {
     case "1.0":
     {
      $function="parse_".$tmp->getName($childs[$c])."_".strtr($this->version, ".", "_");
      if(!in_array($function, get_class_methods($this)))
      {
       $this->e->add($this->type." API unsupported parse ".$tmp->getName($childs[$c])." ".($this->mode=="client"?"response":"request")." for version: \"".$this->version."\"", "error");
       return false;
      }

      if(!$this->$function($tmp->getOuterXML($childs[$c])))
       return false;

      break;
     }
     default:
     {
      $this->e->add($this->type." API unsupported version: \"".$this->version."\"", "error");
      return false;
     }
    }
   }

   return true;
  }


  function format($nr=false)
  {
   if($nr===false) //all commands
   {
    $this->output="";

    $success=true;
    $l=sizeof($this->commands);

    for($c=0;$c<$l;$c++)
     if(!$this->format($c))
      $success=false;

    return $success;
   }

   //format command
   switch($this->version)
   {
    case "1.0":
    {
     $function="format_".$this->commands[$nr]["type"]."_".strtr($this->version, ".", "_");
     if(!in_array($function, get_class_methods($this)))
     {
      $this->e->add($this->type." API unsupported format ".$this->commands[$nr]["type"]." ".($this->mode=="client"?"request":"response")." for version: \"".$this->version."\"", "error");
      return false;
     }

     if(!($output=$this->$function($nr)))
      return false;

     if(!$this->output) //no output set yet
      $this->output.=$output;
     else //first command already in output
     {
      $output=preg_replace("/^.+<jas[^>]*>(.+)<\/jas>.*/s", "\\1", $output);
      $this->output=str_replace("\n</jas>", $output."</jas>", $this->output);
     }

     break;
    }
    default:
    {
     $this->e->add($this->type." API unsupported version: \"".$this->version."\"", "error");
     return false;
    }
   }

   return true;
  }


  /*
   * Data
   */

  function parse_data_1_0($xml)
  {
   if(!($module=$xml->getAttribute("data", "module")))
   {
    $this->e->add($this->type." API no module set for data request", "error");
    return false;
   }

   if(!($action=$xml->getAttribute("data", "action")))
   {
    $this->e->add($this->type." API no action set for data request", "error");
    return false;
   }

   if(!($extra=$xml->getAttribute("data", "extra")))
   {
    $this->e->add($this->type." API no extra action set for data request", "error");
    return false;
   }

   $options=Array("skipCheckTables" => true);
   if(($format=$xml->getAttribute("data", "format")))
    $options["format"]=$format;
   else
    $options["format"]="api";

   if(($db=$xml->getAttribute("data", "db")))
    $options["db"]=$db;

   if(($count=$xml->getAttribute("data", "count"))&&strtolower($count)=="true")
    $count=true;
   else
    $count=false;

   $module=explode(":", $module);
   $d=Data::cache($module[0], $action, $extra, $options);

   if(isset($module[1])) //*:* referenced module
   {
    if(!isset($d->reference[$module[1]]))
    {
     $this->e->add($this->type." API unknown referenced module: \"".$module[1]."\"", "error");
     return false;
    }

    if($d->reference[$module[1]]["type"]!="*:*")
    {
     $this->e->add($this->type." API referenced module: \"".$module[1]."\" isn't \"*:*\" type", "error");
     return false;
    }

    $d=&$d->reference[$module[1]]["data"];
    $d->setAction($action);
    $d->setExtra($extra);
   }

   $d->clearValues(); //ensure no default values are set

   if($d->e->any("fatal"))
   {
    $this->e->add($d->e);
    return false;
   }

   $row=0;
   $show=Array();
   $sort=Array();
   foreach($xml->getChildren("data") as $child)
   {
    switch($xml->getName($child))
    {
     case "row":
     {
      foreach($xml->getChildren($child) as $child2)
      {
       if($xml->getName($child2)!="field")
       {
        $this->e->add($this->type." API unknown tag: \"jas/data/row/".$xml->getName($child2)."\"", "error");
        return false;
       }

       if(!($field=$xml->getAttribute($child2, "name")))
       {
        $this->e->add($this->type." API name attribute not set for tag: \"jas/data/row/".$xml->getName($child2)."\"", "error");
        return false;
       }

       if(($value=$xml->getAttribute($child2, "value"))===false) //value in attribute
       {
        //get value from inside tag
        if($d->field[$field]["type"]=="filedata")
         $value=base64_decode($xml->getInner($child2));
        else
         $value=html_entity_decode($xml->getInner($child2), ENT_QUOTES|ENT_XML1);
       }

       if(!$d->setValue($field, $value, $row))
       {
        $this->e->add($this->type." API unable to set value for field: \"".$field."\"", "error");
        return false;
       }

       if($this->mode=="client")
        if(!in_array($field, $show))
         $show[]=$field;
      }

      if($setFields=$d->check("valuetype", "set")) //set OR operant for last field
      {
       if(!$d->setOperant($setFields[sizeof($setFields)-1], "OR", $row))
       {
        $this->e->add($this->type." API unable to set operant for field: \"".$setFields[sizeof($setFields)-1]."\"", "error");
        return false;
       }
      }

      $row++;
      break;
     }
     case "show":
     {
      if($this->mode=="client")
       break; //show fields set based on rows in client mode

      foreach($xml->getChildren($child) as $child2)
      {
       if($xml->getName($child2)!="field")
       {
        $this->e->add($this->type." API unknown tag: \"jas/data/show/".$xml->getName($child2)."\"", "error");
        return false;
       }

       if(!($field=$xml->getAttribute($child2, "name")))
       {
        $this->e->add($this->type." API name attribute not set for tag: \"jas/data/show/".$xml->getName($child2)."\"", "error");
        return false;
       }

       $show[]=$field;
      }

      break;
     }
     case "sort":
     {
      foreach($xml->getChildren($child) as $child2)
      {
       if($xml->getName($child2)!="field")
       {
        $this->e->add($this->type." API unknown tag: \"jas/data/sort/".$xml->getName($child2)."\"", "error");
        return false;
       }

       if(!($field=$xml->getAttribute($child2, "name")))
       {
        $this->e->add($this->type." API name attribute not set for tag: \"jas/data/sort/".$xml->getName($child2)."\"", "error");
        return false;
       }

       $sort[]=$field;
      }

      break;
     }
     case "mutilations":
     {
      foreach($xml->getChildren($child) as $child2)
      {
       if($xml->getName($child2)!="mutilation")
       {
        $this->e->add($this->type." API unknown tag: \"jas/data/mutilation/".$xml->getName($child2)."\"", "error");
        return false;
       }

       if(!($field=$xml->getAttribute($child2, "name")))
       {
        $this->e->add($this->type." API name attribute not set for tag: \"jas/data/mutilation/".$xml->getName($child2)."\"", "error");
        return false;
       }

       if(!($mutilation=$xml->getAttribute($child2, "type")))
       {
        $this->e->add($this->type." API type attribute not set for tag: \"jas/data/mutilation/".$xml->getName($child2)."\"", "error");
        return false;
       }

       $c=1;
       $args=Array();
       while(true)
       {
        if(!($arg=$xml->getAttribute($child2, "arg".$c)))
         break;

        $args[]=$arg;
        $c++;
       }

       $d->addMutilation($field, $mutilation, $args);
      }

      break;
     }
     default:
     {
      $this->e->add($this->type." API unknown tag: \"jas/data/".$xml->getName($child)."\"", "error");
      return false;
     }
    }
   }

   if($show)
    $d->setShowFields($show);
   if($sort)
    $d->sortOrder($sort);

   if(!$this->addData($d, $count))
    return false;

   return true;
  }

  function format_data_1_0($nr=0)
  {
   //parse XML template
   if(!($xml=$this->parseTemplate($this->commands[$nr]["type"])))
    return false;

   $d=&$this->commands[$nr]["data"];

   //main attributes
   $xml->setAttribute("jas/data", "module", $d->module);
   $xml->setAttribute("jas/data", "action", $d->action);
   $xml->setAttribute("jas/data", "extra", $d->extra);
   $xml->setAttribute("jas/data", "format", $d->dataformat);

   if($this->commands[$nr]["count"])
    $xml->setAttribute("jas/data", "count", "true");
   else
    $xml->setAttribute("jas/data", "count");


   //show fields
   $show=Array();
   if($d->action=="database"&&$d->extra=="info")
   {
    foreach($d->check("use", "info") as  $field)
     $show[]=$field;
   }
   else
   {
    foreach(array_keys($d->field) as $field)
     if($d->check("use", "show", $field)||$d->check("properties", "identifier", $field))
      if(!$d->check("properties", "control", $field))
       $show[]=$field;
   }

   if($show&&$this->mode=="client")
   {
    $tmp="\n";
    $sub=$xml->getOuterXML("jas/data/show/field");
    foreach($show as $field)
    {
     $sub->setAttribute("field", "name", $field);
     $tmp.="   ".$sub->output(false)."\n";
    }

    $xml->setInside("jas/data/show", $tmp."  ");
   }
   else
    $xml->delete("jas/data/show");


   //sorting
   if($d->extra!="info"&&$d->extra!="item")
    $sort=$d->sortOrder;
   else
    $sort=Array();

   if($sort)
   {
    $tmp="\n";
    $sub=$xml->getOuterXML("jas/data/sort/field");
    foreach($sort as $field)
    {
     $sub->setAttribute("field", "name", $field);
     $tmp.="   ".$sub->output(false)."\n";
    }

    $xml->setInside("jas/data/sort", $tmp."  ");
   }
   else
    $xml->delete("jas/data/sort");


   //mutilations
   $mutilations=Array();
   foreach(array_keys($d->field) as $field)
    if($d->field[$field]["mutilation"])
     $mutilations[$field]=$d->field[$field]["mutilation"];

   if($mutilations)
   {
    $tmp="\n";
    $sub=$xml->getOuterXML("jas/data/mutilations/mutilation");
    foreach($mutilations as $field => $mutilation)
    {
     foreach($mutilation as $type => $args)
     {
      $sub->setAttribute("mutilation", "name", $field);
      $sub->setAttribute("mutilation", "type", $type);

      foreach($args as $c => $arg)
       $sub->setAttribute("mutilation", "arg".($c+1), $arg);

      $tmp.="   ".$sub->output(false)."\n";

      foreach($args as $c => $arg) //remove args for next mutilation
       $sub->setAttribute("mutilation", "arg".($c+1));
     }
    }

    $xml->setInside("jas/data/mutilations", $tmp."  ");
   }
   else
    $xml->delete("jas/data/mutilations");


   if($this->mode=="server")
    $fields=$show;
   else //client
   {
    if($d->action=="database"&&$d->extra=="info")
     $fields=Array();
    else //data
     $fields=$d->check("valuetype", "set");
   }

   $d->setShowFields($fields); //ensure Data::next set correct values

   //data
   $tmp="";
   $d->reset();
   $row=$xml->getOuterXML("jas/data/row");
   $sub=$row->getOuterXML("row/field");
   while($d->next())
   {
    $row->setInside("row", "\n  ");
    foreach($fields as $field)
    {
     $sub->reparse();
     $sub->setAttribute("field", "name", $field);

     if(is_array($d->v[$field]))
      $sub->setAttribute("field", "value", join(";", $d->v[$field]));
     else if($d->field[$field]["type"]=="filedata") //set file data inside tag
     {
      $sub->setAttribute("field", "value");
      $sub->setInside("field", base64_encode($d->v[$field]));
     }
     else if(strstr($d->v[$field], "\n")) //set non standard characters inside tag
     {
      $sub->setAttribute("field", "value");
      $sub->setInside("field", htmlentities($d->v[$field], ENT_COMPAT|ENT_XML1));
     }
     else
      $sub->setAttribute("field", "value", $d->v[$field]);

     $row->addInsideAtEnd("row", " ".$sub->output(false)."\n  ");
    }

    if($this->mode=="server")
    {
     if($this->commands[$nr]["count"]&&$d->vc==1)
     {
      $sub->reparse();
      $sub->setAttribute("field", "name", "show_count");
      $sub->setAttribute("field", "value", $d->getValue("show_count"));
      $row->addInsideAtEnd("row", "   ".$sub->output(false)."\n");
     }
    }

    $tmp.=$row->output(false)."\n  ";
   }

   $xml->setReplace("jas/data/row", $tmp); //replace original row tag
   return $xml->output();
  }


  /*
   * Language
   */

  function parse_language_1_0($xml)
  {
   global $_config;

   if(!($set=$xml->getAttribute(0, "set")))
   {
    $this->e->add($this->type." API set attribute not set for tag: \"jas/language\"", "error");
    return false;
   }

   if($this->mode=="server")
   {
    if(!($lang=$xml->getAttribute(0, "language")))
     $lang=$_config->get("language");

    if(!$this->addLanguage($set, $lang))
     return false;
   }
   else //client
   {
    if(!($lang=$xml->getAttribute(0, "translated")))
     $lang=$_config->get("language");

    $l=new Language();

    if(($original=$xml->getAttribute(0, "original")))
     $l->original=$original;

    while($xml->search("translation"))
     if(!$l->setTranslation($set, $l->original, $lang, $xml->getAttribute(false, "original"), $xml->getAttribute(false, "translated")))
      return false;

    if(!$this->addLanguage($set, $lang, $l))
     return false;
   }

   return true;
  }

  function format_language_1_0($nr=0)
  {
   global $_config;

   //parse XML template
   if(!($xml=$this->parseTemplate($this->commands[$nr]["type"])))
    return false;

   $l=&$this->commands[$nr]["language"];

   if($this->mode=="server")
   {
    $xml->setAttribute("jas/language", "original", $l->original);
    $xml->setAttribute("jas/language", "translated", $this->commands[$nr]["lang"]);
    $xml->setAttribute("jas/language", "set", $this->commands[$nr]["set"]);
    $xml->setAttribute("jas/language", "updated", gmdate($_config->get("apidatetimeformat"), $l->updated($this->commands[$nr]["set"], $this->commands[$nr]["lang"])));

    $tmp="";
    $translation=$xml->getOuterXML("jas/language/translation");
    foreach($l->translations["raw"][$l->original][$this->commands[$nr]["lang"]][$this->commands[$nr]["set"]] as $original => $translated)
    {
     $translation->setAttribute("translation", "original", $original);
     $translation->setAttribute("translation", "translated", $translated);
     $tmp.="\n  ".$translation->output();
    }

    $xml->setInside("jas/language", $tmp."\n ");
   }
   else //client
   {
    $xml->setAttribute("jas/language", "language", $this->commands[$nr]["lang"]);
    $xml->setAttribute("jas/language", "set", $this->commands[$nr]["set"]);
    $xml->setAttribute("jas/language", "original");
    $xml->setAttribute("jas/language", "translated");

    $xml->delete("jas/language/translation"); //remove original project tag
   }

   return $xml->output();
  }


  /*
   * Modules
   */

  function parse_modules_1_0($xml)
  {
   $all=false;
   $modules=Array();

   if($this->mode=="server")
   {
    if($xml->getAttribute("modules", "all")=="true")
     $all=true;

    if($childs=$xml->getChildren("modules"))
    {
     foreach($childs as $child) //filtering
     {
      if($xml->getName($child)!="module")
      {
       $this->e->add($this->type." API unknown tag: \"jas/modules/".$xml->getName($child)."\"", "error");
       return false;
      }

      if(!($module=$xml->getAttribute($child, "name")))
      {
       $this->e->add($this->type." API name attribute not set for tag: \"jas/modules/module/".$xml->getName($child)."\"", "error");
       return false;
      }

      if(!in_array($module, $modules))
       $modules[]=$module;
     }
    }
   }
   else //client
   {
    if($childs=$xml->getChildren("modules"))
    {
     foreach($childs as $child) //filtering
     {
      if($xml->getName($child)!="module")
      {
       $this->e->add($this->type." API unknown tag: \"jas/modules/".$xml->getName($child)."\"", "error");
       return false;
      }

      if($xml->getChildren($child)) //module definition
      {
       //load module (empty)
       $d=Data::cache("empty", false, false, Array("skipCheckTables" => true));

       if(!($d->module=$xml->getAttribute($child, "name")))
       {
        $this->e->add($this->type." API name attribute not set for tag: \"jas/modules/module/".$xml->getName($child)."\"", "error");
        return false;
       }

       if(!($d->title=$xml->getAttribute($child, "title")))
       {
        $this->e->add($this->type." API title attribute not set for tag: \"jas/modules/module/".$xml->getName($child)."\"", "error");
        return false;
       }

       foreach($xml->getChildren($child) as $child2)
       {
        switch($xml->getName($child2))
        {
         case "fields":
         {
          foreach($xml->getChildren($child2) as $child3)
          {
           if($xml->getName($child3)!="field")
           {
            $this->e->add($this->type." API unknown tag: \"jas/modules/module/fields/".$xml->getName($child3)."\"", "error");
            return false;
           }

           if($xml->getAttribute($child3, "name")&&isset($d->field[$xml->getAttribute($child3, "name")])) //fields may already exist (e.g. verify password is automatically added)
            continue;

           if(!$d->addField(
            $xml->getAttribute($child3, "name"),
            $xml->getAttribute($child3, "type"),
            $xml->getAttribute($child3, "use"),
            ($xml->getAttribute($child3, "properties")?$xml->getAttribute($child3, "properties"):false),
            ($xml->getAttribute($child3, "min")!==false?(int)$xml->getAttribute($child3, "min"):false),
            ($xml->getAttribute($child3, "max")!==false?(int)$xml->getAttribute($child3, "max"):false),
            ($xml->getAttribute($child3, "default")?$xml->getAttribute($child3, "default"):false),
            ($xml->getAttribute($child3, "title")?$xml->getAttribute($child3, "title"):false),
            ($xml->getAttribute($child3, "text")?$xml->getAttribute($child3, "text"):false)
           ))
           {
            $this->e->add($this->type." API unable to add field for tag: \"jas/modules/module/fields/".$xml->getName($child3)."\"", "error");
            $this->e->add($d->e);
            return false;
           }
          }

          break;
         }
         case "references":
         {
          foreach($xml->getChildren($child2) as $child3)
          {
           if($xml->getName($child3)!="reference")
           {
            $this->e->add($this->type." API unknown tag: \"jas/modules/module/references/".$xml->getName($child3)."\"", "error");
            return false;
           }

           if($xml->getAttribute($child3, "name")!=$xml->getAttribute($child3, "from")&&isset($d->field[$xml->getAttribute($child3, "name")])) //reference field may already exist (added above)
           {
            $tmp=$d->field[$xml->getAttribute($child3, "name")]; //store field
            unset($d->field[$xml->getAttribute($child3, "name")]); //temporarily remove field
           }
           else
            $tmp=false;

           if(!$d->addReference(
            $xml->getAttribute($child3, "name"),
            $xml->getAttribute($child3, "module"),
            $xml->getAttribute($child3, "to"),
            $xml->getAttribute($child3, "type"),
            $xml->getAttribute($child3, "from")
           ))
           {
            $this->e->add($this->type." API unable to add reference for tag: \"jas/modules/module/references/".$xml->getName($child3)."\"", "error");
            $this->e->add($d->e);
            return false;
           }

           if($tmp)
            $d->field[$xml->getAttribute($child3, "name")]=$tmp; //restore field
          }

          break;
         }
         default:
         {
          $this->e->add($this->type." API unknown tag: \"jas/modules/module/".$xml->getName($child2)."\"", "error");
          return false;
         }
        }
       }

       if(!$this->mergeModules($d))
        return false;
      }
      else //module name
      {
       if(!($module=$xml->getAttribute($child, "name")))
       {
        $this->e->add($this->type." API name attribute not set for tag: \"jas/modules/module/".$xml->getName($child)."\"", "error");
        return false;
       }

       if(!in_array($module, $modules))
        $modules[]=$module;
      }
     }
    }

    if(!$modules) //modules already added above
     return true;
   }

   if(!$this->addModules($modules, $all))
    return false;

   return true;
  }

  function format_modules_1_0($nr=0)
  {
   //parse XML template
   if(!($xml=$this->parseTemplate($this->commands[$nr]["type"])))
    return false;

   if($this->mode=="server")
   {
    $tmp="";
    if(!is_object($this->commands[$nr]["modules"][0])) //modules list
    {
     $sub=new Format_XML();
     $sub->parse("<module name=\"\"/>"); //short tags

     foreach($this->commands[$nr]["modules"] as $module)
     {
      $sub->setAttribute("module", "name", $module);
      $tmp.=($tmp?"\n  ":"").$sub->output(false);
     }
    }
    else //get module definition for all request modules
    {
     $sub=$xml->getOuterXML("jas/modules/module");

     $l=sizeof($this->commands[$nr]["modules"]);
     for($c=0;$c<$l;$c++)
     {
      $d=&$this->commands[$nr]["modules"][$c];
      $sub->setAttribute("module", "name", $d->module);

      $sub->setAttribute("module", "title", $d->title);

      $sub2=$sub->getOuterXML("module/fields/field");
      $sub->setInside("module/fields", "\n   ");

      foreach($d->field as $k => $v)
      {
       if($d->check("properties", "control", $k))
        continue;

       $sub2->setAttribute("field", "name", $k);
       $sub2->setAttribute("field", "type", $v["type"]);
       $sub2->setAttribute("field", "use", $v["use"]);
       $sub2->setAttribute("field", "properties", $v["properties"]);
       $sub2->setAttribute("field", "min", $v["min"]);
       $sub2->setAttribute("field", "max", $v["max"]);

       if(is_array($d->field[$k]["original"]["defaultvalue"]))
        $sub2->setAttribute("field", "default", implode(";", $d->field[$k]["original"]["defaultvalue"]));
       else
        $sub2->setAttribute("field", "default", $d->field[$k]["original"]["defaultvalue"]);

       $sub2->setAttribute("field", "title", $v["title"]);
       $sub2->setAttribute("field", "text", $v["text"]);
       $sub->addInsideAtEnd("module/fields", " ".$sub2->output(false)."\n   ");
       $sub2->reparse();
      }

      $sub2=$sub->getOuterXML("module/references/reference");
      $sub->setInside("module/references", "\n   ");

      foreach($d->reference as $k => $v)
      {
       $sub2->setAttribute("reference", "name", $k);
       $sub2->setAttribute("reference", "module", $v["module"]);
       $sub2->setAttribute("reference", "to", $v["to"]);
       $sub2->setAttribute("reference", "type", $v["type"]);
       $sub2->setAttribute("reference", "from", $v["from"]);
       $sub->addInsideAtEnd("module/references", " ".$sub2->output(false)."\n   ");
       $sub2->reparse();
      }


      $tmp.=$sub->output(false)."\n  ";
      $sub->reparse();
     }
    }

    $xml->setReplace("jas/modules/module", $tmp); //replace original module tag
   }
   else //client
   {
    if($this->commands[$nr]["modules"]) //request modules
    {
     $tmp="\n";
     $sub=new Format_XML();
     $sub->parse("  <module name=\"\"/>"); //short tags

     foreach($this->commands[$nr]["modules"] as $module)
     {
      if(is_object($module)) //data object
       $sub->setAttribute("module", "name", $module->module);
      else //string
       $sub->setAttribute("module", "name", $module);

      $tmp.=$sub->output(false)."\n";
     }

     $xml->setReplace("jas/modules/module", $tmp); //replace original module tag
    }
    else //request modules list
     $xml->delete("jas/modules/module"); //remove original module tag
   }

   return $xml->output();
  }


  /*
   * Projects
   */

  function parse_projects_1_0($xml)
  {
   $activate=false;
   $projects=Array();
   if($childs=$xml->getChildren("projects"))
   {
    foreach($childs as $child) //filtering
    {
     if($xml->getName($child)!="project")
     {
      $this->e->add($this->type." API unknown tag: \"jas/projects/".$xml->getName($child)."\"", "error");
      return false;
     }

     if($this->mode=="server")
     {
      $activate=$xml->getAttribute($child, "name");
      break;
     }
     else //client
     {
      $projects[]=$xml->getAttribute($child, "name");

      if($xml->getAttribute($child, "current")=="true")
       $activate=$xml->getAttribute($child, "name");
     }
    }
   }

   if(!$this->addProjects($activate, $projects))
    return false;

   return true;
  }

  function format_projects_1_0($nr=0)
  {
   //parse XML template
   if(!($xml=$this->parseTemplate($this->commands[$nr]["type"])))
    return false;

   if($this->mode=="server")
   {
    $tmp="";
    $sub=$xml->getOuterXML("jas/projects/project");
    foreach($this->commands[$nr]["projects"] as $project)
    {
     $sub->setAttribute("project", "name", $project);

     if($project==$this->commands[$nr]["activate"])
      $sub->setAttribute("project", "current", "true");
     else
      $sub->setAttribute("project", "current");

     $tmp.="\n  ".$sub->output(false);
     $sub->reparse();
    }

    $xml->setInside("jas/projects", $tmp."\n "); //replace original project tag
   }
   else //client
   {
    if($this->commands[$nr]["activate"]) //request project activation
    {
     $xml->setAttribute("jas/projects/project", "name", $this->commands[$nr]["activate"]);
     $xml->setAttribute("jas/projects/project", "current");
    }
    else //projects list
     $xml->delete("jas/projects/project"); //remove original project tag
   }

   return $xml->output();
  }


  /*
   * Errors
   */

  function parse_errors_1_0($xml)
  {
   $this->e->clear(); //remove all existing errors (so the error object only contains remote errors)

   if($childs=$xml->getChildren("errors"))
   {
    foreach($childs as $child) //filtering
    {
     if($xml->getName($child)!="error")
     {
      $this->e->add($this->type." API unknown tag: \"jas/errors/".$xml->getName($child)."\"", "error");
      return false;
     }

     if(!$this->e->add($xml->getAttribute($child, "error"), $xml->getAttribute($child, "level"), $xml->getAttribute($child, "extra")))
      return false;
    }
   }

   return true;
  }

  function format_errors_1_0()
  {
   //parse XML template
   if(!($xml=$this->parseTemplate("errors")))
    return false;

   $sub=$xml->getOuterXML("jas/errors/error");
   $xml->setInside("jas/errors", "\n");

   for($c=0;$c<$this->e->getErrors();$c++)
   {
    $sub->setAttribute("error", "error", $this->e->getError($c));
    $sub->setAttribute("error", "level", $this->e->getLevel($c));
    $sub->setAttribute("error", "extra", $this->e->getExtra($c));
    $xml->addInsideAtEnd("jas/errors", "  ".$sub->output(false)."\n");
    $sub->reparse();
   }

   return $xml->output();
  }

 }
?>
