<?

 /*****************************************************************************
 * Filename: include/cache.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."mime.php");
 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."fs.php");

 class Cache
 {
  function __construct($type=false, $config=false)
  {
   global $_path, $_config;

   $this->fs=new FS();
   $this->e=&$this->fs->e;

   //get Cache config
   $this->config=$_config->merge("Cache", $config);

   $this->memoryConnectionID="JAS"; //memcached persistent ID
   $this->memoryObject=false;

   $this->type=false;
   $this->mime=false;

   $this->e->l->read("cache");

   if(!$type) //no type set: autodetect
   {
    if(isset($this->config["memory"])&&$this->config["memory"])
     $type="memory";
    else if($this->config["module"])
     $type="module";
    else if($this->config["file"])
     $type="file";
    else
    {
     $this->e->add("Cache type autodetection failed", "error", "No enabled cache types");
     return false;
    }
   }

   if($type=="file")
   {
    if(!$this->config["file"])
    {
     $this->e->add("File based caching not enable in configuration", "error");
     return false;
    }

    if(!$this->fs->exists($_path["project"]["cache"]))
    {
     if(!$this->fs->mkdir($_path["project"]["cache"]))
      return false;
    }
    else if(!is_dir($_path["project"]["cache"]))
    {
     $this->e->add("Cache directory: \"".$_path["project"]["cache"]."\" isn't a directory");
     return false;
    }

    $this->type=$type;

    if(!$this->fs->exists($_path["project"]["cache"].".htaccess"))
     if(!$this->fs->copy($_path["include"].".htaccess", $_path["project"]["cache"].".htaccess")) //deny web-access
      return false;

    if($this->config["fileCleanup"]&&$_config->get("cache"))
    {
     if($_config->get("cache.fileCleanupNext")<=time()) //next auto clean up?
     {
      $this->cleanup($this->config["fileCleanup"]);  //clean up everything older the $fileCleanup seconds
      $_config->set("cache.fileCleanupNext", time()+$this->config["fileCleanup"]); //next auto clean up $fileCleanup seconds
     }
    }
   }
   else if($type=="module")
   {
    if(!$this->config["module"])
    {
     $this->e->add("Module based caching not enable in configuration", "error");
     return false;
    }

    $this->d=Data::cache("cache", "view", "item", Array("format" => "sql")); //use SQL data format: no convertion
    $this->d->clearValues();

    if($this->d->e->any("fatal"))
    {
     $this->e->add($this->d->e);
     return false;
    }

    $this->type=$type;
    $this->d2=clone $this->d;

    if($this->config["moduleCleanup"]&&$_config->get("cache"))
    {
     if($_config->get("cache.moduleCleanupNext")<=time()) //next auto clean up?
     {
      $this->cleanup($this->config["moduleCleanup"]);  //clean up everything older the $moduleCleanup seconds
      $_config->set("cache.moduleCleanupNext", time()+$this->config["moduleCleanup"]); //next auto clean up $moduleCleanup seconds
     }
    }
   }
   else if($type=="memory")
   {
    if(!isset($this->config["memory"])||!$this->config["memory"])
    {
     $this->e->add("Memory based caching not enable in configuration", "error");
     return false;
    }

    if(preg_match("/^([a-z0-9_\-\.]+):(\d+)$/i", $this->config["memoryConnection"], $tmp)) //domain:port
    {
     $host=$tmp[1];
     $port=$tmp[2];
    }
    else //socket filename
    {
     $host=$this->config["memoryConnection"];
     $port=0;
    }

    if($this->config["memoryMethod"]=="Memcache")
    {
     $this->memoryObject=new Memcache();

     if(!$port) //local server
     {
      if(!@$this->memoryObject->pconnect("unix://".$host, $port)) //persistent connection, not closed when object is destroyed
      {
       $this->e->add("Unable to connect to Memcached server socket: \"".$host."\"", "error");
       $this->memoryObject=false;
       return false;
      }
     }
     else
     {
      if(!@$this->memoryObject->pconnect($host, $port)) //persistent connection, not closed when object is destroyed
      {
       $this->e->add("Unable to connect to Memcached server: \"".$host."\" on port: ".$port, "error");
       $this->memoryObject=false;
       return false;
      }
     }

     $this->type=$type;
    }
    else if($this->config["memoryMethod"]=="Memcached")
    {
     $this->memoryObject=new Memcached($this->memoryConnectionID); //persistent connection

     //check if server is already in the server list
     $found=false;
     foreach($this->memoryObject->getServerList() as $server)
     {
      if($server["host"]==$host)
      {
       if((!$port&&$server["port"]==11211)||$server["port"]==$port)
       {
        $found=true;
        break;
       }
      }
     }

     if(!$found) //not yet in server list
     {
      if(!@$this->memoryObject->addServer($host, $port)||!@$this->memoryObject->set("__MemCachedTest", 1)) //add server and test connection
      {
       if(!$port) //local server
        $this->e->add("Unable to connect to Memcached server socket: \"".$host."\"", "error");
       else
        $this->e->add("Unable to connect to Memcached server: \"".$host."\" on port: ".$port, "error");

       $this->memoryObject->resetServerList();
       $this->memoryObject=false;
       return false;
      }

      $this->memoryObject->delete("__MemCachedTest"); //clean up connection test
     }
    }
    else if($this->config["memoryMethod"]=="Redis")
    {
     $this->memoryObject=new Redis();

     {
      if(!@$this->memoryObject->pconnect($host, $port, 0, $this->memoryConnectionID)) //persistent connection, not closed when object is destroyed
      {
       if(!$port) //local server
        $this->e->add("Unable to connect to Redis server socket: \"".$host."\"", "error");
       else
        $this->e->add("Unable to connect to Redis server: \"".$host."\" on port: ".$port, "error");

       $this->memoryObject=false;
       return false;
      }
     }

     $this->type=$type;
    }
    else
    {
     $this->e->add("Memory caching method: \"".$this->config["memoryMethod"]."\" not supported", "error");
     return false;
    }

    $this->type=$type;
    if($this->config["memoryCleanup"]&&$_config->get("cache"))
    {
     if($_config->get("cache.memoryCleanupNext")<=time()) //next auto clean up?
     {
      $this->cleanup($this->config["memoryCleanup"]);  //clean up everything older the $memoryCleanup seconds
      $_config->set("cache.memoryCleanupNext", time()+$this->config["memoryCleanup"]); //next auto clean up $memoryCleanup seconds
     }
    }
   }
   else
   {
    $this->e->add("Unknown cache type: \"".$type."\"");
    return false;
   }
  }

  function __clone()
  {
   //save original
   $tmpFS=&$this->fs;
   $tmpE=&$this->fs->e;

   //restore original language (dereference), so it's cloned correctly
   unset($this->fs->e);
   $this->fs->e=$tmpE;

   //clone object
   unset($this->fs);
   $this->fs=clone $tmpFS;

   //restore original
   unset($tmpFS->e);
   $tmpFS->e=&$tmpE;

   //update references
   unset($this->e);
   $this->e=&$this->fs->e;

   //clone other objects
   if($this->mime) //initialized in output()
    $this->mime=clone $this->mime;

   if(isset($this->d)&&$this->d) //module cache
    $this->d=clone $this->d;
   if(isset($this->d2)&&$this->d2)
    $this->d2=clone $this->d2;

   if(isset($this->memoryObject)&&$this->memoryObject) //memory cache
   {
    if($this->config["memoryMethod"]=="Memcache")
     $this->memoryObject=clone $this->memoryObject;
    else if($this->config["memoryMethod"]=="Memcached") //doesn't support clone...
     $this->memoryObject=new Memcached($this->memoryConnectionID); //recreate object (should inherit servers from previous init)
    else if($this->config["memoryMethod"]=="Redis")
     $this->memoryObject=clone $this->memoryObject;
   }
  }


  function exists($key, $log=true)
  {
   global $_path;

   if(!$key||$key[0]==".")
   {
    $this->e->add("Cache key invalid", "error");
    return false;
   }

   if($this->type=="file")
   {
    if(preg_match("!\.\.(/|\\\\)!", $key)) //key for file caching doesn't allow ../ or ..\
    {
     $this->e->add("Cache key invalid", "error");
     return false;
    }

    if($this->fs->exists($_path["project"]["cache"].$key))
     return true;
   }
   else if($this->type=="module")
   {
    if($key==$this->d->getValue("key")) //already got full entry, don't bother getting again
     return true;

    $this->d->clearValues();
    $this->d->setValue("key", $key);

    if(!$this->d->get()) //only count entries
    {
     $this->e->add($this->d->e);
     return false;
    }

    if($this->d->vmax) //any entries?
     return true;
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->config["memoryMethod"]=="Memcache"||$this->config["memoryMethod"]=="Memcached")
    {
     if(@$this->memoryObject->get($key)!==false&&@$this->memoryObject->get($key."_accessed")!==false)
      return true;
    }
    else if($this->config["memoryMethod"]=="Redis")
    {
     if($this->memoryObject->exists($key)&&$this->memoryObject->exists($key."_accessed"))
      return true;
    }
   }
   else
   {
    $this->e->add("Unknown cache type: \"".$this->type."\"");
    return false;
   }

   if($log)
    $this->e->add("Cache key: \"".$key."\" doesn't exist");

   return false;
  }

  function save($key, $value, $time=false)
  {
   global $_path;

   if(!$key||$key[0]==".")
   {
    $this->e->add("Cache key invalid", "error");
    return false;
   }

   if($this->type=="file")
   {
    if(preg_match("!\.\.(/|\\\\)!", $key)) //key for file caching doesn't allow ../ or ..\
    {
     $this->e->add("Cache key invalid", "error");
     return false;
    }

    if(!is_scalar($value))
     $value="53R1AL123D::".serialize($value);

    if($this->fs->exists($_path["project"]["cache"].$key))
    {
     $this->e->add("Cache key: \"".$key."\" already exists");
     return false;
    }

    if(!$this->fs->write($_path["project"]["cache"].".".$key, $value)) //write temporary file
     return false;

    if(!$this->fs->rename($_path["project"]["cache"].".".$key, $_path["project"]["cache"].$key)) //rename temporary file to final destination
     return false;

    if($time!==false)
     if(!$this->fs->setTime($_path["project"]["cache"].$key, $time, "modified"))
      return false;
   }
   else if($this->type=="module")
   {
    if(!is_scalar($value))
     $value="53R1AL123D::".serialize($value);

    $this->d2->setAction("add");
    $this->d2->setExtra("item");

    $value=mb_convert_encoding($value, "UTF-8");

    $this->d2->clearValues();
    $this->d2->setValue("key", $key);
    $this->d2->setValue("value", $value);

    if($time!==false)
     $this->d2->setValue("modified", date($this->d2->field["modified"]["format"][$this->d2->dataformat]["datetimeformat"], $time));

    if(!$this->d2->submit())
    {
     $this->e->add($this->d2->e);
     return false;
    }
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if(!$time)
     $time=time();

    if(!is_scalar($value))
    {
     $type=2; //serialized
     $value=serialize($value);
    }
    else
     $type=1;

    if($this->config["memoryMethod"]=="Memcache"||$this->config["memoryMethod"]=="Memcached"||$this->config["memoryMethod"]=="Redis")
    {
     if( //store key, time and value
      !@$this->memoryObject->set($key, $type)||
      !@$this->memoryObject->set($key."_accessed", time())||
      !@$this->memoryObject->set($key."_modified", $time)||
      !@$this->memoryObject->set($key."_size", strlen($value))||
      !@$this->memoryObject->set($key."_value", $value)
     )
     {
      $this->e->add("Unable to save key: \"".$key."\" for memory method: \"".$this->config["memoryMethod"]."\"", "error");
      return false;
     }
    }
   }
   else
    return false;

   return true;
  }

  function read($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
   {
    $value=$this->fs->read($_path["project"]["cache"].$key);

    if(strpos($value, "53R1AL123D::")!==false) //serialized value
     $value=unserialize(substr($value, 12));

    return $value;
   }
   else if($this->type=="module") //content already gathered in exists function
   {
    //update accessed time
    $this->d2->setAction("edit");
    $this->d2->setExtra("item");

    $this->d2->clearValues();
    $this->d2->setValue("key", $key);
    $this->d2->setValue("accessed", date($this->d2->field["accessed"]["format"][$this->d2->dataformat]["datetimeformat"], time()));

    if(!$this->d2->submit())
    {
     $this->e->add($this->d2->e);
     return false;
    }

    $value=$this->d->getValue("value");

    if(strpos($value, "53R1AL123D::")!==false) //serialized value
     $value=unserialize(substr($value, 12));

    return $value;
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->config["memoryMethod"]=="Memcache"||$this->config["memoryMethod"]=="Memcached"||$this->config["memoryMethod"]=="Redis")
    {
     @$this->memoryObject->set($key."_accessed", time()); //update accessed time

     if(@$this->memoryObject->get($key)==2) //serialized
      return unserialize(@$this->memoryObject->get($key."_value"));
     else
      return @$this->memoryObject->get($key."_value");
    }
   }

   return false;
  }

  function time($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
    return $this->fs->getTime($_path["project"]["cache"].$key, "modified");
   else if($this->type=="module") //content already gathered in exists function
    return DT::dateConvertS($this->d->getValue("modified"), $this->d->field["modified"]["format"][$this->d->dataformat]["datetimeformat"]); //convert to unix timestamp
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->config["memoryMethod"]=="Memcache"||$this->config["memoryMethod"]=="Memcached"||$this->config["memoryMethod"]=="Redis")
     return @$this->memoryObject->get($key."_modified");
   }

   return false;
  }

  function size($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
    return $this->fs->size($_path["project"]["cache"].$key);
   else if($this->type=="module") //content already gathered in exists function
    return strlen($this->d->getValue("value"));
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->config["memoryMethod"]=="Memcache"||$this->config["memoryMethod"]=="Memcached"||$this->config["memoryMethod"]=="Redis")
     return @$this->memoryObject->get($key."_size");
   }

   return false;
  }

  function remove($key)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if($this->type=="file")
   {
    if(!$this->fs->delete($_path["project"]["cache"].$key))
     return false;
   }
   else if($this->type=="module")
   {
    $this->d2->setAction("delete");
    $this->d2->setExtra("item");

    $this->d2->clearValues();
    $this->d2->setValue("key", $key);

    if(!$this->d2->submit())
    {
     $this->e->add($this->d2->e);
     return false;
    }
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->config["memoryMethod"]=="Memcache"||$this->config["memoryMethod"]=="Memcached"||$this->config["memoryMethod"]=="Redis")
    {
     if( //store key, time and value
      !@$this->memoryObject->delete($key)||
      !@$this->memoryObject->delete($key."_accessed")||
      !@$this->memoryObject->delete($key."_modified")||
      !@$this->memoryObject->delete($key."_size")||
      !@$this->memoryObject->delete($key."_value")
     )
     {
      $this->e->add("Unable to remove key: \"".$key."\" for memory method: \"".$this->config["memoryMethod"]."\"", "error");
      return false;
     }
    }
   }
   else
    return false;

   return true;
  }

  function cleanup($lifeTime)
  {
   global $_path;

   $failed=false;
   $lifeTime=time()-$lifeTime;

   if($this->type=="file")
   {
    foreach($this->fs->files($_path["project"]["cache"], true, true) as $key)
     if(!is_dir($_path["project"]["cache"].$key))
      if($this->fs->getTime($_path["project"]["cache"].$key, "accessed")<$lifeTime) //remove all files not accessed longer then lifeTime ago
       if(!$this->remove($key))
        $failed=true;
   }
   else if($this->type=="module")
   {
    $this->d2->setShowFields("key");
    $this->d2->setAction("view");
    $this->d2->setExtra("all");

    $this->d2->clearValues();
    $this->d2->setValue("accessed", date($this->d2->field["accessed"]["format"][$this->d2->dataformat]["datetimeformat"], $lifeTime));
    $this->d2->setOperator("accessed", "<");

    if(!$this->d2->get())
    {
     $this->e->add($this->d2->e);
     return false;
    }

    $keys=Array();
    while($this->d2->next())
     $keys[]=$this->d2->v["key"];

    foreach($keys as $key)
     if(!$this->remove($key))
      $failed=true;
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->config["memoryMethod"]=="Memcache"||$this->config["memoryMethod"]=="Memcached"||$this->config["memoryMethod"]=="Redis")
    {
     foreach($this->keys() as $key)
      if($this->memoryObject->get($key."_accessed")<$lifeTime)  //remove all keys not accessed longer then lifeTime ago
       if(!$this->remove($key))
        $failed=true;
    }
   }
   else
    return false;

   if($failed)
    return false;

   return true;
  }

  function search($search)
  {
   global $_path;

   if(!$search||$search[0]==".")
   {
    $this->e->add("Cache search key invalid", "error");
    return false;
   }

   $matches=Array();
   if($this->type=="file")
   {
    if(preg_match("!\.\.(/|\\\\)!", $search)) //key for file caching doesn't allow ../ or ..\
    {
     $this->e->add("Cache search key invalid", "error");
     return false;
    }

    foreach($this->fs->files($_path["project"]["cache"], false, true) as $key)
     if(stripos($key, $search)!==false)
      $matches[]=$key;
   }
   else if($this->type=="module")
   {
    $this->d2->setShowFields("key");
    $this->d2->setAction("search");
    $this->d2->setExtra("title");

    $this->d2->clearValues();
    $this->d2->setValue("key", $search);
    $this->d2->setOperator("key", "like");

    if(!$this->d2->get())
    {
     $this->e->add($this->d2->e);
     return false;
    }

    while($this->d2->next())
     $matches[]=$this->d2->v["key"];
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->config["memoryMethod"]=="Memcache"||$this->config["memoryMethod"]=="Memcached"||$this->config["memoryMethod"]=="Redis")
    {
     foreach($this->keys() as $key)
      if(stripos($key, $search)!==false)
       $matches[]=$key;
    }
   }
   else
    return false;

   return $matches;
  }

  function keys()
  {
   global $_path;

   $matches=Array();
   if($this->type=="file")
   {
    foreach($this->fs->files($_path["project"]["cache"], false, true) as $key)
     $matches[]=$key;
   }
   else if($this->type=="module")
   {
    $this->d2->setShowFields("key");
    $this->d2->setAction("view");
    $this->d2->setExtra("all");

    $this->d2->clearValues();

    if(!$this->d2->get())
    {
     $this->e->add($this->d2->e);
     return false;
    }

    while($this->d2->next())
     $matches[]=$this->d2->v["key"];
   }
   else if($this->type=="memory")
   {
    if(!$this->memoryObject)
     return false;

    if($this->config["memoryMethod"]=="Memcache")
    {
     if($stats=@$this->memoryObject->getStats("items")) //get stats
      foreach(array_keys($stats["items"]) as $item)
       if($keys=@$this->memoryObject->getStats("cachedump", $item)) //get cache dump per item
        foreach(array_keys($keys) as $key) //validate each key
         if(@$this->memoryObject->get($key."_accessed")) //check if also $key_accessed exists
          $matches[]=$key;
    }
    else if($this->config["memoryMethod"]=="Memcached")
    {
     if($keys=$this->memoryObject->getAllKeys()) //get all keys
      foreach($keys as $key) //validate each key
       if(@$this->memoryObject->get($key."_accessed")) //check if also $key_accessed exists
        $matches[]=$key;
    }
    else if($this->config["memoryMethod"]=="Redis")
    {
     if($keys=$this->memoryObject->getKeys("*_accessed")) //get all keys with _accessed suffix
      foreach($keys as $key) //validate each key
       if(preg_match("/^(.+)_accessed$/", $key, $m)) //get actual key name (without _accessed suffix)
        $matches[]=$m[1];
    }
   }
   else
    return false;

   return $matches;
  }

  function output($key, $headers=true)
  {
   global $_path;

   if(!$this->exists($key))
    return false;

   if(($value=$this->read($key))===false)
    return false;

   if($headers)
   {
    if(!$this->mime)
     $this->mime=new Mime();

    if(preg_match("/\.([a-z]+)$/i", $key, $tmp)) //use "extension"
     $ctype=$this->mime->extension($tmp[1]);
    else
     $ctype=$this->mime->content($value);

    Headers::fileS($key, $ctype, (is_string($value)?strlen($value):false), "inline");
   }

   return $value;
  }

  function lastUpdate($module)
  {
   $d=Data::cache($module, "database", "info");

   if($d->e->any("fatal"))
   {
    $this->e->add($d->e);
    return false;
   }

   $d->setShowFields("info_updated");
   $d->addMutilation("info_updated", "dateConvert", "U");

   if(!$d->get())
   {
    $this->e->add($d->e);
    return false;
   }

   return (int)$d->getValue("info_updated");
  }
 }

?>
