<?

 /*****************************************************************************
 * Filename: include/diff.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."errors.php");

 class Diff
 {
  function __construct()
  {
   $this->e=new Errors();
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  static function strings(&$string1, &$string2, $linebreak=false)
  {
   if($linebreak)
   {
    $linebreak1=$linebreak;
    $linebreak2=$linebreak;
   }
   else
   {
    if(!($linebreak1=Mutilate::detectLineBreaksS($string1)))
     $linebreak1="\n";
    if(!($linebreak2=Mutilate::detectLineBreaksS($string2)))
     $linebreak2="\n";
   }

   $array1=explode($linebreak1, $string1);
   $array2=explode($linebreak2, $string2);

   if(!Diff::arrays($array1, $array2))
    return false;

   $string1=implode($linebreak1, $array1);
   $string2=implode($linebreak2, $array2);

   return true;
  }

  static function arrays(&$array1, &$array2)
  {
   $l1=sizeof($array1);
   $l2=sizeof($array2);

   //align arrays
   for($c=0;$c<$l1;$c++)
   {
    //add missing entries in array2
    if(!isset($array2[$c])) //NULL seems to be the same as !isset
    {
     $array2[$c]=NULL;
     $l2=sizeof($array2); //don't use ++, could be an existing line
    }
    else if($array1[$c]!=$array2[$c]) //line mismatch
    {
     $found=false;
     //search if line exists in array2
     for($x=$c;$x<$l2;$x++)
     {
      if($array1[$c]==$array2[$x]) //found
      {
       $empty=array_fill(0, $x-$c, NULL); //add missing entries to array1
       $array1=array_merge(array_slice($array1, 0, $c), $empty, array_slice($array1, $c));
       $l1=sizeof($array1);
       unset($empty);

       $found=true;
       $c=$x; //step over new entries
       break;
      }
     }

     if($found) //array1 aligned, continue quest for other mismatches
      continue;

     //not aligned yet, search if line exists in array1
     for($x=$c;$x<$l1;$x++)
     {
      if($array1[$x]==$array2[$c]) //found
      {
       $empty=array_fill(0, $x-$c, NULL); //add missing entries to array2
       $array2=array_merge(array_slice($array2, 0, $c), $empty, array_slice($array2, $c));
       $l2=sizeof($array2);
       unset($empty);
       break;
      }
     }
    }
   }

   if($l1<$l2) //add missing lines in array1
   {
    $empty=array_fill(0, $l2-$l1, NULL);
    $array1=array_merge($array1, $empty);
    $l1=sizeof($array1);
   }

   $diffs=array_diff_assoc($array1, $array2);
   for($c=0;$c<$l1;$c++)
   {
    if($array1[$c]===NULL) //deleted
    {
     $array1[$c]="-";
     $array2[$c]="+".$array2[$c];
    }
    else if($array2[$c]===NULL) //added
    {
     $array1[$c]="+".$array1[$c];
     $array2[$c]="-";
    }
    else if($array1[$c]!=$array2[$c])
    {
     $array1[$c]="!".$array1[$c];
     $array2[$c]="!".$array2[$c];
    }
    else
    {
     $array1[$c]="=".$array1[$c];
     $array2[$c]="=".$array2[$c];
    }
   }

   return true;
  }
 }

?>
