<?

 /*****************************************************************************
 * Filename: include/filter.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."cms.php");

 class Filter
 {
  function __construct($config=false)
  {
   global $_config;

   //get Filter config
   $this->config=$_config->merge("Filter", $config);

   $this->cms=new CMS(Array(
    "showText" => false,
    "showTitles" => false,
    "showTips" => false,

    "showTimezone" => $this->config["showTimezone"],
    "showDateformat" => $this->config["showDateformat"],

    "showErrors" => $this->config["showErrors"],
    "showFields" => $this->config["showFields"],
    "showGroups" => $this->config["showGroups"],
    "showSubmit" => $this->config["showSubmit"],
    "showSubmitAdditional" => false,
    "showCancel" => false,

				"useStyling" => $this->config["useStyling"],
				"cmsOptions" => false,

				"handleSubmit" => false,
				"reloadPage" => false,
				"reloadTimeout" => false,
				"returnURL" => false,

				"defaultFormVars" => $this->config["defaultFormVars"],
				"anchoring" => $this->config["anchoring"],

    "multilistStyle" => "dropdown",
    "modulesStyle" => "dropdown",
    "localfilesStyle" => "dropdown",
   ));


   $this->e=&$this->cms->e;
   $this->l=&$this->e->l;

   if(isset($this->config["defaultFormVars"]))
    foreach(explode("&amp;", $this->config["defaultFormVars"]) as $var)
     if(strstr($var, "=")&&list($k, $v)=explode("=", $var))
      $this->cms->addVar($k, $v);

   $this->filterFieldTypes=Array(
    "search",
    "localfile", "localfiles", "localdirectory",
    "date", "time", "datetime",
    "multilist", "modules", "list",
    "enum", "boolean",
    "action", "extra",
    "module", "plugin", "project",
   );
  }

  function __clone()
  {
   //save original
   $tmpCMS=&$this->cms;
   $tmpE=&$this->e;
   $tmpL=&$this->e->l;

   //restore original language+error (dereference), so it's cloned correctly
   unset($this->e->l);
   $this->e->l=$tmpL;
   unset($this->e);
   $this->e=$tmpE;

   //clone object
   unset($this->cms);
   $this->cms=clone $tmpCMS;

   //restore original
   unset($tmpCMS->e);
   $tmpCMS->e=&$tmpE;
   unset($tmpE->l);
   $tmpE->l=&$tmpL;

   //update references
   unset($this->e);
   $this->e=&$this->cms->e;
   unset($this->l);
   $this->l=&$this->e->l;
  }


  function setData(&$data)
  {
   $data->setRequestValues();

   //make fields not mandatory
   foreach($data->check("use", "form") as $field)
   {
    if(isset($data->reference[$field]))
    {
     //make references optional
     if($data->reference[$field]["type"]=="*:1")
     {
      $data->reference[$field]["type"]="*:0";

      if($data->field[$data->reference[$field]["from"]]["min"])
       $data->field[$data->reference[$field]["from"]]["min"]=0;
     }
     else if($data->reference[$field]["type"]=="1:*")
     {
      $data->reference[$field]["type"]="1:*";

      if($data->field[$data->reference[$field]["from"]]["min"])
       $data->field[$data->reference[$field]["from"]]["min"]=0;
     }

     continue; //references used for filtering (not search)
    }

    switch($data->field[$field]["type"])
    {
     case "enum":
     case "list": //add empty list option
     {
      if($data->field[$field]["min"])
       $data->field[$field]["min"]=0;

      if(array_type($data->listOptions[$field])=="indexed")
      {
       if(!in_array("", $data->listOptions[$field]))
        array_unshift($data->listOptions[$field], "");
      }
      else //associative
      {
       if(!isset($data->listOptions[$field][""]))
        $data->listOptions[$field]=array_merge(Array("" => ""), $data->listOptions[$field]);
      }

      break;
     }

     case "label":
     case "submit":
      continue 2; //skip

     default:
     {
      if($data->field[$field]["min"])
       $data->field[$field]["min"]=0;

      break;
     }
    }
   }

   //valuetype = set for submitted values
   foreach($data->check("valuetype", "request") as $field)
   {
    if(isset($data->field[$field]["value"][0])&&$data->field[$field]["value"][0])
    {
     $data->field[$field]["valuetype"]="set";

     if($data->getValue($field, 1))
      $data->setValue($field, $data->getValue($field, 1), 2);

     $data->setValue($field, $data->getValue($field), 1);

     unset($data->field[$field]["operant"][0]); //ensure entry 0 isn't added to query
    }
   }

   //set operators
   foreach($data->check("use", "form") as $field)
   {
    if(preg_match("/^(date|datetime|time)$/", $data->field[$field]["type"])) //from/to field
    {
     $data->setOperator($field, ">=", 1);
     $data->setOperator($field, "<", 2);
    }
    else if(preg_match("/^(multilist|modules|localfiles)$/", $data->field[$field]["type"]))
     $data->setOperator($field, "like", 1);
   }

   $this->cms->setData(clone $data);//set cloned version to ensure above changes aren't reverted before render

   $this->l->read("filter"); //ensure module translations don't overwrite
  }

  function setForm(&$form)
  {
   $this->cms->setForm($form);
  }

  function setDataForm(&$data, &$form)
  {
   $this->cms->setData($data);
   $this->cms->setForm($form);
  }

  function addVar($name, $value)
  {
   $this->cms->addVar($name, $value);
  }


  function form($filterFile=false)
  {
   $r="";

   if(!$this->cms->formDataSetup())
    return false;

   //render form
   $form=str_replace("class=\"cms\"", "class=\"filter\"", $this->cms->formStart($filterFile));

   if($this->config["showFields"])
    $form.=$this->formShowFields();

   if($this->config["showSubmit"]||$this->config["showReset"])
    $form.=$this->formShowSubmit();

   $form.=$this->cms->formEnd();

   //move data errors to filter error object
   if($this->cms->data->e->any())
   {
    $this->cms->e->add($this->cms->data->e);
    $this->cms->data->e->clear();
   }

   //show errors
   if(!isset($this->config["showErrors"])||$this->config["showErrors"])
    $r.=$this->cms->formShowErrors();

   $r.=$form;

   return $r;
  }


  function formShowFields()
  {
   global $_config;

   //determine fields+groups
   $fields=Array();
   $groups=$this->config["showGroups"];

   //control fields
   foreach($this->cms->data->check("properties", "control") as $field)
   {
    if($field=="r"||strpos($field, "aggregation_")===0||strpos($field, "show_")===0) //ignore r, aggregation and show fields
     $fields[$field]="ignore";
   }

   //title/text/form fields
   $done=false;
   switch($this->cms->data->action)
   {
    case "view":
    {
     switch($this->cms->data->extra)
     {
      case "item":
      {
       if($field=$this->cms->data->getID())
       {
        $this->cms->data->field[$field]["properties"]=preg_replace("/hidden;?/", "", $this->cms->data->field[$field]["properties"]);
        $fields[$field]="";
       }

       break;
      }
      case "list":
      {
       if($field=$this->cms->data->getRightsReference())
       {
        $this->cms->data->field[$field]["properties"]=preg_replace("/hidden;?/", "", $this->cms->data->field[$field]["properties"]);
        $fields[$field]="";
       }

       break;
      }
      default:
      {
       foreach($this->cms->data->check("use", "form") as $field)
       {
        if(!in_array($this->cms->data->field[$field]["type"], $this->filterFieldTypes)&&!isset($this->cms->data->reference[$field]))
         continue;

        if($this->cms->data->check("properties", "hidden", $field))
         $fields[$field]="hidden";
        else
         $fields[$field]="";
       }

       break;
      }
     }

     break;
    }
    case "search":
    {
     foreach($this->cms->data->check("use", "form") as $field)
     {
      if(isset($fields[$field])||$field=="searchFields")
       continue;
      if(!in_array($this->cms->data->field[$field]["type"], $this->filterFieldTypes)&&!isset($this->cms->data->reference[$field]))
       continue;

      if($this->cms->data->check("properties", "hidden", $field))
       $fields[$field]="hidden";
      else
       $fields[$field]="";
     }

     break;
    }

    default: break;
   }

   //get groups
   if($groups)
   {
    $groups=Array();
    foreach($this->cms->data->getGroups() as $key => $group)
    {
     foreach($this->cms->data->getGroupFields($key) as $field)
     {
      if(isset($fields[$field]))
      {
       if(!isset($groups[$group]))
        $groups[$group]=Array();

       $groups[$group][]=$field;
      }
     }
    }
   }

   foreach($fields as $field => $type)
   {
    if($type=="ignore")
     continue;
    if(!$this->cms->data->check("valuetype", "default", $field))
     continue;

    $this->cms->data->unsetValue($field);
   }


   //render fields
   $r="";

   if($groups) //group fields
   {
    $r.="<div class=\"groups ".$this->cms->data->module."\">";

    foreach($groups as $group => $groupFields)
    {
     $tmp="";
     $visible=false;

     //render fields
     foreach($fields as $field => $type)
     {
      if($type=="ignore")
       continue;

      if(in_array($field, $groupFields))
      {
       $tmp.=$this->field($field, $type);

       if($type!="hidden")
        $visible=true;
      }
     }

     if($visible)
     {
      $r.="<div class=\"group ".preg_replace("/[^a-z]*/i", "", $group)."\">";
      $r.="<div class=\"groupHeader\">".$this->l->t($group)."</div>";
      $r.="<div class=\"groupBody\">".$tmp."</div>";
      $r.="<div class=\"groupFooter\"></div>";
      $r.="</div>";
     }
     else //no visible fields
      $r.=$tmp;
    }

    $r.="</div>";
   }
   else //no groups
   {
    //render fields
    foreach($fields as $field => $type)
    {
     if($type=="ignore")
      continue;

     $r.=$this->field($field, $type);
    }
   }

   //add vars
   foreach($this->cms->vars as $k => $v)
    $r.=$this->cms->form->input("hidden", $k, $v)."\n";

   return $r;
  }

  function formShowSubmit()
  {
   $r="";

   if($this->cms->data->requestVar)
    $fieldname=$this->cms->data->requestVar."[s]";
   else
    $fieldname="s";

   if($this->config["useStyling"])
   {
    $r.="<div class=\"field\">\n";
    $r.="<div class=\"title\">\n";
    $r.=$this->l->t("Submit");
    $r.="</div>\n";

    $r.="<div class=\"input\">\n";
   }

   if($this->config["showSubmit"])
    $r.=$this->cms->form->input("submit", $fieldname, $this->l->t("Filter"), "submit")."\n";

   if($this->config["showReset"])
    $r.=$this->cms->form->input("reset", "", $this->l->t("Reset"), "submit")."\n";

   if($this->config["useStyling"])
   {
    $r.="</div>\n";
    $r.="</div>\n";
   }

   return $r;
  }

  function field($names, $type=false, $nr=false)
  {
   if(!is_array($names))
    $names=Array($names);

   if(preg_match("/^(date|datetime|time)$/", $this->cms->data->field[$names[0]]["type"])) //make from/to field
   {
    if($nr===false) //force correct first field name
     $nr=0;

    $r=$this->cms->field($names, $type, $nr);
    $tmp=preg_replace("!^.*(<input[^>]+>).*$!smi", "\\1", $this->cms->field($names, $type, $nr+2));
    $r=preg_replace("!(<input[^>]+>)!i", "\\1 ".$tmp, $r);
   }
   else if(preg_match("/^(multilist|modules|localfiles)$/", $this->cms->data->field[$names[0]]["type"]))
   {
    $r=$this->cms->field($names, $type, $nr);
    $r=str_replace(" multi\" multiple=\"multiple\">", "\"><option value=\"\"></option>", $r);
   }
   else //default
    $r=$this->cms->field($names, $type, $nr);

   return $r;
  }
 }
?>
