<?

 /*****************************************************************************
 * Filename: include/format/csv.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["format"]."format.php");

 class Format_CSV extends Format
 {
  function __construct()
  {
   parent::__construct();

   $this->contentType="text/comma-separated-values";

   $this->e->l->read("csv");

   $this->separators();
   $this->header(false);
  }

  function separators($split=false, $quote=false, $linebreak=false, $subSplit=false)
  {
   if($split===false)
    $split=";";
   if($quote===false)
    $quote="\"";
   if($linebreak===false)
    $linebreak="\r\n";
   if($subSplit===false)
   {
    if($quote) //quoted, use linebreak
     $subSplit=$linebreak;
    else if($split==",") //non quoted, use non split
     $subSplit="; ";
    else
     $subSplit=", ";
   }

   if($split==$quote)
   {
    $this->e->add("Split and quote character can't be the same", "error", $this->filename);
    return false;
   }
   if($split==$linebreak)
   {
    $this->e->add("Split and linebreak character can't be the same", "error", $this->filename);
    return false;
   }
   if($quote==$linebreak)
   {
    $this->e->add("Quote and linebreak character can't be the same", "error", $this->filename);
    return false;
   }

   if($quote) //quote
   {
    if($subSplit==$quote)
    {
     $this->e->add("Sub split and quote character can't be the same", "error", $this->filename);
     return false;
    }
   }
   else //non quoted
   {
    if($subSplit==$split)
    {
     $this->e->add("Sub split and split character can't be the same", "error", $this->filename);
     return false;
    }
    if($subSplit==$linebreak)
    {
     $this->e->add("Sub split and linebreak character can't be the same", "error", $this->filename);
     return false;
    }
   }

   $this->split=$split;
   $this->quote=$quote;
   $this->linebreak=$linebreak;
   $this->subSplit=$subSplit;

   return true;
  }

  function header($headerLine)
  {
   $this->headerLine=($headerLine?true:false); //use first line as array keys when reading and manipulating and output/write array keys as first line
   return true;
  }

  function detectLineBreaks($input)
  {
   if($linebreak=Mutilate::detectLineBreaksS($input))
    $this->linebreak=$linebreak;

   $this->e->add("Unable to detect line breaks", "debug");
   return true;
  }


  /*
   * Manipulate
   */

  function checkLine($values)
  {
   foreach($values as $value)
   {
    if(is_array($value))
    {
     $this->e->add("Multi dimensional array lines not allowed", "error", $this->filename);
     return false;
    }
   }

   if(!$this->headerLine) //indexed
   {
    if(array_type($values)!="indexed")
    {
     $this->e->add("Line should be indexed array for non-header mode", "error", $this->filename);
     return false;
    }

    if($this->data) //check size with first line
    {
     if(sizeof($this->data[0])!=sizeof($values))
     {
      $this->e->add("Number of entries in line don't match previous line(s)", "warning", $this->filename);
     }
    }
   }
   else //associative
   {
    if(array_type($values)!="associative")
    {
     $this->e->add("Line should be associative array for header mode", "error", $this->filename);
     return false;
    }

    if($this->data) //check keys with first line
    {
     if(array_keys($this->data[0])!=array_keys($values))
     {
      $this->e->add("Line keys don't match previous line(s)", "error", $this->filename);
      return false;
     }
    }
   }

   return true;
  }

  function addLine()
  {
   $values=func_get_args();
   if(sizeof($values)==1&&is_array($values[0]))
    $values=$values[0];

   if($this->quote) //quote escape
   {
    $values=preg_replace("/([^\\\])".preg_quote($this->quote, "/")."/", "\\1\\".$this->quote, $values);
    $values=preg_replace("/([^\\\])".preg_quote($this->quote, "/")."/", "\\1\\".$this->quote, $values); //run twice to make sure "" gets properly quoted
   }

   if(!$this->checkLine($values))
    return false;

   $this->data[$this->vmax++]=$values;
   return true;
  }

  function editLine($line, $array)
  {
   if(!isset($this->data[$line]))
   {
    $this->e->add("Can't edit line ".$line, "error", $this->filename);
    return false;
   }

   $values=func_get_args();
   array_shift($values); //remove line param
   if(sizeof($values)==1&&is_array($values[0]))
    $values=$values[0];

   if($this->quote) //quote escape
   {
    $values=preg_replace("/([^\\\])".preg_quote($this->quote, "/")."/", "\\1\\".$this->quote, $values);
    $values=preg_replace("/([^\\\])".preg_quote($this->quote, "/")."/", "\\1\\".$this->quote, $values); //run twice to make sure "" gets properly quoted
   }

   if(!$this->checkLine($values))
    return false;

   $this->data[$line]=$array;
   return true;
  }

  function deleteLine($line)
  {
   if(!isset($this->data[$line]))
   {
    $this->e->add("Can't delete line ".$line, "error", $this->filename);
    return false;
   }

   $l=sizeof($this->data);
   for($c=$line+1;$c<$l;$c++)
    $this->data[$c-1]=$this->data[$c];

   unset($this->data[$l-1]);
   $this->vmax--;

   return true;
  }


/*
 * Iterate
 */

  function value($nr)
  {
   if(!parent::value($nr))
    return false;

   $this->v=preg_replace("/\\\\".preg_quote($this->quote, "/")."/", $this->quote, $this->v); //quote unescape
   return true;
  }

  function search($key, $value="")
  {
   //search if the value exists for the key
   for($c=$this->vc;$c<$this->vmax;$c++)
   {
    if(isset($this->data[$c][$key])&&$this->data[$c][$key]==$value)
    {
     //set all values to the v variable
     if(!$this->value($c))
      return false;

     $this->vc=$c+1; //go to next entry
     return true;
    }
   }

   $this->reset();
   return false;
  }


  /*
   * Read / Parse
   */

  function read($file, $headerLine=false, $detectLineBreaks=true)
  {
   if(!parent::read($file, false)) //don't parse automatically
    return false;

   return $this->parse($this->output, $headerLine, $detectLineBreaks);
  }

  function parse($data, $headerLine=false, $detectLineBreaks=true)
  {
   $this->output=$data;

   if(!$this->original)
    $this->original=$data;

   $this->vmax=0;
   $this->reset();
   $this->data=Array();

   if(!$data)
   {
    $this->e->add("No data to parse", "error", $this->filename);
    return false;
   }

   $this->header($headerLine);

   if($detectLineBreaks) //detect used line breaks in input
    if(!$this->detectLineBreaks($data))
     return false;

   if(substr($data, -(strlen($this->linebreak)))==$this->linebreak) //remove last line break
    $data=substr($data, 0, -(strlen($this->linebreak)));

   $data=str_replace($this->split.$this->linebreak, $this->linebreak, $data); //remove extra split character for each line
   if(substr($data, -(strlen($this->split)))==$this->split) //remove last split in case of no linebreak after split
    $data=substr($data, 0, -(strlen($this->split)));

   if(preg_match_all("/((".preg_quote($this->quote, "/").".*?(?<!\\\)".preg_quote($this->quote, "/").preg_quote($this->split, "/")."?)+)".preg_quote($this->linebreak, "/")."/s", $data.$this->linebreak, $lines))
    $lines=$lines[1];
   else
    $lines=explode($this->quote.$this->linebreak, $data);

   $l=sizeof($lines);

   $headers=false;
   for($c=0;$c<$l;$c++) //loop through all lines
   {
    if(!$lines[$c]) //empty line
     $line=Array();
    else
    {
     $line=explode($this->quote.$this->split.$this->quote, $lines[$c]); //split all values

     $l2=sizeof($line); //get number of values on line
     $line[0]=ltrim($line[0], $this->quote); //remove start quote
     $line[$l2-1]=rtrim($line[$l2-1], $this->split); //remove end split
     $line[$l2-1]=rtrim($line[$l2-1], $this->quote); //remove end quote

     $line=str_replace("\\".$this->quote, $this->quote, $line); //remove escapes from quotes
    }

    if($this->headerLine) //associative
    {
     if(!$headers) //get header line
      $headers=$line;
     else //normal lines
     {
      $x=0;
      $line2=Array();
      foreach($headers as $header) //create associative line
       if(isset($line[$x])) //addLine will fail on missing entry,
        $line2[$header]=$line[$x++];

      if(!$this->addLine($line2))
      {
       $this->clear();
       return false;
      }
     }
    }
    else
    {
     if(!$this->addLine($line))
     {
      $this->clear();
      return false;
     }
    }
   }

   return true;
  }


  /*
   * Output
   */

  function output($headers=false)
  {
   if($this->vmax&&$this->headerLine)
    $this->output=$this->quote.implode($this->quote.$this->split.$this->quote, array_keys($this->data[0])).$this->quote.$this->linebreak;
   else
    $this->output="";

   for($c=0;$c<$this->vmax;$c++)
    $this->output.=$this->quote.implode($this->quote.$this->split.$this->quote, $this->data[$c]).$this->quote.$this->linebreak;

   return parent::output($headers);
  }

 }
?>
