<?

 /*****************************************************************************
 * Filename: include/format/xml.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["format"]."format.php");

 class Format_XML extends Format
 {
  function __construct($onlyCommentsAndMarkers=false, $exactPath=true)
  {
   parent::__construct();

   $this->contentType="text/xml";

   $this->e->l->read("xml");

   $this->onlyCommentsAndMarkers=$onlyCommentsAndMarkers;
   $this->exactPath=$exactPath;
  }


/*
 * Parse
 */

  function parse($data)
  {
   $this->output=$data;

   if(!$this->original)
    $this->original=$data;

   $this->vmax=0;
   $this->reset();
   $this->data=Array();

   if(!$data)
   {
    $this->e->add("No data to parse", "error", $this->filename);
    return false;
   }

   //get all tags in xml
   if(!preg_match_all("!<(\!".($this->onlyCommentsAndMarkers?"":"?").")(/?)([^>\s\\n]+)([^>]*)>!si", $data, $tags, PREG_OFFSET_CAPTURE)) //no tags?
   {
    if(!preg_match("!#([^#<>\\n]+)#!si", $data)) //and no markers?
    {
     if($this->onlyCommentsAndMarkers)
      return true; //not fatal in case of only comments and markers (still could contain XML)

     $this->e->add("No XML tags found", "error", $this->filename);
     return false;
    }
   }


   //store all tags with meta data
   $parents=Array();
   $l1=sizeof($tags[0]);
   for($c=0;$c<$l1;$c++)
   {
    if($this->onlyCommentsAndMarkers&&$tags[1][$c][0]!="!")
     continue;

    if($tags[2][$c][0]!="/") //open tag
    {
     if($tags[1][$c][0]=="!") //comment tag
     {
      if(substr($tags[3][$c][0],0,2)=="--") //!-- comment
      {
       $name=preg_replace("![^A-Z0-9]*([A-Z0-9_]+).*!i", "\\1", $tags[4][$c][0]); //get proper name
       $type="!--open";
      }
      else //!comment
      {
       $name=$tags[3][$c][0];
       $type="!open";
      }
     }
     else
     {
      $type="open";
      $name=$tags[3][$c][0];
     }

     //store tag meta data
     $this->data[$this->vmax]=Array();

     //force short tag with end slash
     if(substr($name, -1)=="/")
     {
      $name=substr($name, 0, -1); //remove end slash from name
      $type=str_replace("open", "short", $type);
     }
     else if(substr($tags[4][$c][0], -1)=="/")
      $type=str_replace("open", "short", $type);

     $this->data[$this->vmax]["name"]=$name;
     $this->data[$this->vmax]["type"]=$type;
     $this->data[$this->vmax]["attributes"]=Array();

     $this->data[$this->vmax]["outer"]=Array();
     $this->data[$this->vmax]["outer"]["start"]=$tags[0][$c][1];
     $this->data[$this->vmax]["outer"]["length"]=strlen($tags[0][$c][0]);

     //store tag attributes
     if(preg_match_all("!([^\s]+)=\"([^\"]*)\"!", $tags[4][$c][0], $attributes)||preg_match_all("!([^\s]+)='([^']*)'!", $tags[4][$c][0], $attributes)) //get all key=value pairs
     {
      $l2=sizeof($attributes[0]);
      for($x=0;$x<$l2;$x++)
       $this->data[$this->vmax]["attributes"][$attributes[1][$x]]=$attributes[2][$x];
     }
     else if($tags[4][$c][0]=trim($tags[4][$c][0])) //text/fake attributes
     {
      $this->data[$this->vmax]["attributes"]=explode(" ", $tags[4][$c][0]); //space split values

      if($type=="!--open"||$type=="!--short")
      {
       array_shift($this->data[$this->vmax]["attributes"]); //remove name from attributes
       array_pop($this->data[$this->vmax]["attributes"]); //remove -- tag end from attributes
      }
     }

     $parents[]=$this->vmax; //add to parents list, i.e. assume it's a block tag

     $this->vmax++;
    }
    else //close tag
    {
     if($tags[1][$c][0]=="!") //comment tag
     {
      if(substr($tags[3][$c][0],0,2)=="--") //!-- comment
      {
       $name=preg_replace("![^A-Z0-9]*([A-Z0-9_]+).*!i", "\\1", $tags[4][$c][0]); //get proper name
       $type="!--open";
      }
      else //!comment
      {
       $name=$tags[3][$c][0];
       $type="!open";
      }
     }
     else //normal tag
     {
      $type="open";
      $name=$tags[3][$c][0];
     }

     //close tag
     $open=false;
     for($x=sizeof($parents)-1;$x>=0;$x--) //find matching open tag in parents list
     {
      if($this->data[$parents[$x]]["name"]==$name&&$this->data[$parents[$x]]["type"]==$type) //name and type should match
      {
       //detect number of remaining close tags
       $openTags=0;
       for($z=0;$z<$x;$z++)
        if($this->data[$parents[$z]]["name"]==$name&&$this->data[$parents[$z]]["type"]==$type) //name and type should match
         $openTags++;

       //detect number of remaining close tags
       $closeTags=0;
       for($z=$c+1;$z<$l1;$z++)
       {
        if($this->onlyCommentsAndMarkers&&$tags[1][$z][0]!="!")
         continue;

        if($tags[2][$z][0]=="/") //close tag
        {
         if($tags[1][$z][0]=="!") //comment tag
         {
          if(substr($tags[3][$z][0],0,2)=="--") //!-- comment
          {
           $name2=preg_replace("![^A-Z0-9]*([A-Z0-9_]+).*!i", "\\1", $tags[4][$z][0]); //get proper name
           $type2="!--open";
          }
          else //!comment
          {
           $name2=$tags[3][$z][0];
           $type2="!open";
          }
         }
         else
         {
          $type2="open";
          $name2=$tags[3][$z][0];
         }

         if($name==$name2&&$type==$type2) //name and type should match
          $closeTags++;
        }
       }

       //close tags only matches if there are enough remaining close tags for the amount of open parents (with same name and type)
       //this ensures highest tag is considered block instead of deepest: <tag><tag/></tag>
       if($openTags<=$closeTags)
       {
        $open=$parents[$x]; //get opening counter part
        $parents=array_slice($parents, 0, $x); //remove all deeper parents

        break; //search done
       }

       unset($openTags, $closeTags);
      }
     }

     if($open!==false) //opening counter part found, add close tag and update counter part meta data
     {
      //store tag meta data
      $this->data[$this->vmax]=Array();
      $this->data[$this->vmax]["name"]=$name;

      if($type=="!--open") //!-- comment
       $this->data[$this->vmax]["type"]="--close";
      else if($type=="!open") //!comment
       $this->data[$this->vmax]["type"]="!close";
      else //normal tag
       $this->data[$this->vmax]["type"]="close";

      $this->data[$this->vmax]["open"]=$open; //add open reference to close tag
      $this->data[$open]["close"]=$this->vmax; //add closing reference to open tag

      //update open tag meta data
      $this->data[$open]["inner"]["start"]=$this->data[$open]["outer"]["start"]+$this->data[$open]["outer"]["length"];
      $this->data[$open]["inner"]["length"]=$tags[0][$c][1]-$this->data[$open]["inner"]["start"];

      $this->data[$open]["outer"]["length"]=($tags[0][$c][1]+strlen($tags[0][$c][0]))-$this->data[$open]["outer"]["start"];

      $this->vmax++;
     }
    }
   }

   //set open tags without close as short tag
   for($c=0;$c<$this->vmax;$c++)
    if(strstr($this->data[$c]["type"], "open")) //open tag
     if(!isset($this->data[$c]["close"])) //no close reference
      $this->data[$c]["type"]=str_replace("open", "short", $this->data[$c]["type"]); //interpret as short tag


   //set parents
   $parents=Array();
   for($c=0;$c<$this->vmax;$c++)
   {
    if(isset($this->data[$c]["close"])) //open tag
    {
     $this->data[$c]["parent"]=end($parents); //set parent to most recent open parent
     $parents[]=$c;
    }
    else if(isset($this->data[$c]["open"])) //close tag
     array_pop($parents); //remove most recent open parent
    else //short tag
     $this->data[$c]["parent"]=end($parents); //set parent to most recent open parent
   }


   //get all #markers# in xml
   if(preg_match_all("!#([^#<>\\n]*)#!si", $data, $markers, PREG_OFFSET_CAPTURE))
   {
    $l1=sizeof($markers[0]);
    for($x=0;$x<$l1;$x++)
    {
     if(!$markers[1][$x][0]) //empty marker (##)
      continue; //skip

     //store market meta data
     $tag=Array();

     $marker=$markers[1][$x][0];

     $tag["name"]=preg_replace("/^([^\s]+).*$/", "\\1", $marker);
     $tag["type"]="marker";
     $tag["attributes"]=Array();

     $tag["outer"]=Array();
     $tag["outer"]["start"]=$markers[0][$x][1];
     $tag["outer"]["length"]=strlen($markers[0][$x][0]);

     //store tag attributes
     if(preg_match_all("!([^\s]+)=\"([^\"]*)\"!", $marker, $attributes)||preg_match_all("!([^\s]+)='([^']*)'!", $marker, $attributes)) //get all key=value pairs
     {
      $l2=sizeof($attributes[0]);
      for($x=0;$x<$l2;$x++)
       $tag["attributes"][$attributes[1][$x]]=$attributes[2][$x];
     }
     else if($marker=trim($marker)) //text/fake attributes
     {
      $tag["attributes"]=explode(" ", $marker);
      array_shift($tag["attributes"]); //remove name from attributes
     }

     //find marker's parent
     $tag["parent"]=false;
     for($c=$this->vmax-1;$c>=0;$c--) //backwards: first tag that value falls in is the parent
     {
      if(!isset($this->data[$c]["outer"])) //all except close tags
       continue;

      if($this->data[$c]["outer"]["start"]<=$tag["outer"]["start"]&&$tag["outer"]["start"]<$this->data[$c]["outer"]["start"]+$this->data[$c]["outer"]["length"]) //marker after tag
      {
       $tag["parent"]=$c;
       break;
      }
     }

     //find new entry for values to be inserted
     $entry=false;
     for($c=($tag["parent"]!==false?($tag["parent"]+1):0);$c<$this->vmax;$c++) //start at parent
     {
      if(!isset($this->data[$c]["outer"])) //all except close tags
      {
       if($this->data[$c]["open"]===$tag["parent"]|| //end of parent
          $this->data[$c]["open"]===$this->data[$tag["parent"]]["parent"]) //end of parent's parent
       {
        $entry=$c; //insert before parent's close tag
        break;
       }

       continue;
      }

      if($tag["outer"]["start"]<$this->data[$c]["outer"]["start"]) //marker before tag
      {
       $entry=$c; //insert before tag
       break;
      }

      if(isset($this->data[$c]["inner"])) //block tag
      {
       $c=$this->data[$c]["close"]; //skip over block tag
       continue;
      }
     }

     if($entry===false)
      $entry=$this->vmax;

     //update parent/open/close references
     for($c=0;$c<$this->vmax;$c++)
     {
      if(isset($this->data[$c]["parent"])&&$this->data[$c]["parent"]>=$entry) //parent beyond current entry
       $this->data[$c]["parent"]++; //increase parent with number of new entries

      if(isset($this->data[$c]["open"])&&$this->data[$c]["open"]>=$entry) //open beyond current entry
       $this->data[$c]["open"]++; //increase open with number of new entries
      else if(isset($this->data[$c]["close"])&&$this->data[$c]["close"]>=$entry) //close beyond current entry
       $this->data[$c]["close"]++; //increase close with number of new entries
     }

     array_splice($this->data, $entry, 0, Array($tag)); //insert new entry in tags
     $this->vmax++;
    }
   }

   return true;
  }


/*
 * Manipulate
 */

  function _set($start, $length, $value)
  {
   $end=$start+$length;

   //validate input
   if(is_array($value)||is_object($value))
   {
    $this->e->add("_Setting array/object values is not supported", "error", $this->filename);
    return false;
   }

   if($start<0)
   {
    $this->e->add("_Set start may not be negative", "error", $this->filename);
    return false;
   }

   if($length<0)
   {
    $this->e->add("_Set length may not be negative", "error", $this->filename);
    return false;
   }

   if($end>strlen($this->output))
   {
    $this->e->add("_Set replace end is beyond end of XML", "error", $this->filename);
    return false;
   }


   //replace everything
   if($start==0&&$length==strlen($this->output))
   {
    if(!$this->parse($value)) //parse new value
     $this->e->clear(); //remove parse errors (no data/tags)

    return true;
   }


   //replace partially
   $diff=strlen($value)-$length; //get difference in old and new length

   $xmlBefore=substr($this->output, 0, $start);
   $xmlAfter=substr($this->output, $end);

   $this->output=$xmlBefore.$value.$xmlAfter; //added value in in xml


   //determine scenario and find at which tag the gap starts (and ends)
   $scenario=false;
   $gapStart=false;
   $gapEnd=false;
   for($c=0;$c<$this->vmax;$c++)
   {
    if(!isset($this->data[$c]["outer"])) //all except close tags
     continue;

    if($gapStart!==false&&$c>$gapStart) //start already found in inner tag, current beyond gap start
     break; //done

    //get tag start
    if($start<=$this->data[$c]["outer"]["start"]) //gap starts before tag start
    {
     $gapStart=$c; //gap start tag found
     $scenario="remove";

     if($end<=$this->data[$c]["outer"]["start"]) //gap ends before tag start
     {
      $gapEnd=$c; //same gap end tag (no tags will be deleted)
      $scenario="before";
     }

     break;
    }

    //gap start after tag start
    if(isset($this->data[$c]["inner"])) //block tag
    {
     //assess jump
     $outerEnd=$this->data[$c]["outer"]["start"]+$this->data[$c]["outer"]["length"];
     if($start>=$outerEnd) //gap starts after close tag
     {
      $c=$this->data[$c]["close"]; //step over entire block tag
      continue;
     }

     //assess open tag
     if($start<$this->data[$c]["inner"]["start"]) //gap starts before open tag end (and start after tag starts)
     {
      if($end>=$this->data[$c]["inner"]["start"]) //gap ends after tag
      {
       $this->e->add("_Set gap breaks out of open tag: \"".$this->data[$c]["name"]."\"", "error", $this->filename);
       return false;
      }

      $gapStart=$c; //gap start tag found
      $gapEnd=$c; //same gap end tag (no tags will be deleted)
      $scenario="open";

      break;
     }

     //assess inner
     $innerEnd=$this->data[$c]["inner"]["start"]+$this->data[$c]["inner"]["length"];
     if($start<=$innerEnd) //gap starts before close tag (and after start tag)
     {
      $gapStart=$this->data[$c]["close"]; //gap start before close tag
      $scenario="after";
      //continue looking
     }
     else
     {
      //assess close tag
      if($innerEnd<$start&&$start<$outerEnd) //gap starts after close tag starts and close tag end
      {
       if($end>=$outerEnd) //gap ends after tag
       {
        $this->e->add("_Set gap breaks out of close tag: \"".$this->data[$c]["name"]."\"", "error", $this->filename);
        return false;
       }

       $gapStart=$this->data[$c]["close"]; //gap start tag found
       $gapEnd=$this->data[$c]["close"]; //same gap end tag (no tags will be deleted)
       $scenario="close";
       break;
      }
     }
    }
    else //short tag
    {
     $outerEnd=$this->data[$c]["outer"]["start"]+$this->data[$c]["outer"]["length"];
     if($start<$outerEnd) //gap starts before tag end (and start after tag starts)
     {
      if($end>=$outerEnd) //gap ends after tag
      {
       $this->e->add("_Set gap breaks out of short tag: \"".$this->data[$c]["name"]."\"", "error", $this->filename);
       return false;
      }

      $gapStart=$c; //gap start tag found
      $gapEnd=$c; //same gap end tag (no tags will be deleted)
      $scenario="short";

      break;
     }
    }
   }


   //delete tags / markers
   if($length&&$gapStart!==false)
   {
    if($gapEnd===false) //find gap end
    {
     $gapEnd=$this->vmax; //gap ends at end for now
     for($c=$gapStart;$c<$gapEnd;$c++)
     {
      if(!isset($this->data[$c]["outer"])) //close tag
      {
       //get tag start
       $open=$this->data[$c]["open"];
       $innerEnd=$this->data[$open]["inner"]["start"]+$this->data[$open]["inner"]["length"];
       if($end<=$innerEnd) //gap ends before close tag starts
       {
        $gapEnd=$c; //gap end tag found
        break;
       }
      }
      else //short/open tag
      {
       //get tag start
       if($end<=$this->data[$c]["outer"]["start"]) //gap ends before tag start
       {
        $gapEnd=$c; //gap end tag found
        break;
       }

       //gap ends after tag start
       if(isset($this->data[$c]["inner"])) //block tag
       {
        //assess jump
        $outerEnd=$this->data[$c]["outer"]["start"]+$this->data[$c]["outer"]["length"];
        if($end>=$outerEnd) //gap ends after close tag
         $c=$this->data[$c]["close"]; //step over entire block tag
       }
      }
     }
    }

    if($gapStart!=$gapEnd) //at least one whole tag is removed
    {
     $deleteStart=$gapStart;
     $deleteEnd=$gapEnd;
    }
    else //check if markers in tag are removed
    {
     $deleteStart=false;
     $deleteEnd=false;
     if($scenario=="open"||$scenario=="short") //open/short tag changed
     {
      for($c=$gapStart+1;$c<$this->vmax;$c++) //assess all markers in tag
      {
       if($this->data[$c]["type"]!="marker")
        break;

       if($end<$this->data[$c]["outer"]["start"]) //stop at markers after end position
        break;

       if($this->data[$c]["outer"]["start"]<$start) //ignore markers before start position
        continue;

       if($deleteStart===false)
        $deleteStart=$c;

       $deleteEnd=$c;
      }

      if($deleteStart!==false) //marker(s) delete
       $deleteEnd++; //correct end
     }
    }

    if($deleteStart!=$deleteEnd) //at least one whole tag or marker is removed
    {
     $deleteLength=$deleteEnd-$deleteStart;

     //update parents references
     for($c=$deleteEnd;$c<$this->vmax;$c++) //assess all tags after gap
      if(isset($this->data[$c]["parent"])&&$this->data[$c]["parent"]>=$deleteEnd) //parent beyond current entry
       $this->data[$c]["parent"]-=$deleteLength; //reduce parent

     //update open/close references
     for($c=0;$c<$this->vmax;$c++) //assess all tags
     {
      if(isset($this->data[$c]["open"])&&$this->data[$c]["open"]>=$deleteEnd) //open beyond current entry
       $this->data[$c]["open"]-=$deleteLength; //reduce open
      else if(isset($this->data[$c]["close"])&&$this->data[$c]["close"]>=$deleteEnd) //close beyond current entry
       $this->data[$c]["close"]-=$deleteLength; //reduce close
     }

     //remove tags in gap
     array_splice($this->data, $deleteStart, $deleteLength); //remove entry from tags
     $this->vmax=sizeof($this->data); //update tags counter
    }
   }


   //update positions
   if($diff&&$gapStart<$this->vmax)
   {
    switch($scenario)
    {
     case "before":
     {
      $updateParent=$this->data[$gapStart]["parent"];
      $updateStart=$gapStart;
      break;
     }
     case "after":
     {
      $updateParent=$this->data[$gapStart]["open"]; //use open tag
      $updateStart=$gapStart;
      break;
     }
     case "remove":
     {
      if(!isset($this->data[$gapStart]["outer"])) //close tag
       $updateParent=$this->data[$gapStart]["open"]; //use open tag
      else //short / open
       $updateParent=$this->data[$gapStart]["parent"];

      $updateStart=$gapStart;
      break;
     }
     case "open":
     {
      $updateParent=$this->data[$gapStart]["parent"];
      $updateStart=$gapStart+1;

      $this->data[$gapStart]["outer"]["length"]+=$diff; //update open tag's outer length
      $this->data[$gapStart]["inner"]["start"]+=$diff; //update open tag's inner start

      break;
     }
     case "close":
     {
      $open=$this->data[$gapStart]["open"];
      $updateParent=$this->data[$open]["parent"]; //use open tag's parent
      $updateStart=$gapStart;

      $this->data[$open]["outer"]["length"]+=$diff; //update open tag's outer length
      break;
     }
     case "short":
     {
      $updateParent=$this->data[$gapStart]["parent"];
      $updateStart=$gapStart+1;

      $this->data[$gapStart]["outer"]["length"]+=$diff; //update open tag's outer length
      break;
     }
     default:
     {
      $updateParent=false;
      $updateStart=false;
      break;
     }
    }

    //update parent lengths
    if($updateParent!==false)
    {
     while($updateParent!==false&&($this->data[$updateParent]["parent"]===false||$this->data[$updateParent]["parent"]<$updateParent)) //protect against faulty parent references
     {
      $this->data[$updateParent]["outer"]["length"]+=$diff;
      $this->data[$updateParent]["inner"]["length"]+=$diff;

      $updateParent=$this->data[$updateParent]["parent"]; //next parent
     }
    }

    //update subsequent start positions
    if($updateStart!==false)
    {
     for($c=$updateStart;$c<$this->vmax;$c++)
     {
      if(!isset($this->data[$c]["outer"])) //all except close tags
       continue;

      if($this->data[$c]["outer"]["start"]<$start) //only update tags after start position (updateStart may include markers in open/short that are before the start position)
       continue;

      $this->data[$c]["outer"]["start"]+=$diff; //update tag start

      if(isset($this->data[$c]["inner"])) //block tag
       $this->data[$c]["inner"]["start"]+=$diff; //update tag inner start
     }
    }
   }


   //update tag name and attributes
   if($scenario=="open"||$scenario=="short")
   {
    $name=false;
    if($this->data[$gapStart]["type"]=="marker")
    {
     if(preg_match("/([^\/\s\\n\>\#]+)([^\\n\#]*)\#/", $this->output, $m, 0, $this->data[$gapStart]["outer"]["start"]+1))
     {
      $name=$m[1];
      $attr=$m[2];
     }
    }
    else
    {
     if(preg_match("/([!\-\s]*)([^\/\s\\n\>\#]+)([^>]*)>/", $this->output, $m, 0, $this->data[$gapStart]["outer"]["start"]+1))
     {
      $name=$m[2];
      $attr=$m[3];
     }
    }

    if($name)
    {
     $this->data[$gapStart]["name"]=$name;
     $this->data[$gapStart]["attributes"]=Array();

     if(preg_match_all("!([^\s]+)=\"([^\"]*)\"!", $attr, $attributes)) //get all key=value pairs
     {
      $l=sizeof($attributes[0]);
      for($c=0;$c<$l;$c++)
       $this->data[$gapStart]["attributes"][$attributes[1][$c]]=$attributes[2][$c];
     }
     else if($attr=trim($attr)) //text/fake attributes
     {
      $this->data[$gapStart]["attributes"]=explode(" ", $attr); //space split values

      if($this->data[$gapStart]["type"]=="!--open"||$this->data[$gapStart]["type"]=="!--short")
      {
       array_shift($this->data[$gapStart]["attributes"]); //remove name from attributes
       array_pop($this->data[$gapStart]["attributes"]); //remove -- tag end from attributes
      }
     }
    }
   }
   else if($scenario=="close")
   {
    $innerEnd=$this->getPosition($this->data[$gapStart]["open"], "inner", "end");
    if(preg_match("/([!\-\s]*)\/([^\/\s\\n\>\#]+)/", $this->output, $m, 0, $innerEnd+1))
     $this->data[$gapStart]["name"]=$m[2];
   }


   //new tags/markers inserted
   if(preg_match("/(<".($this->onlyCommentsAndMarkers?"!":"")."|\#)/", $value))
   {
    if($gapStart===false||$gapStart>=$this->vmax)
    {
     $parent=false;
     $offset=$this->vmax; //at end
    }
    else if(!isset($this->data[$gapStart]["outer"])) //close tag
    {
     $parent=$this->data[$gapStart]["open"]; //use open tag as parent
     $offset=$gapStart; //after gap start
    }
    else if($scenario=="open"||$scenario=="short") //open/short tag changed
    {
     $parent=$gapStart;
     $offset=$gapStart+1;

     //determine offset more accurately based on markers
     for($c=$gapStart+1;$c<$this->vmax;$c++) //assess all markers in tag
     {
      if($this->data[$c]["type"]!="marker")
       break;

      if($this->data[$c]["outer"]["start"]<$start) //ignore markers before start position
       continue;

      $offset=$c;
     }
    }
    else //block/short tag
    {
     $parent=$this->data[$gapStart]["parent"];
     $offset=$gapStart; //after gap start
    }

    //get next tags
    $xml=new Format_XML($this->onlyCommentsAndMarkers, $this->exactPath);
    $xml->parse($value);

    //update parent/open/close for new tags
    for($c=0;$c<$xml->vmax;$c++)
    {
     if(isset($xml->data[$c]["parent"])) //set new parents
     {
      if($xml->data[$c]["parent"]===false) //directly under parent
       $xml->data[$c]["parent"]=$parent;
      else //under new entry
       $xml->data[$c]["parent"]+=$offset; //increase parent with gap position
     }

     if(isset($xml->data[$c]["open"])) //set new open references
      $xml->data[$c]["open"]+=$offset; //increase open with gap position
     else if(isset($xml->data[$c]["close"])) //set new close references
      $xml->data[$c]["close"]+=$offset; //increase close with gap position
    }

    //prepare start positions for new tags
    if($start)
    {
     for($c=0;$c<$xml->vmax;$c++)
     {
      if(isset($xml->data[$c]["outer"]))  //set outer positions
       $xml->data[$c]["outer"]{"start"}+=$start;

      if(isset($xml->data[$c]["inner"]))  //set inner positions
       $xml->data[$c]["inner"]{"start"}+=$start;
     }
    }

    $newEntries=$xml->vmax;

    //update parent for existing tags after new tags
    for($c=$offset+1;$c<$this->vmax;$c++)
     if(isset($this->data[$c]["parent"])&&$this->data[$c]["parent"]!==false&&$this->data[$c]["parent"]>=$offset) //parent beyond gap
      $this->data[$c]["parent"]+=$newEntries; //increase parent with gap position

    //update open/close for existing tags after new tags
    for($c=0;$c<$this->vmax;$c++)
    {
     if(isset($this->data[$c]["open"])&&$this->data[$c]["open"]>=$offset) //open beyond gap
      $this->data[$c]["open"]+=$newEntries; //increase open with number of new entries
     else if(isset($this->data[$c]["close"])&&$this->data[$c]["close"]>=$offset) //close beyond gap
      $this->data[$c]["close"]+=$newEntries; //increase close with number of new entries
    }

    array_splice($this->data, $offset, 0, $xml->data); //insert new entries after gap
    $this->vmax=sizeof($this->data); //update tags counter
   }

   return true;
  }


  function _shortToBlock($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if($this->data[$key]["type"]!="short"&&$this->data[$key]["type"]!="!short")
   {
    $this->e->add("Can't convert non short tag: \"".$this->getPath($key)."\" to block tag", "warning", $this->filename);
    return false;
   }

   $type=str_replace("short", "", $this->data[$key]["type"]);
   $value=preg_replace("!\s*/?>$!", ">", $this->getOuter($key));;
   $value.="<".$type."/".$this->data[$key]["name"].">";

   return $this->_set($this->data[$key]["outer"]["start"], $this->data[$key]["outer"]["length"], $value); //replace tag
  }


  function _renderArray($key, $value)
  {
   if(!is_array($value))
    return false;

   //multiple values handling: try to set all values in the inner of the tag and use the new inner as value
   if(array_type($value)=="indexed") //indexed, multiple entries: set all entries in array in inner
   {
    if($key===false) //fake root tag
     $xml=clone($this);
    else if(($xml=$this->getOuterXML($key))===false)
     return false;

    $subPath=preg_replace("!\[\d+\]$!", "", $xml->getPath(0));

    $l=sizeof($value);
    for($c=0;$c<$l;$c++)
     $xml->setBefore($subPath, $value[$c]); //set before (keep tag for next iteration)

    $xml->delete($subPath);
   }
   else //associative array: replace tags/marks that match keys with values
   {
    if($key===false) //fake root tag
     $xml=clone($this);
    else if(($xml=$this->getInnerXML($key))===false)
     return false;

    $xml->exactPath=false;

    foreach($value as $k => $v) //walk through array to replace array based values
    {
     if(is_array($v))
     {
      //to ensure newly inserted tags aren't replace (i.e. endless loop) only existing entries will be replaced
      $entries=0; //find number of entries with $k key
      for($c=0;$c<$xml->vmax;$c++)
       if($xml->data[$c]["name"]==$k) //tag matches
        if(!strstr($xml->data[$c]["type"], "close")) //skip close tags
         $entries++;

      for($c=$entries-1;$c>=0;$c--) //replace number of entries backwards
      {
       if(strstr($xml->getType($k."[".$c."]"), "open"))
        $xml->setInside($k."[".$c."]", ($v?$v:"")); //wrap tag around value
       else //short tag
        $xml->setReplace($k."[".$c."]", ($v?$v:"")); //replace tag
      }
     }
    }

    foreach($value as $k => $v) //walk through array to replace normal values
    {
     if(!is_array($v))
     {
      //to ensure newly inserted tags aren't replace (i.e. endless loop) only existing entries will be replaced
      $entries=0; //find number of entries with $k key
      for($c=0;$c<$xml->vmax;$c++)
       if($xml->data[$c]["name"]==$k) //tag matches
        if(!strstr($xml->data[$c]["type"], "close")) //skip close tags
         $entries++;

      for($c=$entries-1;$c>=0;$c--) //replace number of entries backwards
       $xml->setReplace($k."[".$c."]", $v); //replace tags
     }
    }
   }

   return $xml->output;
  }

  function setReplace($path, $value)
  {
   if($path===false) //fake root tag
    $key=$path;
   else if(($key=$this->getKey($path))===false)
    return false;

   if(is_array($value)) //render array values
    $value=$this->_renderArray($key, $value);

   if($path===false) //fake root tag
    return $this->_set(0, strlen($this->output), $value); //replace full xml

   return $this->_set($this->data[$key]["outer"]["start"], $this->data[$key]["outer"]["length"], $value); //replace tag
  }

  function setBefore($path, $value)
  {
   if($path===false) //fake root tag
    $key=$path;
   else if(($key=$this->getKey($path))===false)
    return false;

   if(is_array($value)) //render array values
    $value=$this->_renderArray($key, $value);

   if($path===false) //fake root tag
    return $this->_set(0, 0, $value); //add at begin of xml

   return $this->_set($this->data[$key]["outer"]["start"], 0, $value); //add before tag
  }

  function setAfter($path, $value)
  {
   if($path===false) //fake root tag
    $key=$path;
   else if(($key=$this->getKey($path))===false)
    return false;

   if(is_array($value)) //render array values
    $value=$this->_renderArray($key, $value);

   if($path===false) //fake root tag
    return $this->_set(strlen($this->output), 0, $value); //add at end of xml

   return $this->_set($this->data[$key]["outer"]["start"]+$this->data[$key]["outer"]["length"], 0, $value); //add after tag
  }

  function setInside($path, $value)
  {
   if($path===false) //fake root tag
    $key=$path;
   else if(($key=$this->getKey($path))===false)
    return false;

   if(is_array($value)) //render array values
    $value=$this->_renderArray($key, $value);

   if($path===false) //fake root tag
    return $this->_set(0, strlen($this->output), $value); //replace full xml

   if(!strstr($this->data[$key]["type"], "open"))
   {
    if($this->data[$key]["type"]!="short"&&$this->data[$key]["type"]!="!short")
    {
     $this->e->add("Can't set inside non block tag: \"".$this->getPath($key)."\"", "warning", $this->filename);
     return false;
    }

    if(!$this->_shortToBlock($key))
     return false;
   }

   return $this->_set($this->data[$key]["inner"]["start"], $this->data[$key]["inner"]["length"], $value); //replace inner tag
  }

  function setInsideCopy($path, $value)
  {
   if($path===false) //fake root tag
    $key=false;
   else if(($key=$this->getKey($path))===false)
    return false;

   if(is_array($value)) //render array values
    $value=$this->_renderArray($key, $value);

   if($path===false) //fake root tag
    return $this->_set(0, 0, $value); //add at begin of xml

   if(strstr($this->data[$key]["type"], "open"))
   {
    //wrap tag around value
    $position=$this->getPosition($key);
    $xmlBefore=substr($this->output, $position["outer"]["start"], $position["inner"]["start"]-$position["outer"][ "start"]);
    $xmlAfter=substr($this->output, $position["inner"]["end"], $position["outer"]["end"]-$position["inner"]["end"]);
    $value=$xmlBefore.$value.$xmlAfter;
   }
   else if($this->data[$key]["type"]=="short"||$this->data[$key]["type"]=="!short")
   {
    //wrap tag around value
    $type=str_replace("short", "", $this->data[$key]["type"]);
    $value=$this->getOuter($key).$value."<".$type."/".$this->data[$key]["name"].">";
   }
   else
   {
    $this->e->add("Can't set inside non block tag: \"".$this->getPath($key)."\"", "warning", $this->filename);
    return false;
   }

   return $this->_set($this->data[$key]["outer"]["start"], 0, $value); //add before tag
  }


  function addBefore($path, $value)
  {
   if($path===false) //fake root tag
    return $this->_set(0, 0, $value); //add at begin of xml

   if(($key=$this->getKey($path))===false)
    return false;

   return $this->_set($this->data[$key]["outer"]["start"], 0, $value);
  }

  function addAfter($path, $value)
  {
   if($path===false) //fake root tag
    return $this->_set(strlen($this->output), 0, $value); //add at end of xml

   if(($key=$this->getKey($path))===false)
    return false;

   return $this->_set($this->getPosition($key, "outer", "end"), 0, $value);
  }

  function addInsideAtBegin($path, $value)
  {
   if($path===false) //fake root tag
    return $this->_set(0, 0, $value); //add at begin of xml

   if(($key=$this->getKey($path))===false)
    return false;

   if(!strstr($this->data[$key]["type"], "open"))
   {
    if($this->data[$key]["type"]!="short"&&$this->data[$key]["type"]!="!short")
    {
     $this->e->add("Can't add inside non block tag: \"".$this->getPath($key)."\"", "warning", $this->filename);
     return false;
    }

    if(!$this->_shortToBlock($key))
     return false;
   }

   return $this->_set($this->data[$key]["inner"]["start"], 0, $value);
  }

  function addInsideAtEnd($path, $value)
  {
   if($path===false) //fake root tag
    return $this->_set(strlen($this->output), 0, $value); //add at end of xml

   if(($key=$this->getKey($path))===false)
    return false;

   if(!strstr($this->data[$key]["type"], "open"))
   {
    if($this->data[$key]["type"]!="short"&&$this->data[$key]["type"]!="!short")
    {
     $this->e->add("Can't add inside non block tag: \"".$this->getPath($key)."\"", "warning", $this->filename);
     return false;
    }

    if(!$this->_shortToBlock($key))
     return false;
   }

   return $this->_set($this->getPosition($key, "inner", "end"), 0, $value);
  }


  function setAttribute($path, $attribute, $value=false)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if(
    !strstr($this->data[$key]["type"], "open")&&
    !strstr($this->data[$key]["type"], "short")&&
    !strstr($this->data[$key]["type"], "marker")
   )
   {
    $this->e->add("Setting attributes is not supported for \"".$this->data[$key]["type"]."\" tags", "error", $this->filename);
    return false;
   }

   if($value!==false)
    $value=htmlentities($value, ENT_COMPAT|ENT_XML1);

   if($this->data[$key]["attributes"])
    $hasAttributes=true;
   else
    $hasAttributes=true;

   $found=false;
   $attributes="";
   foreach($this->data[$key]["attributes"] as $k => $v)
   {
    if($k==$attribute)
    {
     if($value!==false) //edit
     {
      $attributes.=" ".$attribute."=\"".$value."\"";
      $this->data[$key]["attributes"][$attribute]=$value;
     }
     else //delete
      unset($this->data[$key]["attributes"][$attribute]);

     $found=true;
    }
    else
     $attributes.=" ".$k."=\"".$v."\"";
   }

   if(!$found&&$value!==false)
   {
    $attributes.=" ".$attribute."=\"".$value."\"";
    $this->data[$key]["attributes"][$attribute]=$value;
   }

   if(strstr($this->data[$key]["type"], "!--"))
   {
    $outer=$this->getPosition($key, "outer");
    $start=($outer["start"]+4+strlen($this->data[$key]["name"]))+1;
    $length=($outer["end"]-$start)-4;
   }
   else if(strstr($this->data[$key]["type"], "!"))
   {
    $outer=$this->getPosition($key, "outer");
    $start=($outer["start"]+strlen($this->data[$key]["name"]))+2;
    $length=($outer["end"]-$start)-1;
   }
   else if(strstr($this->data[$key]["type"], "open"))
   {
    $start=($this->data[$key]["outer"]["start"]+strlen($this->data[$key]["name"]))+1;
    $length=($this->data[$key]["inner"]["start"]-$start)-1;
   }
   else if(strstr($this->data[$key]["type"], "short"))
   {
    $outer=$this->getPosition($key, "outer");
    $start=($outer["start"]+strlen($this->data[$key]["name"]))+1;
    $length=($outer["end"]-$start)-1;

    if(($outer=$this->getOuter($key)))
    {
     if(substr($outer, -3)==" />"||substr($outer, -3)==" ?>")
      $length-=2;
     else if(substr($outer, -2)=="/>"||substr($outer, -2)=="?>")
      $length--;
    }
   }
   else if(strstr($this->data[$key]["type"], "marker"))
   {
    $outer=$this->getPosition($key, "outer");
    $start=($outer["start"]+strlen($this->data[$key]["name"]))+1;
    $length=($outer["end"]-$start)-1;
   }
   else
   {
    $this->e->add("Setting attributes is not supported for \"".$this->data[$key]["type"]."\" tags", "error", $this->filename);
    return false;
   }

   return $this->_set($start, $length, $attributes);
  }


  function rename($path, $name)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if(strstr($this->data[$key]["type"], "close"))
   {
    $this->e->add("Renaming tag is not supported for \"".$this->data[$key]["type"]."\" tags", "error", $this->filename);
    return false;
   }

   if(strstr($this->data[$key]["type"], "!--"))
    $startOffset=5;
   else if(strstr($this->data[$key]["type"], "!"))
    $startOffset=2;
   else //block/short
    $startOffset=1;

   $start=$this->data[$key]["outer"]["start"]+$startOffset;
   $length=strlen($this->data[$key]["name"]);

   if(!$this->_set($start, $length, $name)) //update xml
    return false;

   if(strstr($this->data[$key]["type"], "open")) //update close tag for block tags
   {
    $start=$this->getPosition($key, "inner", "end")+$startOffset+1;

    if(!$this->_set($start, $length, $name)) //update xml
     return false;
   }

   return true;
  }

  function blank($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if(!strstr($this->data[$key]["type"], "open"))
   {
    $this->e->add("Can't blank non block tag: \"".$this->getPath($key)."\"", "warning", $this->filename);
    return false;
   }

   return $this->_set($this->data[$key]["inner"]["start"], $this->data[$key]["inner"]["length"], ""); //replace inner tag
  }

  function delete($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   return $this->_set($this->data[$key]["outer"]["start"], $this->data[$key]["outer"]["length"], ""); //replace outer tag
  }



/*
 * Info
 */

  function getPathAndKey($path)
  {
   if(is_numeric($path))
   {
    if(!isset($this->data[$path]))
    {
     $this->e->add("Unknown tag key: \"".$path."\"", "warning", $this->filename);
     return false;
    }

    $key=$path;
    $path=$this->getPath($key);
   }
   else if(($key=$this->getKey($path))===false)
   {
    $this->e->add("Unknown tag: \"".$path."\"", "warning", $this->filename);
    return false;
   }

   return Array($path, $key);
  }


  function getKey($path)
  {
   if(is_numeric($path))
   {
    if(!isset($this->data[$path]))
    {
     $this->e->add("Unknown tag key: \"".$path."\"", "warning", $this->filename);
     return false;
    }

    return $path;
   }

   return $this->_getKey($path);
  }

  function _getKey($path, $parent=false)
  {
   if(!is_array($path))
    $path=explode("/", $path);

   $name=array_shift($path);
   $tmp=explode("#", $name);

   //find ID: html/body/div#id
   if(isset($tmp[1]))
   {
    $name=$tmp[0];
    $tagID=$tmp[1];
    $tagNR=false;
   }
   else
   {
    $tagID=false;

    //find tag number:
    //html/body/div[0]: first div in body, same as html/body/div
    //html/body/div[1]: second div in body
    //html/body/div[2]: third...
    if(preg_match("/(.+)\[(\d+)\]/", $name, $tmp))
    {
     $name=$tmp[1];
     $tagNR=$tmp[2];
    }
    else
     $tagNR=false;
   }

   $key=false;
   $tagMatches=0;
   for($c=0;$c<$this->vmax;$c++)
   {
    if(!strstr($this->data[$c]["type"], "close"))
    {
     if(((!$this->exactPath&&$parent===false)||$this->data[$c]["parent"]===$parent)&&$this->data[$c]["name"]===$name)
     {
      if($tagID) //check tag id
      {
       if(!isset($this->data[$c]["attributes"]["id"])||$this->data[$c]["attributes"]["id"]!=$tagID) //tag id doesn't match
        continue; //try next
      }
      else if($tagNR) //check tag number, 0 can be ignored here
      {
       if($tagNR!=$tagMatches) //not the same tag nr
       {
        $tagMatches++; //next match
        continue; //try next
       }
      }

      //found
      if(sizeof($path))
       $key=$this->_getKey($path, $c);
      else
       $key=$c;

      break;
     }
    }
   }

   return $key;
  }


  function getName($key)
  {
   if(!isset($this->data[$key]))
   {
    $this->e->add("Unknown tag key: \"".$key."\"", "warning", $this->filename);
    return false;
   }

   return $this->data[$key]["name"];
  }


  function getPath($key)
  {
   if(!isset($this->data[$key]))
   {
    $this->e->add("Unknown tag key: \"".$key."\"", "warning", $this->filename);
    return false;
   }

   if(strstr($this->data[$key]["type"], "close"))
   {
    $this->e->add("Can't get path for close tag: \"".$key."\"", "warning", $this->filename);
    return false;
   }

   $entry=0;
   for($c=0;$c<$key;$c++)
   {
    if(isset($this->data[$c]["parent"])&&$this->data[$c]["parent"]===$this->data[$key]["parent"]) //same parent
     if($this->data[$c]["name"]==$this->data[$key]["name"])
      $entry++;
   }

   $path=$this->data[$key]["name"]."[".$entry."]";

   if(!strstr($this->data[$key]["type"], "close"))
   {
    if($this->data[$key]["parent"]!==false)
    {
     $entry=0;
     for($c=0;$c<$key;$c++)
     {
      if(isset($this->data[$c]["parent"])&&$this->data[$c]["parent"]===$this->data[$key]["parent"]) //same parent
       if($this->data[$c]["name"]==$this->data[$key]["name"])
        $entry++;
     }

     $path=$this->getPath($this->data[$key]["parent"])."/".$path;
    }
   }

   return $path;
  }


  function getInner($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if(!strstr($this->data[$key]["type"], "open"))
   {
    $this->e->add("Can't get inner for non block tag: \"".$this->getPath($key)."\"", "warning", $this->filename);
    return false;
   }

   return substr($this->output, $this->data[$key]["inner"]["start"], $this->data[$key]["inner"]["length"]);
  }


  function getInnerXML($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   //copy object
   $xml=new Format_XML($this->onlyCommentsAndMarkers, $this->exactPath);
   $xml->vmax=0;
   $xml->reset();
   $xml->data=Array();

   //get XML
   if(!($xml->output=$this->getInner($key)))
    return false;

   $xml->original=$xml->output;

   //copy data
   $entry=$key+1;
   $close=$this->data[$key]["close"];
   for($c=$entry;$c<$close;$c++)
   {
    $xml->data[$xml->vmax]=$this->data[$c];

    if(isset($xml->data[$xml->vmax]["parent"])) //set new parents
    {
     if($xml->data[$xml->vmax]["parent"]===$key) //directly under parent
      $xml->data[$xml->vmax]["parent"]=false;
     else //under new entry
      $xml->data[$xml->vmax]["parent"]-=$entry; //decrease parent with number of entries
    }

    if(isset($xml->data[$xml->vmax]["open"])) //set new open references
     $xml->data[$xml->vmax]["open"]-=$entry; //decrease open with number of entries
    else if(isset($xml->data[$xml->vmax]["close"])) //set new close references
     $xml->data[$xml->vmax]["close"]-=$entry; //decrease close with number of entries

    $xml->vmax++;
   }

   //adjust start positions
   $start=$this->data[$key]["inner"]["start"];
   for($c=0;$c<$xml->vmax;$c++)
   {
    if(isset($xml->data[$c]["outer"]))  //set outer positions
     $xml->data[$c]["outer"]{"start"}-=$start;

    if(isset($xml->data[$c]["inner"]))  //set inner positions
     $xml->data[$c]["inner"]{"start"}-=$start;
   }

   return $xml;
  }


  function getOuter($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if(strstr($this->data[$key]["type"], "close"))
   {
    $this->e->add("Can't get outer for close tag: \"".$this->getPath($key)."\"", "warning", $this->filename);
    return false;
   }

   return substr($this->output, $this->data[$key]["outer"]["start"], $this->data[$key]["outer"]["length"]);
  }


  function getOuterXML($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   //copy object
   $xml=new Format_XML($this->onlyCommentsAndMarkers, $this->exactPath);
   $xml->vmax=0;
   $xml->reset();
   $xml->data=Array();

   //get XML
   if(!($xml->output=$this->getOuter($key)))
    return false;

   $xml->original=$xml->output;

   //copy data
   $entry=$key;
   $parent=$this->data[$key]["parent"];

   if(isset($this->data[$key]["close"])) //block tag
    $close=$this->data[$key]["close"]+1;
   else //short tag
    $close=$entry+1;

   for($c=$entry;$c<$close;$c++)
   {
    $xml->data[$xml->vmax]=$this->data[$c];

    if(isset($xml->data[$xml->vmax]["parent"])) //set new parents
    {
     if($xml->data[$xml->vmax]["parent"]===$parent) //directly under parent
      $xml->data[$xml->vmax]["parent"]=false;
     else //under new entry
      $xml->data[$xml->vmax]["parent"]-=$entry; //decrease parent with number of entries
    }

    if(isset($xml->data[$xml->vmax]["open"])) //set new open references
     $xml->data[$xml->vmax]["open"]-=$entry; //decrease open with number of entries
    else if(isset($xml->data[$xml->vmax]["close"])) //set new close references
     $xml->data[$xml->vmax]["close"]-=$entry; //decrease close with number of entries

    $xml->vmax++;
   }

   //adjust start positions
   $start=$this->data[$key]["outer"]["start"];
   for($c=0;$c<$xml->vmax;$c++)
   {
    if(isset($xml->data[$c]["outer"]))  //set outer positions
     $xml->data[$c]["outer"]{"start"}-=$start;

    if(isset($xml->data[$c]["inner"]))  //set inner positions
     $xml->data[$c]["inner"]{"start"}-=$start;
   }

   return $xml;
  }


  function getAttributes($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if(!isset($this->data[$key]["attributes"]))
    return false;

   $attributes=Array();
   foreach($this->data[$key]["attributes"] as $k => $v)
    $attributes[$k]=html_entity_decode($v, ENT_QUOTES|ENT_XML1);

   return $attributes;
  }


  function getAttribute($path, $attribute)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if(!isset($this->data[$key]["attributes"])||!isset($this->data[$key]["attributes"][$attribute]))
    return false;

   return html_entity_decode($this->data[$key]["attributes"][$attribute], ENT_QUOTES|ENT_XML1);
  }


  function getChildren($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   $childs=Array();
   for($c=0;$c<$this->vmax;$c++)
    if(isset($this->data[$c]["parent"]))
     if($this->data[$c]["parent"]===$key)
      $childs[]=$c;

   return $childs;
  }


  function getType($path)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   return $this->data[$key]["type"];
  }


  function getPosition($path, $category=false, $type=false)
  {
   if(($key=$this->getKey($path))===false)
    return false;

   if(!isset($this->data[$key]["outer"]))
   {
    $this->e->add("No outer position for tag: \"".$this->getPath($key)."\"", "error", $this->filename);
    return false;
   }

   if($category=="inner"&&!isset($this->data[$key]["inner"]))
   {
    $this->e->add("No inner position for tag: \"".$this->getPath($key)."\"", "error", $this->filename);
    return false;
   }

   if(isset($this->data[$key][$category])) //outer/inner position(s)
   {
    switch($type)
    {
     case "start":
      return $this->data[$key][$category]["start"];

     case "length":
      return $this->data[$key][$category]["length"];

     case "end":
      return $this->data[$key][$category]["start"]+$this->data[$key][$category]["length"];

     case false: //all outer/inner positions
     {
      return Array(
       "start" => $this->data[$key][$category]["start"],
       "length" => $this->data[$key][$category]["length"],
       "end" => $this->data[$key][$category]["start"]+$this->data[$key][$category]["length"],
      );
     }

     default: break;
    }
   }
   else if(!$category) //all position categories
   {
    switch($type)
    {
     case "start":
     {
      $position=Array("outer" => $this->data[$key]["outer"]["start"]);

      if(isset($this->data[$key]["inner"]))
       $position["inner"]=$this->data[$key]["inner"]["start"];

      return $position;
     }

     case "length":
     {
      $position=Array("outer" => $this->data[$key]["outer"]["length"]);

      if(isset($this->data[$key]["inner"]))
       $position["inner"]=$this->data[$key]["inner"]["length"];

      return $position;
     }

     case "end":
     {
      $position=Array("outer" => $this->data[$key]["outer"]["start"]+$this->data[$key]["outer"]["length"]);

      if(isset($this->data[$key]["inner"]))
       $position["inner"]=$this->data[$key]["inner"]["start"]+$this->data[$key]["inner"]["length"];

      return $position;
     }

     case false: //all outer/inner positions
     {
      $position=Array("outer" => Array(
       "start" => $this->data[$key]["outer"]["start"],
       "length" => $this->data[$key]["outer"]["length"],
       "end" => $this->data[$key]["outer"]["start"]+$this->data[$key]["outer"]["length"],
      ));

      if(isset($this->data[$key]["inner"]))
       $position["inner"]=Array(
        "start" => $this->data[$key]["inner"]["start"],
        "length" => $this->data[$key]["inner"]["length"],
        "end" => $this->data[$key]["inner"]["start"]+$this->data[$key]["inner"]["length"],
       );

      return $position;
     }

     default: break;
    }
   }

   $this->e->add("Unknown position category: \"".$category."\" and type: \"".$type."\" for tag: \"".$this->getPath($key)."\"", "error", $this->filename);
   return false;
  }


/*
 * Iterate
 */

  function current($path)
  {
   if(($key=$this->getKey($path))===false)
   {
    $this->e->add("Unknown tag: \"".$path."\"", "warning", $this->filename);
    return false;
   }

   if(!$this->value($key))
    return false;

   $this->vc=$key+1;
   return $key;
  }

  function next($comment=true, $marker=true, $close=false)
  {
   if($this->vc>=$this->vmax)
   {
    $this->reset();
    return false;
   }

   while($this->vc<$this->vmax)
   {
    if($this->value($this->vc, $comment, $marker, $close))
     return $this->vc++;

    $this->vc++;
   }

   $this->reset();
   return false;
  }

  function previous($comment=true, $marker=true, $close=false)
  {
   if($this->vc>=$this->vmax)
   {
    $this->reset();
    return false;
   }

   while($this->vc<$this->vmax)
   {
    if($this->value($this->vmax-($this->vc+1), $comment, $marker, $close))
     return $this->vmax-(++$this->vc);

    $this->vc++;
   }


   $this->reset();
   return false;
  }

  function search($tag, $comment=true, $marker=true, $close=false)
  {
   $tag=strtolower($tag);

   if($this->vc>=$this->vmax)
   {
    $this->reset();
    return false;
   }

   while($this->vc<$this->vmax)
   {
    if(strtolower($this->data[$this->vc]["name"])==$tag) //tag matches
     if($this->value($this->vc, $comment, $marker, $close))
      return $this->vc++;

    $this->vc++;
   }


   $this->reset();
   return false;
  }

  function value($nr, $comment=true, $marker=true, $close=false)
  {
//illegal entry
   if($nr<0||$nr>=$this->vmax)
   {
    $this->v=false;
    return false;
   }

   if(!$comment&&preg_match("/^(\!|\-\-)/", $this->data[$nr]["type"])) //skip comment tags + comment close tags
    return false;

   if(!$marker&&strstr($this->data[$nr]["type"], "marker")) //skip marker tags
    return false;

   if(!$close&&strstr($this->data[$nr]["type"], "close")) //skip close tags
    return false;

   $this->v=$this->data[$nr];
   return true;
  }


/*
 * Output
 */

  function display($data=false)
  {
   if($data)
   {
    $output="<tr><th>#</th><th>Name</th><th>Type</th><th>Attributes</th><th colspan=\"5\">Outer</th><th colspan=\"5\">Inner</th><th>Parent</th><th>Open/Close</th></tr>\n";

    for($c=0;$c<$this->vmax;$c++)
    {
     $output.="<tr><td>".$c."</td><td>".$this->data[$c]["name"]."</td><td>".$this->data[$c]["type"]."</td>";

     $output.="<td>";
     if(isset($this->data[$c]["attributes"]))
      foreach($this->data[$c]["attributes"] as $k => $v)
       $output.=$k." = ".$v."<br/>";
     $output.="</td>";

     if(isset($this->data[$c]["outer"]))
      $output.="<td class=\"nr\">".$this->data[$c]["outer"]["start"]."</td><td>+</td><td class=\"nr\">".$this->data[$c]["outer"]["length"]."</td><td>=</td><td class=\"nr\">".($this->data[$c]["outer"]["start"]+$this->data[$c]["outer"]["length"])."</td>";
     else
      $output.="<td colspan=\"5\"></td>";

     if(isset($this->data[$c]["inner"]))
      $output.="<td class=\"nr\">".$this->data[$c]["inner"]["start"]."</td><td>+</td><td class=\"nr\">".$this->data[$c]["inner"]["length"]."</td><td>=</td><td class=\"nr\">".($this->data[$c]["inner"]["start"]+$this->data[$c]["inner"]["length"])."</td>";
     else
      $output.="<td colspan=\"5\"></td>";

     if(isset($this->data[$c]["parent"]))
      $output.="<td>".$this->data[$c]["parent"]."</td>";
     else
      $output.="<td></td>";

     if(isset($this->data[$c]["open"]))
      $output.="<td>".$this->data[$c]["open"]."</td>";
     else if(isset($this->data[$c]["close"]))
      $output.="<td>".$this->data[$c]["close"]."</td>";
     else
      $output.="<td></td>";

     $output.="</tr>\n";
    }

    $output="<table class=\"xml data\">\n".$output."</table>\n";
   }
   else //html
   {
    $output=htmlentities($this->output(false), ENT_COMPAT|ENT_XML1);

    $output=preg_replace("/^(.+)$/m", "<li>\\1</li>", $output);
    $output=preg_replace("/^$/m", "<li>&nbsp; </li>", $output);
    $output=preg_replace_callback("/^<li>(\s+)/m", create_function('$m', 'return "<li>".str_repeat("&nbsp;", strlen($m[1]));'), $output);
    $output=preg_replace("/(&gt;)(.*)(&lt;)/Uims", "\\1<font color='#222222'>\\2</font>\\3", $output);
    $output=preg_replace("/(&lt;!--)(.*)(--&gt;)/Uims", "\\1<font color='#CCCC88'>\\2</font>\\3", $output);
    $output=preg_replace("/(&lt;)(.+)(&gt;)/Um", "<font color='#FF8888'>\\1</font><font color='#4444AA'>\\2</font><font color='#FF8888'>\\3</font>", $output);
    $output=preg_replace("/([^\s]+)(=&quot;)(.*)(&quot;)/U", "<font color='#5999A4'>\\1</font>\\2<font color='#666666'>\\3</font>\\4", $output);

    $output="<ul class=\"list decimal\">\n".$output."</ul>\n";
   }

   return $output;
  }

 }

?>
