<?

 /*****************************************************************************
 * Filename: include/fs.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."cli.php");
 require_once($_path["include"]."diff.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."mutilate.php");

 class FS
 {
  function __construct()
  {
   $this->e=new Errors();
   $this->e->l->read("fs");

   $this->cli=new CLI();
   $this->_getACL="getfacl";
   $this->_setACL="setfacl";

   if(preg_match("/^(\d+\.\d+)/", phpversion(), $m))
    $this->phpversion=(float)$m[1];
   else
    $this->phpversion=0; //?
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
   $this->cli=clone $this->cli;
  }

  static function __callStatic($method, $arguments)
  {
   $object=new self();
   $object->e->logging(false); //disable logging for static calls

   if(substr($method, -1)=="S") //method must have a S suffix
   {
    $method=substr($method, 0, -1); //remove S suffix
    if(method_exists($object, $method)) //check if dynamic method exists
     return call_user_func_array(Array($object, $method), $arguments);
   }

   $bt=debug_backtrace();
   print "<br/>\n<b>Fatal error</b>:  Call to undefined method ".get_class($object)."::".$method." in <b>".$bt[0]["file"]."</b> on line <b>".$bt[0]["line"]."</b><br />\n";
   exit;
  }


  function which($file)
  {
   $PATH=explode(":", $_SERVER["PATH"]);

   foreach($PATH as $dir)
    if($this->exists($dir."/".$file))
     if(($mode=$this->getMode($dir."/".$file, "array"))&&$mode["user"]["execute"])
      return $dir."/".$file;

   return false;
  }

  function inDir($dir, $file)
  {
   $dir=explode("/", $this->realPath($dir));
   $file=explode("/", $this->realPath($file));

   $l=sizeof($dir);
   if(!$dir[$l-1]) //empty entry should not be checked
    $l--;

   for($c=0;$c<$l;$c++) //see if all subdirs are the same
    if(!isset($file[$c])||$dir[$c]!=$file[$c])
     return false;

   return true;
  }

  function realPath($file)
  {
   $file=preg_replace("!/{2,}!", "/", $file); //remove bogus slashed

   if($file[0]!="/") //relative path, add current dir
    $file=@getcwd()."/".$file;

   $file=explode("/", $file);
   $l=sizeof($file);

   $new=Array();
   for($c=0;$c<$l;$c++)
   {
    if($file[$c]==".") //same dir, skip
     continue;
    else if($file[$c]=="..") //dir up, remove previous
    {
     $l2=sizeof($new);
     if($l2>1||($l2==1&&$new[0]!="")) //not on first level, leave first slash in tact
      array_pop($new);
    }
    else //dir down, add
     $new[]=$file[$c];
   }

   return implode("/", $new);
  }

  function filename($file)
  {
   if(($pos=strrpos($file, "/"))===false) //get last slash
    return $file; //no path in file

   return substr($file, $pos+1); //strip path from file
  }

  function filepath($file)
  {
   if(($pos=strrpos($file, "/"))===false) //get last slash
    return ""; //no path in file

   return substr($file, 0, $pos+1); //strip filename from file
  }

  function fileext($file)
  {
   //get filename
   $file=$this->filename($file);

   if(($pos=strrpos($file, "."))===false) //get last dot
    return ""; //no extension in filename

   return substr($file, $pos+1); //strip extension from file
  }

  function tmpfilename($prefix=false)
  {
   $file=sys_get_temp_dir();
   if(substr($prefix, -1)!="/")
    $file.="/";

   //determine prefix
   if($prefix&&substr($prefix, -1)!="_")
    $prefix.="_";

   $prefix="JAS_".$prefix;

   $file.=$prefix;

   //generate random suffix
   $chars="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
   while(true)
   {
    $suffix="";
    for($c=0;$c<16;$c++)
     $suffix.=$chars[rand(0, 61)];

    if(!file_exists($file.$suffix))
     break;
   }

   $file.=$suffix;

   return $file;
  }


  function files($dir, $recursive=false, $stripRoot=false, $getDotFiles=false, $include=false, $exclude=false)
  {
   if(!file_exists($dir))
   {
    $this->e->add("Directory: \"".$dir."\" doesn't exist", "error");
    return false;
   }

   if(!is_dir($dir))
   {
    $this->e->add("Directory: \"".$dir."\" isn't a directory", "error");
    return false;
   }

   if($include)
   {
    if($include[0]!="#")
     $include="#".str_replace("#", "\\#", $include)."#";

    if(preg_match($include, "")===false)
    {
     $this->e->add("Include regexp: \"".$include."\" incorrect", "error");
     return false;
    }
   }

   if($exclude)
   {
    if($exclude[0]!="#")
     $exclude="#".str_replace("#", "\\#", $exclude)."#";

    if(preg_match($exclude, "")===false)
    {
     $this->e->add("Exclude regexp: \"".$exclude."\" incorrect", "error");
     return false;
    }
   }

   if(substr($dir, -1)!="/") //add end slash
    $dir.="/";

   $files=Array();
   if(($dh=opendir($dir)))
   {
    while($file=readdir($dh))
    {
     if($file[0]==".")
     {
      if($file=="."||$file=="..")
       continue;
      if(!$getDotFiles)
       continue;
     }

     $file=$dir.$file;

     if(!is_dir($file))
      $files[]=$file;
     else //dir
     {
      $files[]=$file."/";
      if($recursive)
      {
       if(($subfiles=$this->files($file, true, false, $getDotFiles))===false)
        return false;

       $files=array_merge($files, $subfiles);
      }
     }
    }
   }

   //strip root
   if($stripRoot||$include||$exclude)
    $files=preg_replace("!^".preg_quote($dir, "!")."!", "", $files);

   //remove not included files
   if($include)
    foreach($files as $k => $file)
     if(!preg_match($include, $file))
      unset($files[$k]);

   //remove excluded files
   if($exclude)
    foreach($files as $k => $file)
     if(preg_match($exclude, $file))
      unset($files[$k]);

   //reattach root
   if(!$stripRoot&&($include||$exclude))
    foreach($files as $k => $file)
     $files[$k]=$dir.$file;

   sort($files);
   return $files;
  }


  function directories($dir, $recursive=false, $stripRoot=false, $getDotFiles=false, $include=false, $exclude=false)
  {
   if(!file_exists($dir))
   {
    $this->e->add("Directory: \"".$dir."\" doesn't exist", "error");
    return false;
   }

   if(!is_dir($dir))
   {
    $this->e->add("Directory: \"".$dir."\" isn't a directory", "error");
    return false;
   }

   if($include)
   {
    if($include[0]!="#")
     $include="#".str_replace("#", "\\#", $include)."#";

    if(preg_match($include, "")===false)
    {
     $this->e->add("Include regexp: \"".$include."\" incorrect", "error");
     return false;
    }
   }

   if($exclude)
   {
    if($exclude[0]!="#")
     $exclude="#".str_replace("#", "\\#", $exclude)."#";

    if(preg_match($exclude, "")===false)
    {
     $this->e->add("Exclude regexp: \"".$exclude."\" incorrect", "error");
     return false;
    }
   }

   if(substr($dir, -1)!="/") //add end slash
    $dir.="/";

   $directories=Array();
   if(($dh=opendir($dir)))
   {
    while($directory=readdir($dh))
    {
     if(!is_dir($dir.$directory)) //only directories
      continue;

     if($directory[0]==".")
     {
      if($directory=="."||$directory=="..")
       continue;
      if(!$getDotFiles)
       continue;
     }

     $directory=$dir.$directory."/";
     $directories[]=$directory;

     if($recursive)
     {
      if(($subdirectories=$this->directories($directory, true, false, $getDotFiles))===false)
       return false;

      $directories=array_merge($directories, $subdirectories);
     }
    }
   }

   //strip root
   if($stripRoot||$include||$exclude)
    $directories=preg_replace("!^".preg_quote($dir, "!")."!", "", $directories);

   //remove not included directories
   if($include)
    foreach($directories as $k => $directory)
     if(!preg_match($include, $directory))
      unset($directories[$k]);

   //remove excluded directories
   if($exclude)
    foreach($directories as $k => $directory)
     if(preg_match($exclude, $directory))
      unset($directories[$k]);

   //reattach root
   if(!$stripRoot&&($include||$exclude))
    foreach($directories as $k => $directory)
     $directories[$k]=$dir.$directory;

   sort($directories);
   return $directories;
  }


  function size($file)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(is_dir($file))
   {
    $this->e->add("Can't get file size for directory: \"".$file."\"", "error");
    return false;
   }

   return @filesize($file);
  }

  function exists($file, $type=false)
  {
   if(!file_exists($file))
    return false;

   if($type)
   {
    switch($type)
    {
     case "file":
     {
      if(!is_file($file))
       return false;

      break;
     }
     case "symlink":
     {
      if(!is_link($file))
       return false;

      break;
     }
     case "directory":
     {
      if(!is_dir($file))
       return false;

      break;
     }
     default:
     {
      $this->e->add("Unknown file type: \"".$type."\"", "error");
      return false;
     }
    }
   }

   return true;
  }

  function mkdir($dir, $parents=false)
  {
   if(file_exists($dir))
   {
    $this->e->add("Directory: \"".$dir."\" already exist", "error");
    return false;
   }

   if(!@mkdir($dir, 0755, $parents))
   {
    $this->e->add("Can't create directory: \"".$dir."\"", "error");
    return false;
   }

   return true;
  }

  function delete($file, $recursive=false)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(is_file($file)||is_link($file))
   {
    if(!@unlink($file))
    {
     $this->e->add("Can't delete file: \"".$file."\"", "error");
     return false;
    }
   }
   else //directory
   {
    if(($files=$this->files($file, false, false, false))===false)
     return false;

    if(!$recursive&&$files)
    {
     $this->e->add("Refusing to delete non-empty directory: \"".$file."\"", "error");
     return false;
    }

    $l=sizeof($files);
    for($c=0;$c<$l;$c++)
    {
     if(is_dir($files[$c]))
     {
      if(!$this->delete($files[$c], $recursive))
       return false;
     }
     else //file
     {
      if(!@unlink($files[$c]))
      {
       $this->e->add("Can't delete file: \"".$files[$c]."\"", "error");
       return false;
      }
     }
    }

    if(!@rmdir($file))
    {
     $this->e->add("Can't delete directory: \"".$file."\"", "error");
     return false;
    }
   }

   return true;
  }

  function read($file)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!is_readable($file))
   {
    $this->e->add("File: \"".$file."\" is not readable", "error");
    return false;
   }

   if(($data=@file_get_contents($file))===false)
   {
    $this->e->add("Can't read file: \"".$file."\"", "error");
    return false;
   }

   return $data;
  }

  function readLines($file, $skipEmptyLines=false)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!is_readable($file))
   {
    $this->e->add("File: \"".$file."\" is not readable", "error");
    return false;
   }

   $flags=FILE_IGNORE_NEW_LINES;

   if($skipEmptyLines)
    $flags|=FILE_SKIP_EMPTY_LINES;

   if(($data=@file($file, $flags))===false)
   {
    $this->e->add("Can't read file: \"".$file."\"", "error");
    return false;
   }

   return $data;
  }

  function grep($pattern, $file)
  {
   if(!($data=$this->read($file)))
    return false;

   if(preg_match_all($pattern, $data, $m))
   {
    if(sizeof($m)>1) //matches stored
     return array_slice($m, 1); //return matches

    return true;
   }

   return false;
  }

  function replace($pattern, $replacement, $file)
  {
   if(!($data=$this->read($file)))
    return false;

   $count=0;
   $data=preg_replace($pattern, $replacement, $data, -1, $count);

   if(!$count)
    return false;

   return $this->write($file, $data, "overwrite");
  }

  function write($file, $data, $option=false, $lock=false)
  {
   if(file_exists($file))
   {
    if($option!="overwrite"&&$option!="append")
    {
     $this->e->add("File: \"".$file."\" already exist", "error");
     return false;
    }

    if(!is_writable($file))
    {
     $this->e->add("File: \"".$file."\" is not writable", "error");
     return false;
    }
   }

   $flags=0;

   if($lock)
    $flags|=LOCK_EX;

   if($option=="append")
    $flags|=FILE_APPEND;

   if(($bytes=@file_put_contents($file, $data, $flags))===false)
   {
    $this->e->add("Can't write file: \"".$file."\"", "error");
    return false;
   }

   if($bytes!=strlen($data))
   {
    $this->e->add("File: \"".$file."\" wasn't written correctly", "error");
    return false;
   }

   return true;
  }

  function copy($file, $destination)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!is_readable($file))
   {
    $this->e->add("File: \"".$file."\" is not readable", "error");
    return false;
   }

   if(is_dir($destination))
   {
    if(substr($destination, -1)!="/")
     $destination.="/";

    $destination.=preg_replace("!^.*/([^/]+)$!", "\\1", $file);
   }

   if(!@copy($file, $destination))
   {
    $this->e->add("Can't copy file: \"".$file."\" to: \"".$destination."\"", "error");
    return false;
   }

   return true;
  }

  function symlink($file, $destination)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!is_readable($file))
   {
    $this->e->add("File: \"".$file."\" is not readable", "error");
    return false;
   }

   $file=str_repeat("../", substr_count($destination, "/")).$file; //make relative link target

   if(!@symlink($file, $destination))
   {
    $this->e->add("Can't symlink file: \"".$file."\" to: \"".$destination."\"", "error");
    return false;
   }

   return true;
  }

  function rename($file, $destination)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!is_readable($file))
   {
    $this->e->add("File: \"".$file."\" is not readable", "error");
    return false;
   }

   if(is_dir($destination))
   {
    if(substr($destination, -1)!="/")
     $destination.="/";

    $destination.=preg_replace("!^.*/([^/]+)$!", "\\1", $file);
   }

   if(!@rename($file, $destination))
   {
    $this->e->add("Can't rename file: \"".$file."\" to: \"".$destination."\"", "error");
    return false;
   }

   return true;
  }

  function fileModeConvert($mode, $type) //type: array, int, string
  {
   if(!in_array($type, Array("array", "string", "int")))
   {
    $this->e->add("Unknown conversion type: \"".$type."\"", "error");
    return false;
   }

   if(is_array($mode)) //array mode
   {
    if($type=="array")
    {
     $this->e->add("No conversion need", "warning");
     return $mode;
    }

    if(isset($mode["user"])&&isset($mode["group"])&&isset($mode["other"])) //user, group and other
    {
     if(($user=$this->fileModeConvert($mode["user"], "int"))===false)
      return false;
     if(($group=$this->fileModeConvert($mode["group"], "int"))===false)
      return false;
     if(($other=$this->fileModeConvert($mode["other"], "int"))===false)
      return false;

     $nr=$user.$group.$other;
    }
    else if(isset($mode["read"])&&isset($mode["write"])&&isset($mode["execute"])) //single mode
    {
     $nr=0;
     if($mode["read"])
      $nr+=4;
     if($mode["write"])
      $nr+=2;
     if($mode["execute"])
      $nr+=1;
     $nr=(string)$nr;
    }
    else
    {
     $this->e->add("Can't convert unknown array based file mode", "error");
     return false;
    }
   }
   else if(preg_match("/^[rwx\-]+$/", $mode)) //string mode
   {
/*
 * allow string "rectification" conversion (rx => r-x)
    if($type=="string")
    {
     $this->e->add("No conversion need", "warning");
     return $mode;
    }
 */

    if(strlen($mode)==9) //user, group and other
    {
     $user=substr($mode, 0, 3);
     $group=substr($mode, 3, 3);
     $other=substr($mode, 6, 3);

     if(($user=$this->fileModeConvert($user, "int"))===false)
      return false;
     if(($group=$this->fileModeConvert($group, "int"))===false)
      return false;
     if(($other=$this->fileModeConvert($other, "int"))===false)
      return false;

     $nr=$user.$group.$other;
    }
    else //single mode
    {
     $nr=0;
     if(strchr($mode, "r")) //read
      $nr+=4;
     if(strchr($mode, "w")) //write
      $nr+=2;
     if(strchr($mode, "x")) //execute
      $nr+=1;

     $nr=(string)$nr;
    }
   }
   else if(preg_match("/^[0-7]+$/", $mode)) //int mode
   {
    if($type=="int")
    {
     $this->e->add("No conversion need", "warning");
     return $mode;
    }

    $nr=(string)$mode;
    //no convertion needed here
   }
   else if(!$mode)
    $nr="0";
   else
   {
    $this->e->add("Can't convert unknown file mode: \"".$mode."\"", "error");
    return false;
   }

   if($type=="int")
   {
    $output=$nr;
   }
   else if($type=="string")
   {
    $output="";
    $l=strlen($nr);
    for($c=0;$c<$l;$c++)
    {
     $output.=($nr[$c]&4?"r":"-");
     $output.=($nr[$c]&2?"w":"-");
     $output.=($nr[$c]&1?"x":"-");
    }
   }
   else if($type=="array")
   {
    $output=Array();
    $l=strlen($nr);
    if($l==3) //user, group and other
    {
     $tmp=Array("user", "group", "other");
     for($c=0;$c<$l;$c++)
     {
      $output[$tmp[$c]]["read"]=($nr[$c]&4?true:false);
      $output[$tmp[$c]]["write"]=($nr[$c]&2?true:false);
      $output[$tmp[$c]]["execute"]=($nr[$c]&1?true:false);
     }
    }
    else //single mode
    {
     $c=0;
     $output["read"]=($nr[$c]&4?true:false);
     $output["write"]=($nr[$c]&2?true:false);
     $output["execute"]=($nr[$c]&1?true:false);
    }
   }
   else
   {
    $this->e->add("Can't convert to unknown file mode type: \"".$type."\"", "error");
    return false;
   }

   return $output;
  }

  function setUser($file, $user)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!@chown($file, $user))
   {
    $this->e->add("Unable to change user for file: \"".$file."\"", "error");
    return false;
   }

   return true;
  }

  function getUser($file, $returnID=false)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(($id=@fileowner($file))===false)
   {
    $this->e->add("Unable to get user for file: \"".$file."\"", "error");
    return false;
   }

   if($returnID)
    return $id;

   $user=posix_getpwuid($id);
   return $user["name"];
  }

  function setGroup($file, $group)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(!@chgrp($file, $group))
   {
    $this->e->add("Unable to change group for file: \"".$file."\"", "error");
    return false;
   }

   return true;
  }

  function getGroup($file, $returnID=false)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(($id=@filegroup($file))===false)
   {
    $this->e->add("Unable to get group for file: \"".$file."\"", "error");
    return false;
   }

   if($returnID)
    return $id;

   $group=posix_getgrgid($id);
   return $group["name"];
  }

  function setUserMode($file, $mode)
  {
   return $this->setMode($file, $mode, false, false);
  }
  function setGroupMode($file, $mode)
  {
   return $this->setMode($file, false, $mode, false);
  }
  function setOtherMode($file, $mode)
  {
   return $this->setMode($file, false, false, $mode);
  }
  function setMode($file, $userMode=false, $groupMode=false, $otherMode=false)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   $mode=$this->getMode($file);
   if($userMode!==false)
    $mode[0]=$this->fileModeConvert($userMode, "int");
   if($groupMode!==false)
    $mode[1]=$this->fileModeConvert($groupMode, "int");
   if($otherMode!==false)
    $mode[2]=$this->fileModeConvert($otherMode, "int");

   $mode=octdec("100".$mode);

   if((@chmod($file, $mode))===false)
   {
    $this->e->add("Unable to set mode: \"".$mode."\" for file: \"".$file."\"", "error");
    return false;
   }

   //update file status cache
   if($this->phpversion>=5.3)
    clearstatcache(true, $file);
   else
    clearstatcache();

   return true;
  }

  function getUserMode($file, $type="int")
  {
   return $this->getMode($file, $type, "user");
  }
  function getGroupMode($file, $type="int")
  {
   return $this->getMode($file, $type, "group");
  }
  function getOtherMode($file, $type="int")
  {
   return $this->getMode($file, $type, "other");
  }
  function getMode($file, $type="int", $userGroupOther=true)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if(($mode=@fileperms($file))===false)
   {
    $this->e->add("Unable to get mode for file: \"".$file."\"", "error");
    return false;
   }

   $mode=substr(decoct($mode), -3);

   if($userGroupOther==="user")
    $mode=$mode[0];
   else if($userGroupOther==="group")
    $mode=$mode[1];
   else if($userGroupOther==="other")
    $mode=$mode[2];

   if($type=="int")
    return $mode;

   return $this->fileModeConvert($mode, $type);
  }

  function getTime($file, $type=false)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if($type)
   {
    if($type=="created")
     return @filectime($file);
    else if($type=="modified")
     return @filemtime($file);
    else if($type=="accessed")
     return @fileatime($file);
    else
    {
     $this->e->add("Unknown file time type: \"".$type."\"", "error");
     return false;
    }
   }

   $times=Array(
    "created" => @filectime($file),
    "modified" => @filemtime($file),
    "accessed" => @fileatime($file),
   );

   return $times;
  }

  function setTime($file, $time, $type=false)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   if($type)
   {
    if($type=="modified")
     @touch($file, $time, $this->getTime($file, "accessed"));
    else if($type=="accessed")
     @touch($file, $this->getTime($file, "modified"), $time);
    else
    {
     $this->e->add("Can't set file time type: \"".$type."\"", "error");
     return false;
    }
   }
   else
    @touch($file, $time, $time);

   //update file status cache
   if($this->phpversion>=5.3)
    clearstatcache(true, $file);
   else
    clearstatcache();

   return true;
  }


  function diff($file1, $file2)
  {
   if(($data1=$this->read($file1))===false)
    return false;
   if(($data2=$this->read($file2))===false)
    return false;

   $data1=rtrim($data1);
   $data2=rtrim($data2);

   $linebreak1=Mutilate::detectLineBreaksS($data1);
   $linebreak2=Mutilate::detectLineBreaksS($data2);

   $array1=explode($linebreak1, $data1);
   $array2=explode($linebreak2, $data2);

   if(!Diff::arrays($array1, $array2))
    return false;

   $diff=Array();
   $l=sizeof($array2);
   for($c=0;$c<$l;$c++)
    if($array2[$c][0]!="=")
     $diff[$c]=$array2[$c];

   return $diff;
  }


  function getACLUser($files, $user, $type=false)
  {
   if(!($acl=$this->getACL($files, "int")))
    return false;

   if(!$type)
    $type="string";

   if(!is_array($files)) //single file
   {
    if(isset($acl["users"][$user]))
     return $this->fileModeConvert($acl["users"][$user], $type);
   }
   else
   {
    $r=Array();
    foreach(array_keys($acl) as $file)
    {
     if(isset($acl[$file]["users"][$user]))
      $r[$file]=$this->fileModeConvert($acl[$file]["users"][$user], $type);
     else
      $r[$file]=$this->fileModeConvert(0, $type);
    }
    return $r;
   }

   return $this->fileModeConvert(0, $type);
  }

  function getACLGroup($files, $group, $type=false)
  {
   if(!($acl=$this->getACL($files, "int")))
    return false;

   if(!$type)
    $type="string";

   if(!is_array($files)) //single file
   {
    if(isset($acl["groups"][$group]))
     return $this->fileModeConvert($acl["groups"][$group], $type);
   }
   else
   {
    $r=Array();
    foreach(array_keys($acl) as $file)
    {
     if(isset($acl[$file]["groups"][$group]))
      $r[$file]=$this->fileModeConvert($acl[$file]["groups"][$group], $type);
     else
      $r[$file]=$this->fileModeConvert(0, $type);
    }
    return $r;
   }

   return $this->fileModeConvert(0, $type);
  }

  function getACL($files, $type=false)
  {
   if(!is_array($files))
    $files=Array($files);

   $l=sizeof($files);
   for($c=0;$c<$l;$c++)
   {
    if(!file_exists($files[$c]))
    {
     $this->e->add("File: \"".$files[$c]."\" doesn't exist", "error");
     return false;
    }
   }

   if(!$type)
    $type="string";

   if(!in_array($type, Array("array", "string", "int")))
   {
    $this->e->add("Unknown conversion type: \"".$type."\"", "error");
    return false;
   }

   if(!$this->cli->exec($this->_getACL." \"".implode("\" \"", $files)."\""))
   {
    $this->e->add($this->cli->e);
    return false;
   }

   $lines=&$this->cli->output;

   $file=false;
   $acl=Array();
   $l=sizeof($lines);
   for($c=0;$c<$l;$c++)
   {
    if(preg_match("/^# file: (.+)$/i", $lines[$c], $m))
    {
     $file=$m[1];
     $acl[$file]=Array(
      //normal rights
      "userName" => false,
      "groupName" => false,
      "user" => false,
      "group" => false,
      "other" => false,
      "mask" => false,
      //acl rights
      "users" => Array(),
      "groups" => Array(),
     );
    }
    else if($file)
    {
     if(preg_match("/^default/i", $lines[$c])) //skip default mask
      continue;
     else if(preg_match("/^# (owner|group): ([^\s]+)$/i", $lines[$c], $m))
     {
      if($m[1]=="owner")
       $acl[$file]["userName"]=$m[2];
      else
       $acl[$file]["groupName"]=$m[2];
     }
     else if(preg_match("/^(user|group|other|mask):([^:]*):([^\s]+)$/i", $lines[$c], $m))
     {
      if($m[2]) //acl
      {
       if($m[1]=="other") // huh?
        continue;

       $acl[$file][$m[1]."s"][$m[2]]=($type=="string"?$m[3]:$this->fileModeConvert($m[3], $type));
      }
      else //normal rights
       $acl[$file][$m[1]]=($type=="string"?$m[3]:$this->fileModeConvert($m[3], $type));
     }
    }
   }

   if(sizeof($files)==1)
   {
    $file=($files[0][0]=="/"?substr($files[0], 1):$files[0]); //remove leading slash
    return $acl[$file];
   }

   return $acl;
  }


  function setACLUser($file, $user, $mode, $recursive=false)
  {
   $acl=Array("users" => Array($user => $mode));
   return $this->setACL($file, $acl, $recursive);
  }

  function setACLGroup($file, $group, $mode, $recursive=false)
  {
   $acl=Array("groups" => Array($group => $mode));
   return $this->setACL($file, $acl, $recursive);
  }

  function setACL($file, $acl, $recursive=false)
  {
   if(!file_exists($file))
   {
    $this->e->add("File: \"".$file."\" doesn't exist", "error");
    return false;
   }

   $args="";

   if($acl===false)
    $args.=" --remove-all";
   else //if(is_array($acl))
   {
    if(!($cur=$this->getACL($file, "string")))
     return false;

    if(isset($acl["user"]))
    {
     if(($mode=$this->fileModeConvert($acl["user"], "string"))===false)
      return false;

     if($mode!=$cur["user"])
      $args.=" --modify user::".$mode; //change user rights
    }

    if(isset($acl["group"]))
    {
     if(($mode=$this->fileModeConvert($acl["group"], "string"))===false)
      return false;

     if($mode!=$cur["group"])
      $args.=" --modify group::".$mode; //change group rights
    }

    if(isset($acl["other"]))
    {
     if(($mode=$this->fileModeConvert($acl["other"], "string"))===false)
      return false;

     if($mode!=$cur["other"])
      $args.=" --modify other::".$mode; //change other rights
    }

    if(isset($acl["users"]))
    {
     foreach($acl["users"] as $user => $mode)
     {
      if(!preg_match("/^[a-z0-9]+$/", $user))
      {
       $this->e->add("Illegal username: \"".$user."\"", "error");
       return false;
      }

      if(($mode=$this->fileModeConvert($mode, "string"))===false)
       return false;

      if(!isset($cur["users"][$user])||$mode!=$cur["users"][$user])
      {
       if($mode=="---")
        $args.=" --remove user:".$user;
       else
        $args.=" --modify user:".$user.":".$mode;
      }
     }
    }

    if(isset($acl["groups"]))
    {
     foreach($acl["groups"] as $group => $mode)
     {
      if(!preg_match("/^[a-z0-9]+$/", $group))
      {
       $this->e->add("Illegal groupname: \"".$group."\"", "error");
       return false;
      }

      if(($mode=$this->fileModeConvert($mode, "string"))===false)
       return false;

      if(!isset($cur["groups"][$group])||$mode!=$cur["groups"][$group])
      {
       if($mode=="---")
        $args.=" --remove group:".$group;
       else
        $args.=" --modify group:".$group.":".$mode;
      }
     }
    }
   }

   if(!$args)
   {
    $this->e->add("No ACL changes for: \"".$file."\"", "note");
    return true;
   }

   if($recursive)
    $args.=" --recursive --physical"; //ignore symlinks

   $args.=" \"".$file."\"";

   if(!$this->cli->exec($this->_setACL.$args))
   {
    if($this->cli->output)
    {
     if(stristr($this->cli->output[0], "not supported"))
      $this->e->add("Set ACL not support", "error");
     else if(stristr($this->cli->output[0], "not permitted"))
      $this->e->add("Set ACL not permitted", "error");
     else
      $this->e->add($this->cli->e);
    }
    else
     $this->e->add($this->cli->e);

    return false;
   }

   return true;
  }


 }
?>
