<?

 /*****************************************************************************
 * Filename: include/history.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");

 class History
 {
  function __construct()
  {
   global $_config, $_session, $_historyLockProject;

   $this->e=new Errors();

   $this->setLimit(10);
   $this->setDB("session://history/global"); //use global session storage instead of project session to store information

   if(isset($_historyLockProject)&&$_historyLockProject)
    $this->setProject($_historyLockProject);
   else
    $this->setProject($_session->get("project"));
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  function setLimit($limit)
  {
   $this->limit=$limit;
   return true;
  }

  function setDB($db)
  {
   $this->db=$db;
   return true;
  }

  function setProject($project)
  {
   if(!in_array($project, JAS::projects()))
   {
    $this->e->add("Project: \"".$project."\" doesn't exist");
    return false;
   }

   $this->project=$project;
   return true;
  }

  /*
   *
   * locks project setting for subsequent History objects in this page call
   * true (default): locks to current project
   * $project: locks to $project
   * false: unlock project
   * */
  function lockProject($project=true)
  {
   global $_historyLockProject;

   if($project===false) //unlock
   {
    $_historyLockProject=false;
    return true;
   }

   if($project===true) //lock to current
    $project=$this->project;

   if(!$this->setProject($project))
    return false;

   $_historyLockProject=$project;
   return true;
  }


  function clear()
  {
   $d=Data::cache("history", "delete", "all", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->setShowFields("nr");

   $d->clearValues();

   if(!$d->get()) //get all history IDs
   {
    $this->e->add($d->e);
    return false;
   }

   if($d->vmax==0) //clean already
    return true;

   if(!$d->submit()) //delete all
   {
    $this->e->add($d->e);
    return false;
   }

   return true;
  }


  function store($cms=false)
  {
   $d=Data::cache("history", "add", "item", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->clearValues();
   $d->setDefaultValue("uri");
   $d->setDefaultValue("time");
   $d->setDefaultValue("method");
   $d->setValue("project", $this->project);
   $d->setValue("cms", ($cms?true:false));

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return false;
   }

   if($this->limit) //automagically clean up
    $this->cleanUp();

   return true;
  }

  function cms($value=true)
  {
   if(!$this->isAlreadyStored()) //current page not in db
   {
    $this->e->add("Can't change CMS flag, current page not in history");
    return false;
   }

   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->setShowFields("nr", "cms");
   $d->sortOrder("nr-"); //get newest

   $d->clearValues();
   $d->setValue("show_total", 1);

   if(!$d->get()) //get current entry
   {
    $this->e->add($d->e);
    return false;
   }

   $nr=$d->getValue("nr");

   if($d->getValue("cms")==$value) //already set to correct state
    return true;

   $d->setAction("edit");
   $d->setExtra("item");
   $d->clearValues();
   $d->setValue("nr", $nr);
   $d->setValue("cms", $value);

   if(!$d->submit())
   {
    $this->e->add($d->e);
    return false;
   }

   return true;
  }


  function isAlreadyStored()
  {
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->setShowFields();

   $d->clearValues();
   $d->setValue("uri", $d->getDefaultValue("uri"));
   $d->setValue("time", $d->getDefaultValue("time"));
   $d->setValue("method", $d->getDefaultValue("method"));
   $d->setValue("project", $this->project);

   if(!$d->get(false, true))
   {
    $this->e->add($d->e);
    return false;
   }

   if($d->getValue("show_count"))
    return true;

   return false;
  }


  function getTotal()
  {
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->setShowFields();

   $d->clearValues();

   if(!$d->get(false, true))
   {
    $this->e->add($d->e);
    return false;
   }

   return $d->getValue("show_count");
  }


  function cleanUp()
  {
   if(!$this->limit||(($total=$this->getTotal())<=$this->limit)) //limit not breached
    return true; //don't clean up

   $d=Data::cache("history", "delete", "all", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->setShowFields("nr");

   $d->clearValues();

   //remove oldest entries
   $d->setValue("show_total", ($total-$this->limit));
   $d->sortOrder("nr+");

   if(!$d->get())
   {
    $this->e->add($d->e);
    return false;
   }

   if(!$d->submit()) //delete all
   {
    $this->e->add($d->e);
    return false;
   }

   return true;
  }


  function previous($entry=1, $method=false, $projectSpecific=true, $cmsSkip=true)
  {
   if($entry<1)
   {
    $this->e->add("Entry should at least be 1");
    return false;
   }

   //get current page history nr
   $current=false;
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->setShowFields("nr");

   $d->clearValues();
   $d->setValue("uri", $d->getDefaultValue("uri"));
   $d->setValue("time", $d->getDefaultValue("time"));
   $d->setValue("method", $d->getDefaultValue("method"));
   $d->setValue("project", $this->project);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return false;
   }

   if($d->next())
    $current=$d->v["nr"];

   //get (non current) page from history
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->setShowFields("uri");
   $d->sortOrder("nr-"); //get newest

   $d->clearValues();
   $d->setValue("show_from", $entry-1); //get previous "entry" (-1 since from start at 0)
   $d->setValue("show_total", 1);

   //skip current
   if($current)
   {
    $d->setValue("nr", $current);
    $d->setOperator("nr", "!=");
   }

   if($method) //filter on method
    $d->setValue("method", strtoupper($method));

   if($projectSpecific) //filter on current project
    $d->setValue("project", $this->project);

   if($cmsSkip) //filter non cms pages
    $d->setValue("cms", false);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return false;
   }

   if(!$d->next())
    return false;

   return html_entity_decode($d->v["uri"]); //convert to proper URI
  }


  function debug()
  {
   $d=Data::cache("history", "view", "all", Array("db" => $this->db));
   unset($d->field["_stp"]); //disable stp

   $d->sortOrder("nr-"); //get newest

   if(!$d->get())
    return $d->e->show();

   $c=1;
   $tmp="";
   while($d->next())
   {
    $tmp.=$c.": ";
    $tmp.=$d->v["method"];
    $tmp.=($d->v["cms"]?"-CMS":"");
    $tmp.=": ";

    $tmp.=$d->v["uri"]." (";
    $tmp.="Project: ". $d->v["project"]." ";
    $tmp.="Time: ". $d->v["time"].")<br/>";
    $c++;
   }

   return $tmp;
  }
 }

?>
