<?

 /*****************************************************************************
 * Filename: include/modules/configCMS.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");
 require_once($_path["modules"]."config.php");

 class System_Module_configCMS extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="CMS configuration";

   $this->data->defaultFields("id;added;updated");
   $this->data->setShowFields(); //clear show fields

   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 256); //view rights for all
   $this->data->addField("faketitle", "text", "show", "title;hidden", 0, false, $this->data->title, "Title");

   $this->data->addField("showText", "boolean", "all", false, false, false, true, "Show instruction", "Shows instruction line at the top");
   $this->data->addField("showTitles", "boolean", "all", false, false, false, true, "Show titles", "Shows field titles");
   $this->data->addField("showTips", "boolean", "all", false, false, false, false, "Show tips", "Shows additional field information");

   $this->data->addField("showTimezone", "boolean", "all", false, false, false, false, "Show timezone", "Shows timezone information for datetime fields");
   $this->data->addField("showDateformat", "boolean", "all", false, false, false, false, "Show date/time format", "Shows date/time format for date/time/datetime fields");

   $this->data->addField("showErrors", "boolean", "all", false, false, false, true, "Show errors", "Show errors generated by submitting information");
   $this->data->addField("showFields", "boolean", "all", false, false, false, true, "Show field", "If you want to be able to input something leave this enabled");
   $this->data->addField("showGroups", "boolean", "all", false, false, false, true, "Show groups", "Group fields together");
   $this->data->addField("showSubmit", "boolean", "all", false, false, false, true, "Show submit", "Shows submit buttons");
   $this->data->addField("showSubmitAdditional", "boolean", "all", false, false, false, true, "Show additional submits", "Shows additional submit buttons");
   $this->data->addField("showCancel", "boolean", "all", false, false, false, true, "Show cancel", "Shows cancel button");

   $this->data->addField("useStyling", "boolean", "all", "reload", false, false, true, "Use styling", "Use HTML styling");
   $this->data->addField("cmsOptions", "boolean", "all", false, false, false, true, "CMS options", "Advanced CMS javascript options");

   $this->data->addField("handleSubmit", "boolean", "all", false, false, false, true, "Handle submit", "Let CMS handle the submitted form information");
   $this->data->addField("reloadPage", "boolean", "all", "reload", false, false, true, "Reload page", "Reload page after succesfull submit");
   $this->data->addField("reloadTimeout", "nr", "all", "", 0, false, 500, "Reload timeout", "Amount of milliseconds to wait for reload");
   $this->data->addField("returnURL", "text", "all", false, 0, 255, false, "Default return URL", "Only used in case no history is found");

   $this->data->addField("defaultFormVars", "text", "all", false, 0, 255, false, "Default form variables", "eg: cms=1&title=asd&");

   $this->data->addField("listStyle", "enum", "all", false, false, false, "dropdown", "Style for list fields", "The way list fields are displayed");
   $this->data->enumOptions("listStyle", "dropdown", "list");
   $this->data->addField("multilistStyle", "enum", "all", false, false, false, "list", "Style for multilist fields", "The way multilist fields are displayed");
   $this->data->enumOptions("multilistStyle", "dropdown", "list");
   $this->data->addField("moduleStyle", "enum", "all", false, false, false, "dropdown", "Style for module fields", "The way module fields are displayed");
   $this->data->enumOptions("moduleStyle", "dropdown", "list");
   $this->data->addField("modulesStyle", "enum", "all", false, false, false, "list", "Style for modules fields", "The way modules fields are displayed");
   $this->data->enumOptions("modulesStyle", "dropdown", "list");
   $this->data->addField("localfileStyle", "enum", "all", false, false, false, "dropdown", "Style for localfile fields", "The way localfile fields are displayed");
   $this->data->enumOptions("localfileStyle", "dropdown", "list");
   $this->data->addField("localfilesStyle", "enum", "all", false, false, false, "list", "Style for localfiles fields", "The way localfiles fields are displayed");
   $this->data->enumOptions("localfilesStyle", "dropdown", "list");
   $this->data->addField("pluginStyle", "enum", "all", false, false, false, "dropdown", "Style for plugin fields", "The way plugin fields are displayed");
   $this->data->enumOptions("pluginStyle", "dropdown", "list");
   $this->data->addField("projectStyle", "enum", "all", false, false, false, "dropdown", "Style for project fields", "The way project fields are displayed");
   $this->data->enumOptions("projectStyle", "dropdown", "list");
   $this->data->addField("actionStyle", "enum", "all", false, false, false, "dropdown", "Style for action fields", "The way action fields are displayed");
   $this->data->enumOptions("actionStyle", "dropdown", "list");
   $this->data->addField("extraStyle", "enum", "all", false, false, false, "dropdown", "Style for extra fields", "The way extra fields are displayed");
   $this->data->enumOptions("extraStyle", "dropdown", "list");

   $this->data->addField("requiredIndicatorFile", "localfile", "all", false, false, 255, "", "Required indicator", "Leave empty for asterisk");

   $this->data->addField("anchoring", "boolean", "all", false, false, false, 1, "Add field anchors", "Reload field property jumps to anchor");

   $this->data->addField("manytoReferenceStyle", "enum", "all", "", false, false, "dropdown", "Style for many-to-one references", "The way *:0 and *:1 references are displayed");
   $this->data->enumOptions("manytoReferenceStyle", "dropdown", "list");
   $this->data->addField("manytoReferenceManip", "boolean", "all", "", false, false, true, "Many-to-one reference manipulation", "Show *:0 and *:1 reference manipulation options");

   $this->data->addField("tomanyReferenceStyle", "enum", "all", "", false, false, "list", "Style for many-to-many references", "The way *:*, 0:* and 1:* references are displayed");
   $this->data->enumOptions("tomanyReferenceStyle", "dropdown", "list");
   $this->data->addField("tomanyReferenceManip", "boolean", "all", "", false, false, true, "Many-to-many reference manipulation", "Show *:*, 0:* and 1:* reference manipulation options");

   $this->data->group("Show", "showText", "showTitles", "showTips", "showTimezone", "showDateformat", "showErrors", "showFields", "showGroups", "showSubmit", "showSubmitAdditional", "showCancel", "useStyling", "cmsOptions");
   $this->data->group("List style", "listStyle", "multilistStyle", "moduleStyle", "modulesStyle", "localfileStyle", "localfilesStyle" , "pluginStyle", "projectStyle", "actionStyle", "extraStyle");
   $this->data->group("References", "manytoReferenceStyle", "manytoReferenceManip", "tomanyReferenceStyle", "tomanyReferenceManip");
  }

  function postInit()
  {
   global $_path;

   if(is_dir($_path["project"]["images"]))
    $this->data->setFileDir("requiredIndicatorFile", $_path["project"]["images"]);
   else
    $this->data->setFileDir("requiredIndicatorFile", $_path["project"]["root"]);

   $this->data->setFileExtensions("requiredIndicatorFile", "gif", "png", "jpg");

   return $this->customizeFields();
  }
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   if($this->data->getValue("useStyling"))
    $this->data->resetField("cmsOptions");
   else
    $this->data->field["cmsOptions"]["properties"].=";hidden";

   if($this->data->getValue("reloadPage"))
   {
    $this->data->resetField("reloadTimeout");
    $this->data->resetField("returnURL");
   }
   else
   {
    $this->data->field["reloadTimeout"]["properties"].=";hidden";
    $this->data->field["returnURL"]["properties"].=";hidden";
   }

   return true;
  }

  function viewPostGet()
  {
   for($c=0;$c<$this->data->vmax;$c++)
    $this->data->setValue("faketitle", $this->data->title, $c);

   return true;
  }


  function addPostSubmit(){return System_Module_config::updateConfig();}
  function editPostSubmit(){return System_Module_config::updateConfig();}
  function deletePostSubmit(){return System_Module_config::updateConfig();}
 }

?>
