<?

 /*****************************************************************************
 * Filename: include/modules/configUsers.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_configUsers extends Module
 {
  function __construct(&$data)
  {
   global $_config;

   parent::__construct($data);

   $d=Data::cache("config", "view", "item"); //init config for default values

   $this->data->title="Users configuration";

   $this->data->defaultFields("id;owner;added;updated");
   $this->data->addReference("user", $_config->get("usermodule"), "nr", "*:1", "nr");
   $this->data->setShowFields(); //clear show fields

   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 512); //add rights for all
   $this->data->addField("faketitle", "text", "", "title;hidden", 0, false, $this->data->title, "Title");


   $this->data->addField("language", "enum", "all", "", 1, 3, $d->getDefaultValue("language"), "Language code", "Default project language");
   $this->data->enumOptions("language", Array("NL" => "Dutch", "EN" => "English"));

   if(function_exists("timezone_identifiers_list")) //no timezone support for PHP < 5.1.0
   {
    $this->data->addField("timezone", "list", "all", false, 1, 25, $d->getDefaultValue("timezone"), "Display timezone");
    $this->data->listOptions("timezone", timezone_identifiers_list());
   }

   $this->data->addField("dateformat", "text", "all", false, 1, 10, $d->getDefaultValue("dateformat"), "Display date format");
   $this->data->addField("timeformat", "text", "all", false, 1, 10, $d->getDefaultValue("timeformat"), "Display time format");
   $this->data->addField("datetimeformat", "text", "all", false, 1, 25, $d->getDefaultValue("datetimeformat"), "Display date and time format");

   $this->data->addField("decimalseparator", "text", "all", false, 1, 1, $d->getDefaultValue("decimalseparator"), "Display decimal separation character");
   $this->data->addField("thousandsseparator", "text", "all", false, 0, 1, $d->getDefaultValue("thousandsseparator"), "Display thousands separation character");

   $this->data->group("Date / Time", "timezone", "dateformat", "timeformat", "datetimeformat");
   $this->data->group("Numbers", "decimalseparator", "thousandsseparator");
  }

  function postInit(){return $this->customizeFields();}
//  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
//  function movePostGet(){return $this->customizeFields();}
//  function deletePostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   global $_config;

   if(!$_config->userRoot())
   {
    $this->data->setValue("user", $_config->get("usernr")); //force current users
    $this->data->field["user"]["properties"].="hidden"; //don't show user dropdown when editting
   }
   else //root
   {
    if($this->data->action=="add")
    {
     if($this->data->check("valuetype", "default", "user")||$this->data->check("valuetype", "none", "user"))
      $this->data->setValue("user", $_config->get("usernr"));
    }
    else if($this->data->action=="edit")
     $this->data->field["user"]["properties"].="hidden"; //don't show user dropdown when editting
   }

   return true;
  }

  function viewPostGet()
  {
   global $_config;

   //get fake titles users
   if($this->data->check("use", "show", "faketitle")&&$this->data->getValue("nr"))
   {
    $d=Data::cache($_config->get("usermodule"), "view", "all");
    $d->field["nr"]["type"]="array";
    $d->setValue("nr", $this->data->field["nr"]["value"]);

    $d->setShowFields($d->check("properties", "title"));

    if(!$d->get())
    {
     $this->data->e->add($d->e);
     return false;
    }

    for($c=0;$c<$this->data->vmax;$c++)
    {
     $d->reset();
     if($d->search("nr", $this->data->getValue("nr", $c)))
      $this->data->setValue("faketitle", $d->v("title"), $c);
     else
      $this->data->setValue("faketitle", $this->data->getValue("nr", $c), $c);
    }
   }

   return true;
  }


  function addPreSubmitErrorCheck(){return $this->checkFields();}
  function copyPreSubmitErrorCheck(){return $this->checkFields();}

  function checkFields()
  {
   global $_config;

   if($this->data->getValue("nr")==$_config->get("anonnr")) //not allowed to add user config for anonymous
    return false;

   return true;
  }

  function addPostSubmit(){return $this->updateConfig();}
  function copyPostSubmit(){return $this->updateConfig();}
  function editPostSubmit(){return $this->updateConfig();}
  function deletePostSubmit(){return $this->updateConfig();}

  function updateConfig()
  {
   global $_config;

   if($this->data->getValue("nr")==$_config->get("usernr")) //config for current user updated
   {
    if($this->data->action=="delete")
     $_config->reload();
    else
     $_config->loadUserConfig();
   }

   return true;
  }
 }

?>
