<?

 /*****************************************************************************
 * Filename: include/modules/onlinePlugins.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."archive.php");
 require_once($_path["include"]."module.php");
 require_once($_path["include"]."fs.php");

 class System_Module_onlinePlugins extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Online plugins";

   $this->data->defaultFields();

   $this->data->addField("plugin", "text", "all", "title;disabled", 1, 25, false, "Plugin name");
   $this->data->addField("title", "text", "all", "title", 1, 25, false, "Title");
   $this->data->addField("description", "text", "all", "", 0, 65535, false, "Description");

   $this->data->addField("version", "version", "all", "title", 0, false, false, "Version");
   $this->data->addField("status", "enum", "all", false, false, false, "", "Status");
   $this->data->enumOptions("status", "", "alpha", "beta", "stable", "deprecated");
   $this->data->addField("status2", "text", "show", false, false, false, "", "Status");

   $this->data->addField("license", "text", "all", false, 0, 25, "", "License");
   $this->data->listOptions("license", Array(
    " " => "",
    "APACHE" => "Apache License",
    "BSD" => "BSD License",
    "CCL" => "Creative Common License",
    "GPL" => "GNU General Public License",
    "LGPL" => "GNU Lesser General Public License",
    "MIT" => "MIT License",
    "MOZILLA" => "Mozilla Public License",
    "  " => "",
    "PUBLIC" => "Public domain",
    "PROPRIETARY" => "Proprietary license",
    "   " => "",
    "JAS" => "Part of JAS project",
    "" => "None"
   ));
   $this->data->addMutilation("license", "trim");
   $this->data->addField("license2", "text", "show", false, false, false, "", "License");

   $this->data->addField("author", "text", "all", false, 0, 25, false, "Author");
   $this->data->addField("email", "email", "all", false, 0, 25, false, "Email address");

   $this->data->addField("dependencies", "text", "all", "", 0, 65535, false, "Dependencies");
   $this->data->addField("changelog", "text", "all", "", 0, 1024*1024*1, false, "Changelog");

   $this->data->addField("file", "file", "all", "md5;count", 100, 1024*1024*10, false, "File");

   $this->data->addField("system", "boolean", "all", "", false, false, false, "System plugin");

   $this->data->addUnique("pluginVersion", "plugin", "version");
  }


  function viewPreGet(){return $this->showStatusLicense2();}
  function searchPreGet(){return $this->showStatusLicense2();}

  function viewPostGet(){return $this->setStatusLicense2();}
  function searchPostGet(){return $this->setStatusLicense2();}

  function showStatusLicense2()
  {
   if(!$this->data->check("use", "show", "status")&&$this->data->check("use", "show", "status2")) //get status if status2 is show field
    $this->data->field["status"]["use"].=";show";

   if(!$this->data->check("use", "show", "license")&&$this->data->check("use", "show", "license2")) //get license if license2 is show field
    $this->data->field["license"]["use"].=";show";

   return true;
  }

  function setStatusLicense2()
  {
   if($this->data->check("use", "show", "status2"))
   {
    $statusConvert=Array("alpha" => "&alpha;", "beta" => "&beta;", "stable" => "&gamma;", "deprecated" => "&delta;");

    for($c=0;$c<$this->data->vmax;$c++)
     if(isset($statusConvert[$this->data->getValue("status", $c)]))
      $this->data->setValue("status2", $statusConvert[$this->data->getValue("status", $c)], $c);
   }

   if($this->data->check("use", "show", "license2"))
   {
    for($c=0;$c<$this->data->vmax;$c++)
     if(isset($this->data->listOptions["license"][$this->data->getValue("license", $c)]))
      $this->data->setValue("license2", $this->data->listOptions["license"][$this->data->getValue("license", $c)], $c);
   }

   return true;
  }


  function postInit()
  {
   if($this->data->action=="add"||$this->data->action=="copy") //only show file field
    foreach(Array("plugin", "title", "description", "version", "status", "license", "author", "email", "changelog", "dependencies", "system") as $field)
     $this->data->field[$field]["use"]=str_replace("form;", "", $this->data->field[$field]["use"]);

   return true;
  }

  function addPreSubmitErrorCheck(){return $this->readFile();}
  function copyPreSubmitErrorCheck(){return $this->readFile();}
  function editPreSubmitErrorCheck(){return $this->readFile();}

  function readFile()
  {
   global $_path;

   if(!$this->data->vmax)
    return true;

   $fs=new FS();
   $zip=new Archive();

   //get plugin information for all uploaded plugin files
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(!$this->data->getValue("file", $c))
     continue;

    //check release file
    if(!preg_match("/\.zip$/i", $this->data->getValue("file", $c)))
    {
     $this->data->e->add("Not a \".zip\" file extension for file: \"".$this->data->getValue("file", $c)."\"", "error");
     return false;
    }

    if($this->data->getValue("fileType", $c)!="application/zip")
    {
     $this->data->e->add("Incorrect MIME type: \"".$this->data->getValue("fileType", $c)."\", should be: \"application/zip\"", "error");
     return false;
    }

    //check plugin filename
    if(!preg_match("/^([a-z0-9\_]{1,25})-(\d+\.\d+(\.\d+)?)/i", $this->data->getValue("file", $c), $m))
    {
     $this->data->e->add("Unable to detect plugin information from filename: \"".$this->data->getValue("file", $c)."\"", "error", "e.g. lister-1.0.zip");
     return false;
    }

    $plugin=$m[1];
    $version=$m[2];
    unset($m);

    //open plugin archive
    if(!$zip->openString($this->data->getValue("fileData", $c), "ZIP"))
    {
     $this->data->e->add($zip->e);
     return false;
    }

    //get pluginInfo file from plugin archive
    $pluginInfo=false;
    foreach($zip->getFiles() as $file)
    {
     if(preg_match("!pluginInfo\.xml$!", $file)&&!preg_match("!language/[a-z]+/modules/pluginInfo.xml$!i", $file))
     {
      $pluginInfo=$file;
      break;
     }
    }

    if(!$pluginInfo)
    {
     $this->data->e->add("Plugin info XML not found", "error");
     return false;
    }

    //get pluginInfo
    if(!($pluginInfo=$zip->getContents($pluginInfo)))
    {
     $this->data->e->add($zip->e);
     return false;
    }

    //get pluginDependencies file from plugin archive
    $pluginDependencies=false;
    foreach($zip->getFiles() as $file)
    {
     if(preg_match("!pluginDependencies.xml$!", $file))
     {
      $pluginDependencies=$file;
      break;
     }
    }

    if($pluginDependencies)
    {
     //get pluginDependencies
     if(!($pluginDependencies=$zip->getContents($pluginDependencies)))
     {
      $this->data->e->add($zip->e);
      return false;
     }
    }

    //get changelog file from plugin archive
    $changelog=false;
    foreach($zip->getFiles() as $file)
    {
     if(preg_match("!changelog\.txt$!", $file))
     {
      $changelog=$file;
      break;
     }
    }

    if($changelog)
    {
     //get changelog
     if(!($changelog=$zip->getContents($changelog)))
     {
      $this->data->e->add($zip->e);
      return false;
     }
    }

    $tmpfile=$_path["project"]["cache"]."onlinePlugins_".$plugin."pluginInfo.xml";
    if(!$fs->write($tmpfile, $pluginInfo))
    {
     $this->data->e->add($fs->e);
     $fs->delete($tmpfile);
     return false;
    }

    //get plugin information
    $di=Data::cache("pluginInfo", "view", "all", Array("db" => "xml://".$tmpfile));

    if(!$di->get())
    {
     $this->data->e->add($di->e);
     $fs->delete($tmpfile);
     return false;
    }

    $di->next();

    //check plugin version of pluginInfo and archive
    if($di->v["version"]!=$version)
    {
     $this->data->e->add("Plugin \"".$plugin."\" info version: \"".$di->v["version"]."\" doesn't match archive version: \"".$version."\"");
     $fs->delete($tmpfile);
     return false;
    }

    $fs->delete($tmpfile);


    $dependencies="";
    if($pluginDependencies)
    {
     $tmpfile=$_path["project"]["cache"]."onlinePlugins_".$plugin."_pluginDependencies.xml";
     if(!$fs->write($tmpfile, $pluginDependencies))
     {
      $this->data->e->add($fs->e);
      $fs->delete($tmpfile);
      return false;
     }

     //get plugin dependencies
     $dd=Data::cache("pluginDependencies", "view", "all", Array("db" => "xml://".$tmpfile));

     if(!$dd->get())
     {
      $this->data->e->add($dd->e);
      $fs->delete($tmpfile);
      return false;
     }

     while($dd->next())
     {
      if(!$dd->v["plugin"])
       $dependencies.="JAS-".$dd->v["version"].($dd->v["required"]?" - required":"")."\n";
      else
       $dependencies.=$dd->v["plugin"]."-".$dd->v["version"].($dd->v["required"]?" - required":"")."\n";
     }

     $fs->delete($tmpfile);
    }


    $this->data->setValue("plugin", $plugin, $c);
    $this->data->setValue("title", $di->v["title"], $c);
    $this->data->setValue("description", $di->v["description"], $c);

    $this->data->setValue("version", $di->v["version"], $c);
    $this->data->setValue("status", $di->v["status"], $c);

    $this->data->setValue("license", $di->v["license"], $c);

    $this->data->setValue("author", $di->v["author"], $c);
    $this->data->setValue("email", $di->v["email"], $c);

    if($changelog)
     $this->data->setValue("changelog", $changelog, $c);
    else
     $this->data->setValue("changelog", "", $c);

    $this->data->setValue("dependencies", $dependencies, $c);

    $this->data->setValue("system", in_array($plugin, JAS::plugins("system")), $c);
   }

   return true;
  }

 }
?>
