<?

 /*****************************************************************************
 * Filename: include/modules/onlineReleases.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."archive.php");
 require_once($_path["include"]."module.php");
 require_once($_path["include"]."fs.php");

 class System_Module_onlineReleases extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Online releases";

   $this->data->defaultFields();

   $this->data->addField("title", "text", "all", "title", 1, 25, false, "Title");

   $this->data->addField("version", "version", "all", "title", 0, false, false, "Version");
   $this->data->addField("tag", "text", "all", "title", 0, 25, "", "Tag");
   $this->data->enumOptions("tag", "", "Alpha", "Beta", "RC1", "RC2", "RC3", "RC4");

   $this->data->addField("date", "date", "all", "", 0, false, false, "Date");
   $this->data->addField("changelog", "text", "all", "", 0, 1024*1024*2, false, "Changelog");

   $this->data->addField("file", "file", "all", "md5;count", 100, 1024*1024*10, false, "File");

   $this->data->addUnique("releaseVersion", "title", "version", "tag");
  }

  function viewPostGet(){return $this->fixFinalStatus();}
  function searchPostGet(){return $this->fixFinalStatus();}

  function fixFinalStatus()
  {
   for($c=0;$c<$this->data->vmax;$c++)
    if($this->data->getValue("tag", $c)=="_") //remove _ from tag (used for sorting)
     $this->data->setValue("tag", "", $c);

   return true;
  }


  function postInit()
  {
   if($this->data->action=="add"||$this->data->action=="copy") //only show file field
    foreach(Array("title", "version", "tag", "changelog", "date") as $field)
     $this->data->field[$field]["use"]=str_replace("form;", "", $this->data->field[$field]["use"]);

   return true;
  }

  function addPreSubmitErrorCheck(){return $this->readFile();}
  function copyPreSubmitErrorCheck(){return $this->readFile();}
  function editPreSubmitErrorCheck(){return $this->readFile();}

  function readFile()
  {
   global $_path;

   if(!$this->data->vmax)
    return true;

   $zip=new Archive();

   //get release information for all uploaded release files
   for($c=0;$c<$this->data->vmax;$c++)
   {
    //check release file
    if(!$this->data->getValue("file", $c))
     continue;

    if(!preg_match("/\.zip$/i", $this->data->getValue("file", $c)))
    {
     $this->data->e->add("Not a \".zip\" file extension for file: \"".$this->data->getValue("file", $c)."\"", "error");
     return false;
    }

    if($this->data->getValue("fileType", $c)!="application/zip")
    {
     $this->data->e->add("Incorrect MIME type: \"".$this->data->getValue("fileType", $c)."\", should be: \"application/zip\"", "error");
     return false;
    }

    if(!preg_match("/^([a-z0-9\_]{1,25})-(\d+\.\d+(\.\d+)?)(-(\w+))?/i", $this->data->getValue("file", $c), $m))
    {
     $this->data->e->add("Unable to detect release information from filename: \"".$this->data->getValue("file", $c)."\"", "error", "e.g. JAS-1.0.zip");
     return false;
    }

    $release=$m[1];
    $version=$m[2];
    $tag=(isset($m[5])?$m[5]:"");
    unset($m);

    //open release archive
    if(!$zip->openString($this->data->getValue("fileData", $c), "ZIP"))
    {
     $this->data->e->add($zip->e);
     return false;
    }

    //get changelog file from release archive
    $changelog=false;
    foreach($zip->getFiles() as $file)
    {
     if(preg_match("!changelog\.txt$!", $file))
     {
      $changelog=$file;
      break;
     }
    }

    if(!$changelog)
    {
     $this->data->e->add("No changelog found in file: \"".$this->data->getValue("file", $c)."\"", "error");
     return false;
    }

    //get changelog content
    if(!($changelog=$zip->getContents($changelog)))
    {
     $this->data->e->add($zip->e);
     return false;
    }

    if(!preg_match("/\n\t?(\d{4}-\d{2}-\d{2})/", $changelog, $date))
    {
     $this->data->e->add("No changelog date found in file: \"".$this->data->getValue("file", $c)."\"", "error");
     return false;
    }

    $date=DT::dateConvertS($date[1], "Y-m-d", $this->data->field["date"]["format"][$this->data->dataformat]["dateformat"]);

    $this->data->setValue("title", $release, $c);

    $this->data->setValue("version", $version, $c);
    $this->data->setValue("tag", $tag, $c);

    $this->data->setValue("changelog", $changelog, $c);
    $this->data->setValue("date", $date, $c);
   }

   //set _ tag for final releases to ensure they're sorted after RC4
   for($c=0;$c<$this->data->vmax;$c++)
    if($this->data->getValue("tag", $c)==="")
     $this->data->setValue("tag", "_", $c);

   return true;
  }

 }
?>
