<?

 /*****************************************************************************
 * Filename: include/modules/workflowTransitions.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class System_Module_workflowTransitions extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="Workflow transitions";

   $this->data->defaultFields();

   $this->data->addField("workflow", "nr", "all", "title;reload", false, false, 1, "Workflow");
   $this->data->addReference("workflow", "workflow", "nr", "*:1");

   $this->data->addField("previous", "nr", "all", "", false, false, 1, "Previous state");
   $this->data->addReference("previous", "workflow", "nr", "*:1");
   $this->data->addField("next", "nr", "all", "", false, false, 1, "Next state");
   $this->data->addReference("next", "workflow", "nr", "*:1");


   $this->data->addField("text", "text", "all", "title", 1, 100, false, "Text");
   $this->data->addField("code", "text", "all", "", 1, 25, false, "Code");


   //checks
   $this->data->addField("users", "multilist", "all", "", 0, 65536, false, "Allowed users");
   $this->data->addField("groups", "multilist", "all", "", 0, 65536, false, "Allowed user groups");


   $this->data->group("Workflow state", "workflow", "previous", "next");
   $this->data->group("Restrictions", "users", "groups");

   $this->data->addUnique("transition", "previous", "next");

   $this->data->sortOrder("workflow", "nr");
  }


  function postInit(){return $this->customizeFields();}
  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}

  function customizeFields()
  {
   //get workflows
   $d=Data::cache("workflow", "view", "all");

   $d->clearValues();
   $d->setValue("workflow", 0);

   $d->setShowFields("name");
   $d->sortOrder("text");

   $workflows=Array();
   if($d->get())
    while($d->next())
     $workflows[$d->v["nr"]]=$d->v["name"];

   $this->data->listOptions("workflow", $workflows);

   //get previous/next states
   if($workflow=$this->data->getValue("workflow"))
   {
    $d=Data::cache("workflow", "view", "all");

    $d->clearValues();
    $d->setValue("nr", $workflow);
    $d->setOperant("nr", "OR");
    $d->setValue("workflow", $workflow);

    $d->setShowFields("text");
    $d->sortOrder("nr");

    $states=Array();
    if($d->get())
     while($d->next())
      $states[$d->v["nr"]]=$d->v["text"];

    $this->data->listOptions("previous", $states);
    $this->data->listOptions("next", $states);
   }

   //get users+groups from categories rights fields
   $d=Data::cache("categories", "view", "all");
   $d->moduleSpecificFunction("initRightsFields", false);

   $this->data->listOptions("users", $d->listOptions["usernr"]);
   $this->data->listOptions("groups", $d->listOptions["groupnr"]);

   unset($this->data->listOptions["users"][""]);
   unset($this->data->listOptions["groups"][""]);

   return true;
  }


  function addPreSubmitErrorCheck(){return $this->checkFields();}
  function copyPreSubmitErrorCheck(){return $this->checkFields();}
  function editPreSubmitErrorCheck(){return $this->checkFields();}

  function checkFields()
  {
   if(!$this->data->getValue("code")) //no code
    $this->data->setValue("code", strtoupper($this->data->getValue("text")));

   //force workflow field based on previous
   if($this->data->getValue("previous"))
   {
    $d=Data::cache("workflow", "view", "item");
    $d->setValue("nr", $this->data->getValue("previous"));
    $d->setShowFields("workflow");

    if($d->get()&&$d->next()) //state found
    {
     if($d->getValue("workflow")) //state has a parent workflow
      $this->data->setValue("workflow", $d->getValue("workflow"));
     else //state is parent
      $this->data->setValue("workflow", $this->data->getValue("previous"));
    }
   }

   return true;
  }
 }

?>
