<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/dt.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."dt.php");

 class DT_EXTENDS_TEST extends DT
 {
 }

 class UT_DT extends UT_Classes
 {
  var $dutchMons=Array("jan", "feb", "mrt", "apr", "mei", "jun", "jul", "aug", "sep", "okt", "nov", "dec");
  var $dutchMonths=Array("januari", "februari", "maart", "april", "mei", "juni", "juli", "augustus", "september", "oktober", "november", "december");

  function Constructor()
  {
   $object=new DT();

   if(!is_object($object)||get_class($object)!="DT")
   {
    $this->e->add("Constructor doesn't return a DT object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new DT();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("DT object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("DT object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   //init language object
   $orig=new DT();
   $orig->setLanguage("nl");
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("DT object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("DT object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("DT object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetLanguage()
  {
   $dt=new DT();

   if($dt->getLanguage()!="en")
   {
    $this->e->add("Get language doesn't match: en", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   $dt->l=new Language();
   if(!$dt->l->setLanguage("nl"))
   {
    $this->e->add("Set language shouldn't fail", "error");
    $this->e->add($dt->l->e);
    return UT_FAILED;
   }

   if($dt->getLanguage()!="nl")
   {
    $this->e->add("Get language doesn't match: nl", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if(!$dt->l->setLanguage("en"))
   {
    $this->e->add("Set language shouldn't fail", "error");
    $this->e->add($dt->l->e);
    return UT_FAILED;
   }

   if($dt->getLanguage()!="en")
   {
    $this->e->add("Get language doesn't match: en", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   $dt->l=false;
   if($dt->getLanguage()!="en")
   {
    $this->e->add("Get language doesn't match: en", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }


   return UT_OK;
  }

  function SetLanguage()
  {
   global $_config;

   $dt=new DT();

   if(!$dt->setLanguage("nl"))
   {
    $this->e->add("Set date language to \"nl\" shouldn't fail", "error");
    return UT_FAILED;
   }

   if($dt->getLanguage()!="nl")
   {
    $this->e->add("Date language should be set to: \"nl\"", "error");
    return UT_FAILED;
   }

   if(!$dt->setLanguage("en"))
   {
    $this->e->add("Set date language to \"en\" shouldn't fail", "error");
    return UT_FAILED;
   }

   if($dt->getLanguage()!="en")
   {
    $this->e->add("Date language should be set to: \"en\"", "error");
    return UT_FAILED;
   }

   if(!$dt->setLanguage())
   {
    $this->e->add("Set date language to default shouldn't fail", "error");
    return UT_FAILED;
   }

   if($dt->getLanguage()!=$_config->get("language"))
   {
    $this->e->add("Date language should be set to default", "error");
    return UT_FAILED;
   }

   if($dt->setLanguage("adasdasd"))
   {
    $this->e->add("Set date language to \"adasdasd\" should fail", "error");
    return UT_FAILED;
   }

   if($dt->getLanguage()!="en")
   {
    $this->e->add("Date language should be set to: \"en\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function GetTimezone()
  {
   $dt=new DT();
   $current=date_default_timezone_get();

   if($dt->getTimezone()!=$current)
   {
    $this->e->add("Get timezone doesn't match: ".$current, "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   date_default_timezone_set("UTC");

   if($dt->getTimezone()!="UTC")
   {
    date_default_timezone_set($current);
    $this->e->add("Get timezone doesn't match: UTC", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   date_default_timezone_set("Europe/Amsterdam");

   if($dt->getTimezone()!="Europe/Amsterdam")
   {
    date_default_timezone_set($current);
    $this->e->add("Get timezone doesn't match: Europe/Amsterdam", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   date_default_timezone_set("Asia/Tokyo");

   if($dt->getTimezone()!="Asia/Tokyo")
   {
    date_default_timezone_set($current);
    $this->e->add("Get timezone doesn't match: Asia/Tokyo", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   date_default_timezone_set($current);

   //system timezone
   if(!($system=exec("date +%Z")))
   {
    $this->e->add("Get system timezone failed", "error");
    return UT_FAILED;
   }
   $system=timezone_name_from_abbr($system);

   if($dt->getTimezone(true)!=$system)
   {
    $this->e->add("Get system timezone doesn't match: ".$system, "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetTimezone()
  {
   global $_config;

   $dt=new DT();
   $current=date_default_timezone_get();

   if(!$dt->setTimezone("UTC"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if($dt->getTimezone()!="UTC")
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: UTC", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if(!$dt->setTimezone("Europe/Amsterdam"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if($dt->getTimezone()!="Europe/Amsterdam")
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: Europe/Amsterdam", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if(!$dt->setTimezone("Asia/Tokyo"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if($dt->getTimezone()!="Asia/Tokyo")
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: Asia/Tokyo", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if(!$dt->setTimezone($current))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if($dt->getTimezone()!=$current)
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: ".$current, "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if(!$dt->setTimezone())
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone shouldn't have failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if($dt->getTimezone()!=$_config->get("timezone"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone didn't set timezone to: ".$_config->get("timezone"), "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   if($dt->setTimezone("asdasdasd"))
   {
    date_default_timezone_set($current);
    $this->e->add("Set timezone should have failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   date_default_timezone_set($current);

   return UT_OK;
  }


  function Date()
  {
   $dt=new DT();

   $format="l j F Y H:i:s";
   $date=date($format);
   if(!($output=$dt->date($format)))
   {
    $this->e->add("Date shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=$output)
   {
    $this->e->add("Date output incorrect", "error");
    return UT_FAILED;
   }

//timestamp
   $timestamp=946684800;
   $date=date($format, $timestamp);
   if(!($output=$dt->date($format, false, $timestamp)))
   {
    $this->e->add("Date shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=$output)
   {
    $this->e->add("Date output incorrect", "error");
    return UT_FAILED;
   }

//timezone
   if(!($output=$dt->date($format, "UTC", $timestamp)))
   {
    $this->e->add("Date shouldn't fail", "error");
    return UT_FAILED;
   }

   if("Saturday 1 January 2000 00:00:00"!=$output)
   {
    $this->e->add("Date output incorrect", "error");
    return UT_FAILED;
   }

   if(!($output=$dt->date($format, "Europe/Amsterdam", $timestamp)))
   {
    $this->e->add("Date shouldn't fail", "error");
    return UT_FAILED;
   }

   if("Saturday 1 January 2000 01:00:00"!=$output)
   {
    $this->e->add("Date output incorrect", "error");
    return UT_FAILED;
   }

   if(!($output=$dt->date($format, "America/New_York", $timestamp)))
   {
    $this->e->add("Date shouldn't fail", "error");
    return UT_FAILED;
   }

   if("Friday 31 December 1999 19:00:00"!=$output)
   {
    $this->e->add("Date output incorrect", "error");
    return UT_FAILED;
   }


//language
   if(!$dt->setLanguage("nl"))
   {
    $this->e->add("Date set language failed", "error");
    return UT_FAILED;
   }

   $format="l j F Y H:i:s";
   if(!($output=$dt->date($format, "Europe/Amsterdam", $timestamp)))
   {
    $this->e->add("Date shouldn't fail", "error");
    return UT_FAILED;
   }

   if("Zaterdag 1 Januari 2000 01:00:00"!=$output)
   {
    $this->e->add("Date output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtract()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 9,
    "day" => 4,
    "hour" => 23,
    "minute" => 15,
    "second" => 10,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractMonths()
  {
   $dt=new DT();

//numeric month
   $format="Y m";
   $time="2012 02";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 2,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   $format="Y n";
   $time="2012 4";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 4,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

//month name
   $format="Y M";
   $time="2012 Jul";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 7,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   $format="Y F";
   $time="2012 November";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 11,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractMonthsDutch()
  {
   $dt=new DT();

   if(!$dt->setLanguage("nl"))
   {
    $this->e->add("Set date language shouldn't fail", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   $format="Y M";
   foreach($this->dutchMons as $nr => $month)
   {
    $time="2016 ".$month;
    if(!($date=$dt->dateExtract($time, $format)))
    {
     $this->e->add("Date extract shouldn't fail", "error");
     return UT_FAILED;
    }

    if($date!=Array(
     "year" => 2016,
     "month" => $nr+1,
     "day" => 1,
     "hour" => 0,
     "minute" => 0,
     "second" => 0,
    ))
    {
     $this->e->add("Date extract output incorrect", "error");
     return UT_FAILED;
    }
   }

   $format="Y F";
   foreach($this->dutchMonths as $nr => $month)
   {
    $time="2016 ".$month;
    if(!($date=$dt->dateExtract($time, $format)))
    {
     $this->e->add("Date extract shouldn't fail", "error");
     return UT_FAILED;
    }

    if($date!=Array(
     "year" => 2016,
     "month" => $nr+1,
     "day" => 1,
     "hour" => 0,
     "minute" => 0,
     "second" => 0,
    ))
    {
     $this->e->add("Date extract output incorrect", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


  function DateExtractWeeks()
  {
   $dt=new DT();

   $format="Y W";

//normal year
   $time="2012 01";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 1,
    "day" => 2,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 26";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 6,
    "day" => 25,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 52";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 12,
    "day" => 24,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//first week starting in previous year
   $time="2013 1";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 12,
    "day" => 31,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//week number over 52 weeks (next year)
   $time="2012 60";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2013,
    "month" => 2,
    "day" => 18,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//year starting on the first of jan
   $time="2006 52";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2006,
    "month" => 12,
    "day" => 25,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2007 1";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2007,
    "month" => 1,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//53 weeks year
   $time="2009 1";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2008,
    "month" => 12,
    "day" => 29,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2009 26";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2009,
    "month" => 6,
    "day" => 22,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2009 53";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2009,
    "month" => 12,
    "day" => 28,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

//leap day
   $time="2012 9";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 2,
    "day" => 27,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2011 9";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2011,
    "month" => 2,
    "day" => 28,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractDays()
  {
   $dt=new DT();

   $format="Y z";

//normal year
   $time="2012 01";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 1,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 182";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 6,
    "day" => 30,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 365";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 12,
    "day" => 30,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


//leap day
   $time="2012 366";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 12,
    "day" => 31,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }


   $time="2012 60";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 2,
    "day" => 29,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function DateExtractEscaped()
  {
   $dt=new DT();
   $timezone=date_default_timezone_get();

   $format="\D\a\y d \M\o\\n\\t\h m \Y\\e\a\\r Y";
   $time="Day 21 Month 06 Year 2015";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 6,
    "day" => 21,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateExtractOffset()
  {
   $dt=new DT();
   $timezone=date_default_timezone_get();

//Europe/Amsterdam (postive offset) with DST
   $format="Y-m-d H:i:sO";
   $time="2015-09-21 11:42:16+0200";
   date_default_timezone_set("UTC");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 9,
    "day" => 21,
    "hour" => 9,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   date_default_timezone_set("Asia/Tokyo"); //after UTC

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 9,
    "day" => 21,
    "hour" => 18,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   date_default_timezone_set("America/New_York"); //before UTC

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 9,
    "day" => 21,
    "hour" => 5,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//America/Sao_Paulo (negative offset) without DST
   $format="Y-m-d H:i:sP";
   $time="2015-02-21 11:42:16-03:00";
   date_default_timezone_set("UTC");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 2,
    "day" => 21,
    "hour" => 14,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   date_default_timezone_set("Asia/Tokyo"); //after UTC

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 2,
    "day" => 21,
    "hour" => 23,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   date_default_timezone_set("America/New_York"); //before UTC

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 2,
    "day" => 21,
    "hour" => 9,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//cross date boundary
   $format="Y-m-d H:i:sO";
   $time="2015-09-21 00:42:16+0200";
   date_default_timezone_set("UTC");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 9,
    "day" => 20,
    "hour" => 22,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   $time="2015-12-31 23:42:16-0500";
   date_default_timezone_set("UTC");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2016,
    "month" => 1,
    "day" => 1,
    "hour" => 4,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//DST
   $time="2015-10-25 02:42:16+0200"; //CEST
   date_default_timezone_set("UTC");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 10,
    "day" => 25,
    "hour" => 0,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   $time="2015-10-25 02:42:16+0100"; //CET
   date_default_timezone_set("UTC");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2015,
    "month" => 10,
    "day" => 25,
    "hour" => 1,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   $time="2016-03-13 01:42:16-0500"; //EST
   date_default_timezone_set("UTC");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2016,
    "month" => 3,
    "day" => 13,
    "hour" => 6,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   $time="2016-03-13 03:42:16-0400"; //EDT
   date_default_timezone_set("UTC");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2016,
    "month" => 3,
    "day" => 13,
    "hour" => 7,
    "minute" => 42,
    "second" => 16,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   date_default_timezone_set($timezone); //revert timezone
   return UT_OK;
  }

  function DateExtractComposite()
  {
   $dt=new DT();
   $timezone=date_default_timezone_get();

//RFC 2822
   $format="r";
   $time="Thu, 21 Dec 2000 16:01:07 +0200"; //EET
   date_default_timezone_set("Europe/Amsterdam");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2000,
    "month" => 12,
    "day" => 21,
    "hour" => 15,
    "minute" => 01,
    "second" => 07,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//RFC 2822
   $format="c";
   $time="2004-02-12T15:19:21+00:00"; //GMT
   date_default_timezone_set("America/New_York");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2004,
    "month" => 2,
    "day" => 12,
    "hour" => 10,
    "minute" => 19,
    "second" => 21,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//RFC 2822
   $format="c";
   $time="2004-02-12T15:19:21Z"; //UTC
   date_default_timezone_set("Europe/Amsterdam");

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2004,
    "month" => 2,
    "day" => 12,
    "hour" => 16,
    "minute" => 19,
    "second" => 21,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


   date_default_timezone_set($timezone); //revert timezone
   return UT_OK;
  }


  function DateExtractUnixTimestamp()
  {
   $dt=new DT();

   //set timezone to UTC
   $timezone=date_default_timezone_get();
   date_default_timezone_set("UTC");

   $format="U";

//Unix epoch
   $time=0;

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 1970,
    "month" => 1,
    "day" => 1,
    "hour" => 0,
    "minute" => 0,
    "second" => 0,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//"now"
   $time=1351271722;

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 10,
    "day" => 26,
    "hour" => 17,
    "minute" => 15,
    "second" => 22,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }


//multi value date
   $format="d-m-Y U H:i:s";
   $time="26-10-2012 1351271722 17:15:22";

   if(!($date=$dt->dateExtract($time, $format)))
   {
    $this->e->add("Date extract shouldn't fail", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   if($date!=Array(
    "year" => 2012,
    "month" => 10,
    "day" => 26,
    "hour" => 17,
    "minute" => 15,
    "second" => 22,
   ))
   {
    $this->e->add("Date extract output incorrect", "error");
    date_default_timezone_set($timezone); //revert timezone
    return UT_FAILED;
   }

   date_default_timezone_set($timezone); //revert timezone
   return UT_OK;
  }


  function DateExtractArray()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";
   $time1="2012-09-04 23:15:10";
   $time2="1920-10-23 01:02:03";

   if(!($date=$dt->dateExtract(Array($time1, $time2), $format)))
   {
    $this->e->add("Date extract array shouldn't fail", "error");
    return UT_FAILED;
   }

   if($date!=Array(
    Array(
     "year" => 2012,
     "month" => 9,
     "day" => 4,
     "hour" => 23,
     "minute" => 15,
     "second" => 10,
    ),
    Array(
     "year" => 1920,
     "month" => 10,
     "day" => 23,
     "hour" => 1,
     "minute" => 2,
     "second" => 3,
    ),
   ))
   {
    $this->e->add("Date extract array output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateConvert()
  {
   $dt=new DT();

//date
   $tstamp=time();
   $inf="Y-m-d";
   $outf="d-m-Y";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if($dt->dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

//date time
   $inf="Y-m-d H:i:s";
   $outf="H:i d-m-Y";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if($dt->dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

//to unix timestamp
   $inf="d m Y H i s";
   $outf=false;

   $in=date($inf, $tstamp);
   $out=$tstamp;

   if($dt->dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

//one big input string
   $inf="YmdHis";
   $outf="j n y G i s";

   $in=date($inf, $tstamp);
   $out=date($outf, $tstamp);

   if($dt->dateConvert($in, $inf, $outf)!==$out)
   {
    $this->e->add("Converting datetime from ".$inf." to ".$outf." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateConvertArray()
  {
   $dt=new DT();

   $tstamp=time();
   $inf="Y-m-d H:i:s";
   $outf="H:i d-m-Y";

   $in1=date($inf, $tstamp);
   $out1=date($outf, $tstamp);

   $in2=date($inf, $tstamp+73945);
   $out2=date($outf, $tstamp+73945);

   if(!($values=$dt->dateConvert(Array($in1, $in2), $inf, $outf)))
   {
    $this->e->add("Date time array convert failed", "error");
    return UT_FAILED;
   }
   if($values[0]!=$out1)
   {
    $this->e->add("Date time array convert argument 1 incorrect", "error");
    return UT_FAILED;
   }
   if($values[1]!=$out2)
   {
    $this->e->add("Date time array convert argument 2 incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateDelta()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";
   $time="2012-09-04 23:15:10";

   if($dt->dateDelta($time, 5, "second", $format)!="2012-09-04 23:15:15")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -15, "seconds", $format)!="2012-09-04 23:14:55")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 55, "minute", $format)!="2012-09-05 00:10:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -16, "minutes", $format)!="2012-09-04 22:59:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 2, "hour", $format)!="2012-09-05 01:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -28, "hours", $format)!="2012-09-03 19:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 13, "day", $format)!="2012-09-17 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -15, "days", $format)!="2012-08-20 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 5, "week", $format)!="2012-10-09 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -2, "weeks", $format)!="2012-08-21 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 5, "month", $format)!="2013-02-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -15, "months", $format)!="2011-06-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, 24, "year", $format)!="2036-09-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   if($dt->dateDelta($time, -15, "years", $format)!="1997-09-04 23:15:10")
   {
    $this->e->add("Date delta output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function DateDeltaArray()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";
   $time1="2012-09-04 23:15:10";
   $time2="1920-10-23 01:02:03";

   if($dt->dateDelta(Array($time1, $time2), -70, "second", $format)!=Array("2012-09-04 23:14:00", "1920-10-23 01:00:53"))
   {
    $this->e->add("Date delta array output incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TimezoneConvert()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";

   $intime="2012-05-27 13:39:29";
   $intz="Europe/Amsterdam";

   $outtime="2012-05-27 20:39:29";
   $outtz="Asia/Tokyo";

   if($dt->timezoneConvert($intime, $intz, $outtz, $format)!==$outtime)
   {
    $this->e->add("Converting timezone from ".$intz." to ".$outtz." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TimezoneConvertArray()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";

   $intime1="2012-01-15 23:15:10";
   $intime2="2012-05-27 13:39:29";
   $intz="Europe/Amsterdam";

   $outtime1="2012-01-16 07:15:10";
   $outtime2="2012-05-27 20:39:29";
   $outtz="Asia/Tokyo";

   if(!($values=$dt->timezoneConvert(Array($intime1, $intime2), $intz, $outtz, $format)))
   {
    $this->e->add("Converting timezone from ".$intz." to ".$outtz." failed", "error");
    $this->e->add($dt->e);
    return UT_FAILED;
   }
   if($values[0]!=$outtime1)
   {
    $this->e->add("Timezone array convert argument 1 incorrect", "error");
    return UT_FAILED;
   }
   if($values[1]!=$outtime2)
   {
    $this->e->add("Timezone array convert argument 2 incorrect", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function TimeAgo()
  {
   $dt=new DT();

   $format="Y-m-d H:i:s";

   //seconds
   if($dt->timeAgo(date($format, time()-86400), $format, "seconds")!="86400 seconds ago")
   {
    $this->e->add("Time 86400 seconds ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-2), $format, "seconds")!="2 seconds ago")
   {
    $this->e->add("Time 2 seconds ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-1), $format, "seconds")!="1 second ago")
   {
    $this->e->add("Time 1 second ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-0), $format, "seconds")!="just now")
   {
    $this->e->add("Time 0 seconds ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+1), $format, "seconds")!="in 1 second")
   {
    $this->e->add("Time in 1 second doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+2), $format, "seconds")!="in 2 seconds")
   {
    $this->e->add("Time in 2 seconds doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+86400), $format, "seconds")!="in 86400 seconds")
   {
    $this->e->add("Time in 86400 seconds doesn't match", "error");
    return UT_FAILED;
   }

   //minutes
   if($dt->timeAgo(date($format, time()-86400), $format, "minutes")!="1440 minutes ago")
   {
    $this->e->add("Time 1440 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-2*60), $format, "minutes")!="2 minutes ago")
   {
    $this->e->add("Time 2 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-60), $format, "minutes")!="1 minute ago")
   {
    $this->e->add("Time 1 minute ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-30), $format, "minutes")!="less than a minute ago")
   {
    $this->e->add("Time 0 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-0), $format, "minutes")!="less than a minute ago")
   {
    $this->e->add("Time 0 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+30), $format, "minutes")!="in less than a minute")
   {
    $this->e->add("Time in 0 minutes doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+60), $format, "minutes")!="in 1 minute")
   {
    $this->e->add("Time in 1 minute doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+2*60), $format, "minutes")!="in 2 minutes")
   {
    $this->e->add("Time in 2 minutes doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+86400), $format, "minutes")!="in 1440 minutes")
   {
    $this->e->add("Time in 1440 minutes doesn't match", "error");
    return UT_FAILED;
   }

   //hours
   if($dt->timeAgo(date($format, time()-86400), $format, "hours")!="24 hours ago")
   {
    $this->e->add("Time 24 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-2*3600), $format, "hours")!="2 hours ago")
   {
    $this->e->add("Time 2 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-3600), $format, "hours")!="1 hour ago")
   {
    $this->e->add("Time 1 hour ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-1800), $format, "hours")!="less than an hour ago")
   {
    $this->e->add("Time 0 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-0), $format, "hours")!="less than an hour ago")
   {
    $this->e->add("Time 0 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+1800), $format, "hours")!="in less than an hour")
   {
    $this->e->add("Time in 0 hours doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+3600), $format, "hours")!="in 1 hour")
   {
    $this->e->add("Time in 1 hour doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+2*3600), $format, "hours")!="in 2 hours")
   {
    $this->e->add("Time in 2 hours doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+86400), $format, "hours")!="in 24 hours")
   {
    $this->e->add("Time in 24 hours doesn't match", "error");
    return UT_FAILED;
   }

   //days
   if($dt->timeAgo(date($format, time()-24*86400), $format, "days")!="24 days ago")
   {
    $this->e->add("Time 24 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-2*86400), $format, "days")!="2 days ago")
   {
    $this->e->add("Time 2 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-86400), $format, "days")!="1 day ago")
   {
    $this->e->add("Time 1 day ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-43200), $format, "days")!="today")
   {
    $this->e->add("Time 0 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-0), $format, "days")!="today")
   {
    $this->e->add("Time 0 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+43200), $format, "days")!="today")
   {
    $this->e->add("Time in 0 days doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+86400), $format, "days")!="in 1 day")
   {
    $this->e->add("Time in 1 day doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+2*86400), $format, "days")!="in 2 days")
   {
    $this->e->add("Time in 2 days doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+24*86400), $format, "days")!="in 24 days")
   {
    $this->e->add("Time in 24 days doesn't match", "error");
    return UT_FAILED;
   }

   //weeks
   if($dt->timeAgo(date($format, time()-24*7*86400), $format, "weeks")!="24 weeks ago")
   {
    $this->e->add("Time 24 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-2*7*86400), $format, "weeks")!="2 weeks ago")
   {
    $this->e->add("Time 2 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-7*86400), $format, "weeks")!="1 week ago")
   {
    $this->e->add("Time 1 week ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-4*86400), $format, "weeks")!="this week")
   {
    $this->e->add("Time 0 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-0), $format, "weeks")!="this week")
   {
    $this->e->add("Time 0 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+4*86400), $format, "weeks")!="coming week")
   {
    $this->e->add("Time in 0 weeks doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+7*86400), $format, "weeks")!="in 1 week")
   {
    $this->e->add("Time in 1 week doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+2*7*86400), $format, "weeks")!="in 2 weeks")
   {
    $this->e->add("Time in 2 weeks doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+24*7*86400), $format, "weeks")!="in 24 weeks")
   {
    $this->e->add("Time in 24 weeks doesn't match", "error");
    return UT_FAILED;
   }

   //months
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")-24)), $format, "months")!="24 months ago")
   {
    $this->e->add("Time 24 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")-2)), $format, "months")!="2 months ago")
   {
    $this->e->add("Time 2 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")-1)), $format, "months")!="1 month ago")
   {
    $this->e->add("Time 1 month ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-10), $format, "months")!="this month")
   {
    $this->e->add("Time 0 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-0), $format, "months")!="this month")
   {
    $this->e->add("Time 0 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+10), $format, "months")!="coming month")
   {
    $this->e->add("Time in 0 months doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")+1)), $format, "months")!="in 1 month")
   {
    $this->e->add("Time in 1 month doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")+2)), $format, "months")!="in 2 months")
   {
    $this->e->add("Time in 2 months doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")+24)), $format, "months")!="in 24 months")
   {
    $this->e->add("Time in 24 months doesn't match", "error");
    return UT_FAILED;
   }

   //years
   if($dt->timeAgo(date($format, time()-24*365*86400), $format, "years")!="24 years ago")
   {
    $this->e->add("Time 24 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-2*365*86400), $format, "years")!="2 years ago")
   {
    $this->e->add("Time 2 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-365*86400), $format, "years")!="1 year ago")
   {
    $this->e->add("Time 1 year ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-180*86400), $format, "years")!="this year")
   {
    $this->e->add("Time 0 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-0), $format, "years")!="this year")
   {
    $this->e->add("Time 0 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+180*86400), $format, "years")!="coming year")
   {
    $this->e->add("Time in 0 years doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+365*86400), $format, "years")!="in 1 year")
   {
    $this->e->add("Time in 1 year doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+2*365*86400), $format, "years")!="in 2 years")
   {
    $this->e->add("Time in 2 years doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+10*365*86400), $format, "years")!="in 10 years")
   {
    $this->e->add("Time in 10 years doesn't match", "error");
    return UT_FAILED;
   }

   //autodetect
   if($dt->timeAgo(date($format, time()-24*365*86400), $format)!="24 years ago")
   {
    $this->e->add("Time 24 years ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-365*86400), $format)!="1 year ago")
   {
    $this->e->add("Time 1 year ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")-11)), $format)!="11 months ago")
   {
    $this->e->add("Time 11 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")-1)), $format)!="1 month ago")
   {
    $this->e->add("Time 1 months ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-3*7*86400), $format)!="3 weeks ago")
   {
    $this->e->add("Time 3 weeks ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-7*86400), $format)!="1 week ago")
   {
    $this->e->add("Time 1 week ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-6*86400), $format)!="6 days ago")
   {
    $this->e->add("Time 6 days ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-86400), $format)!="1 day ago")
   {
    $this->e->add("Time 1 day ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-23*3600), $format)!="23 hours ago")
   {
    $this->e->add("Time 23 hours ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-3600), $format)!="1 hour ago")
   {
    $this->e->add("Time 1 hour ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-59*60), $format)!="59 minutes ago")
   {
    $this->e->add("Time 59 minutes ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-60), $format)!="1 minute ago")
   {
    $this->e->add("Time 1 minute ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-59), $format)!="59 seconds ago")
   {
    $this->e->add("Time 59 seconds ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()-1), $format)!="1 second ago")
   {
    $this->e->add("Time 1 second ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()), $format)!="just now")
   {
    $this->e->add("Time 0 second ago doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+1), $format)!="in 1 second")
   {
    $this->e->add("Time in 1 second doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+59), $format)!="in 59 seconds")
   {
    $this->e->add("Time in 59 seconds doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+60), $format)!="in 1 minute")
   {
    $this->e->add("Time in 1 minute doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+59*60), $format)!="in 59 minutes")
   {
    $this->e->add("Time in 59 minutes doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+3600), $format)!="in 1 hour")
   {
    $this->e->add("Time in 1 hour doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+23*3600), $format)!="in 23 hours")
   {
    $this->e->add("Time in 23 hours doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+86400), $format)!="in 1 day")
   {
    $this->e->add("Time in 1 day doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+6*86400), $format)!="in 6 days")
   {
    $this->e->add("Time in 6 days doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+7*86400), $format)!="in 1 week")
   {
    $this->e->add("Time in 1 week doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+3*7*86400), $format)!="in 3 weeks")
   {
    $this->e->add("Time in 3 weeks doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")+1)), $format)!="in 1 month")
   {
    $this->e->add("Time in 1 months doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, mktime(0, 0, 0, date("m")+11)), $format)!="in 11 months")
   {
    $this->e->add("Time in 11 months doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+365*86400), $format)!="in 1 year")
   {
    $this->e->add("Time in 1 year doesn't match", "error");
    return UT_FAILED;
   }
   if($dt->timeAgo(date($format, time()+10*365*86400), $format)!="in 10 years")
   {
    $this->e->add("Time in 10 years doesn't match", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }

?>
