<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/format_email.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/format.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["format"]."email.php");

 class UT_Format_Email extends UT_Format
 {
  function Constructor()
  {
   $object=new Format_Email();

   if(!is_object($object)||get_class($object)!="Format_Email")
   {
    $this->e->add("Constructor doesn't return a Email object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   //config merge
   $object=new Format_Email(Array("mimeVersion" => "0.1", "encoding" => "TEST"));

   if($object->mimeVersion!="0.1")
   {
    $this->e->add("Email mime version should be \"0.1\"");
    return UT_FAILED;
   }

   if($object->encoding!="TEST")
   {
    $this->e->add("Email encoding should be \"TEST\"");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Format_Email();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Format_Email object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->userAgent=-1;
   if($orig->userAgent==$copy->userAgent)
   {
    $this->e->add("Format_Email object copy also modified original \"userAgent\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Format_Email object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->mime->fake=-1;
   if(isset($orig->mime->fake)&&$orig->mime->fake==$copy->mime->fake)
   {
    $this->e->add("Format_Email object copy also modified original \"Mime\" object", "error");
    return UT_FAILED;
   }

   return UT_Format::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function Clear()
  {
   $email=new Format_Email();

   if(!$email->from("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $email->clear("From");
   if($email->data["From"])
   {
    $this->e->add("Clear from address failed", "error");
    return UT_FAILED;
   }

   if(!$email->to("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $email->clear("To");
   if($email->data["To"])
   {
    $this->e->add("Clear to address failed", "error");
    return UT_FAILED;
   }

   if(!$email->cc("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $email->clear("Cc");
   if($email->data["Cc"])
   {
    $this->e->add("Clear cc address failed", "error");
    return UT_FAILED;
   }

   if(!$email->bcc("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $email->clear("Bcc");
   if($email->data["Bcc"])
   {
    $this->e->add("Clear bcc address failed", "error");
    return UT_FAILED;
   }

   if(!$email->subject("Test subject"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $email->clear("Subject");
   if($email->data["Subject"])
   {
    $this->e->add("Clear subject failed", "error");
    return UT_FAILED;
   }

   if(!$email->message("Test message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $email->clear("Attachment");
   if($email->data["Attachment"])
   {
    $this->e->add("Clear attachments failed", "error");
    return UT_FAILED;
   }

   $email=new Format_Email();
   $email->from("test@unit.jas");
   $email->to("test@unit.jas");
   $email->cc("test@unit.jas");
   $email->bcc("test@unit.jas");
   $email->subject("Test subject");
   $email->message("Test message");


   $email->clear();
   if($email->data["From"])
   {
    $this->e->add("Clear all didn't clear from", "error");
    return UT_FAILED;
   }
   if($email->data["To"])
   {
    $this->e->add("Clear all didn't clear to", "error");
    return UT_FAILED;
   }
   if($email->data["Cc"])
   {
    $this->e->add("Clear all didn't clear cc", "error");
    return UT_FAILED;
   }
   if($email->data["Bcc"])
   {
    $this->e->add("Clear all didn't clear bcc", "error");
    return UT_FAILED;
   }
   if($email->data["Subject"])
   {
    $this->e->add("Clear all didn't clear subject", "error");
    return UT_FAILED;
   }
   if($email->data["Attachment"])
   {
    $this->e->add("Clear all didn't clear attachments", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Set()
  {
   $email=new Format_Email();

//single
   if(!$email->set("Return-Path", "test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Return-Path"][0]!="test@unit.jas")
   {
    $this->e->add("Set single value failed", "error");
    return UT_FAILED;
   }
   if($email->vmax["Return-Path"]!=1)
   {
    $this->e->add("Set single value vmax failed", "error");
    return UT_FAILED;
   }

//single overwrite
   if(!$email->set("Return-Path", "test2@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Return-Path"][0]!="test2@unit.jas")
   {
    $this->e->add("Overwrite single value failed", "error");
    return UT_FAILED;
   }
   if($email->vmax["Return-Path"]!=1)
   {
    $this->e->add("Set single value vmax failed", "error");
    return UT_FAILED;
   }

//list / multi
   if(!$email->set("Received", "from unit.jas (123@1.2.3.4)"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->set("Received", "from unit.jas (345@3.4.5.6)"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Received"][0]!="from unit.jas (123@1.2.3.4)")
   {
    $this->e->add("Set multi / list value failed", "error");
    return UT_FAILED;
   }
   if($email->data["Received"][1]!="from unit.jas (345@3.4.5.6)")
   {
    $this->e->add("Set multi / list value failed", "error");
    return UT_FAILED;
   }
   if($email->vmax["Received"]!=2)
   {
    $this->e->add("Set multi / list value vmax failed", "error");
    return UT_FAILED;
   }

//ovewrite list / multi
   if(!$email->set("Received", "from unit.jas (test1@1.2.3.4)", 0))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->set("Received", "from unit.jas (test2@3.4.5.6)", 1))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Received"][0]!="from unit.jas (test1@1.2.3.4)")
   {
    $this->e->add("Overwrite multi / list value failed", "error");
    return UT_FAILED;
   }
   if($email->data["Received"][1]!="from unit.jas (test2@3.4.5.6)")
   {
    $this->e->add("Overwrite multi / list value failed", "error");
    return UT_FAILED;
   }
   if($email->vmax["Received"]!=2)
   {
    $this->e->add("Set multi / list value vmax failed", "error");
    return UT_FAILED;
   }

//new key
   if(!$email->set("ASDASD", "test"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["ASDASD"][0]!="test")
   {
    $this->e->add("Set new value failed", "error");
    return UT_FAILED;
   }
   if($email->vmax["ASDASD"]!=1)
   {
    $this->e->add("Set new value vmax failed", "error");
    return UT_FAILED;
   }

//out of bound
   if($email->set("ASDASD", "test2", 23423))
   {
    $this->e->add("Set out of bound should have failed");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Get()
  {
   $email=new Format_Email();

//single
   if(!$email->set("Return-Path", "test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->get("Return-Path")!="test@unit.jas")
   {
    $this->e->add("Get single value failed", "error");
    return UT_FAILED;
   }

//single overwrite
   if(!$email->set("Return-Path", "test2@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->get("Return-Path")!="test2@unit.jas")
   {
    $this->e->add("Get single value failed", "error");
    return UT_FAILED;
   }

//list / multi
   if(!$email->set("Received", "from unit.jas (123@1.2.3.4)"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$email->set("Received", "from unit.jas (345@3.4.5.6)"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->get("Received")!="from unit.jas (123@1.2.3.4)")
   {
    $this->e->add("Get multi / list value failed", "error");
    return UT_FAILED;
   }
   if($email->get("Received", 1)!="from unit.jas (345@3.4.5.6)")
   {
    $this->e->add("Get multi / list value failed", "error");
    return UT_FAILED;
   }

//new key
   if(!$email->set("ASDASD", "test"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->get("ASDASD")!="test")
   {
    $this->e->add("Get single value failed", "error");
    return UT_FAILED;
   }

//out of bound
   if($email->get("ASDASD", 23423))
   {
    $this->e->add("Get out of bound should have failed");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Reset()
  {
   $email=new Format_Email();

//key
   $email->v="asd";
   $email->vc["To"]=1;

   $email->reset("To");

   if($email->v!==false)
   {
    $this->e->add("Reset value failed");
    return UT_FAILED;
   }

   if($email->vc["To"]!==0)
   {
    $this->e->add("Reset value counter key failed");
    return UT_FAILED;
   }

//all
   $email->v="asd";
   $email->vc["To"]=1;
   $email->vc["Cc"]=1;

   $email->reset();

   if($email->v!==false)
   {
    $this->e->add("Reset v failed");
    return UT_FAILED;
   }

   if($email->vc["To"]!==0)
   {
    $this->e->add("Reset value counter key failed");
    return UT_FAILED;
   }

   if($email->vc["Cc"]!==0)
   {
    $this->e->add("Reset value counter key failed");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Next()
  {
   $email=new Format_Email();

//multi
   if(
    !$email->set("test", 1)||
    !$email->set("test", 2)||
    !$email->set("test", 3)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->next("test"))
   {
    $this->e->add("Next shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==1)
   {
    $this->e->add("Next value doesn't match");
    return UT_FAILED;
   }

   if(!$email->next("test"))
   {
    $this->e->add("Next shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==2)
   {
    $this->e->add("Next value doesn't match");
    return UT_FAILED;
   }

   if(!$email->next("test"))
   {
    $this->e->add("Next shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==3)
   {
    $this->e->add("Next value doesn't match");
    return UT_FAILED;
   }

   if($email->next("test"))
   {
    $this->e->add("Next should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Next value doesn't match");
    return UT_FAILED;
   }

//single
   if(
    !$email->set("From", 1)||
    !$email->set("From", 2)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->next("From"))
   {
    $this->e->add("Next shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==2)
   {
    $this->e->add("Next value doesn't match");
    return UT_FAILED;
   }

   if($email->next("From"))
   {
    $this->e->add("Next should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Next value doesn't match");
    return UT_FAILED;
   }

//incorrect key
   $email->v="asd";
   if($email->next("ASDASD"))
   {
    $this->e->add("Next should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Next value doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Previous()
  {
   $email=new Format_Email();

//multi
   if(
    !$email->set("test", 1)||
    !$email->set("test", 2)||
    !$email->set("test", 3)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->previous("test"))
   {
    $this->e->add("Previous shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==3)
   {
    $this->e->add("Previous value doesn't match");
    return UT_FAILED;
   }

   if(!$email->previous("test"))
   {
    $this->e->add("Previous shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==2)
   {
    $this->e->add("Previous value doesn't match");
    return UT_FAILED;
   }

   if(!$email->previous("test"))
   {
    $this->e->add("Previous shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==1)
   {
    $this->e->add("Previous value doesn't match");
    return UT_FAILED;
   }

   if($email->previous("test"))
   {
    $this->e->add("Previous should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Previous value doesn't match");
    return UT_FAILED;
   }

//single
   if(
    !$email->set("From", 1)||
    !$email->set("From", 2)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->previous("From"))
   {
    $this->e->add("Previous shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==2)
   {
    $this->e->add("Previous value doesn't match");
    return UT_FAILED;
   }

   if($email->previous("From"))
   {
    $this->e->add("Previous should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Previous value doesn't match");
    return UT_FAILED;
   }

//incorrect key
   $email->v="asd";
   if($email->previous("ASDASD"))
   {
    $this->e->add("Previous should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Previous value doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function First()
  {
   $email=new Format_Email();

//multi
   if(
    !$email->set("test", 1)||
    !$email->set("test", 2)||
    !$email->set("test", 3)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->first("test"))
   {
    $this->e->add("First shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==1)
   {
    $this->e->add("First value doesn't match");
    return UT_FAILED;
   }

//single
   if(
    !$email->set("From", 1)||
    !$email->set("From", 2)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->first("From"))
   {
    $this->e->add("First shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==2)
   {
    $this->e->add("First value doesn't match");
    return UT_FAILED;
   }

//incorrect key
   $email->v="asd";
   if($email->first("ASDASD"))
   {
    $this->e->add("First should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("First value doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Last()
  {
   $email=new Format_Email();

//multi
   if(
    !$email->set("test", 1)||
    !$email->set("test", 2)||
    !$email->set("test", 3)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->last("test"))
   {
    $this->e->add("Last shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==3)
   {
    $this->e->add("Last value doesn't match");
    return UT_FAILED;
   }

//single
   if(
    !$email->set("From", 1)||
    !$email->set("From", 2)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->last("From"))
   {
    $this->e->add("Last shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==2)
   {
    $this->e->add("Last value doesn't match");
    return UT_FAILED;
   }

//incorrect key
   $email->v="asd";
   if($email->last("ASDASD"))
   {
    $this->e->add("Last should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Last value doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Search()
  {
   $email=new Format_Email();

//multi
   if(
    !$email->set("test", "test1")||
    !$email->set("test", "test2")||
    !$email->set("test", "test3")
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->search("test", "1"))
   {
    $this->e->add("Search shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!=="test1")
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if($email->search("test", "1"))
   {
    $this->e->add("Search should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if(!$email->search("test", "2"))
   {
    $this->e->add("Search shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!=="test2")
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if($email->search("test", "2"))
   {
    $this->e->add("Search should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if(!$email->search("test", "3"))
   {
    $this->e->add("Search shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!=="test3")
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if($email->search("test", "3"))
   {
    $this->e->add("Search should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }


   if(!$email->search("test", "test"))
   {
    $this->e->add("Search shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!=="test1")
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if(!$email->search("test", "test"))
   {
    $this->e->add("Search shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!=="test2")
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if(!$email->search("test", "test"))
   {
    $this->e->add("Search shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!=="test3")
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if($email->search("test", "test"))
   {
    $this->e->add("Search should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

//single
   if(
    !$email->set("From", "test1")||
    !$email->set("From", "test2")
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->search("From", "1"))
   {
    $this->e->add("Search should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if(!$email->search("From", "2"))
   {
    $this->e->add("Search shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!=="test2")
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if($email->search("From", "2"))
   {
    $this->e->add("Search should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }


   if(!$email->search("From", "test"))
   {
    $this->e->add("Search shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!=="test2")
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   if($email->search("From", "test"))
   {
    $this->e->add("Search should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

//incorrect key
   $email->v="asd";
   if($email->search("ASDASD", "test"))
   {
    $this->e->add("Search should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Search value doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Value()
  {
   $email=new Format_Email();

//multi
   if(
    !$email->set("test", 1)||
    !$email->set("test", 2)||
    !$email->set("test", 3)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->value("test", 0))
   {
    $this->e->add("Value shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==1)
   {
    $this->e->add("Value value doesn't match");
    return UT_FAILED;
   }

   if(!$email->value("test", 1))
   {
    $this->e->add("Value shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==2)
   {
    $this->e->add("Value value doesn't match");
    return UT_FAILED;
   }

   if(!$email->value("test", 2))
   {
    $this->e->add("Value shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==3)
   {
    $this->e->add("Value value doesn't match");
    return UT_FAILED;
   }

   if($email->value("test", 123))
   {
    $this->e->add("Value should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Value value doesn't match");
    return UT_FAILED;
   }

//single
   if(
    !$email->set("From", 1)||
    !$email->set("From", 2)
   )
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->value("From", 0))
   {
    $this->e->add("Value shouldn't have failed");
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->v!==2)
   {
    $this->e->add("Value value doesn't match");
    return UT_FAILED;
   }

   if($email->value("From", 1))
   {
    $this->e->add("Value should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Value value doesn't match");
    return UT_FAILED;
   }

//incorrect key
   $email->v="asd";
   if($email->value("ASDASD", 0))
   {
    $this->e->add("Value should have failed");
    return UT_FAILED;
   }
   if($email->v!==false)
   {
    $this->e->add("Value value doesn't match");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse()
  {
   $email=new Format_Email();

//text
   $content="Content-Type: text/plain; charset=utf-8; format=flowed; delsp=yes
To: \"Unit test\" <test@unit.jas>, Unit <unit@test.jas>
Cc: <test@unit.jas>, unit@test.jas
Date: Mon, 16 Jun 2014 13:49:23 +0200
Subject: test1
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
From: \"JAS\" <jas@unit.jas>
Organization: JAS
Message-ID: <ojn213g0808nafg@unit.jas>
User-Agent: Custom Client/1.0
Status: RO


Test message
";

$expected="From: \"JAS\" <jas@unit.jas>\r
To: \"Unit test\" <test@unit.jas>, \"Unit\" <unit@test.jas>\r
Cc: test@unit.jas, unit@test.jas\r
Subject: test1\r
Message-ID: <ojn213g0808nafg@unit.jas>\r
Date: Mon, 16 Jun 2014 13:49:23 +0200\r
MIME-Version: 1.0\r
Organization: JAS\r
User-Agent: Custom Client/1.0\r
Status: RO\r
Content-Type: text/plain; charset=utf-8\r
Content-Transfer-Encoding: 7bit\r
\r
\r
Test message\r
\r
";

   if(!$email->parse($content))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$this->_compare($email->output(), $expected))
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }


//html
   $content="Return-Path: jas@unit.jas
Delivered-To: test@unit.jas
Received: from mail.unit.jas (@1.2.3.4)
        by mail.unit.jas with SMTP; Mon, 16 Jun 2014 13:49:23 +0200
Delivered-To: testing@unit.jas
Received: from mail.unit.jas (@1.2.3.4)
        by mail.unit.jas with SMTP; Mon, 16 Jun 2014 13:49:23 +0200
From: \"JAS\" <jas@unit.jas>
To: \"Unit test\" <test@unit.jas>
Subject: test2
Message-ID: <ojn213g0808nafg@unit.jas>
Date: Mon, 16 Jun 2014 13:49:23 +0200
Content-Type: text/html; charset=utf-8


<html><body>Test HTML message</body></html>
";

   $expected="Return-Path: jas@unit.jas\r
Delivered-To: test@unit.jas\r
Received: from mail.unit.jas (@1.2.3.4) by mail.unit.jas with SMTP\r
          ; Mon, 16 Jun 2014 13:49:23 +0200\r
Delivered-To: testing@unit.jas\r
Received: from mail.unit.jas (@1.2.3.4) by mail.unit.jas with SMTP\r
          ; Mon, 16 Jun 2014 13:49:23 +0200\r
From: \"JAS\" <jas@unit.jas>\r
To: \"Unit test\" <test@unit.jas>\r
Subject: test2\r
Message-ID: <ojn213g0808nafg@unit.jas>\r
Date: Mon, 16 Jun 2014 13:49:23 +0200\r
MIME-Version: 1.0\r
User-Agent: JAS/1.0\r
Content-Type: text/html; charset=utf-8\r
Content-Transfer-Encoding: 7bit\r
\r
\r
<html><body>Test HTML message</body></html>\r
\r
";

   if(!$email->parse($content))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$this->_compare($email->output(), $expected))
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }


//multipart/alternative
   $content="From: jas@unit.jas
Message-ID: <ojn213g0808nafg@unit.jas>
Date: Mon, 16 Jun 2014 13:49:23 +0200
Content-Type: multipart/alternative; boundary=----------kTpZ8hm0Y3rptroBHYbvf2


------------kTpZ8hm0Y3rptroBHYbvf2
Content-Type: text/plain; charset=utf-8; format=flowed; delsp=yes
Content-Transfer-Encoding: 7bit

Test message

------------kTpZ8hm0Y3rptroBHYbvf2
Content-Type: text/html; charset=utf-8
Content-ID: <op.1402919507350.2b2414645d696208@2001:960:7bc:1:da50:e6ff:febb:ba40>
Content-Transfer-Encoding: Quoted-Printable


<html><body>Test HTML message</body></html>
------------kTpZ8hm0Y3rptroBHYbvf2--

";


   $expected="From: jas@unit.jas\r
Message-ID: <ojn213g0808nafg@unit.jas>\r
Date: Mon, 16 Jun 2014 13:49:23 +0200\r
MIME-Version: 1.0\r
User-Agent: JAS/1.0\r
Content-Type: multipart/alternative; boundary=----------kTpZ8hm0Y3rptroBHYbvf2\r
\r
------------kTpZ8hm0Y3rptroBHYbvf2\r
Content-Type: text/plain; charset=utf-8\r
Content-Transfer-Encoding: 7bit\r
Content-Disposition: inline\r
\r
Test message\r
\r
------------kTpZ8hm0Y3rptroBHYbvf2\r
Content-Type: text/html; charset=utf-8\r
Content-Transfer-Encoding: Quoted-Printable\r
\r
\r
<html><body>Test HTML message</body></html>\r
------------kTpZ8hm0Y3rptroBHYbvf2--\r
";


   if(!$email->parse($content))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$this->_compare($email->output(), $expected))
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }


//multipart/mixed
   $content="From: jas@unit.jas
Message-ID: <ojn213g0808nafg@unit.jas>
Date: Mon, 16 Jun 2014 13:49:23 +0200
Content-Type: multipart/mixed; boundary=----------kTpZ8hm0Y3rptroBHYbvf2


------------kTpZ8hm0Y3rptroBHYbvf2
Content-Type: text/plain; charset=utf-8; format=flowed; delsp=yes
Content-Transfer-Encoding: 7bit

Test message

------------kTpZ8hm0Y3rptroBHYbvf2
Content-Type: image/png
Content-Transfer-Encoding: base64

iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJ
bWFnZVJlYWR5ccllPAAAAdpJREFUeNo8Ur9rIkEYHXWyu+rC+otzhWMrXU7rdMJeawzkIG0qIYWk
C4FrUl8ZuCpcEch/cLG7w0I4YQvhCis5AiIuQaKy6Gr8sa4/8j4vZGCYmW++ee997xtfs9lkPp+P
0dhsNmy9Xuc5599WqxUbjUbXOJu07/f7bDqdMs7+j49IvsKs41JLJBKfKanX6x0C7AP2BnJuMJ/4
brdTgPgjHo8fj8fj8mw2s5bLJfM8j4AuYrGYFolEpEajkQHgGcdrZ7FYPPv9fg9oUjKZ1Ofz+V5m
JpPR6SGYvHa7/bzdbh2OVyXXdW1cuAgcdLvdVavV+hUMBlk2mz1SFEVAjhsKhWwoKXFJku5UVfVD
DiPdlmX9tm37NBwOs8Fg8IDEE+zlYrH4tVarbbnjOAzI5A6DLBaNRt8dIzm0kkSqCwCMdzqdcwQ/
6bp+ATZZ07QCjPhJbKlUqkC1wNKXarV6i9g/LsvyPYJ3oBWJBauQTqe/EHIgENgzwwgRtcTBeE8u
KbhQ0ZQDFLscDoeWYRg6SahUKo+5XE5DjRLO6mQyUTiSHSSV6/X6vnHohwbG73COOn1rmqYFQwxR
FG8wnX2nUeyTIAiX0E4G5IH8h74BHv5F3ERtD28/gr0KMAB7fRvr2KRa3wAAAABJRU5ErkJggg==
------------kTpZ8hm0Y3rptroBHYbvf2--

";


   $expected="From: jas@unit.jas\r
Message-ID: <ojn213g0808nafg@unit.jas>\r
Date: Mon, 16 Jun 2014 13:49:23 +0200\r
MIME-Version: 1.0\r
User-Agent: JAS/1.0\r
Content-Type: multipart/mixed; boundary=----------kTpZ8hm0Y3rptroBHYbvf2\r
\r
------------kTpZ8hm0Y3rptroBHYbvf2\r
Content-Type: text/plain; charset=utf-8\r
Content-Transfer-Encoding: 7bit\r
\r
Test message\r
\r
------------kTpZ8hm0Y3rptroBHYbvf2\r
Content-Type: image/png\r
Content-Transfer-Encoding: base64\r
\r
iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJ\r
bWFnZVJlYWR5ccllPAAAAdpJREFUeNo8Ur9rIkEYHXWyu+rC+otzhWMrXU7rdMJeawzkIG0qIYWk\r
C4FrUl8ZuCpcEch/cLG7w0I4YQvhCis5AiIuQaKy6Gr8sa4/8j4vZGCYmW++ee997xtfs9lkPp+P\r
0dhsNmy9Xuc5599WqxUbjUbXOJu07/f7bDqdMs7+j49IvsKs41JLJBKfKanX6x0C7AP2BnJuMJ/4\r
brdTgPgjHo8fj8fj8mw2s5bLJfM8j4AuYrGYFolEpEajkQHgGcdrZ7FYPPv9fg9oUjKZ1Ofz+V5m\r
JpPR6SGYvHa7/bzdbh2OVyXXdW1cuAgcdLvdVavV+hUMBlk2mz1SFEVAjhsKhWwoKXFJku5UVfVD\r
DiPdlmX9tm37NBwOs8Fg8IDEE+zlYrH4tVarbbnjOAzI5A6DLBaNRt8dIzm0kkSqCwCMdzqdcwQ/\r
6bp+ATZZ07QCjPhJbKlUqkC1wNKXarV6i9g/LsvyPYJ3oBWJBauQTqe/EHIgENgzwwgRtcTBeE8u\r
KbhQ0ZQDFLscDoeWYRg6SahUKo+5XE5DjRLO6mQyUTiSHSSV6/X6vnHohwbG73COOn1rmqYFQwxR\r
FG8wnX2nUeyTIAiX0E4G5IH8h74BHv5F3ERtD28/gr0KMAB7fRvr2KRa3wAAAABJRU5ErkJggg==\r
------------kTpZ8hm0Y3rptroBHYbvf2--\r
";


   if(!$email->parse($content))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$this->_compare($email->output(), $expected))
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }


//multipart/alternative+related
   $content="From: jas@unit.jas
Message-ID: <ojn213g0808nafg@unit.jas>
Date: Mon, 16 Jun 2014 13:49:23 +0200
Content-Type: multipart/alternative; boundary=----------kTpZ8hm0Y3rptroBHYbvf2


------------kTpZ8hm0Y3rptroBHYbvf2
Content-Type: text/plain; charset=utf-8; format=flowed; delsp=yes
Content-Transfer-Encoding: 7bit

Test message

------------kTpZ8hm0Y3rptroBHYbvf2
Content-Type: multipart/related; boundary=----------Du6XBpaZLMluAJfTnmUlnv

------------Du6XBpaZLMluAJfTnmUlnv
Content-Type: text/html; charset=utf-8
Content-ID: <op.1402919507350.2b2414645d696208@2001:960:7bc:1:da50:e6ff:febb:ba40>
Content-Transfer-Encoding: Quoted-Printable


<html><body>Test HTML message</body></html>

------------Du6XBpaZLMluAJfTnmUlnv
Content-Type: text/css; charset=utf-8
Content-Transfer-Encoding: 7bit


body{font-size: 100%}
------------Du6XBpaZLMluAJfTnmUlnv--

------------kTpZ8hm0Y3rptroBHYbvf2--

";


   $expected="From: jas@unit.jas\r
Message-ID: <ojn213g0808nafg@unit.jas>\r
Date: Mon, 16 Jun 2014 13:49:23 +0200\r
MIME-Version: 1.0\r
User-Agent: JAS/1.0\r
Content-Type: multipart/alternative; boundary=----------kTpZ8hm0Y3rptroBHYbvf2\r
\r
------------kTpZ8hm0Y3rptroBHYbvf2\r
Content-Type: text/plain; charset=utf-8\r
Content-Transfer-Encoding: 7bit\r
Content-Disposition: inline\r
\r
Test message\r
\r
------------kTpZ8hm0Y3rptroBHYbvf2\r
Content-Type: multipart/related; boundary=----------Du6XBpaZLMluAJfTnmUlnv\r
\r
\r
------------Du6XBpaZLMluAJfTnmUlnv\r
Content-Type: text/html; charset=utf-8\r
Content-Transfer-Encoding: Quoted-Printable\r
\r
\r
<html><body>Test HTML message</body></html>\r
\r
------------Du6XBpaZLMluAJfTnmUlnv\r
Content-Type: text/css; charset=utf-8\r
Content-Transfer-Encoding: 7bit\r
\r
\r
body{font-size: 100%}\r
------------Du6XBpaZLMluAJfTnmUlnv--\r
------------kTpZ8hm0Y3rptroBHYbvf2--\r
";


   if(!$email->parse($content))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$this->_compare($email->output(), $expected))
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }


//multipart/mixed+alternative+related
   $content="From: jas@unit.jas
Message-ID: <ojn213g0808nafg@unit.jas>
Date: Mon, 16 Jun 2014 13:49:23 +0200
Content-Type: multipart/mixed; boundary=----------kTpZ8hm0Y3rptroBHYbvf2


------------kTpZ8hm0Y3rptroBHYbvf2
Content-Type: multipart/alternative; boundary=----------gUKwjBgeVG0MCjDBCqTXVr


------------gUKwjBgeVG0MCjDBCqTXVr
Content-Type: text/plain; charset=utf-8; format=flowed; delsp=yes
Content-Transfer-Encoding: 7bit

Test message

------------gUKwjBgeVG0MCjDBCqTXVr
Content-Type: multipart/related; boundary=----------Du6XBpaZLMluAJfTnmUlnv

------------Du6XBpaZLMluAJfTnmUlnv
Content-Type: text/html; charset=utf-8
Content-ID: <op.1402919507350.2b2414645d696208@2001:960:7bc:1:da50:e6ff:febb:ba40>
Content-Transfer-Encoding: Quoted-Printable


<html><body>Test HTML message</body></html>

------------Du6XBpaZLMluAJfTnmUlnv
Content-Type: text/css; charset=utf-8
Content-Transfer-Encoding: 7bit


body{font-size: 100%}
------------Du6XBpaZLMluAJfTnmUlnv--

------------gUKwjBgeVG0MCjDBCqTXVr--

------------kTpZ8hm0Y3rptroBHYbvf2
Content-Type: image/png
Content-Transfer-Encoding: base64

iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJ
bWFnZVJlYWR5ccllPAAAAdpJREFUeNo8Ur9rIkEYHXWyu+rC+otzhWMrXU7rdMJeawzkIG0qIYWk
C4FrUl8ZuCpcEch/cLG7w0I4YQvhCis5AiIuQaKy6Gr8sa4/8j4vZGCYmW++ee997xtfs9lkPp+P
0dhsNmy9Xuc5599WqxUbjUbXOJu07/f7bDqdMs7+j49IvsKs41JLJBKfKanX6x0C7AP2BnJuMJ/4
brdTgPgjHo8fj8fj8mw2s5bLJfM8j4AuYrGYFolEpEajkQHgGcdrZ7FYPPv9fg9oUjKZ1Ofz+V5m
JpPR6SGYvHa7/bzdbh2OVyXXdW1cuAgcdLvdVavV+hUMBlk2mz1SFEVAjhsKhWwoKXFJku5UVfVD
DiPdlmX9tm37NBwOs8Fg8IDEE+zlYrH4tVarbbnjOAzI5A6DLBaNRt8dIzm0kkSqCwCMdzqdcwQ/
6bp+ATZZ07QCjPhJbKlUqkC1wNKXarV6i9g/LsvyPYJ3oBWJBauQTqe/EHIgENgzwwgRtcTBeE8u
KbhQ0ZQDFLscDoeWYRg6SahUKo+5XE5DjRLO6mQyUTiSHSSV6/X6vnHohwbG73COOn1rmqYFQwxR
FG8wnX2nUeyTIAiX0E4G5IH8h74BHv5F3ERtD28/gr0KMAB7fRvr2KRa3wAAAABJRU5ErkJggg==
------------kTpZ8hm0Y3rptroBHYbvf2--

";


   $expected="From: jas@unit.jas\r
Message-ID: <ojn213g0808nafg@unit.jas>\r
Date: Mon, 16 Jun 2014 13:49:23 +0200\r
MIME-Version: 1.0\r
User-Agent: JAS/1.0\r
Content-Type: multipart/mixed; boundary=----------kTpZ8hm0Y3rptroBHYbvf2\r
\r
------------kTpZ8hm0Y3rptroBHYbvf2\r
Content-Type: multipart/alternative; boundary=----------gUKwjBgeVG0MCjDBCqTXVr\r
\r
\r
------------gUKwjBgeVG0MCjDBCqTXVr\r
Content-Type: text/plain; charset=utf-8\r
Content-Transfer-Encoding: 7bit\r
Content-Disposition: inline\r
\r
Test message\r
\r
------------gUKwjBgeVG0MCjDBCqTXVr\r
Content-Type: multipart/related; boundary=----------Du6XBpaZLMluAJfTnmUlnv\r
\r
\r
------------Du6XBpaZLMluAJfTnmUlnv\r
Content-Type: text/html; charset=utf-8\r
Content-Transfer-Encoding: Quoted-Printable\r
\r
\r
<html><body>Test HTML message</body></html>\r
\r
------------Du6XBpaZLMluAJfTnmUlnv\r
Content-Type: text/css; charset=utf-8\r
Content-Transfer-Encoding: 7bit\r
\r
\r
body{font-size: 100%}\r
------------Du6XBpaZLMluAJfTnmUlnv--\r
------------gUKwjBgeVG0MCjDBCqTXVr--\r
------------kTpZ8hm0Y3rptroBHYbvf2\r
Content-Type: image/png\r
Content-Transfer-Encoding: base64\r
\r
iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJ\r
bWFnZVJlYWR5ccllPAAAAdpJREFUeNo8Ur9rIkEYHXWyu+rC+otzhWMrXU7rdMJeawzkIG0qIYWk\r
C4FrUl8ZuCpcEch/cLG7w0I4YQvhCis5AiIuQaKy6Gr8sa4/8j4vZGCYmW++ee997xtfs9lkPp+P\r
0dhsNmy9Xuc5599WqxUbjUbXOJu07/f7bDqdMs7+j49IvsKs41JLJBKfKanX6x0C7AP2BnJuMJ/4\r
brdTgPgjHo8fj8fj8mw2s5bLJfM8j4AuYrGYFolEpEajkQHgGcdrZ7FYPPv9fg9oUjKZ1Ofz+V5m\r
JpPR6SGYvHa7/bzdbh2OVyXXdW1cuAgcdLvdVavV+hUMBlk2mz1SFEVAjhsKhWwoKXFJku5UVfVD\r
DiPdlmX9tm37NBwOs8Fg8IDEE+zlYrH4tVarbbnjOAzI5A6DLBaNRt8dIzm0kkSqCwCMdzqdcwQ/\r
6bp+ATZZ07QCjPhJbKlUqkC1wNKXarV6i9g/LsvyPYJ3oBWJBauQTqe/EHIgENgzwwgRtcTBeE8u\r
KbhQ0ZQDFLscDoeWYRg6SahUKo+5XE5DjRLO6mQyUTiSHSSV6/X6vnHohwbG73COOn1rmqYFQwxR\r
FG8wnX2nUeyTIAiX0E4G5IH8h74BHv5F3ERtD28/gr0KMAB7fRvr2KRa3wAAAABJRU5ErkJggg==\r
------------kTpZ8hm0Y3rptroBHYbvf2--\r
";


   if(!$email->parse($content))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!$this->_compare($email->output(), $expected))
   {
    $this->e->add("Output doesn't match expected output", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function From()
  {
   $email=new Format_Email();

   if(!$email->from("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["From"][0]["address"]!="test@unit.jas")
   {
    $this->e->add("Adding from address without name failed", "error");
    return UT_FAILED;
   }

   if(!$email->from("test2@unit.jas", "Test2"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["From"][0]["address"]!="test2@unit.jas")
   {
    $this->e->add("Adding from address with name failed", "error");
    return UT_FAILED;
   }
   if($email->data["From"][0]["name"]!="Test2")
   {
    $this->e->add("Adding from address with name failed", "error");
    return UT_FAILED;
   }

   if(!$email->from("test3@unit.jas", "\"Test3\""))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["From"][0]["address"]!="test3@unit.jas")
   {
    $this->e->add("Adding from address with quoted name failed", "error");
    return UT_FAILED;
   }
   if($email->data["From"][0]["name"]!="Test3")
   {
    $this->e->add("Adding from address with quoted name failed", "error");
    return UT_FAILED;
   }

   if($email->from("test4.unit.jas"))
   {
    $this->e->add("Adding illegal from address didn't fail", "error");
    return UT_FAILED;
   }
   if($email->data["From"][0]["address"]=="test4.unit.jas")
   {
    $this->e->add("Adding illegal from address didn't fail", "error");
    return UT_FAILED;
   }
   if($email->data["From"][0]["name"]=="")
   {
    $this->e->add("Adding illegal from address didn't fail", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function To()
  {
   $email=new Format_Email();

   if(!$email->to("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["To"][0]["address"]!="test@unit.jas")
   {
    $this->e->add("Adding to address without name failed", "error");
    return UT_FAILED;
   }

   if(!$email->to("test2@unit.jas", "Test2"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["To"][1]["address"]!="test2@unit.jas")
   {
    $this->e->add("Adding to address with name failed", "error");
    return UT_FAILED;
   }
   if($email->data["To"][1]["name"]!="Test2")
   {
    $this->e->add("Adding to address with name failed", "error");
    return UT_FAILED;
   }

   if(!$email->to("test3@unit.jas", "\"Test3\""))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["To"][2]["address"]!="test3@unit.jas")
   {
    $this->e->add("Adding to address with quoted name failed", "error");
    return UT_FAILED;
   }
   if($email->data["To"][2]["name"]!="Test3")
   {
    $this->e->add("Adding to address with quoted name failed", "error");
    return UT_FAILED;
   }

   if($email->to("test4.unit.jas"))
   {
    $this->e->add("Adding illegal to address didn't fail", "error");
    return UT_FAILED;
   }
   if(sizeof($email->data["To"])!=3)
   {
    $this->e->add("Incorrect number of to addresses", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CC()
  {
   $email=new Format_Email();

   if(!$email->cc("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Cc"][0]["address"]!="test@unit.jas")
   {
    $this->e->add("Adding cc address without name failed", "error");
    return UT_FAILED;
   }

   if(!$email->cc("test2@unit.jas", "Test2"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Cc"][1]["address"]!="test2@unit.jas")
   {
    $this->e->add("Adding cc address with name failed", "error");
    return UT_FAILED;
   }
   if($email->data["Cc"][1]["name"]!="Test2")
   {
    $this->e->add("Adding cc address with name failed", "error");
    return UT_FAILED;
   }

   if(!$email->cc("test3@unit.jas", "\"Test3\""))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Cc"][2]["address"]!="test3@unit.jas")
   {
    $this->e->add("Adding cc address with quoted name failed", "error");
    return UT_FAILED;
   }
   if($email->data["Cc"][2]["name"]!="Test3")
   {
    $this->e->add("Adding cc address with quoted name failed", "error");
    return UT_FAILED;
   }

   if($email->cc("test4.unit.jas"))
   {
    $this->e->add("Adding illegal cc address didn't fail", "error");
    return UT_FAILED;
   }
   if(sizeof($email->data["Cc"])!=3)
   {
    $this->e->add("Incorrect number of cc addresses", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function BCC()
  {
   $email=new Format_Email();

   if(!$email->bcc("test@unit.jas"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Bcc"][0]["address"]!="test@unit.jas")
   {
    $this->e->add("Adding bcc address without name failed", "error");
    return UT_FAILED;
   }

   if(!$email->bcc("test2@unit.jas", "Test2"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Bcc"][1]["address"]!="test2@unit.jas")
   {
    $this->e->add("Adding bcc address with name failed", "error");
    return UT_FAILED;
   }
   if($email->data["Bcc"][1]["name"]!="Test2")
   {
    $this->e->add("Adding bcc address with name failed", "error");
    return UT_FAILED;
   }

   if(!$email->bcc("test3@unit.jas", "\"Test3\""))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Bcc"][2]["address"]!="test3@unit.jas")
   {
    $this->e->add("Adding bcc address with quoted name failed", "error");
    return UT_FAILED;
   }
   if($email->data["Bcc"][2]["name"]!="Test3")
   {
    $this->e->add("Adding bcc address with quoted name failed", "error");
    return UT_FAILED;
   }

   if($email->bcc("test4.unit.jas"))
   {
    $this->e->add("Adding illegal bcc address didn't fail", "error");
    return UT_FAILED;
   }
   if(sizeof($email->data["Bcc"])!=3)
   {
    $this->e->add("Incorrect number of bcc addresses", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Subject()
  {
   $email=new Format_Email();

   if(!$email->subject("Test subject"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Subject"][0]!="Test subject")
   {
    $this->e->add("Adding subject failed", "error");
    return UT_FAILED;
   }

   if(!$email->subject("Test subject\n\n"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($email->data["Subject"][0]!="Test subject")
   {
    $this->e->add("Adding subject failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AttachText()
  {
   $email=new Format_Email();

   $attachment=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => "text",
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->attach($attachment["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   //custom content type
   $email->clear();
   $attachment["contentType"]="text/html";

   if(!$email->attach($attachment["content"], $attachment["contentType"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   //quoted-printable encoding
   $email->clear();
   $attachment["contentEncoding"]="Quoted-Printable";

   if(!$email->attach($attachment["content"], $attachment["contentType"], $attachment["contentEncoding"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=quoted_printable_encode($attachment["content"]);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   //base64 encoding
   $email->clear();
   $attachment["contentEncoding"]="base64";

   if(!$email->attach($attachment["content"], $attachment["contentType"], $attachment["contentEncoding"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=base64_encode($attachment["content"]);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   return UT_OK;
  }

  function AttachTextFile()
  {
   global $_path;

   $email=new Format_Email();

   $file=$_path["project"]["plugins"]."status.txt";

   $attachment=Array(
    "name" => "status.txt",
    "type" => false,
    "parent" => false,
    "content" => @file_get_contents($file),
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->attachFile($file))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   //custom content type
   $email->clear();
   $attachment["contentType"]="text/html";

   if(!$email->attachFile($file, $attachment["contentType"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   //quoted-printable encoding
   $email->clear();
   $attachment["contentEncoding"]="Quoted-Printable";

   if(!$email->attachFile($file, $attachment["contentType"], $attachment["contentEncoding"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=quoted_printable_encode(Mutilate::fixLineBreaksS(@file_get_contents($file), $email->bodyLinebreak));

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   //base64 encoding
   $email->clear();
   $attachment["contentEncoding"]="base64";

   if(!$email->attachFile($file, $attachment["contentType"], $attachment["contentEncoding"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=base64_encode(@file_get_contents($file));

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   return UT_OK;
  }

  function AttachPNGFile()
  {
   global $_path;

   $email=new Format_Email();

   $file=$_path["images"]."imagenotfound.png";

   $attachment=Array(
    "name" => "imagenotfound.png",
    "type" => false,
    "parent" => false,
    "content" => @file_get_contents($file),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );

   if(!$email->attachFile($file))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=base64_encode($attachment["content"]);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   return UT_OK;
  }

  function AttachMultiMixed()
  {
   global $_path;

   $email=new Format_Email();

//new attachment
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => "text1",
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->attach($attachments[0]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//1 existing attachment + 1 new attachment, generates multipart/mixed
   array_unshift($attachments, Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/mixed",
    "contentCharset" => false,
    "contentEncoding" => false,
   ));

   $attachments[1]["parent"]=0;
   $attachments[2]=$attachments[1];
   $attachments[2]["content"]="text2";

   if(!$email->attach($attachments[2]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//multipart/mixed + 2 existing attachments + 1 new attachment
   $attachments[3]=$attachments[1];
   $attachments[3]["content"]="text3";

   if(!$email->attach($attachments[3]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[3]["boundary"]=$email->data["Attachment"][3]["boundary"];

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//new text message
   $email->clear();
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => "message",
    "parent" => false,
    "content" => "text1",
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->attach($attachments[0]["content"], $attachments[0]["contentType"], false, false, "message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//overwrite text message
   $attachments[0]["content"]="text2";
   if(!$email->attach($attachments[0]["content"], $attachments[0]["contentType"], false, false, "message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//1 existing text message + 1 new html message, generates multipart/alternative
   array_unshift($attachments, Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/alternative",
    "contentCharset" => false,
    "contentEncoding" => false,
   ));

   $attachments[1]["parent"]=0;
   $attachments[2]=$attachments[1];
   $attachments[2]["content"]="html";
   $attachments[2]["contentType"]="text/html";

   if(!$email->attach($attachments[2]["content"], $attachments[2]["contentType"], false, false, "message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//2 existing messages + 1 new "png" message
   $attachments[3]=$attachments[1];
   $attachments[3]["content"]="PNG";
   $attachments[3]["contentType"]="image/png";
   $attachments[3]["contentCharset"]=false;
   $attachments[3]["contentEncoding"]="base64";

   if(!$email->attach($attachments[3]["content"], $attachments[3]["contentType"], false, false, "message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[3]["boundary"]=$email->data["Attachment"][3]["boundary"];
   $attachments[3]["content"]=base64_encode($attachments[3]["content"]);

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//new attachment
   $email->clear();
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => "text1",
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->attach($attachments[0]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//1 existing attachment + 1 new text message, generates multipart/mixed
   array_unshift($attachments, Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/mixed",
    "contentCharset" => false,
    "contentEncoding" => false,
   ));

   $attachments[1]["parent"]=0;
   $attachments[2]=$attachments[1];
   $attachments[2]["type"]="message";

   if(!$email->attach($attachments[2]["content"], $attachments[2]["contentType"], false, false, "message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

//1 existing attachment + 1 existing text message + 1 new html message, generates multipart/alternative
   $attachments[3]=$attachments[2];
   $attachments[4]=$attachments[3];
   $attachments[3]["parent"]=2;
   $attachments[4]["parent"]=2;

   $attachments[2]["type"]=false;
   $attachments[2]["content"]=false;
   $attachments[2]["contentType"]="multipart/alternative";
   $attachments[2]["contentCharset"]=false;
   $attachments[2]["contentEncoding"]=false;

   $attachments[4]["content"]="html";
   $attachments[4]["contentType"]="text/html";

   if(!$email->attach($attachments[4]["content"], $attachments[4]["contentType"], false, false, "message"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];
   $attachments[4]["boundary"]=$email->data["Attachment"][4]["boundary"];

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;


   return UT_OK;
  }


  function Detach()
  {
   global $_path;

   $email=new Format_Email();

   //7bit text file
   $file=$_path["include"]."changelog.txt";
   $content=@file_get_contents($file);

   if(!$email->attach($content, false, "7bit", "text"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $content=Mutilate::fixLineBreaksS($content, $email->bodyLinebreak);
   if($email->detach(0)!=$content)
   {
    $this->e->add("Detach text doesn't match original contents", "error");
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->detach("text")!=$content)
   {
    $this->e->add("Detach text by name doesn't match original contents", "error");
    $this->e->add($email->e);
    return UT_FAILED;
   }


   //quoted-printable
   $content=@file_get_contents($file);
   if(!$email->attach($content, false, "Quoted-Printable", "quoted"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $content=Mutilate::fixLineBreaksS($content, $email->bodyLinebreak);
   if($email->detach(2)!=$content)
   {
    $this->e->add("Detach quoted doesn't match original contents", "error");
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->detach("quoted")!=$content)
   {
    $this->e->add("Detach quoted by file doesn't match original contents", "error");
    $this->e->add($email->e);
    return UT_FAILED;
   }


   //png
   $file=$_path["images"]."imagenotfound.png";
   $content=@file_get_contents($file);

   if(!$email->attach($content, false, "base64", "PNG"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->detach(3)!=$content)
   {
    $this->e->add("Detach png doesn't match original contents", "error");
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->detach("PNG")!=$content)
   {
    $this->e->add("Detach png by file doesn't match original contents", "error");
    $this->e->add($email->e);
    return UT_FAILED;
   }


   //fail
   if($email->detach(-1))
   {
    $this->e->add("Detach out-of-bound should have failed", "error");
    return UT_FAILED;
   }

   if($email->detach(100))
   {
    $this->e->add("Detach out-of-bound should have failed", "error");
    return UT_FAILED;
   }

   if($email->detach("doesn'texist.txt"))
   {
    $this->e->add("Detach unknown file should have failed", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function DetachFile()
  {
   global $_path;

   $path=$_path["project"]["cache"];

   if(!is_dir($path)||!is_writable($path))
   {
    $this->e->add("Detach file skipped, can't write to: ".$path, "error");
    return UT_SKIPPED;
   }

   $email=new Format_Email();

   //7bit text file
   $file=$_path["include"]."changelog.txt";
   $content=@file_get_contents($file);

   if(!$email->attachFile($file, false, "7bit"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $content=Mutilate::fixLineBreaksS($content, $email->bodyLinebreak);
   if(!$email->detachFile(0, $path."changelog.text"))
   {
    $this->e->add("Detach text failed", "error");
    $this->e->add($email->e);

    @unlink($path."changelog.text");
    return UT_FAILED;
   }

   if($content!=@file_get_contents($path."changelog.text"))
   {
    $this->e->add("Detach text doesn't match original contents", "error");

    @unlink($path."changelog.text");
    return UT_FAILED;
   }

   @unlink($path."changelog.text");


   if(!$email->detachFile("changelog.txt", $path))
   {
    $this->e->add("Detach text by name failed", "error");
    $this->e->add($email->e);

    @unlink($path."changelog.txt");
    return UT_FAILED;
   }

   if($content!=@file_get_contents($path."changelog.txt"))
   {
    $this->e->add("Detach text by name doesn't match original contents", "error");

    @unlink($path."changelog.txt");
    return UT_FAILED;
   }

   @unlink($path."changelog.txt");


   //quoted printable file
   $email->clear();
   $content=@file_get_contents($file);

   if(!$email->attachFile($file, false, "Quoted-Printable"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $content=Mutilate::fixLineBreaksS($content, $email->bodyLinebreak);
   if(!$email->detachFile(0, $path."changelog.text"))
   {
    $this->e->add("Detach quoted text failed", "error");
    $this->e->add($email->e);

    @unlink($path."changelog.text");
    return UT_FAILED;
   }

   if($content!=@file_get_contents($path."changelog.text"))
   {
    $this->e->add("Detach quoted text doesn't match original contents", "error");

    @unlink($path."changelog.text");
    return UT_FAILED;
   }

   @unlink($path."changelog.text");


   if(!$email->detachFile("changelog.txt", $path))
   {
    $this->e->add("Detach quoted text by name failed", "error");
    $this->e->add($email->e);

    @unlink($path."changelog.txt");
    return UT_FAILED;
   }

   if($content!=@file_get_contents($path."changelog.txt"))
   {
    $this->e->add("Detach quoted text by name doesn't match original contents", "error");

    @unlink($path."changelog.txt");
    return UT_FAILED;
   }

   @unlink($path."changelog.txt");


   //png
   $email->clear();
   $file=$_path["images"]."imagenotfound.png";
   $content=@file_get_contents($file);

   if(!$email->attachFile($file, false, "base64"))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!$email->detachFile(0, $path."testimage.png"))
   {
    $this->e->add("Detach png failed", "error");
    $this->e->add($email->e);

    @unlink($path."testimage.png");
    return UT_FAILED;
   }

   if($content!=@file_get_contents($path."testimage.png"))
   {
    $this->e->add("Detach png doesn't match original contents", "error");

    @unlink($path."testimage.png");
    return UT_FAILED;
   }

   @unlink($path."testimage.png");


   if(!$email->detachFile("imagenotfound.png", $path))
   {
    $this->e->add("Detach png by name failed", "error");
    $this->e->add($email->e);

    @unlink($path."imagenotfound.png");
    return UT_FAILED;
   }

   if($content!=@file_get_contents($path."imagenotfound.png"))
   {
    $this->e->add("Detach png by name doesn't match original contents", "error");

    @unlink($path."imagenotfound.png");
    return UT_FAILED;
   }

   @unlink($path."imagenotfound.png");


   //fail
   if($email->detachFile(-1, $path))
   {
    $this->e->add("Detach out-of-bound should have failed", "error");
    return UT_FAILED;
   }

   if($email->detachFile(100, $path))
   {
    $this->e->add("Detach out-of-bound should have failed", "error");
    return UT_FAILED;
   }

   if($email->detachFile("doesn'texist.txt", $path))
   {
    $this->e->add("Detach unknown file should have failed", "error");
    return UT_FAILED;
   }

   //file already exists
   $email->clear();
   $file=$_path["include"]."changelog.txt";

   if(!$email->attachFile($file))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if(!touch($path."changelog.txt"))
   {
    $this->e->add("Unable to create test file in: ".$path, "error");
    @unlink($path."changelog.txt");
    return UT_FAILED;
   }

   if($email->detachFile(0, $path."changelog.txt"))
   {
    $this->e->add("Detach text should have failed", "error");
    @unlink($path."changelog.txt");
    return UT_FAILED;
   }

   @unlink($path."changelog.txt");

   return UT_OK;
  }


  function MessageText()
  {
   $email=new Format_Email();

   $attachment=Array(
    "name" => false,
    "type" => "message",
    "parent" => false,
    "content" => "Test message",
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->message($attachment["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;


   //alternative copy
   $email->clear();

   $attachment=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => "Test message",
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   $alternativeContent="<html>
 <head>
 </head>
 <body>
".$attachment["content"]." </body>
</html>
";

   if(!$email->message($attachment["content"], true)) //also add HTML copy
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=3)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][1]["boundary"]; //text message
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][1]))
    return UT_FAILED;

   $attachment["boundary"]=$email->data["Attachment"][2]["boundary"]; //html message
   $attachment["content"]=Mutilate::fixLineBreaksS($alternativeContent, $email->bodyLinebreak);
   $attachment["contentType"]="text/html";

   if(!$this->_compare($attachment, $email->data["Attachment"][2]))
    return UT_FAILED;

   $attachment=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/alternative",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"]; //multipart attachment

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;


   //alternative content
   $email->clear();

   $attachment=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => "Test message",
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   $alternativeContent="<html><body>Test message</body></html>";

   if(!$email->message($attachment["content"], $alternativeContent)) //also add HTML content
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=3)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][1]["boundary"]; //text message
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][1]))
    return UT_FAILED;

   $attachment["boundary"]=$email->data["Attachment"][2]["boundary"]; //html message
   $attachment["content"]=$alternativeContent;
   $attachment["contentType"]="text/html";

   if(!$this->_compare($attachment, $email->data["Attachment"][2]))
    return UT_FAILED;

   $attachment=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/alternative",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"]; //multipart attachment

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   return UT_OK;
  }

  function MessageTextFile()
  {
   global $_path;

   $email=new Format_Email();

   $file=$_path["project"]["plugins"]."status.txt";

   $attachment=Array(
    "name" => false,
    "type" => "message",
    "parent" => false,
    "content" => @file_get_contents($file),
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->messageFile($file))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

//alternative content
   $email->clear();
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/alternative",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachments[]=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => @file_get_contents($file),
    "contentType" => "text/plain",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );
   $attachments[2]=$attachments[1];
   $attachments[2]["content"]="<html>\r
 <head>\r
 </head>\r
 <body>\r
".str_replace("\r\n", "<br/>".$email->bodyLinebreak, $attachment["content"])." </body>\r
</html>\r
";
   $attachments[2]["contentType"]="text/html";

   if(!$email->messageFile($file, true))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[1]["boundary"]=$email->data["Attachment"][1]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];
   $attachments[1]["content"]=Mutilate::fixLineBreaksS($attachments[1]["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

   return UT_OK;
  }

  function MessageHTML()
  {
   $email=new Format_Email();

   $attachment=Array(
    "name" => false,
    "type" => "message",
    "parent" => false,
    "content" => "<html><body>Test message</body></html>",
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->message($attachment["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;


   //alternative copy
   $email->clear();

   $attachment=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => "<html>
 <head>
 <title>Test</title>
 </head>
 <body>Test message</body>
</html>",
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   $alternativeContent="Test message";

   if(!$email->message($attachment["content"], true)) //also add HTML copy
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=3)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][2]["boundary"]; //html message
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][2]))
    return UT_FAILED;

   $attachment["boundary"]=$email->data["Attachment"][1]["boundary"]; //text message
   $attachment["content"]=$alternativeContent;
   $attachment["contentType"]="text/plain";

   if(!$this->_compare($attachment, $email->data["Attachment"][1]))
    return UT_FAILED;

   $attachment=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/alternative",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"]; //multipart attachment

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;


   //alternative content
   $email->clear();

   $attachment=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => "<html><body>Test message</body></html>",
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   $alternativeContent="Test message";

   if(!$email->message($attachment["content"], $alternativeContent)) //also add HTML content
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=3)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][2]["boundary"]; //html message
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][2]))
    return UT_FAILED;

   $attachment["boundary"]=$email->data["Attachment"][1]["boundary"]; //text message
   $attachment["content"]=$alternativeContent;
   $attachment["contentType"]="text/plain";

   if(!$this->_compare($attachment, $email->data["Attachment"][1]))
    return UT_FAILED;

   $attachment=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/alternative",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"]; //multipart attachment

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   return UT_OK;
  }

  function MessageHTMLFile()
  {
   global $_path;

   $email=new Format_Email();

   $file=$_path["plugins"]."pager/templates/error.html";

   $attachment=Array(
    "name" => false,
    "type" => "message",
    "parent" => false,
    "content" => @file_get_contents($file),
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->messageFile($file))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   if($email->vmax["Attachment"]!=1)
   {
    $this->e->add("No attachments added", "error");
    return UT_FAILED;
   }

   $attachment["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachment["content"]=Mutilate::fixLineBreaksS($attachment["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachment, $email->data["Attachment"][0]))
    return UT_FAILED;

   return UT_OK;
  }

  function Embed()
  {
   global $_path;

   $email=new Format_Email();

//no embedded files
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => "message",
    "parent" => false,
    "content" => "<html>
 <head>
 </head>
 <body>
 </body>
</html>",
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );

   if(!$email->embed($attachments[0]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[0]["content"]=Mutilate::fixLineBreaksS($attachments[0]["content"], $email->bodyLinebreak);

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;


//embedded file
   $email->clear();
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/related",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachments[]=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => "<html>
 <head>
 </head>
 <body>
  <img src=\"".$_path["root"]."project/jas/images/jas.png\" alt=\"JAS\"/>
 </body>
</html>",
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "jas.png",
    "type" => "embedded",
    "parent" => 0,
    "content" => base64_encode(@file_get_contents($_path["root"]."project/jas/images/jas.png")),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );

   if(!$email->embed($attachments[1]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[1]["boundary"]=$email->data["Attachment"][1]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];

   $attachments[1]["content"]=Mutilate::fixLineBreaksS($attachments[1]["content"], $email->bodyLinebreak);
   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/images/jas.png", "cid:".$email->data["Attachment"][2]["boundary"], $attachments[1]["content"]);

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;


//embedded duplicate files
   $email->clear();
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/related",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachments[]=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => "<html>
 <head>
 </head>
 <body>
  <img src=\"".$_path["root"]."project/jas/images/jas.png\" alt=\"JAS\"/>
  <img src=\"".$_path["root"]."project/jas/../jas/images/jas.png\" alt=\"JAS\"/>
 </body>
</html>",
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "jas.png",
    "type" => "embedded",
    "parent" => 0,
    "content" => base64_encode(@file_get_contents($_path["root"]."project/jas/images/jas.png")),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );

   if(!$email->embed($attachments[1]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[1]["boundary"]=$email->data["Attachment"][1]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];

   $attachments[1]["content"]=Mutilate::fixLineBreaksS($attachments[1]["content"], $email->bodyLinebreak);
   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/images/jas.png", "cid:".$email->data["Attachment"][2]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/../jas/images/jas.png", "cid:".$email->data["Attachment"][2]["boundary"], $attachments[1]["content"]);

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;


//embedded files with css embedded images
   $email->clear();
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/related",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachments[]=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => "<html>
 <head>
  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["root"]."project/jas/style/default.css\"/> <!-- has embedded css -->
  <script type=\"text/javascript\" src=\"".$_path["root"]."script/functions.js\"></script>
 </head>
 <body>
  <img src=\"".$_path["root"]."project/jas/images/jas.png\" alt=\"JAS\"/>
 </body>
</html>",
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "jas.png",
    "type" => "embedded",
    "parent" => 0,
    "content" => base64_encode(@file_get_contents($_path["root"]."project/jas/images/jas.png")),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );
   $attachments[]=Array(
    "name" => "functions.js",
    "type" => "embedded",
    "parent" => 0,
    "content" => @file_get_contents($_path["root"]."script/functions.js"),
    "contentType" => "text/javascript",
    "contentCharset" => false,
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "default.css",
    "type" => "embedded",
    "parent" => 0,
    "content" => @file_get_contents($_path["root"]."project/jas/style/default.css"),
    "contentType" => "text/css",
    "contentCharset" => false,
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "list.png",
    "type" => "embedded",
    "parent" => 0,
    "content" => base64_encode(@file_get_contents($_path["root"]."project/jas/images/list.png")),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );

   if(!$email->embed($attachments[1]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[1]["boundary"]=$email->data["Attachment"][1]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];
   $attachments[3]["boundary"]=$email->data["Attachment"][3]["boundary"];
   $attachments[4]["boundary"]=$email->data["Attachment"][4]["boundary"];
   $attachments[5]["boundary"]=$email->data["Attachment"][5]["boundary"];

   $attachments[1]["content"]=Mutilate::fixLineBreaksS($attachments[1]["content"], $email->bodyLinebreak);
   $attachments[3]["content"]=Mutilate::fixLineBreaksS($attachments[3]["content"], $email->bodyLinebreak);
   $attachments[4]["content"]=Mutilate::fixLineBreaksS($attachments[4]["content"], $email->bodyLinebreak);

   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/images/jas.png", "cid:".$email->data["Attachment"][2]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace($_path["root"]."script/functions.js", "cid:".$email->data["Attachment"][3]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/style/default.css", "cid:".$email->data["Attachment"][4]["boundary"], $attachments[1]["content"]);
   $attachments[4]["content"]=str_replace("url(../images/list.png)", "url(cid:".$email->data["Attachment"][5]["boundary"].")", $attachments[4]["content"]);

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;


//embedded duplicate files with css embedded images
   $email->clear();
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/related",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachments[]=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => "<html>
 <head>
  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["root"]."project/jas/style/default.css\"/> <!-- has embedded css -->
  <script type=\"text/javascript\" src=\"".$_path["root"]."script/functions.js\"></script>
 </head>
 <body>
  <img src=\"".$_path["root"]."project/jas/images/jas.png\" alt=\"JAS\"/>
  <img src=\"".$_path["root"]."project/jas/../jas/images/jas.png\" alt=\"JAS\"/>
  <img src=\"".$_path["root"]."project/jas/images/list.png\" alt=\"JAS\"/>
 </body>
</html>",
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "jas.png",
    "type" => "embedded",
    "parent" => 0,
    "content" => base64_encode(@file_get_contents($_path["root"]."project/jas/images/jas.png")),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );
   $attachments[]=Array(
    "name" => "list.png",
    "type" => "embedded",
    "parent" => 0,
    "content" => base64_encode(@file_get_contents($_path["root"]."project/jas/images/list.png")),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );
   $attachments[]=Array(
    "name" => "functions.js",
    "type" => "embedded",
    "parent" => 0,
    "content" => @file_get_contents($_path["root"]."script/functions.js"),
    "contentType" => "text/javascript",
    "contentCharset" => false,
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "default.css",
    "type" => "embedded",
    "parent" => 0,
    "content" => @file_get_contents($_path["root"]."project/jas/style/default.css"),
    "contentType" => "text/css",
    "contentCharset" => false,
    "contentEncoding" => "7bit",
   );

   if(!$email->embed($attachments[1]["content"]))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[1]["boundary"]=$email->data["Attachment"][1]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];
   $attachments[3]["boundary"]=$email->data["Attachment"][3]["boundary"];
   $attachments[4]["boundary"]=$email->data["Attachment"][4]["boundary"];
   $attachments[5]["boundary"]=$email->data["Attachment"][5]["boundary"];

   $attachments[1]["content"]=Mutilate::fixLineBreaksS($attachments[1]["content"], $email->bodyLinebreak);
   $attachments[4]["content"]=Mutilate::fixLineBreaksS($attachments[4]["content"], $email->bodyLinebreak);
   $attachments[5]["content"]=Mutilate::fixLineBreaksS($attachments[5]["content"], $email->bodyLinebreak);

   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/images/jas.png", "cid:".$email->data["Attachment"][2]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/../jas/images/jas.png", "cid:".$email->data["Attachment"][2]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/images/list.png", "cid:".$email->data["Attachment"][3]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace($_path["root"]."script/functions.js", "cid:".$email->data["Attachment"][4]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace($_path["root"]."project/jas/style/default.css", "cid:".$email->data["Attachment"][5]["boundary"], $attachments[1]["content"]);
   $attachments[5]["content"]=str_replace("url(../images/list.png)", "url(cid:".$email->data["Attachment"][3]["boundary"].")", $attachments[5]["content"]);

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;


   return UT_OK;
  }

  function EmbedFile()
  {
   global $_path;

   $email=new Format_Email();

   $file=$_path["plugins"]."unit_testing/misc/embedded.eml";

//embedded files with css embedded images
   $attachments=Array();
   $attachments[]=Array(
    "name" => false,
    "type" => false,
    "parent" => false,
    "content" => false,
    "contentType" => "multipart/related",
    "contentCharset" => false,
    "contentEncoding" => false,
   );
   $attachments[]=Array(
    "name" => false,
    "type" => "message",
    "parent" => 0,
    "content" => @file_get_contents($file),
    "contentType" => "text/html",
    "contentCharset" => "ISO-8859-1",
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "jas.png",
    "type" => "embedded",
    "parent" => 0,
    "content" => base64_encode(@file_get_contents($_path["root"]."project/jas/images/jas.png")),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );
   $attachments[]=Array(
    "name" => "functions.js",
    "type" => "embedded",
    "parent" => 0,
    "content" => @file_get_contents($_path["root"]."script/functions.js"),
    "contentType" => "text/javascript",
    "contentCharset" => false,
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "default.css",
    "type" => "embedded",
    "parent" => 0,
    "content" => @file_get_contents($_path["root"]."project/jas/style/default.css"),
    "contentType" => "text/css",
    "contentCharset" => false,
    "contentEncoding" => "7bit",
   );
   $attachments[]=Array(
    "name" => "list.png",
    "type" => "embedded",
    "parent" => 0,
    "content" => base64_encode(@file_get_contents($_path["root"]."project/jas/images/list.png")),
    "contentType" => "image/png",
    "contentCharset" => false,
    "contentEncoding" => "base64",
   );

   if(!$email->embedFile($file))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }

   $attachments[0]["boundary"]=$email->data["Attachment"][0]["boundary"];
   $attachments[1]["boundary"]=$email->data["Attachment"][1]["boundary"];
   $attachments[2]["boundary"]=$email->data["Attachment"][2]["boundary"];
   $attachments[3]["boundary"]=$email->data["Attachment"][3]["boundary"];
   $attachments[4]["boundary"]=$email->data["Attachment"][4]["boundary"];
   $attachments[5]["boundary"]=$email->data["Attachment"][5]["boundary"];

   $attachments[1]["content"]=Mutilate::fixLineBreaksS($attachments[1]["content"], $email->bodyLinebreak);
   $attachments[3]["content"]=Mutilate::fixLineBreaksS($attachments[3]["content"], $email->bodyLinebreak);
   $attachments[4]["content"]=Mutilate::fixLineBreaksS($attachments[4]["content"], $email->bodyLinebreak);

   $attachments[1]["content"]=str_replace("project/jas/images/jas.png", "cid:".$email->data["Attachment"][2]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace("script/functions.js", "cid:".$email->data["Attachment"][3]["boundary"], $attachments[1]["content"]);
   $attachments[1]["content"]=str_replace("project/jas/style/default.css", "cid:".$email->data["Attachment"][4]["boundary"], $attachments[1]["content"]);
   $attachments[4]["content"]=str_replace("url(../images/list.png)", "url(cid:".$email->data["Attachment"][5]["boundary"].")", $attachments[4]["content"]);

   if(!$this->_compare($attachments, $email->data["Attachment"]))
    return UT_FAILED;

   return UT_OK;
  }


  function Header()
  {
   $email=new Format_Email();

//text
   $email->from("test@unit.jas");
   $email->to("test@unit.jas");
   $email->cc("test@unit.jas");
   $email->bcc("test@unit.jas");
   $email->subject("Test subject");
   $email->message("Test message");

   if(!($header=$email->header()))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!preg_match("/From: test@unit.jas/", $header))
   {
    $this->e->add("From address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/To: test@unit.jas/", $header))
   {
    $this->e->add("To address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Cc: test@unit.jas/", $header))
   {
    $this->e->add("CC address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Bcc: test@unit.jas/", $header))
   {
    $this->e->add("BCC address not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Subject: Test subject/", $header))
   {
    $this->e->add("Subject not found in header", "error");
    return UT_FAILED;
   }

   if(!preg_match("/Message-ID: <[^>]+>/", $header))
   {
    $this->e->add("Message-ID not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/MIME-Version: [0-9\.]+/", $header))
   {
    $this->e->add("MIME-Version not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Date: /", $header))
   {
    $this->e->add("Date not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Date: [A-Za-z]{3}, [0-9]{1,2} [A-Za-z]{3} [0-9]{4} [0-9]{2}:[0-9]{2}:[0-9]{2} \+[0-9]{4}/", $header))
   {
    $this->e->add("Date format incorrect in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/User-Agent: ".preg_quote($email->userAgent, "/")."/", $header))
   {
    $this->e->add("User-Agent not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Type: text\/plain; charset=".preg_quote($email->encoding)."/", $header))
   {
    $this->e->add("Content-Type not found in header", "error");
    return UT_FAILED;
   }
   if(!preg_match("/Content-Transfer-Encoding: 7bit/", $header))
   {
    $this->e->add("Content-Transfer-Encoding not found in header", "error");
    return UT_FAILED;
   }

//multipart/alternative
   $email->message("Test message", true);

   if(!($header=$email->header()))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!preg_match("/Content-Type: multipart\/alternative; boundary=".$email->boundaryPrefix.$email->data["Attachment"][0]["boundary"]."/", $header))
   {
    $this->e->add("Content-Type not found in header", "error");
    return UT_FAILED;
   }

//multipart/mixed
   $email->attach("Attachment");

   if(!($header=$email->header()))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if(!preg_match("/Content-Type: multipart\/mixed; boundary=".$email->boundaryPrefix.$email->data["Attachment"][0]["boundary"]."/", $header))
   {
    $this->e->add("Content-Type not found in header", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Body()
  {
   $email=new Format_Email();




   $email->from("lala@lala.com");
   $email->to("lala@lala.com");
   $email->subject("Test");




//text
   $email->message("Test message");
   $expected="Test message\r
";

   if(!($body=$email->body()))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($body!=$expected)
   {
    $this->e->add("Body doesn't match expected output", "error");
    return UT_FAILED;
   }

//multipart/alternative
   $alternativeContent="<html><body>Test HTML message</body></html>";

   $email->message("Test message", $alternativeContent);
   $expected="--".$email->boundaryPrefix.$email->data["Attachment"][0]["boundary"]."\r
Content-Type: text/plain; charset=".$email->encoding."\r
Content-Transfer-Encoding: 7bit\r
Content-Disposition: inline\r
\r
Test message\r
--".$email->boundaryPrefix.$email->data["Attachment"][0]["boundary"]."\r
Content-Type: text/html; charset=".$email->encoding."\r
Content-Transfer-Encoding: 7bit\r
\r
".$alternativeContent."\r
--".$email->boundaryPrefix.$email->data["Attachment"][0]["boundary"]."--\r
";

   if(!($body=$email->body()))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($body!=$expected)
   {
    $this->e->add("Body doesn't contain message", "error");
    return UT_FAILED;
   }

//multipart/mixed
   $email->attach("Attachment");
   $email->data["Attachment"][4]["name"]="attachment.tst"; //fake filename

   $expected="--".$email->boundaryPrefix.$email->data["Attachment"][0]["boundary"]."\r
Content-Type: multipart/alternative; boundary=".$email->boundaryPrefix.$email->data["Attachment"][1]["boundary"]."\r
\r
\r
--".$email->boundaryPrefix.$email->data["Attachment"][1]["boundary"]."\r
Content-Type: text/plain; charset=".$email->encoding."\r
Content-Transfer-Encoding: 7bit\r
Content-Disposition: inline\r
\r
Test message\r
--".$email->boundaryPrefix.$email->data["Attachment"][1]["boundary"]."\r
Content-Type: text/html; charset=".$email->encoding."\r
Content-Transfer-Encoding: 7bit\r
\r
".$alternativeContent."\r
--".$email->boundaryPrefix.$email->data["Attachment"][1]["boundary"]."--\r
--".$email->boundaryPrefix.$email->data["Attachment"][0]["boundary"]."\r
Content-Type: text/plain; charset=".$email->encoding."; name=\"attachment.tst\"\r
Content-Transfer-Encoding: 7bit\r
Content-Disposition: attachment; filename=\"attachment.tst\"\r
Content-ID: <".$email->data["Attachment"][4]["boundary"].">\r
\r
Attachment\r
--".$email->boundaryPrefix.$email->data["Attachment"][0]["boundary"]."--\r
";

   if(!($body=$email->body()))
   {
    $this->e->add($email->e);
    return UT_FAILED;
   }
   if($body!=$expected)
   {
    $this->e->add("Body doesn't contain message", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

 }

?>
