<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/plugin.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."archive.php");
 require_once($_path["include"]."plugin.php");

 class UT_Plugin extends UT_Classes
 {
  function Constructor()
  {
   $object=new Plugin("unit_testing");

   if(!is_object($object)||get_class($object)!="Plugin")
   {
    $this->e->add("Constructor doesn't return a Plugin object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Plugin("unit_testing");
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Plugin object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->class=Array();
   if($orig->class==$copy->class)
   {
    $this->e->add("Plugin object copy also modified original \"class\" variable", "error");
    return UT_FAILED;
   }

   $copy->fs->phpversion=-1;
   if($orig->fs->phpversion==$copy->fs->phpversion)
   {
    $this->e->add("Plugin object copy also modified original \"FS\" object", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Plugin object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("Plugin object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   $orig->l->sets=Array("test");
   if($orig->l->sets!=$orig->e->l->sets)
   {
    $this->e->add("Plugin object original didn't modify original \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test2");
   if($copy->l->sets!=$copy->e->l->sets)
   {
    $this->e->add("Plugin object copy didn't modify copy \"Language\" object", "error");
    return UT_FAILED;
   }


   //error
   $orig=new Plugin("unit_testing");
   $orig->e->add("test 1");
   $orig->e->add("test 2");
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Plugin object copy doesn't match original", "error");
    return UT_FAILED;
   }


   //info/deps objects
   $orig=new Plugin("unit_testing");
   $orig->info();
   $orig->dependencies();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Plugin object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->info->format=Array();
   if($orig->info->format==$copy->info->format)
   {
    $this->e->add("Plugin info object copy also modified original \"format\" variable", "error");
    return UT_FAILED;
   }

   $copy->info->e->errors=-1;
   if($orig->info->e->errors==$copy->info->e->errors)
   {
    $this->e->add("Plugin info object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->deps->format=Array();
   if($orig->deps->format==$copy->deps->format)
   {
    $this->e->add("Plugin deps object copy also modified original \"format\" variable", "error");
    return UT_FAILED;
   }

   $copy->deps->e->errors=-1;
   if($orig->deps->e->errors==$copy->deps->e->errors)
   {
    $this->e->add("Plugin deps object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }


   //frontend object
   $orig=new Plugin("pager");
   $orig->initClass("frontend");
   $copy=clone $orig;

   if(spl_object_hash($orig->class["frontend"]["object"]->plugin)!=spl_object_hash($orig))
   {
    $this->e->add("Backend plugin reference object original doesn't match original", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($orig->class["frontend"]["object"]->plugin->e)!=spl_object_hash($orig->e))
   {
    $this->e->add("Backend plugin error reference object original doesn't match original", "error");
    return UT_FAILED;
   }

   if(spl_object_hash($copy->class["frontend"]["object"]->plugin)!=spl_object_hash($copy))
   {
    $this->e->add("Backend plugin reference object copy doesn't match copy", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($copy->class["frontend"]["object"]->plugin->e)!=spl_object_hash($copy->e))
   {
    $this->e->add("Backend plugin error reference object copy doesn't match copy", "error");
    return UT_FAILED;
   }

   if(spl_object_hash($orig->class["frontend"]["object"]->plugin)==spl_object_hash($copy->class["frontend"]["object"]->plugin))
   {
    $this->e->add("Backend plugin reference object copy doesn't match original", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($orig->class["frontend"]["object"]->plugin->e)==spl_object_hash($copy->class["frontend"]["object"]->plugin->e))
   {
    $this->e->add("Backend plugin error reference object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->vars=Array(1);
   if($orig->vars==$copy->vars)
   {
    $this->e->add("Plugin object copy also modified original \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->class["frontend"]["object"]->plugin->vars==$copy->vars)
   {
    $this->e->add("Plugin object copy also modified original frontend reference \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->vars==$copy->class["frontend"]["object"]->plugin->vars)
   {
    $this->e->add("Plugin object copy frontend reference also modified original \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->class["frontend"]["object"]->plugin->vars==$copy->class["frontend"]["object"]->plugin->vars)
   {
    $this->e->add("Plugin object copy frontend reference also modified original frontend reference \"vars\" variable", "error");
    return UT_FAILED;
   }

   $copy->config=Array(1);
   if($orig->config==$copy->config)
   {
    $this->e->add("Plugin object copy also modified original \"config\" variable", "error");
    return UT_FAILED;
   }
   if($orig->class["frontend"]["object"]->plugin->config==$copy->config)
   {
    $this->e->add("Plugin object copy also modified original frontend reference \"config\" variable", "error");
    return UT_FAILED;
   }
   if($orig->config==$copy->class["frontend"]["object"]->plugin->config)
   {
    $this->e->add("Plugin object copy frontend reference also modified original \"config\" variable", "error");
    return UT_FAILED;
   }
   if($orig->class["frontend"]["object"]->plugin->config==$copy->class["frontend"]["object"]->plugin->config)
   {
    $this->e->add("Plugin object copy frontend reference also modified original frontend reference \"config\" variable", "error");
    return UT_FAILED;
   }

   $copy->fs->phpversion=-1;
   if($orig->fs->phpversion==$copy->fs->phpversion)
   {
    $this->e->add("Plugin object copy also modified original \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["frontend"]["object"]->plugin->fs->phpversion==$copy->fs->phpversion)
   {
    $this->e->add("Plugin object copy also modified original frontend reference \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->fs->phpversion==$copy->class["frontend"]["object"]->plugin->fs->phpversion)
   {
    $this->e->add("Plugin object copy frontend reference also modified original \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["frontend"]["object"]->plugin->fs->phpversion==$copy->class["frontend"]["object"]->plugin->fs->phpversion)
   {
    $this->e->add("Plugin object copy frontend reference also modified original frontend reference \"FS\" object", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Plugin object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["frontend"]["object"]->plugin->e->errors==$copy->e->errors)
   {
    $this->e->add("Plugin object copy also modified original frontend reference \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->e->errors==$copy->class["frontend"]["object"]->plugin->e->errors)
   {
    $this->e->add("Plugin object copy frontend reference also modified original \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["frontend"]["object"]->plugin->e->errors==$copy->class["frontend"]["object"]->plugin->e->errors)
   {
    $this->e->add("Plugin object copy frontend reference also modified original frontend reference \"Error\" object", "error");
    return UT_FAILED;
   }


   //backend object
   $orig=new Plugin("unit_testing");
   $orig->initClass("backend");
   $copy=clone $orig;

   if(spl_object_hash($orig->class["backend"]["object"]->plugin)!=spl_object_hash($orig))
   {
    $this->e->add("Backend plugin reference object original doesn't match original", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($orig->class["backend"]["object"]->plugin->e)!=spl_object_hash($orig->e))
   {
    $this->e->add("Backend plugin error reference object original doesn't match original", "error");
    return UT_FAILED;
   }

   if(spl_object_hash($copy->class["backend"]["object"]->plugin)!=spl_object_hash($copy))
   {
    $this->e->add("Backend plugin reference object copy doesn't match copy", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($copy->class["backend"]["object"]->plugin->e)!=spl_object_hash($copy->e))
   {
    $this->e->add("Backend plugin error reference object copy doesn't match copy", "error");
    return UT_FAILED;
   }

   if(spl_object_hash($orig->class["backend"]["object"]->plugin)==spl_object_hash($copy->class["backend"]["object"]->plugin))
   {
    $this->e->add("Backend plugin reference object copy doesn't match original", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($orig->class["backend"]["object"]->plugin->e)==spl_object_hash($copy->class["backend"]["object"]->plugin->e))
   {
    $this->e->add("Backend plugin error reference object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->vars=Array(1);
   if($orig->vars==$copy->vars)
   {
    $this->e->add("Plugin object copy also modified original \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->class["backend"]["object"]->plugin->vars==$copy->vars)
   {
    $this->e->add("Plugin object copy also modified original backend reference \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->vars==$copy->class["backend"]["object"]->plugin->vars)
   {
    $this->e->add("Plugin object copy backend reference also modified original \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->class["backend"]["object"]->plugin->vars==$copy->class["backend"]["object"]->plugin->vars)
   {
    $this->e->add("Plugin object copy backend reference also modified original backend reference \"vars\" variable", "error");
    return UT_FAILED;
   }

   $copy->fs->phpversion=-1;
   if($orig->fs->phpversion==$copy->fs->phpversion)
   {
    $this->e->add("Plugin object copy also modified original \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["backend"]["object"]->plugin->fs->phpversion==$copy->fs->phpversion)
   {
    $this->e->add("Plugin object copy also modified original backend reference \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->fs->phpversion==$copy->class["backend"]["object"]->plugin->fs->phpversion)
   {
    $this->e->add("Plugin object copy backend reference also modified original \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["backend"]["object"]->plugin->fs->phpversion==$copy->class["backend"]["object"]->plugin->fs->phpversion)
   {
    $this->e->add("Plugin object copy backend reference also modified original backend reference \"FS\" object", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Plugin object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["backend"]["object"]->plugin->e->errors==$copy->e->errors)
   {
    $this->e->add("Plugin object copy also modified original backend reference \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->e->errors==$copy->class["backend"]["object"]->plugin->e->errors)
   {
    $this->e->add("Plugin object copy backend reference also modified original \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["backend"]["object"]->plugin->e->errors==$copy->class["backend"]["object"]->plugin->e->errors)
   {
    $this->e->add("Plugin object copy backend reference also modified original backend reference \"Error\" object", "error");
    return UT_FAILED;
   }



   //setup object
   $orig=new Plugin("unit_testing");
   $orig->initClass("setup");
   $copy=clone $orig;

   if(spl_object_hash($orig->class["setup"]["object"]->plugin)!=spl_object_hash($orig))
   {
    $this->e->add("Backend plugin reference object original doesn't match original", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($orig->class["setup"]["object"]->plugin->e)!=spl_object_hash($orig->e))
   {
    $this->e->add("Backend plugin error reference object original doesn't match original", "error");
    return UT_FAILED;
   }

   if(spl_object_hash($copy->class["setup"]["object"]->plugin)!=spl_object_hash($copy))
   {
    $this->e->add("Backend plugin reference object copy doesn't match copy", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($copy->class["setup"]["object"]->plugin->e)!=spl_object_hash($copy->e))
   {
    $this->e->add("Backend plugin error reference object copy doesn't match copy", "error");
    return UT_FAILED;
   }

   if(spl_object_hash($orig->class["setup"]["object"]->plugin)==spl_object_hash($copy->class["setup"]["object"]->plugin))
   {
    $this->e->add("Backend plugin reference object copy doesn't match original", "error");
    return UT_FAILED;
   }
   if(spl_object_hash($orig->class["setup"]["object"]->plugin->e)==spl_object_hash($copy->class["setup"]["object"]->plugin->e))
   {
    $this->e->add("Backend plugin error reference object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->class["setup"]["object"]->fs->phpversion=-1;
   if($orig->class["setup"]["object"]->fs->phpversion==$copy->class["setup"]["object"]->fs->phpversion)
   {
    $this->e->add("Plugin setup object copy also modified original \"FS\" object", "error");
    return UT_FAILED;
   }

   $copy->vars=Array(1);
   if($orig->vars==$copy->vars)
   {
    $this->e->add("Plugin object copy also modified original \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->class["setup"]["object"]->plugin->vars==$copy->vars)
   {
    $this->e->add("Plugin object copy also modified original setup reference \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->vars==$copy->class["setup"]["object"]->plugin->vars)
   {
    $this->e->add("Plugin object copy setup reference also modified original \"vars\" variable", "error");
    return UT_FAILED;
   }
   if($orig->class["setup"]["object"]->plugin->vars==$copy->class["setup"]["object"]->plugin->vars)
   {
    $this->e->add("Plugin object copy setup reference also modified original setup reference \"vars\" variable", "error");
    return UT_FAILED;
   }

   $copy->fs->phpversion=-1;
   if($orig->fs->phpversion==$copy->fs->phpversion)
   {
    $this->e->add("Plugin object copy also modified original \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["setup"]["object"]->plugin->fs->phpversion==$copy->fs->phpversion)
   {
    $this->e->add("Plugin object copy also modified original setup reference \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->fs->phpversion==$copy->class["setup"]["object"]->plugin->fs->phpversion)
   {
    $this->e->add("Plugin object copy setup reference also modified original \"FS\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["setup"]["object"]->plugin->fs->phpversion==$copy->class["setup"]["object"]->plugin->fs->phpversion)
   {
    $this->e->add("Plugin object copy setup reference also modified original setup reference \"FS\" object", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Plugin object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["setup"]["object"]->plugin->e->errors==$copy->e->errors)
   {
    $this->e->add("Plugin object copy also modified original setup reference \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->e->errors==$copy->class["setup"]["object"]->plugin->e->errors)
   {
    $this->e->add("Plugin object copy setup reference also modified original \"Error\" object", "error");
    return UT_FAILED;
   }
   if($orig->class["setup"]["object"]->plugin->e->errors==$copy->class["setup"]["object"]->plugin->e->errors)
   {
    $this->e->add("Plugin object copy setup reference also modified original setup reference \"Error\" object", "error");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function Export()
  {
   global $_path;

   $plugin=new Plugin("unit_testing");
   $plugin->info();

   $ar=new Archive();

   if(!($zip=$plugin->export("zip", false)))
   {
    $this->e->add("Plugins should be able to be exported using a ZIP compression method", "error");
    return UT_FAILED;
   }

   if(!$ar->openString($zip, "zip"))
   {
    $this->e->add("Unable to open plugin export ZIP archive", "error");
    return UT_FAILED;
   }

   if($ar->getType("unit_testing/")!="directory")
   {
    $this->e->add("Can't find main directory in plugin export archive", "error");
    return UT_FAILED;
   }

   if($ar->getType("unit_testing/backend.php")!="file")
   {
    $this->e->add("Can't find backend file in plugin export archive", "error");
    return UT_FAILED;
   }

   if($ar->getType("unit_testing/images/.htaccess")!="file")
   {
    $this->e->add("Can't find .htaccess file in plugin export archive", "error");
    return UT_FAILED;
   }

//ignore version control
   $ar->clear();
   $plugdir=$_path["plugins"]."unit_testing/";
   $testdir=$plugdir.".svn/";
   if(!@mkdir($testdir))
   {
    $this->e->add("Unable to create test directory: \"".$testdir."\"", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   if(!($zip=$plugin->export("zip", false)))
   {
    $this->e->add("Plugins should be able to be exported using a ZIP compression method", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   if(!$ar->openString($zip, "zip"))
   {
    $this->e->add("Unable to open plugin export ZIP archive", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   if($ar->getType("unit_testing/")!="directory")
   {
    $this->e->add("Can't find main directory in plugin export archive", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   if($ar->getType("unit_testing/backend.php")!="file")
   {
    $this->e->add("Can't find backend file in plugin export archive", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   if($ar->getType("unit_testing/images/.htaccess")!="file")
   {
    $this->e->add("Can't find .htaccess file in plugin export archive", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   if($ar->getType("unit_testing/.svn/"))
   {
    $this->e->add("Shouldn't find .svn directory in plugin export archive", "error");
    @rmdir($testdir);
    return UT_FAILED;
   }

   @rmdir($testdir);

//non supported compression
   if($zip=$plugin->export("asdasd", false))
   {
    $this->e->add("Plugins shouldn't be able to be exported using a non supported compression method", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Import()
  {
   global $_config, $_path;

   if(in_array("unit_testing_test", JAS::plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config->su(true);
    $remove->remove();
    $_config->su();
   }

   $plugin=new Plugin(false);
   $plugin->e->clear();

   $_config->su(true);
   $imported=$plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_test-0.1.tgz");
   $_config->su();


   if(in_array("unit_testing_test", JAS::plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config->su(true);
    $remove->remove();
    $_config->su();
   }

   if(!$imported)
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function ImportOverwrite()
  {
   global $_config, $_path;

   if(in_array("unit_testing_test", JAS::plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config->su(true);
    $remove->remove();
    $_config->su();
   }

   $plugin=new Plugin(false);
   $plugin->e->clear();

   $_config->su(true);
   $imported=$plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_test-0.1.tgz");
   $imported=$plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_test-0.1.tgz");
   $imported=$plugin->importOverwrite("unit_testing_test");
   $_config->su();

   if(in_array("unit_testing_test", JAS::plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config->su(true);
    $remove->remove();
    $_config->su();
   }

   if(!$imported)
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Remove()
  {
   global $_config, $_path;

   if(in_array("unit_testing_test", JAS::plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config->su(true);
    $remove->remove();
    $_config->su();
   }

   $plugin=new Plugin(false);

   $_config->su(true);
   $imported=$plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_test-0.1.tgz");
   $_config->su();

   if(in_array("unit_testing_test", JAS::plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config->su(true);
    $removed=$remove->remove();
    $_config->su();
   }
   else
    $removed=false;

   if(!$removed)
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }


  function IsEnabled()
  {
   global $_config;

   $plugin=new Plugin("unit_testing");

   if($plugin->isEnabled()!==true)
   {
    $this->e->add("Plugin should be enabled", "error");
    return UT_FAILED;
   }

   $_config->su(true);
   $plugin->setStatus(false);
   $isEnabled=$plugin->isEnabled();
   $plugin->setStatus(true);
   $_config->su();

   if($isEnabled!==false)
   {
    $this->e->add("Plugin should be disabled", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetStatus()
  {
   global $_config, $_path;

   $plugin=new Plugin("unit_testing");

   if($plugin->enabled!==true)
   {
    $this->e->add("Plugin should be enabled", "error");
    return UT_FAILED;
   }

   if(in_array("unit_testing_test", JAS::plugins()))
   {
    $remove=new Plugin("unit_testing_test");

    $_config->su(true);
    $remove->remove();
    $_config->su();
   }


   $plugin=new Plugin(false);

   $_config->su(true);
   $plugin->import($_path["plugins"]."unit_testing/misc/unit_testing_testtest-0.1.tgz");
   $_config->su();


   if(in_array("unit_testing_test", JAS::plugins()))
   {
    $remove=new Plugin("unit_testing_test");
    $enabled=$remove->enabled;

    $_config->su(true);
    $remove->remove();
    $_config->su();
   }
   else
    $enabled=false;

   if($enabled!==false) //newly import plugin always disabled by default
   {
    $this->e->add("Plugin should be disabled", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function SetStatus()
  {
   global $_config;

   $plugin=new Plugin("unit_testing");

   $_config->su(true);

   $plugin->setStatus(false);
   $enabled1=$plugin->enabled;
   $plugin->setStatus(true);
   $enabled2=$plugin->enabled;

   $_config->su();

   if($enabled1!==false)
   {
    $this->e->add("Plugin should be disabled", "error");
    return UT_FAILED;
   }
   if($enabled2!==true)
   {
    $this->e->add("Plugin should be enabled", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Info()
  {
   global $_path;

   $plugin=new Plugin("unit_testing");

   if(!$plugin->info())
   {
    $this->e->add($plugin->e, "error");
    return UT_FAILED;
   }

   if($plugin->title=="")
   {
    $this->e->add("Plugin title not set", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Dependencies()
  {
   global $_path;

   $plugin=new Plugin("unit_testing");

   if(!$plugin->dependencies())
   {
    $this->e->add($plugin->e, "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Config()
  {
   $d=Data::cache("pluginConfigs", "view", "all");
   $d->setShowFields("plugin", "key");
   $d->setValue("show_total", 1);

   if($d->e->any("fatal"))
   {
    $this->e->add("No plugin configs found", "warning");
    return UT_SKIPPED;
   }

   if(!$d->get())
   {
    $this->e->add("Unable to get plugin configs", "error");
    $this->e->add($d->e);
    return UT_FAILED;
   }

   if(!$d->next())
   {
    $this->e->add("No plugin configs found", "warning");
    return UT_SKIPPED;
   }

   $plugin=new Plugin($d->v["plugin"]);

   if(!$plugin->config($d->v["key"]))
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   if($plugin->config("asdasd")) //bogus... we hope
   {
    $this->e->add($plugin->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CheckClass()
  {
   $unit_testing=new Plugin("unit_testing");
   $pager=new Plugin("pager");
   $cms=new Plugin("cms");

//frontend
   if($unit_testing->checkClass("frontend"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." should fail on frontend class check", "error");
    return UT_FAILED;
   }

   if(!$pager->checkClass("frontend"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on frontend class check", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$cms->checkClass("frontend"))
   {
    $this->e->add("Plugin: ".$cms->name." shouldn't fail on frontend class check", "error");
    $this->e->add($cms->e);
    return UT_FAILED;
   }

//backend
   if(!$unit_testing->checkClass("backend"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on backend class check", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$pager->checkClass("backend"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on backend class check", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if($cms->checkClass("backend"))
   {
    $this->e->add("Plugin: ".$cms->name." should fail on backend class check", "error");
    return UT_FAILED;
   }

//setup
   if(!$unit_testing->checkClass("setup"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on setup class check", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$pager->checkClass("setup"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on setup class check", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$cms->checkClass("setup"))
   {
    $this->e->add("Plugin: ".$cms->name." shouldn't fail on setup class check", "error");
    $this->e->add($cms->e);
    return UT_FAILED;
   }

//fail
   if($unit_testing->checkClass("asdasd"))
   {
    $this->e->add("Plugin class check should fail on illegal class type: asdasd", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function InitClass()
  {
   $unit_testing=new Plugin("unit_testing");
   $pager=new Plugin("pager");
   $cms=new Plugin("cms");

//frontend
   if($unit_testing->initClass("frontend"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." should fail on frontend class init", "error");
    return UT_FAILED;
   }

   if(!$pager->initClass("frontend"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on frontend class init", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$cms->initClass("frontend"))
   {
    $this->e->add("Plugin: ".$cms->name." shouldn't fail on frontend class init", "error");
    $this->e->add($cms->e);
    return UT_FAILED;
   }

//backend
   if(!$unit_testing->initClass("backend"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on backend class init", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$pager->initClass("backend"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on backend class init", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if($cms->initClass("backend"))
   {
    $this->e->add("Plugin: ".$cms->name." should fail on backend class init", "error");
    return UT_FAILED;
   }

//setup
   if(!$unit_testing->initClass("setup"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on setup class init", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$pager->initClass("setup"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on setup class init", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$cms->initClass("setup"))
   {
    $this->e->add("Plugin: ".$cms->name." shouldn't fail on setup class init", "error");
    $this->e->add($cms->e);
    return UT_FAILED;
   }

//fail
   if($unit_testing->initClass("asdasd"))
   {
    $this->e->add("Plugin class init should fail on illegal class type: asdasd", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function CallExists()
  {
   $unit_testing=new Plugin("unit_testing");
   $pager=new Plugin("pager");
   $cms=new Plugin("cms");

//frontend
   if($unit_testing->callExists("frontend", "main"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." should fail on frontend main call exists", "error");
    return UT_FAILED;
   }
   if($unit_testing->callExists("frontend", "output"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." should fail on frontend output call exists", "error");
    return UT_FAILED;
   }

   if(!$pager->callExists("frontend", "main"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on frontend main call exists", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }
   if(!$pager->callExists("frontend", "output"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on frontend output call exists", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$cms->callExists("frontend", "main"))
   {
    $this->e->add("Plugin: ".$cms->name." shouldn't fail on frontend main call exists", "error");
    $this->e->add($cms->e);
    return UT_FAILED;
   }
   if(!$cms->callExists("frontend", "output"))
   {
    $this->e->add("Plugin: ".$cms->name." shouldn't fail on frontend output call exists", "error");
    $this->e->add($cms->e);
    return UT_FAILED;
   }

//backend
   if(!$unit_testing->callExists("backend", "main"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on backend main call exists", "error");
    $this->e->add($unit_testing->e);
    return UT_FAILED;
   }
   $this->runtests=false; //ensure tests aren't executed
   if(!$unit_testing->callExists("backend", "output"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on backend output call exists", "error");
    $this->e->add($unit_testing->e);
    return UT_FAILED;
   }

   if(!$pager->callExists("backend", "main"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on backend main call exists", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }
   if(!$pager->callExists("backend", "output"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on backend output call exists", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if($cms->callExists("backend", "main"))
   {
    $this->e->add("Plugin: ".$cms->name." should fail on backend main call exists", "error");
    return UT_FAILED;
   }
   if($cms->callExists("backend", "output"))
   {
    $this->e->add("Plugin: ".$cms->name." should fail on backend output call exists", "error");
    return UT_FAILED;
   }

//setup
   if(!$unit_testing->callExists("setup", "init"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on setup init call exists", "error");
    $this->e->add($unit_testing->e);
    return UT_FAILED;
   }

   if(!$pager->callExists("setup", "init"))
   {
    $this->e->add("Plugin: ".$pager->name." shouldn't fail on setup init call exists", "error");
    $this->e->add($pager->e);
    return UT_FAILED;
   }

   if(!$cms->callExists("setup", "init"))
   {
    $this->e->add("Plugin: ".$cms->name." shouldn't fail on setup init call exists", "error");
    $this->e->add($cms->e);
    return UT_FAILED;
   }

//fail
   if($cms->callExists("asdasd", "main"))
   {
    $this->e->add("Plugin call exists should fail on illegal class type: asdasd", "error");
    return UT_FAILED;
   }

   if($cms->callExists("setup", "asdasd"))
   {
    $this->e->add("Plugin call exists should fail on illegal call: asdasd", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Call()
  {
   $unit_testing=new Plugin("unit_testing");
   $pager=new Plugin("pager");
   $cms=new Plugin("cms");

//frontend
   if($unit_testing->call("frontend", "main"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." should fail on frontend main call", "error");
    return UT_FAILED;
   }
   if($unit_testing->call("frontend", "output"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." should fail on frontend output call", "error");
    return UT_FAILED;
   }

   if($pager->isEnabled())
   {
    if(!$pager->call("frontend", "main"))
    {
     $this->e->add("Plugin: ".$pager->name." shouldn't fail on frontend main call", "error");
     $this->e->add($pager->e);
     return UT_FAILED;
    }
    if(!$pager->call("frontend", "output"))
    {
     $this->e->add("Plugin: ".$pager->name." shouldn't fail on frontend output call", "error");
     $this->e->add($pager->e);
     return UT_FAILED;
    }
   }

   if($cms->isEnabled())
   {
    if($cms->call("frontend", "main")) //no config
    {
     $this->e->add("Plugin: ".$cms->name." should fail on frontend main call", "error");
     return UT_FAILED;
    }
    if($cms->call("frontend", "output")) //no config
    {
     $this->e->add("Plugin: ".$cms->name." shouldt fail on frontend output call", "error");
     $this->e->add($cms->e);
     return UT_FAILED;
    }
   }

//backend
   if(!$unit_testing->call("backend", "main"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on backend main call", "error");
    $this->e->add($unit_testing->e);
    return UT_FAILED;
   }
   $unit_testing->getClass("backend")->runtests=false; //ensure tests aren't executed
   if(!$unit_testing->call("backend", "output"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on backend output call", "error");
    $this->e->add($unit_testing->e);
    return UT_FAILED;
   }
/* fails
   if($pager->isEnabled())
   {
    if(!$pager->call("backend", "main"))
    {
     $this->e->add("Plugin: ".$pager->name." shouldn't fail on backend main call", "error");
     $this->e->add($pager->e);
     return UT_FAILED;
    }
    if(!$pager->call("backend", "output"))
    {
     $this->e->add("Plugin: ".$pager->name." shouldn't fail on backend output call", "error");
     $this->e->add($pager->e);
     return UT_FAILED;
    }
   }
*/
   if($cms->isEnabled())
   {
    if($cms->call("backend", "main"))
    {
     $this->e->add("Plugin: ".$cms->name." should fail on backend main call", "error");
     return UT_FAILED;
    }
    if($cms->call("backend", "output"))
    {
     $this->e->add("Plugin: ".$cms->name." should fail on backend output call", "error");
     return UT_FAILED;
    }
   }

//setup
   if(!$unit_testing->call("setup", "init"))
   {
    $this->e->add("Plugin: ".$unit_testing->name." shouldn't fail on setup init call", "error");
    $this->e->add($unit_testing->e);
    return UT_FAILED;
   }

   if($pager->isEnabled())
   {
    if(!$pager->call("setup", "init"))
    {
     $this->e->add("Plugin: ".$pager->name." shouldn't fail on setup init call", "error");
     $this->e->add($pager->e);
     return UT_FAILED;
    }
   }

   if($cms->isEnabled())
   {
    if(!$cms->call("setup", "init"))
    {
     $this->e->add("Plugin: ".$cms->name." shouldn't fail on setup init call", "error");
     $this->e->add($cms->e);
     return UT_FAILED;
    }
   }

//fail
   if($unit_testing->call("asdasd", "main"))
   {
    $this->e->add("Plugin call should fail on illegal class type: asdasd", "error");
    return UT_FAILED;
   }

   if($unit_testing->call("setup", "asdasd"))
   {
    $this->e->add("Plugin call should fail on illegal call: asdasd", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function AddVar()
  {
   $plugin=new Plugin("unit_testing");

   if($plugin->vars)
   {
    $this->e->add("Plugins vars not empty", "error");
    return UT_FAILED;
   }

   $plugin->addVar("asd", "123");

   if(!isset($plugin->vars["asd"])||$plugin->vars["asd"]!=123)
   {
    $this->e->add("Plugins vars test value not found", "error");
    return UT_FAILED;
   }

   $plugin->addVar("asd", "123");

   if(!$plugin->e->any("warning"))
   {
    $this->e->add("Plugins vars overwrite should generate a warning", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function URL()
  {
   $plugin=new Plugin("unit_testing");

   if($plugin->url()!="?")
   {
    $this->e->add("Plugins URL should be \"?\"", "error");
    return UT_FAILED;
   }

   $plugin->addVar("asd", "123");

   if($plugin->url()!="?asd=123&amp;")
   {
    $this->e->add("Plugins URL should be \"?ad=123&amp;\"", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

 class UT_Base_Plugin extends UT_Plugin
 {
  function Constructor()
  {
   $plugin=new Plugin("unit_testing");
   $object=new Base_Plugin($plugin, $plugin->config);

   if(!is_object($object)||get_class($object)!="Base_Plugin")
   {
    $this->e->add("Constructor doesn't return a Base_Plugin object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->plugin!==$plugin)
   {
    $this->e->add("Base_Plugin plugin not set correctly", "error");
    return UT_FAILED;
   }

   if($object->config!==$plugin->config)
   {
    $this->e->add("Base_Plugin plugin config not set correctly", "error");
    return UT_FAILED;
   }

   if($object->e!==$plugin->e)
   {
    $this->e->add("Base_Plugin error not set correctly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Main()
  {
   $plugin=new Plugin("unit_testing");
   $p=new Base_Plugin($plugin, $plugin->config);

   if($p->main()!==true)
   {
    $this->e->add("Base_Plugin main function should return true", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $plugin=new Plugin("unit_testing");
   $p=new Base_Plugin($plugin, $plugin->config);

   if($p->output()!="")
   {
    $this->e->add("Base_Plugin output function should return nothing", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }


 class UT_Base_Plugin_Backend extends UT_Plugin
 {
  function Constructor()
  {
   $plugin=new Plugin("unit_testing");
   $object=new Base_Plugin_Backend($plugin);

   if(!is_object($object)||get_class($object)!="Base_Plugin_Backend")
   {
    $this->e->add("Constructor doesn't return a Base_Plugin_Backend object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->plugin!==$plugin)
   {
    $this->e->add("Base_Plugin_Backend plugin not set correctly", "error");
    return UT_FAILED;
   }

   if($object->e!==$plugin->e)
   {
    $this->e->add("Base_Plugin_Backend error not set correctly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Main()
  {
   $plugin=new Plugin("unit_testing");
   $p=new Base_Plugin_Backend($plugin);

   if($p->main()!==true)
   {
    $this->e->add("Base_Plugin_Backend main function should return true", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Output()
  {
   $plugin=new Plugin("unit_testing");
   $p=new Base_Plugin_Backend($plugin);

   if($p->output()!="")
   {
    $this->e->add("Base_Plugin_Backend output function should return nothing", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }
 }

 class UT_Base_Plugin_Setup extends UT_Plugin
 {
  function Constructor()
  {
   $plugin=new Plugin("unit_testing");
   $object=new Base_Plugin_Setup($plugin);

   if(!is_object($object)||get_class($object)!="Base_Plugin_Setup")
   {
    $this->e->add("Constructor doesn't return a Base_Plugin_Setup object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->fs)||get_class($object->fs)!="FS")
   {
    $this->e->add("Object: \"fs\" isn't an FS object", "FS");
    return UT_FAILED;
   }

   if($object->plugin!==$plugin)
   {
    $this->e->add("Base_Plugin_Setup plugin not set correctly", "error");
    return UT_FAILED;
   }

   if($object->e!==$plugin->e)
   {
    $this->e->add("Base_Plugin_Setup error not set correctly", "error");
    return UT_FAILED;
   }

   if($object->force!==false)
   {
    $this->e->add("Base_Plugin_Setup force should be false", "error");
    return UT_FAILED;
   }

   if($object->deSetup!==false)
   {
    $this->e->add("Base_Plugin_Setup deSetup should be false", "error");
    return UT_FAILED;
   }

   if($object->directories!==Array())
   {
    $this->e->add("Base_Plugin_Setup directories should be an empty array", "error");
    return UT_FAILED;
   }

   if($object->files!==Array())
   {
    $this->e->add("Base_Plugin_Setup files should be an empty array", "error");
    return UT_FAILED;
   }

   if($object->modules!==Array())
   {
    $this->e->add("Base_Plugin_Setup modules should be an empty array", "error");
    return UT_FAILED;
   }

   if($object->done!==true)
   {
    $this->e->add("Base_Plugin_Setup done should be true", "error");
    return UT_FAILED;
   }

   $object=new Base_Plugin_Setup($plugin, true, true);

   if($object->force!==true)
   {
    $this->e->add("Base_Plugin_Setup force should be true", "error");
    return UT_FAILED;
   }

   if($object->deSetup!==true)
   {
    $this->e->add("Base_Plugin_Setup deSetup should be true", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Clear()
  {
   $plugin=new Plugin("unit_testing");
   $p=new Base_Plugin_Setup($plugin);

   $p->directories=Array("directories");
   $p->files=Array("files");
   $p->modules=Array("modules");
   $p->done=false;

   $p->clear();

   if($p->directories!==Array())
   {
    $this->e->add("Base_Plugin_Setup clear function should have reset directories", "error");
    return UT_FAILED;
   }

   if($p->files!==Array())
   {
    $this->e->add("Base_Plugin_Setup clear function should have reset files", "error");
    return UT_FAILED;
   }

   if($p->modules!==Array())
   {
    $this->e->add("Base_Plugin_Setup clear function should have reset modules", "error");
    return UT_FAILED;
   }

   if($p->done!==true)
   {
    $this->e->add("Base_Plugin_Setup clear function should have reset done", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Get()
  {
   $plugin=new Plugin("unit_testing");
   $p=new Base_Plugin_Setup($plugin);

   if($p->get("directories")!==Array())
   {
    $this->e->add("Base_Plugin_Setup get function should have returned an empty directories array", "error");
    return UT_FAILED;
   }

   if($p->get("files")!==Array())
   {
    $this->e->add("Base_Plugin_Setup get function should have returned an empty files array", "error");
    return UT_FAILED;
   }

   if($p->get("modules")!==Array())
   {
    $this->e->add("Base_Plugin_Setup get function should have returned an empty modules array", "error");
    return UT_FAILED;
   }

   $p->directories=Array("directories");
   $p->files=Array("files");
   $p->modules=Array("modules");

   if($p->get("directories")!==Array("directories"))
   {
    $this->e->add("Base_Plugin_Setup get function should have returned the directories array", "error");
    return UT_FAILED;
   }

   if($p->get("files")!==Array("files"))
   {
    $this->e->add("Base_Plugin_Setup get function should have returned the files array", "error");
    return UT_FAILED;
   }

   if($p->get("files")!==Array("files"))
   {
    $this->e->add("Base_Plugin_Setup get function should have returned the files array", "error");
    return UT_FAILED;
   }

   if($p->get("asdasd"))
   {
    $this->e->add("Base_Plugin_Setup get function should have returned the files array", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Mkdir()
  {
   global $_path;

   $plugin=new Plugin("unit_testing");
   $p=new Base_Plugin_Setup($plugin);

   //new
   if(!$p->mkdir($_path["root"]."asdasd"))
   {
    $this->e->add("Base_Plugin_Setup mkdir function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("directories"), Array(Array("directory" => $_path["root"]."asdasd", "status" => "Setup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup mkdir function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //exists
   if(!$p->mkdir($_path["root"]."jas"))
   {
    $this->e->add("Base_Plugin_Setup mkdir function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("directories"), Array(Array("directory" => $_path["root"]."jas", "status" => "Setup correctly"))))
    return UT_FAILED;

   if($p->done!==true)
   {
    $this->e->add("Base_Plugin_Setup mkdir function should have set done to true", "error");
    return UT_FAILED;
   }

   $p->clear();

   //exists as file
   if(!$p->mkdir($_path["root"]."index.php"))
   {
    $this->e->add("Base_Plugin_Setup mkdir function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("directories"), Array(Array("directory" => $_path["root"]."index.php", "status" => "Directory already exists as file"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup mkdir function should have set done to false", "error");
    return UT_FAILED;
   }


//DeSetup
   $p=new Base_Plugin_Setup($plugin, false, true);

   //new
   if(!$p->mkdir($_path["root"]."asdasd"))
   {
    $this->e->add("Base_Plugin_Setup mkdir function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("directories"), Array(Array("directory" => $_path["root"]."asdasd", "status" => "Not setup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup mkdir function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //exists
   if(!$p->mkdir($_path["root"]."jas"))
   {
    $this->e->add("Base_Plugin_Setup mkdir function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("directories"), Array(Array("directory" => $_path["root"]."jas", "status" => "DeSetup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup mkdir function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //exists as file
   if(!$p->mkdir($_path["root"]."index.php"))
   {
    $this->e->add("Base_Plugin_Setup mkdir function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("directories"), Array(Array("directory" => $_path["root"]."index.php", "status" => "Directory is a file"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup mkdir function should have set done to false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Files()
  {
   $plugin=new Plugin("unit_testing");
   $p=new Base_Plugin_Setup($plugin);

   //new
   if(!$p->file("backend.php", "asdasd"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => "asdasd", "symlink" => false, "status" => "Setup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //exists
   if(!$p->file("backend.php", $plugin->path["root"], true))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => $plugin->path["root"]."backend.php", "symlink" => true, "status" => "Setup correctly"))))
    return UT_FAILED;

   if($p->done!==true)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to true", "error");
    return UT_FAILED;
   }

   $p->clear();

   //changed
   if(!$p->file("backend.php", "index.php"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => "index.php", "symlink" => false, "status" => "File already exists"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //no source
   if(!$p->file("asdasd", "index.php"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "asdasd", "destination" => "index.php", "symlink" => false, "status" => "File not found"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }


//Force
   $p=new Base_Plugin_Setup($plugin, true, false);

   //new
   if(!$p->file("backend.php", "asdasd"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => "asdasd", "symlink" => false, "status" => "Setup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //exists
   if(!$p->file("backend.php", $plugin->path["root"], true))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => $plugin->path["root"]."backend.php", "symlink" => true, "status" => "Forced setup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //changed
   if(!$p->file("backend.php", "index.php"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => "index.php", "symlink" => false, "status" => "Forced setup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //no source
   if(!$p->file("asdasd", "index.php"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "asdasd", "destination" => "index.php", "symlink" => false, "status" => "File not found"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }


//DeSetup
   $p=new Base_Plugin_Setup($plugin, false, true);

   //new
   if(!$p->file("backend.php", "asdasd"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => "asdasd", "symlink" => false, "status" => "Not setup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //exists
   if(!$p->file("backend.php", $plugin->path["root"], true))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => $plugin->path["root"]."backend.php", "symlink" => true, "status" => "DeSetup"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //changed
   if(!$p->file("backend.php", "index.php"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "backend.php", "destination" => "index.php", "symlink" => false, "status" => "File changed"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   $p->clear();

   //no source
   if(!$p->file("asdasd", "index.php"))
   {
    $this->e->add("Base_Plugin_Setup file function shouldn't have failed", "error");
    $this->e->add($p->e);
    return UT_FAILED;
   }

   if(!$this->_compare($p->get("files"), Array(Array("source" => "asdasd", "destination" => "index.php", "symlink" => false, "status" => "File not found"))))
    return UT_FAILED;

   if($p->done!==false)
   {
    $this->e->add("Base_Plugin_Setup file function should have set done to false", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Modules()
  {
   return UT_SKIPPED;
  }

  function Data()
  {
   return UT_SKIPPED;
  }

  function Init()
  {
   return UT_SKIPPED;
  }

  function Exec()
  {
   return UT_SKIPPED;
  }
 }

?>
