<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/script.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/core.php");
 require_once($_path["include"]."script.php");

 class ScriptTest extends Script
 {
  function output()
  {
   return "something is returned";
  }

  function outputEach()
  {
   global $_ScriptTest, $_ScriptCounter;

   if(!isset($_ScriptCounter))
    $_ScriptCounter=0;

   if(!isset($_ScriptTest[$_ScriptCounter]))
   {
    $_ScriptCounter=0;
    return false;
   }

   return $_ScriptTest[$_ScriptCounter++];
  }

  function input($key, $value)
  {
   global $_ScriptTest;

   if(isset($_ScriptTest[$key]))
    return false;

   $_ScriptTest[$key]=$value;
   return true;
  }
 }

 class UT_Script extends UT_Classes
 {
  function Constructor()
  {
   $object=new Script();

   if(!is_object($object)||get_class($object)!="Script")
   {
    $this->e->add("Constructor doesn't return a Script object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if(!is_array($object->markers))
   {
    $this->e->add("Markers isn't an array", "error");
    return UT_FAILED;
   }

   if($object->m!="|")
   {
    $this->e->add("Marker doesn't match", "error");
    return UT_FAILED;
   }

   if($object->mq!="\|")
   {
    $this->e->add("Quoted marker doesn't match", "error");
    return UT_FAILED;
   }

   if($object->commands)
   {
    $this->e->add("Commands list should be empty", "error");
    return UT_FAILED;
   }

   if($object->values)
   {
    $this->e->add("Values list should be empty", "error");
    return UT_FAILED;
   }

   if($object->valuesReadonly)
   {
    $this->e->add("Values readonly list should be empty", "error");
    return UT_FAILED;
   }

   if(!$object->blacklist)
   {
    $this->e->add("Blacklist shouldn't be empty", "error");
    return UT_FAILED;
   }

   $blacklist=Array(
    "Script", "__construct", "__destruct", "__clone",
    "clear", "marker",
    "convert", "values",
    "parse", "parseFlow",
   );

   if(!$this->_compare($blacklist, $object->blacklist))
    return UT_FAILED;

   $object=new ScriptTest();

   $blacklist=Array(
    "Script", "ScriptTest", "__construct", "__destruct", "__clone",
    "clear", "marker",
    "convert", "values",
    "parse", "parseFlow",
   );

   if(!$this->_compare($blacklist, $object->blacklist))
    return UT_FAILED;

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $orig=new Script();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("Script object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->debug=true;
   if($orig->debug==$copy->debug)
   {
    $this->e->add("Script object copy also modified original \"debug\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("Script object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Clear()
  {
   $s=new Script();
   $s->commands[]="test";
   $s->values[]="test";
   $s->valuesReadonly[]="test";

   $s->clear();

   if($s->commands)
   {
    $this->e->add("Commands list should be empty", "error");
    return UT_FAILED;
   }

   if($s->values)
   {
    $this->e->add("Values list should be empty", "error");
    return UT_FAILED;
   }

   if($s->valuesReadonly)
   {
    $this->e->add("Values readonly list should be empty", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Marker()
  {
   $s=new Script();

   if(!$s->marker("|"))
   {
    $this->e->add("Marker shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->m!="|")
   {
    $this->e->add("Marker doesn't match", "error");
    return UT_FAILED;
   }

   if($s->mq!="\|")
   {
    $this->e->add("Quoted marker doesn't match", "error");
    return UT_FAILED;
   }

   if(!$s->marker("/"))
   {
    $this->e->add("Marker shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->m!="/")
   {
    $this->e->add("Marker doesn't match", "error");
    return UT_FAILED;
   }

   if($s->mq!="\/")
   {
    $this->e->add("Quoted marker doesn't match", "error");
    return UT_FAILED;
   }

   if(!$s->marker("!"))
   {
    $this->e->add("Marker shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->m!="!")
   {
    $this->e->add("Marker doesn't match", "error");
    return UT_FAILED;
   }

   if($s->mq!="\!")
   {
    $this->e->add("Quoted marker doesn't match", "error");
    return UT_FAILED;
   }

   if(!$s->marker("`"))
   {
    $this->e->add("Marker shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->m!="`")
   {
    $this->e->add("Marker doesn't match", "error");
    return UT_FAILED;
   }

   if($s->mq!="`")
   {
    $this->e->add("Quoted marker doesn't match", "error");
    return UT_FAILED;
   }

   if($s->marker("\\"))
   {
    $this->e->add("Marker should have failed", "error");
    return UT_FAILED;
   }

   if($s->marker("$"))
   {
    $this->e->add("Marker should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Convert()
  {
   $s=new Script();

   if($s->convert("123")!==123)
   {
    $this->e->add("Convert didn't convert int correctly", "error");
    return UT_FAILED;
   }

   if($s->convert(123)!==123)
   {
    $this->e->add("Convert didn't convert int correctly", "error");
    return UT_FAILED;
   }

   if($s->convert("123.456")!==123.456)
   {
    $this->e->add("Convert didn't convert float correctly", "error");
    return UT_FAILED;
   }

   if($s->convert(123.0)!==123.0)
   {
    $this->e->add("Convert didn't convert float correctly", "error");
    return UT_FAILED;
   }

   if($s->convert("false")!==false)
   {
    $this->e->add("Convert didn't convert false correctly", "error");
    return UT_FAILED;
   }

   if($s->convert(false)!==false)
   {
    $this->e->add("Convert didn't convert false correctly", "error");
    return UT_FAILED;
   }

   if($s->convert("true")!==true)
   {
    $this->e->add("Convert didn't convert true correctly", "error");
    return UT_FAILED;
   }

   if($s->convert(true)!==true)
   {
    $this->e->add("Convert didn't convert true correctly", "error");
    return UT_FAILED;
   }

   if($s->convert("test")!=="test")
   {
    $this->e->add("Convert didn't convert string correctly", "error");
    return UT_FAILED;
   }

   if($s->convert(Array("123", "123.0", "false", "test"))!==Array(123, 123.0, false, "test"))
   {
    $this->e->add("Convert didn't convert string correctly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Values()
  {
   $s=new Script();
   $s->set("var1", 123);
   $s->set("var2", 123.456);
   $s->set("var3.var1", "Testing");

   if($s->values("'var1 testing'")!=Array("var1 testing"))
   {
    $this->e->add("Values didn't convert value correctly", "error");
    return UT_FAILED;
   }

   if($s->values("var1 testing")!=Array("var1", "testing"))
   {
    $this->e->add("Values didn't convert value correctly", "error");
    return UT_FAILED;
   }

   if($s->values(Array(
    "value" => "|0|",
    "keys" => Array("var1"),
   ))!=Array(123))
   {
    $this->e->add("Values didn't convert int correctly", "error");
    return UT_FAILED;
   }

   if($s->values(Array(
    "value" => "'|0| |1| |2|'",
    "keys" => Array("var1", "var2", "var3.var1"),
   ))!=Array("123 123.456 Testing"))
   {
    $this->e->add("Values didn't convert string correctly", "error");
    return UT_FAILED;
   }

   if($s->values(Array(
    "value" => "|0| |1| |2|",
    "keys" => Array("var1", "var2", "var3.var1"),
   ))!=Array(123, 123.456, "Testing"))
   {
    $this->e->add("Values didn't convert array correctly", "error");
    return UT_FAILED;
   }

   if($s->values(Array(
    "value" => "'|0|' '' |1| |2|",
    "keys" => Array("var1", "var2", "var3.var1"),
   ))!=Array(123, "", 123.456, "Testing"))
   {
    $this->e->add("Values didn't convert array correctly", "error");
    return UT_FAILED;
   }

   if($s->values("")!=Array())
   {
    $this->e->add("Values didn't convert array correctly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Parse()
  {
   $s=new ScriptTest();

   if(!$s->parse("log", "
log error Test
 s.execute test == true
else |||failure
 exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["log"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "log",
     "input" => "error Test",
     "output" => NULL,
     "comments" => false,
     "success" => 1,
     "failure" => 2,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => true,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
    Array(
     "depth" => 0,
     "type" => "else",
     "comments" => "failure",
     "success" => 3,
     "failure" => -1,
    ),
    Array(
     "depth" => 1,
     "type" => "exit",
     "input" => false,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;


   if(!$s->parse("test", "
set var1 'Test 1' == true
 set var2 123 |||comment
  set var3 Testing
output == |var1|[a-z]+| is returned
 s.input |var2|\d+| |var3|
exit 1.0"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands["test"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "set",
     "input" => "var1 'Test 1'",
     "output" => true,
     "comments" => false,
     "success" => 1,
     "failure" => 3,
    ),
    Array(
     "depth" => 1,
     "object" => false,
     "type" => "set",
     "input" => "var2 123",
     "output" => NULL,
     "comments" => "comment",
     "success" => 2,
     "failure" => 3,
    ),
    Array(
     "depth" => 2,
     "object" => false,
     "type" => "set",
     "input" => "var3 Testing",
     "output" => NULL,
     "comments" => false,
     "success" => 3,
     "failure" => 3,
    ),
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "output",
     "input" => "",
     "output" => Array(
      "value" => "|0| is returned",
      "keys" => Array("var1"),
      "regexps" => Array("[a-z]+"),
     ),
     "comments" => false,
     "success" => 4,
     "failure" => 5,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "input",
     "input" => Array(
      "value" => "|0| |1|",
      "keys"  => Array("var2", "var3"),
      "regexps" => Array("\d+", ".+"),
     ),
     "output" => NULL,
     "comments" => false,
     "success" => 5,
     "failure" => 5,
    ),
    Array(
     "depth" => 0,
     "type" => "exit",
     "input" => 1.0,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;


   //overwrite test
   if(!$s->parse("test", "
log error Test
 s.execute test == true
else |||failure
 exit 0
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands["test"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "log",
     "input" => "error Test",
     "output" => NULL,
     "comments" => false,
     "success" => 1,
     "failure" => 2,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => true,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
    Array(
     "depth" => 0,
     "type" => "else",
     "comments" => "failure",
     "success" => 3,
     "failure" => -1,
    ),
    Array(
     "depth" => 1,
     "type" => "exit",
     "input" => 0,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;

   return UT_OK;
  }

  function Parse_Slash()
  {
   $s=new ScriptTest();
   $s->marker("/");

   if(!$s->parse("log", "
log error Test
 s.execute test == true
else ///failure
 exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["log"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "log",
     "input" => "error Test",
     "output" => NULL,
     "comments" => false,
     "success" => 1,
     "failure" => 2,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => true,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
    Array(
     "depth" => 0,
     "type" => "else",
     "comments" => "failure",
     "success" => 3,
     "failure" => -1,
    ),
    Array(
     "depth" => 1,
     "type" => "exit",
     "input" => false,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;


   if(!$s->parse("test", "
set var1 'Test 1' == true
 set var2 123 ///comment
  set var3 Testing
output == /var1/[a-z]+/ is returned
 s.input /var2/\d+/ /var3/
exit 1.0"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands["test"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "set",
     "input" => "var1 'Test 1'",
     "output" => true,
     "comments" => false,
     "success" => 1,
     "failure" => 3,
    ),
    Array(
     "depth" => 1,
     "object" => false,
     "type" => "set",
     "input" => "var2 123",
     "output" => NULL,
     "comments" => "comment",
     "success" => 2,
     "failure" => 3,
    ),
    Array(
     "depth" => 2,
     "object" => false,
     "type" => "set",
     "input" => "var3 Testing",
     "output" => NULL,
     "comments" => false,
     "success" => 3,
     "failure" => 3,
    ),
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "output",
     "input" => "",
     "output" => Array(
      "value" => "/0/ is returned",
      "keys" => Array("var1"),
      "regexps" => Array("[a-z]+"),
     ),
     "comments" => false,
     "success" => 4,
     "failure" => 5,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "input",
     "input" => Array(
      "value" => "/0/ /1/",
      "keys"  => Array("var2", "var3"),
      "regexps" => Array("\d+", ".+"),
     ),
     "output" => NULL,
     "comments" => false,
     "success" => 5,
     "failure" => 5,
    ),
    Array(
     "depth" => 0,
     "type" => "exit",
     "input" => 1.0,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;

   return UT_OK;
  }

  function Parse_Backtick()
  {
   $s=new ScriptTest();
   $s->marker("`");

   if(!$s->parse("log", "
log error Test
 s.execute test == true
else ```failure
 exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["log"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "log",
     "input" => "error Test",
     "output" => NULL,
     "comments" => false,
     "success" => 1,
     "failure" => 2,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => true,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
    Array(
     "depth" => 0,
     "type" => "else",
     "comments" => "failure",
     "success" => 3,
     "failure" => -1,
    ),
    Array(
     "depth" => 1,
     "type" => "exit",
     "input" => false,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;


   if(!$s->parse("test", "
set var1 'Test 1' == true
 set var2 123 ```comment
  set var3 Testing
output == `var1`[a-z]+` is returned
 s.input `var2`\d+` `var3`
exit 1.0"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands["test"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "set",
     "input" => "var1 'Test 1'",
     "output" => true,
     "comments" => false,
     "success" => 1,
     "failure" => 3,
    ),
    Array(
     "depth" => 1,
     "object" => false,
     "type" => "set",
     "input" => "var2 123",
     "output" => NULL,
     "comments" => "comment",
     "success" => 2,
     "failure" => 3,
    ),
    Array(
     "depth" => 2,
     "object" => false,
     "type" => "set",
     "input" => "var3 Testing",
     "output" => NULL,
     "comments" => false,
     "success" => 3,
     "failure" => 3,
    ),
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "output",
     "input" => "",
     "output" => Array(
      "value" => "`0` is returned",
      "keys" => Array("var1"),
      "regexps" => Array("[a-z]+"),
     ),
     "comments" => false,
     "success" => 4,
     "failure" => 5,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "input",
     "input" => Array(
      "value" => "`0` `1`",
      "keys"  => Array("var2", "var3"),
      "regexps" => Array("\d+", ".+"),
     ),
     "output" => NULL,
     "comments" => false,
     "success" => 5,
     "failure" => 5,
    ),
    Array(
     "depth" => 0,
     "type" => "exit",
     "input" => 1.0,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;

   return UT_OK;
  }

  function Parse_Exclamation()
  {
   $s=new ScriptTest();
   $s->marker("!");

   if(!$s->parse("log", "
log error Test
 s.execute test == true
else !!!failure
 exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["log"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "log",
     "input" => "error Test",
     "output" => NULL,
     "comments" => false,
     "success" => 1,
     "failure" => 2,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => true,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
    Array(
     "depth" => 0,
     "type" => "else",
     "comments" => "failure",
     "success" => 3,
     "failure" => -1,
    ),
    Array(
     "depth" => 1,
     "type" => "exit",
     "input" => false,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;


   if(!$s->parse("test", "
set var1 'Test 1' == true
 set var2 123 !!!comment
  set var3 Testing
output == !var1![a-z]+! is returned
 s.input !var2!\d+! !var3!
exit 1.0"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands["test"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "set",
     "input" => "var1 'Test 1'",
     "output" => true,
     "comments" => false,
     "success" => 1,
     "failure" => 3,
    ),
    Array(
     "depth" => 1,
     "object" => false,
     "type" => "set",
     "input" => "var2 123",
     "output" => NULL,
     "comments" => "comment",
     "success" => 2,
     "failure" => 3,
    ),
    Array(
     "depth" => 2,
     "object" => false,
     "type" => "set",
     "input" => "var3 Testing",
     "output" => NULL,
     "comments" => false,
     "success" => 3,
     "failure" => 3,
    ),
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "output",
     "input" => "",
     "output" => Array(
      "value" => "!0! is returned",
      "keys" => Array("var1"),
      "regexps" => Array("[a-z]+"),
     ),
     "comments" => false,
     "success" => 4,
     "failure" => 5,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "input",
     "input" => Array(
      "value" => "!0! !1!",
      "keys"  => Array("var2", "var3"),
      "regexps" => Array("\d+", ".+"),
     ),
     "output" => NULL,
     "comments" => false,
     "success" => 5,
     "failure" => 5,
    ),
    Array(
     "depth" => 0,
     "type" => "exit",
     "input" => 1.0,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;

   return UT_OK;
  }

  function Parse_Advanced()
  {
   $s=new ScriptTest();

   if(!$s->parse("log", "
log error Test
 s.execute test == true
  s.execute test == false
 s.execute test == true
else |||failure
 s.execute test == true
 exit false
exit true
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["log"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "log",
     "input" => "error Test",
     "output" => NULL,
     "comments" => false,
     "success" => 1,
     "failure" => 4,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => true,
     "comments" => false,
     "success" => 2,
     "failure" => 3,
    ),
    Array(
     "depth" => 2,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => false,
     "comments" => false,
     "success" => 3,
     "failure" => 3,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => true,
     "comments" => false,
     "success" => 7,
     "failure" => 7,
    ),
    Array(
     "depth" => 0,
     "type" => "else",
     "comments" => "failure",
     "success" => 5,
     "failure" => 7,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => "test",
     "output" => true,
     "comments" => false,
     "success" => 6,
     "failure" => 6,
    ),
    Array(
     "depth" => 1,
     "type" => "exit",
     "input" => false,
     "output" => NULL,
     "comments" => false,
     "success" => 7,
     "failure" => 7,
    ),
    Array(
     "depth" => 0,
     "type" => "exit",
     "input" => true,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;

   return UT_OK;
  }

  function Parse_Else()
  {
   $s=new ScriptTest();

   if(!$s->parse("else", "
set asd 123
 set asd 456
 else
  log fail 456
  exit false
else
 log fail 123
 exit false
exit true
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["else"]=Array(
    Array(
     "depth" => 0,
     "object" => false,
     "type" => "set",
     "input" => "asd 123",
     "output" => NULL,
     "comments" => false,
     "success" => 1,
     "failure" => 5,
    ),
    Array(
     "depth" => 1,
     "object" => false,
     "type" => "set",
     "input" => "asd 456",
     "output" => NULL,
     "comments" => false,
     "success" => 8,
     "failure" => 2,
    ),
    Array(
     "depth" => 1,
     "type" => "else",
     "comments" => false,
     "success" => 3,
     "failure" => 8,
    ),
    Array(
     "depth" => 2,
     "object" => false,
     "type" => "log",
     "input" => "fail 456",
     "output" => NULL,
     "comments" => false,
     "success" => 4,
     "failure" => 4,
    ),
    Array(
     "depth" => 2,
     "type" => "exit",
     "input" => false,
     "output" => NULL,
     "comments" => false,
     "success" => 8,
     "failure" => 8,
    ),
    Array(
     "depth" => 0,
     "type" => "else",
     "comments" => false,
     "success" => 6,
     "failure" => 8,
    ),
    Array(
     "depth" => 1,
     "object" => false,
     "type" => "log",
     "input" => "fail 123",
     "output" => NULL,
     "comments" => false,
     "success" => 7,
     "failure" => 7,
    ),
    Array(
     "depth" => 1,
     "type" => "exit",
     "input" => false,
     "output" => NULL,
     "comments" => false,
     "success" => 8,
     "failure" => 8,
    ),
    Array(
     "depth" => 0,
     "type" => "exit",
     "input" => true,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;

   return UT_OK;
  }

  function Parse_Each()
  {
   $s=new ScriptTest();

   if(!$s->parse("each", "
each list k v
 s.execute |k| == |v|
 else |||failure
  s.execute |k| == true
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["each"]=Array(
    Array(
     "depth" => 0,
     "type" => "each",
     "input" => Array("list", "k", "v"),
     "comments" => false,
     "success" => 1,
     "failure" => -1,
    ),
    Array(
     "depth" => 1,
     "object" => "s",
     "type" => "execute",
     "input" => Array(
      "value" => "|0|",
      "keys" => Array("k"),
      "regexps" => Array(".+"),
     ),
     "output" => Array(
      "value" => "|0|",
      "keys" => Array("v"),
      "regexps" => Array(".+"),
     ),
     "comments" => false,
     "success" => 0,
     "failure" => 2,
    ),
    Array(
     "depth" => 1,
     "type" => "else",
     "comments" => "failure",
     "success" => 3,
     "failure" => 0,
    ),
    Array(
     "depth" => 2,
     "object" => "s",
     "type" => "execute",
     "input" => Array(
      "value" => "|0|",
      "keys" => Array("k"),
      "regexps" => Array(".+"),
     ),
     "output" => true,
     "comments" => false,
     "success" => 0,
     "failure" => 0,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;


   if(!$s->parse("each", "
each list k v
 each list k2 v2
  s.execute |k| |k2| == true
exit true
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["each"]=Array(
    Array(
     "depth" => 0,
     "type" => "each",
     "input" => Array("list", "k", "v"),
     "comments" => false,
     "success" => 1,
     "failure" => 3,
    ),
    Array(
     "depth" => 1,
     "type" => "each",
     "input" => Array("list", "k2", "v2"),
     "comments" => false,
     "success" => 2,
     "failure" => 0,
    ),
    Array(
     "depth" => 2,
     "object" => "s",
     "type" => "execute",
     "input" => Array(
      "value" => "|0| |1|",
      "keys" => Array("k", "k2"),
      "regexps" => Array(".+", ".+"),
     ),
     "output" => true,
     "comments" => false,
     "success" => 1,
     "failure" => 1,
    ),
    Array(
     "depth" => 0,
     "type" => "exit",
     "input" => true,
     "output" => NULL,
     "comments" => false,
     "success" => -1,
     "failure" => -1,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;

   return UT_OK;
  }

  function Parse_Each_Else()
  {
   $s=new ScriptTest();

   if(!$s->parse("each", "
each list k v
 get v
  each list k2 v2
   s.execute |k| |k2| == true
 else
  each list k2 v2
   set entry 'no show'
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $commands=Array();
   $commands["each"]=Array(
    Array(
     "depth" => 0,
     "type" => "each",
     "input" => Array("list", "k", "v"),
     "comments" => false,
     "success" => 1,
     "failure" => -1,
    ),
    Array(
     "depth" => 1,
     "object" => false,
     "type" => "get",
     "input" => "v",
     "output" => NULL,
     "comments" => false,
     "success" => 2,
     "failure" => 4,
    ),
    Array(
     "depth" => 2,
     "type" => "each",
     "input" => Array("list", "k2", "v2"),
     "comments" => false,
     "success" => 3,
     "failure" => 0,
    ),
    Array(
     "depth" => 3,
     "object" => "s",
     "type" => "execute",
     "input" => Array(
      "value" => "|0| |1|",
      "keys" => Array("k", "k2"),
      "regexps" => Array(".+", ".+"),
     ),
     "output" => true,
     "comments" => false,
     "success" => 2,
     "failure" => 2,
    ),
    Array(
     "depth" => 1,
     "type" => "else",
     "comments" => false,
     "success" => 5,
     "failure" => 0,
    ),
    Array(
     "depth" => 2,
     "type" => "each",
     "input" => Array("list", "k2", "v2"),
     "comments" => false,
     "success" => 6,
     "failure" => 0,
    ),
    Array(
     "depth" => 3,
     "object" => false,
     "type" => "set",
     "input" => "entry 'no show'" ,
     "output" => NULL,
     "comments" => false,
     "success" => 5,
     "failure" => 5,
    ),
   );

   if(!$this->_compare($commands, $s->commands))
    return UT_FAILED;


   return UT_OK;
  }

  function Parse_BlackList()
  {
   $commands=Array();
   $s=new Script();

   foreach($s->blacklist as $function)
   {
    if($s->parse("blacklist", "
".$function."
exit true
 "))
    {
     $this->e->add("Parse should have failed on blacklisted: ".$function, "error");
     $this->e->add($s->e);
     return UT_FAILED;
    }

    if(!$this->_compare($commands, $s->commands))
     return UT_FAILED;
   }

   return UT_OK;
  }



  function Execute()
  {
   $s=new Script();

   if(!$s->parse("test", "
exit 2.0"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test")!=2.0)
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Exist()
  {
   $s=new Script();

   if(!$s->parse("test", "
exit 2.0"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->exist("test"))
   {
    $this->e->add("Exist shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }


   //fail
   if($s->exist("asd"))
   {
    $this->e->add("Exist should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Set()
  {
   $s=new Script();

   foreach(Array("value", 123, 0, 0.1, false, true, Array()) as $value)
   {
    if(!$s->set("key", $value))
    {
     $this->e->add("Set shouldn't have failed", "error", $value);
     $this->e->add($s->e);
     return UT_FAILED;
    }

    if(!isset($s->values["key"]))
    {
     $this->e->add("Values list doesn't have a key: key", "error", $value);
     return UT_FAILED;
    }

    if($s->values["key"]!==$value)
    {
     $this->e->add("Values list value doesn't match for key: key", "error", $value);
     return UT_FAILED;
    }
   }

   if(!$s->set("key"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(isset($s->values["key"]))
   {
    $this->e->add("Values list shouldn't have a key: key", "error");
    return UT_FAILED;
   }


   //array
   unset($s->values["key"]);
   if(!$s->set("key.0", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!isset($s->values["key"]))
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    return UT_FAILED;
   }

   if(!is_array($s->values["key"]))
   {
    $this->e->add("Values list value isn't an array for key: key", "error");
    return UT_FAILED;
   }

   if(!isset($s->values["key"][0]))
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    return UT_FAILED;
   }

   if($s->values["key"][0]!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   unset($s->values["key"]);
   if(!$s->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!isset($s->values["key"]))
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    return UT_FAILED;
   }

   if(!is_array($s->values["key"]))
   {
    $this->e->add("Values list value isn't an array for key: key", "error");
    return UT_FAILED;
   }

   if(!isset($s->values["key"]["sub"]))
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    return UT_FAILED;
   }

   if($s->values["key"]["sub"]!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }


   //object
   unset($s->values["key"]);
   if(!$s->set("key", $s))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!isset($s->values["key"]))
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    return UT_FAILED;
   }

   if(!is_object($s->values["key"]))
   {
    $this->e->add("Values list value isn't an object for key: key", "error");
    return UT_FAILED;
   }

   if(!$s->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!isset($s->values["key"]->sub))
   {
    $this->e->add("Values list doesn't have a key: key.sub", "error");
    return UT_FAILED;
   }

   if($s->values["key"]->sub!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   if(!$s->set("key.e.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!isset($s->values["key"]->e->sub))
   {
    $this->e->add("Values list doesn't have a key: key.e.sub", "error");
    return UT_FAILED;
   }

   if($s->values["key"]->e->sub!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.e.sub", "error");
    return UT_FAILED;
   }


   //readonly
   unset($s->values["key"]);
   unset($s->sub, $s->e->sub);
   if(!$s->set("key", $s, true))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!isset($s->values["key"]))
   {
    $this->e->add("Values list doesn't have a key: key", "error");
    return UT_FAILED;
   }

   if(!is_object($s->values["key"]))
   {
    $this->e->add("Values list value isn't an object for key: key", "error");
    return UT_FAILED;
   }

   if($s->set("key", "value"))
   {
    $this->e->add("Set should have failed (readonly)", "error");
    return UT_FAILED;
   }

   if(!is_object($s->values["key"]))
   {
    $this->e->add("Values list value isn't an object for key: key", "error");
    return UT_FAILED;
   }

   if($s->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed (readonly sub key)", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(isset($s->values["key"]->sub))
   {
    $this->e->add("Values list shouldn't have a key: key.sub", "error");
    return UT_FAILED;
   }

   if($s->set("key.e.sub", "value"))
   {
    $this->e->add("Set should have failed (readonly sub key)", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(isset($s->values["key"]->e->sub))
   {
    $this->e->add("Values list shouldn't have a key: key.e.sub", "error");
    return UT_FAILED;
   }


   //failures
   unset($s->values["key"]);
   $s->valuesReadonly=Array();
   if(!$s->set("key", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->set("key.sub", "value")) //try to use string as object/array
   {
    $this->e->add("Set should have failed", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Get()
  {
   $s=new Script();

   foreach(Array("value", 123, 0, 0.1, false, true, Array()) as $value)
   {
    if(!$s->set("key", $value))
    {
     $this->e->add("Set shouldn't have failed", "error", $value);
     $this->e->add($s->e);
     return UT_FAILED;
    }

    if($s->get("key", true)!==true)
    {
     $this->e->add("Values list value isn't set for key: key", "error", $value);
     return UT_FAILED;
    }

    if($s->get("key")!==$value)
    {
     $this->e->add("Values list value doesn't match for key: key", "error", $value);
     return UT_FAILED;
    }
   }


   //array
   unset($s->values["key"]);
   if(!$s->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("key", true)!==true)
   {
    $this->e->add("Values list value isn't set for key: key", "error");
    return UT_FAILED;
   }

   if($s->get("key")!=Array("sub" => "value"))
   {
    $this->e->add("Values list value doesn't match for key: key", "error");
    return UT_FAILED;
   }

   if($s->get("key.sub", true)!==true)
   {
    $this->e->add("Values list value isn't set for key: key.sub", "error");
    return UT_FAILED;
   }

   if($s->get("key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }


   //object
   unset($s->values["key"]);
   if(!$s->set("key", $s))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("key", true)!==true)
   {
    $this->e->add("Values list value isn't set for key: key", "error");
    return UT_FAILED;
   }

   if($s->get("key")!=$s)
   {
    $this->e->add("Values list value doesn't match for key: key", "error");
    return UT_FAILED;
   }

   if(!$s->set("key.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("key.sub", true)!==true)
   {
    $this->e->add("Values list value isn't set for key: key.sub", "error");
    return UT_FAILED;
   }

   if($s->get("key.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.sub", "error");
    return UT_FAILED;
   }

   if(!$s->set("key.e.sub", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("key.e.sub", true)!==true)
   {
    $this->e->add("Values list value isn't set for key: key.e.sub", "error");
    return UT_FAILED;
   }

   if($s->get("key.e.sub")!="value")
   {
    $this->e->add("Values list value doesn't match for key: key.e.sub", "error");
    return UT_FAILED;
   }

   //failures
   if($s->get("asd", true)!==false)
   {
    $this->e->add("Values list value shouldn't be set for key: asd", "error");
    return UT_FAILED;
   }

   if($s->get("asd")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: asd", "error");
    return UT_FAILED;
   }

   unset($s->values["key"]);
   if(!$s->set("key", "value"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("key.sub", true)!==false)
   {
    $this->e->add("Values list value shouldn't be set for key: key.sub", "error");
    return UT_FAILED;
   }

   if($s->get("key.sub")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: key.sub", "error");
    return UT_FAILED;
   }


   unset($s->values["key"]);
   if(!$s->set("key", Array()))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("key.sub", true)!==false)
   {
    $this->e->add("Values list value shouldn't be set for key: key.sub", "error");
    return UT_FAILED;
   }

   if($s->get("key.sub")!==NULL)
   {
    $this->e->add("Values list value shouldn't be set for key: key.sub", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Log()
  {
   $s=new Script();

   if(!$s->log("error", "Test"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->e->errorlevel[0]!=3)
   {
    $this->e->add("Log level not logged correctly", "error");
    return UT_FAILED;
   }

   if($s->e->error[0]!="Test")
   {
    $this->e->add("Log error not logged correctly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }


  function Script_Debug()
  {
   $s=new Script();

   if(!$s->parse("test", "
debug
 get output == Debug on
  exit true
exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   ob_start(); //prevent debug info from being printed
   if(!$s->execute("test"))
   {
    ob_end_clean();
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }
   ob_end_clean();

   if(!$s->debug)
   {
    $this->e->add("Execute should have enabled debug", "error");
    return UT_FAILED;
   }


   //fail
   if(!$s->parse("test", "
debug
 get output == Debug on
  exit true
exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   ob_start(); //prevent debug info from being printed
   if($s->execute("test"))
   {
    ob_end_clean();
    $this->e->add("Execute should have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }
   ob_end_clean();

   if($s->debug)
   {
    $this->e->add("Execute should have disabled debug", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Script_Exists()
  {
   $s=new Script();

   if(!$s->parse("test", "
exist test
 exit true
exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->execute("test"))
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }


   //fail
   if(!$s->parse("test", "
exist asd
 exit true
exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test"))
   {
    $this->e->add("Execute should have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Script_Set()
  {
   $s=new Script();

   if(!$s->parse("test", "
set var1 'Test 1' == true
 set var2 123
  set var3 'Test/ing'
exit"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->execute("test"))
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("var1")!="Test 1")
   {
    $this->e->add("Execute didn't set var1 properly", "error");
    return UT_FAILED;
   }

   if($s->get("var2")!==123)
   {
    $this->e->add("Execute didn't set var2 properly", "error");
    return UT_FAILED;
   }

   if($s->get("var3")!="Test/ing")
   {
    $this->e->add("Execute didn't set var3 properly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Script_Get()
  {
   $s=new Script();

   if(!$s->set("var1", 123))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->set("var2", 123.456))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->set("var3", "Test/ing"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test", "
get var1 == 123
 set 1 true
get var2 == 123.456
 set 2 true
get var3 == Test/ing
 set 3 true
get var4 == Testing
 set 4 true
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->execute("test"))
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->get("1"))
   {
    $this->e->add("Execute didn't set 1 properly", "error");
    return UT_FAILED;
   }

   if(!$s->get("2"))
   {
    $this->e->add("Execute didn't set 2 properly", "error");
    return UT_FAILED;
   }

   if(!$s->get("3"))
   {
    $this->e->add("Execute didn't set 3 properly", "error");
    return UT_FAILED;
   }

   if($s->get("4"))
   {
    $this->e->add("Execute didn't set 4 properly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Script_Log()
  {
   $s=new Script();

   if(!$s->parse("test", "
log error Test
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->execute("test"))
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->e->errorlevel[0]!=3)
   {
    $this->e->add("Log level not logged correctly", "error");
    return UT_FAILED;
   }

   if($s->e->error[0]!="Test")
   {
    $this->e->add("Log error not logged correctly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Script_Exit()
  {
   $s=new Script();

   if(!$s->parse("test", "
set var1 123.456
exit |var1|
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test")!==123.456)
   {
    $this->e->add("Execute should have return a float", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("var1")!==123.456)
   {
    $this->e->add("Execute didn't set var1 properly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Script_Array()
  {
   global $_ScriptTest;
   $_ScriptTest=Array();

   $s=new ScriptTest();

   if(!$s->set("var1", Array("first", "second", "third")))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->set("var2", Array(123, 123.456, "Testing")))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test", "
input 1 '|var1| = |var2|' == true
 set 1 true
input 1 '|var1| |var2|' == false |||setting var1 twice expected to fail
 set 2 true
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->execute("test"))
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->get("1"))
   {
    $this->e->add("Execute didn't set 1 properly", "error");
    return UT_FAILED;
   }

   if(!$s->get("2"))
   {
    $this->e->add("Execute didn't set 2 properly", "error");
    return UT_FAILED;
   }

   $values=Array(
    1 => Array(
     "first = 123",
     "second = 123.456",
     "third = Testing",
    )
   );

   if(!$this->_compare($values, $_ScriptTest))
    return UT_FAILED;

   return UT_OK;
  }

  function Script_Return()
  {
   $s=new ScriptTest();

   if(!$s->set("var1", "Testing"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test", "
get var1 == |var2|[A-z]+|
 exit true
exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->execute("test"))
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("var2")!="Testing")
   {
    $this->e->add("Execute didn't set var2 properly", "error");
    return UT_FAILED;
   }


   $s->clear();
   if(!$s->set("var1", Array("first", "second", "third")))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test", "
get var1 == |var2|first|
 exit true
exit false
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->execute("test"))
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->get("output"))
   {
    $this->e->add("Execute didn't set output properly", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Script_Object()
  {
   $s=new ScriptTest();
   $d=new Data();

   if(!$s->set("var1", $d))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test1", "
var1.getValue updated == |var2|
 exit 1
exit 2
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test1")!==1)
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("var2")!=$d->getValue("updated"))
   {
    $this->e->add("Execute didn't set var2 properly", "error");
    return UT_FAILED;
   }


   //sub object
   $s->clear();
   if(!$s->set("var1.sub", $d))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test1", "
var1.sub.getValue updated == |var2|
 exit 1
exit 2
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test1")!==1)
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->get("var2")!=$d->getValue("updated"))
   {
    $this->e->add("Execute didn't set var2 properly", "error");
    return UT_FAILED;
   }


   //incorrect arguments count
   $s->clear();
   if(!$s->set("var1", $d))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test1", "
var1.getValue == |var2|
 exit 1
exit 2
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test1")!==2)
   {
    $this->e->add("Execute should have failed", "error");
    return UT_FAILED;
   }
   $this->e->add($s->e);
   $s->e->clear();


   //incorrect function
   $s->clear();
   if(!$s->set("var1", $d))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test1", "
var1.asd == |var2|
 exit 1
exit 2
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test1")!==2)
   {
    $this->e->add("Execute should have failed", "error");
    return UT_FAILED;
   }
   $this->e->add($s->e);
   $s->e->clear();


   //incorrect object
   $s->clear();
   if(!$s->set("var1", "lalal"))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("test1", "
var1.getValue nr == |var2|
 exit 1
exit 2
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test1")!==2)
   {
    $this->e->add("Execute should have failed", "error");
    return UT_FAILED;
   }
   $this->e->add($s->e);
   $s->e->clear();


   //no object
   $s->clear();
   if(!$s->parse("test1", "
var1.getValue nr == |var2|
 exit 1
exit 2
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("test1")!==2)
   {
    $this->e->add("Execute should have failed", "error");
    return UT_FAILED;
   }
   $this->e->add($s->e);
   $s->e->clear();

   return UT_OK;
  }

  function Script_Each()
  {
   global $_ScriptTest;
   $_ScriptTest=Array();

   $s=new ScriptTest();
   $d=new Data();

   $values=Array("first", "second", "third");
   if(!$s->set("list", $values))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("set", "
each list k v
 input |k| |v|
 else
  exit 2
exit 1
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("set")!==1)
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($values, $_ScriptTest))
    return UT_FAILED;


   //double set
   $_ScriptTest=Array();
   $values=Array("first", "second", "third");
   if(!$s->set("list", $values))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("set", "
each list k v
 each list k2 v2
  input '|k| |k2|' '|v| |v2|'
exit 1
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("set")!==1)
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $tmp=Array();
   foreach($values as $k => $v)
    foreach($values as $k2 => $v2)
     $tmp[$k." ".$k2]=$v." ".$v2;

   if(!$this->_compare($tmp, $_ScriptTest))
    return UT_FAILED;


   //get
   $_ScriptTest=Array("first", "second", "third");
   if(!$s->set("list", $values))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("get", "
set list |||empty list to set each in get mode
each list v
 outputEach == |v|
exit 1
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("get")!==1)
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if((!$values=$s->get("list")))
   {
    $this->e->add("Get shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$this->_compare($values, $_ScriptTest))
    return UT_FAILED;


   //get key/values
   $_ScriptTest=Array("1 first", "2 second", "3 third");
   if(!$s->set("list", $values))
   {
    $this->e->add("Set shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if(!$s->parse("get", "
set list |||empty list to set each in get mode
each list k v
 outputEach == |k| |v|
exit 1
"))
   {
    $this->e->add("Parse shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if($s->execute("get")!==1)
   {
    $this->e->add("Execute shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   if((!$values=$s->get("list")))
   {
    $this->e->add("Get shouldn't have failed", "error");
    $this->e->add($s->e);
    return UT_FAILED;
   }

   $_ScriptTest=Array(
    1 => "first",
    2 => "second",
    3 => "third",
   );

   if(!$this->_compare($values, $_ScriptTest))
    return UT_FAILED;


   return UT_OK;
  }
 }

?>
