<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/storage.api.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/storage.php");
 require_once($_path["include"]."storage/api.php");
 require_once($_path["include"]."data.php");

 class UT_Storage_API extends UT_Storage
 {
  function Constructor()
  {
   $d=new Data();
   $object=new System_Storage_api($d);

   if(!is_object($object)||get_class($object)!="System_Storage_api")
   {
    $this->e->add("Constructor doesn't return a System_Storage_api object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $d=Data::cache(false, false, false, Array("db" => "api://jas.junerules.com"));
   $orig=new System_Storage_api($d);
   $orig->http->disconnect();
   $orig->http->e->clear();
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("System_Storage_api object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->type="different";
   if($orig->type==$copy->type)
   {
    $this->e->add("System_Storage_api object copy also modified original \"type\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("System_Storage_api object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("System_Storage_api object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   $orig->l->sets=Array("test");
   if($orig->l->sets!=$orig->e->l->sets)
   {
    $this->e->add("System_Storage_api object original didn't modify original \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test2");
   if($copy->l->sets!=$copy->e->l->sets)
   {
    $this->e->add("System_Storage_api object copy didn't modify copy \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->data->format=Array();
   if($orig->data->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_api object copy didn't modify original \"Data\" object", "error");
    return UT_FAILED;
   }

   if($d->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_api object copy didn't modify referenced \"Data\" object", "error");
    return UT_FAILED;
   }

   $session=$copy->session; //store session id
   $copy->session="";
   if($orig->session!=$copy->session)
   {
    $this->e->add("System_Storage_api object copy didn't modify original \"session\" reference", "error");
    $copy->session=$session; //restore session id
    return UT_FAILED;
   }
   $copy->session=$session; //restore session id

   $copy->api->type="different";
   if($orig->api->type==$copy->api->type)
   {
    $this->e->add("System_Storage_api object copy also modified original \"API\" object", "error");
    return UT_FAILED;
   }

   $copy->http->linebreak=-1;
   if($orig->http->linebreak==$copy->http->linebreak)
   {
    $this->e->add("System_Storage_api object copy also modified original \"HTTP\" object", "error");
    return UT_FAILED;
   }

   return UT_Storage::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function _init()
  {
   global $_path;

//   $this->_db="api://".$_SERVER["HTTP_HOST"]."/".$_path["HTML"]["root"]."/#secure";
   $this->_db="api://".$_SERVER["HTTP_HOST"]."/".$_path["HTML"]["root"]."/";

   return true;
  }

  function TableExists()
  {
   global $_config;

   if(!$_config->get("api")||!$_config->get("api.modules"))
    return UT_SKIPPED;

   if(!$this->_init())
    return UT_FAILED;

   $enabled=false;
   foreach(Array("data", "xml", "csv") as $type)
   {
    $d=new Data("categories", "view", "all", Array("skipCheckTables" => true, "db" => $this->_db."?type=".$type));
    $s=new System_Storage_api($d);

    if($s->table())
    {
     $enabled=$type;
     break;
    }
    else
    {
     if(strstr($s->e->getError(0), "API is disabled"))
      continue;

     $this->e->add($s->e);
     return UT_FAILED;
    }
   }

   if(!$enabled)
   {
    $this->e->add("No API enabled");
    return UT_SKIPPED;
   }

   return UT_OK;
  }

 }

?>
