<?

 /*****************************************************************************
 * Filename: include/plugins/unit_testing/ut/storage.sql.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["plugins"]."unit_testing/ut/storage.php");
 require_once($_path["include"]."storage/sql.php");
 require_once($_path["include"]."data.php");

 class UT_Storage_SQL extends UT_Storage
 {
  function Constructor()
  {
   $d=new Data();
   $object=new System_Storage_sql($d, false);

   if(!is_object($object)||get_class($object)!="System_Storage_sql")
   {
    $this->e->add("Constructor doesn't return a System_Storage_sql object", "error");
    return UT_FAILED;
   }

   if(!is_object($object->e)||get_class($object->e)!="Errors")
   {
    $this->e->add("Object: \"e\" isn't an Errors object", "error");
    return UT_FAILED;
   }

   if($object->dbcon!==false)
   {
    $this->e->add("Variable: \"dbcon\" isn't false", "error");
    return UT_FAILED;
   }

   if($object->engine!==false)
   {
    $this->e->add("Variable: \"engine\" isn't false", "error");
    return UT_FAILED;
   }
   if($object->collation!==false)
   {
    $this->e->add("Variable: \"collation\" isn't false", "error");
    return UT_FAILED;
   }
   if($object->charset!==false)
   {
    $this->e->add("Variable: \"charset\" isn't false", "error");
    return UT_FAILED;
   }
   if($object->enum_type!==false)
   {
    $this->e->add("Variable: \"enum_type\" isn't false", "error");
    return UT_FAILED;
   }

   if($object->qv!=="\"")
   {
    $this->e->add("Variable: \"value quotes\" isn't a \"", "error");
    return UT_FAILED;
   }
   if($object->qt!=="`")
   {
    $this->e->add("Variable: \"table quotes\" isn't a `", "error");
    return UT_FAILED;
   }
   if($object->qf!=="`")
   {
    $this->e->add("Variable: \"field quotes\" isn't a `", "error");
    return UT_FAILED;
   }
   if($object->qi!=="`")
   {
    $this->e->add("Variable: \"index quotes\" isn't a \"", "error");
    return UT_FAILED;
   }

   if(!is_array($object->crossConvert))
   {
    $this->e->add("Variable: \"crossConvert\" isn't an array", "error");
    return UT_FAILED;
   }
   if(!isset($object->crossConvert["int"]))
   {
    $this->e->add("Variable: \"crossConvert\" doesn't have an entry: int", "error");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function Copy_Constructor()
  {
   $d=Data::cache();
   $orig=new System_Storage_sql($d, "sql");
   $copy=clone $orig;

   if(!$this->_compare($orig, $copy))
   {
    $this->e->add("System_Storage_sql object copy doesn't match original", "error");
    return UT_FAILED;
   }

   $copy->type="different";
   if($orig->type==$copy->type)
   {
    $this->e->add("System_Storage_sql object copy also modified original \"type\" variable", "error");
    return UT_FAILED;
   }

   $copy->e->errors=-1;
   if($orig->e->errors==$copy->e->errors)
   {
    $this->e->add("System_Storage_sql object copy also modified original \"Error\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test");
   if($orig->l->sets==$copy->l->sets)
   {
    $this->e->add("System_Storage_sql object copy also modified original \"Language\" object", "error");
    return UT_FAILED;
   }

   $orig->l->sets=Array("test");
   if($orig->l->sets!=$orig->e->l->sets)
   {
    $this->e->add("System_Storage_sql object original didn't modify original \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->l->sets=Array("test2");
   if($copy->l->sets!=$copy->e->l->sets)
   {
    $this->e->add("System_Storage_sql object copy didn't modify copy \"Language\" object", "error");
    return UT_FAILED;
   }

   $copy->data->format=Array();
   if($orig->data->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_sql object copy didn't modify original \"Data\" object", "error");
    return UT_FAILED;
   }

   if($d->format!=$copy->data->format)
   {
    $this->e->add("System_Storage_sql object copy didn't modify referenced \"Data\" object", "error");
    return UT_FAILED;
   }

   $copy->dbcon="test";
   if($orig->dbcon==$copy->dbcon)
   {
    $this->e->add("System_Storage_sql object copy also modified original \"SQL link\" object", "error");
    return UT_FAILED;
   }

   return UT_Storage::Copy_Constructor(); //run parent Copy_Constructor tests
  }


  function GetJoin()
  {
   global $_config;

   $d=new Data("unit_testing_pluginConfigs", "view", "all");
   $s=new System_Storage_sql($d, "sql");

   //   SELECT nr
   //     FROM unit_testing_pluginConfigs
   //    WHERE (true)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(preg_match("/JOIN .?pluginConfigs.? AS .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Query shouldn't be joined to \"pluginConfigs\"");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr, plugin,
   //     FROM unit_testing_pluginConfigs
   //LEFT JOIN pluginConfigs on nr=nr
   //    WHERE (true)
   $d->setShowFields("plugin");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?.*.?pluginConfigs.?\..?plugin.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" and \"plugin\" fields in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/JOIN .?pluginConfigs.? AS .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"pluginsConfigs\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr, display,
   //     FROM unit_testing_pluginConfigs
   //    WHERE (true)
   $d->setShowFields("display");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?.*.?pluginConfigs.?\..?display.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" and \"display\" fields in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(preg_match("/JOIN .?pluginConfigs.? AS .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Query shouldn't be joined to \"pluginConfigs\"");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr
   //     FROM unit_testing_pluginConfigs
   //LEFT JOIN pluginConfigs on nr=nr
   //    WHERE (plugin="unit_testing")
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("plugin", "unit_testing");
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/JOIN .?pluginConfigs.? AS .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"pluginsConfigs\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?plugin.?=.?unit_testing.?/", $s->sql))
   {
    $this->e->add("Can't find \"plugin=unit_testing\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr
   //     FROM unit_testing_pluginConfigs
   //    WHERE (display="list_units")
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("display", "list_units");
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(preg_match("/JOIN .?pluginConfigs.? AS .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Query shouldn't be joined to \"pluginConfigs\"");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?display.?=.?list_units.?/", $s->sql))
   {
    $this->e->add("Can't find \"display=list_units\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM unit_testing_pluginConfigs
   //LEFT JOIN pluginConfigs on nr=nr
   //    WHERE (true)
   //    ORDER BY plugin DESC
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder("plugin-");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/JOIN .?pluginConfigs.? AS .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"pluginsConfigs\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?plugin.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"plugin DESC\" in ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM unit_testing_pluginConfigs
   //    WHERE (true)
   //    ORDER BY display DESC
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder("display-");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(preg_match("/JOIN .?pluginConfigs.? AS .?pluginConfigs.? ON .?unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Query shouldn't be joined to \"pluginConfigs\"");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?display.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"display DESC\" in ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT COUNT(*),
   //     FROM unit_testing_pluginConfigs
   //    WHERE (true)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder("display-");

   $s->e->clear();

   $_config->su(true);
   $s->get("count"); //count
   $_config->su();

   if(!preg_match("/SELECT COUNT\(\*\) AS .?show_count.?/", $s->sql))
   {
    $this->e->add("Unable to find \"count\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?unit_testing_pluginConfigs.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   return UT_OK;
  }


  function GetItem()
  {
   global $_config;

   $d=new Data("categories", "view", "item");
   $s=new System_Storage_sql($d, "sql");

   //   SELECT nr,
   //     FROM categories
   //    WHERE (false)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?false.?/", $s->sql))
   {
    $this->e->add("Can't find \"false\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM categories
   //    WHERE (nr=1)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("nr", 1);
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?nr.?=.?1.?/", $s->sql))
   {
    $this->e->add("Can't find \"nr=1\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM categories
   //    WHERE (nr=1 OR nr=2)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("nr", 1);
   $d->setValue("nr", 2, 1);
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?nr.?=.?1.?.*OR.*\.?nr.?=.?2.?/", $s->sql))
   {
    $this->e->add("Can't find \"nr=1 OR nr=2\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function GetList()
  {
   global $_config;

   $d=new Data("categories", "view", "list");
   $s=new System_Storage_sql($d, "sql");

   //   SELECT nr,
   //     FROM categories
   //    WHERE (false)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?false.?/", $s->sql))
   {
    $this->e->add("Can't find \"false\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM categories
   //    WHERE (catnr=1)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("catnr", 1);
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?catnr.?=.?1.?/", $s->sql))
   {
    $this->e->add("Can't find \"catnr=1\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr,
   //     FROM categories
   //    WHERE (catnr=1 OR catnr=2)
   $d->setShowFields("nr");
   $d->clearValues();
   $d->setValue("catnr", 1);
   $d->setValue("catnr", 2, 1);
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/SELECT.*.?categories.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Unable to find \"nr\" field in SELECT clause");
    return UT_FAILED;
   }

   if(!preg_match("/FROM .?categories.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .*\.?catnr.?=.?1.?.*OR.*\.?catnr.?=.?2.?/", $s->sql))
   {
    $this->e->add("Can't find \"catnr=1 OR catnr=2\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }


   return UT_OK;
  }

  function GetAll()
  {
   global $_config;

   $values=Array(
    "nr" => 1,
    "added" => "2016-01-02 12:34:56",
    "updated" => "2016-03-04 13:45:01",
    "text" => "some text",
   );

   $operators=Array(
    "nr" => "!=",
    "added" => ">=",
    "updated" => "in",
    "text" => "like",
   );


   $d=new Data("empty", "view", "all", Array("format" => "sql"));
   $d->defaultFields();

   $d->addField("text", "text", "sql;show", "", 0, 255, "", "Added");

   $s=new System_Storage_sql($d, "sql");

   //   SELECT nr, added, updated, text
   //     FROM empty
   //    WHERE (false)
   $d->setShowFields(array_keys($values));
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   foreach($values as $k => $v)
   {
    if(!preg_match("/SELECT.*.?empty.?\..?".$k.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in SELECT clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/FROM .?empty.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }

   //   SELECT nr, added, updated, text
   //     FROM empty
   //    WHERE nr = 1
   //      AND added = "2016-01-02 12:34:56"
   //      AND updated = "2016-03-04 13:45:01"
   //      AND text = "some text"
   //    ORDER BY nr ASC
   $d->setShowFields(array_keys($values));
   $d->clearValues();

   foreach($values as $k => $v)
    $d->setValue($k, $v);

   $d->sortOrder("nr");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   foreach($values as $k => $v)
   {
    if(!preg_match("/SELECT.*.?empty.?\..?".$k.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in SELECT clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/FROM .?empty.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   foreach($values as $k => $v)
   {
    if(!preg_match("/WHERE.*.?empty.?\..?".$k.".?=.?".$v.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in SELECT clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/ORDER BY .*.?nr.? ASC/", $s->sql))
   {
    $this->e->add("Can't find \"nr ASC\" in ORDER BY clause");
    return UT_FAILED;
   }

   $d->field["updated"]["type"]="array";
   $values["updated"]=Array("2016-03-04 13:45:01", "2016-03-04 13:45:02");
   //   SELECT nr, added, updated, text
   //     FROM empty
   //    WHERE nr != 1
   //      AND added >= "2016-01-02 12:34:56"
   //      AND updated IN ("2016-03-04 13:45:01", "2016-03-04 13:45:02")
   //      AND text LIKE "%some text%"
   //    ORDER BY text ASC
   $d->setShowFields(array_keys($values));
   $d->clearValues();

   foreach($values as $k => $v)
    $d->setValue($k, $v);

   foreach($operators as $k => $v)
    $d->setOperator($k, $v);

   $d->sortOrder("text+", "updated-");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   foreach($values as $k => $v)
   {
    if(!preg_match("/SELECT.*.?empty.?\..?".$k.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in SELECT clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/FROM .?empty.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   $operators["nr"]="<>";
   $operators["updated"]=" IN ";
   $operators["text"]=" LIKE ";
   $values["text"]="%".$values["text"]."%";
   $values["updated"]="(\"".implode("\", \"", $values["updated"])."\")";
   foreach($values as $k => $v)
   {
    if(!preg_match("/WHERE.*.?empty.?\..?".$k.".?".$operators[$k].".?".$v.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in WHERE clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/ORDER BY .*.?text.? ASC, .*.?updated.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"text ASC, updated DESC\" in ORDER BY clause");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetAllReferences()
  {
   global $_config;

   $d=new Data("notes", "view", "all", Array("format" => "sql"));

   $d->addReference("one2many", "files", "catnr", "1:*", "nr");
   $d->addReference("many2many", "images", "nr", "*:*", "nr");

   $s=new System_Storage_sql($d, "sql");

   //   SELECT nr, catnr
   //     FROM notes
   //    WHERE (false)
   $d->setShowFields("catnr", "one2many", "many2many");
   $d->clearValues();
   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   foreach(Array("nr", "catnr") as $k)
   {
    if(!preg_match("/SELECT.*.?notes.?\..?".$k.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in SELECT clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/FROM .?notes.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE .?true.?/", $s->sql))
   {
    $this->e->add("Can't find \"true\" in WHERE clause");
    return UT_FAILED;
   }

   if(preg_match("/ORDER BY/", $s->sql))
   {
    $this->e->add("Query shouldn't contain ORDER BY clause");
    return UT_FAILED;
   }

   //   SELECT nr, catnr
   //     FROM notes
   //    WHERE catnr = 1
   //      AND one2many.nr = 2
   //      AND many2many.files = 3
   //    ORDER BY catnr.name DESC
   $d->setShowFields("catnr", "one2many", "many2many");
   $d->clearValues();

   $d->setValue("catnr", 1);
   $d->setValue("one2many", 2);
   $d->setValue("many2many", 3);

   $d->sortOrder("catnr");
   $d->getReferenceData("catnr")->sortOrder("name-");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   foreach(Array("nr", "catnr") as $k)
   {
    if(!preg_match("/SELECT.*.?notes.?\..?".$k.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in SELECT clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/FROM .?notes.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?categories.? AS .?catnr.? ON .?notes.?\..?catnr.?=.?catnr.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"catnr\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?files.? AS .?one2many.? ON .?notes.?\..?nr.?=.?one2many.?\..?catnr.?/", $s->sql))
   {
    $this->e->add("Can't find \"catnr\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?many2many.? AS .?many2many.? ON .?notes.?\..?nr.?=.?many2many.?\..?notes.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?images.? AS .?many2many\.images.? ON .?many2many.?\..?images.?=.?many2many\.images.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE.*.?notes.?\..?catnr.?=.?1.?/", $s->sql))
   {
    $this->e->add("Unable to find \"catnr\" field in WHERE clause");
    return UT_FAILED;
   }
   if(!preg_match("/AND.*.?one2many.?\..?nr.?=.?2.?/", $s->sql))
   {
    $this->e->add("Unable to find \"one2many\" field in WHERE clause");
    return UT_FAILED;
   }
   if(!preg_match("/AND.*.?many2many.?\..?images.?=.?3.?/", $s->sql))
   {
    $this->e->add("Unable to find \"many2many\" field in WHERE clause");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?catnr.?\..?name.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"catnr.name DESC\" in ORDER BY clause");
    return UT_FAILED;
   }

   //   SELECT nr, catnr
   //     FROM notes
   //    WHERE catnr IN (1, 2)
   //      AND one2many.nr NOT IN (2, 3, 4)
   //      AND many2many.images IN (3, 4, 5)
   //    ORDER BY notes.text DESC, catnr.name ASC
   $d->setShowFields("catnr", "one2many", "many2many");
   $d->clearValues();

   $d->field["catnr"]["type"]="array";
   $d->field["one2many"]["type"]="array";
   $d->field["many2many"]["type"]="array";

   $d->setValue("catnr", Array(1, 2));
   $d->setValue("one2many", Array(2, 3, 4));
   $d->setValue("many2many", Array(3, 4, 5));

   $d->setOperator("one2many", "not in");

   $d->sortOrder("text-", "catnr");
   $d->getReferenceData("catnr")->sortOrder("name");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   foreach(Array("nr", "catnr") as $k)
   {
    if(!preg_match("/SELECT.*.?notes.?\..?".$k.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in SELECT clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/FROM .?notes.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?categories.? AS .?catnr.? ON .?notes.?\..?catnr.?=.?catnr.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"catnr\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?files.? AS .?one2many.? ON .?notes.?\..?nr.?=.?one2many.?\..?catnr.?/", $s->sql))
   {
    $this->e->add("Can't find \"catnr\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?many2many.? AS .?many2many.? ON .?notes.?\..?nr.?=.?many2many.?\..?notes.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?images.? AS .?many2many\.images.? ON .?many2many.?\..?images.?=.?many2many\.images.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE.*.?notes.?\..?catnr.? IN \(.?1.?, .?2.?\)/", $s->sql))
   {
    $this->e->add("Unable to find \"catnr\" field in WHERE clause");
    return UT_FAILED;
   }
   if(!preg_match("/AND.*.?one2many.?\..?nr.? NOT IN \(.?2.?, .?3.?, .?4.?\)/", $s->sql))
   {
    $this->e->add("Unable to find \"one2many\" field in WHERE clause");
    return UT_FAILED;
   }
   if(!preg_match("/AND.*.?many2many.?\..?images.? IN \(.?3.?, .?4.?, .?5.?\)/", $s->sql))
   {
    $this->e->add("Unable to find \"many2many\" field in WHERE clause");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?notes.?\..?text.? DESC, .*.?catnr.?\..?name.? ASC/", $s->sql))
   {
    $this->e->add("Can't find \"catnr.name DESC\" in ORDER BY clause");
    return UT_FAILED;
   }


//extended module references
   $d=new Data("notes", "view", "all", Array("format" => "sql"));

   unset($d->reference["catnr"]);
   $d->addReference("many2one", "unit_testing_pluginConfigs", "nr", "*:1", "catnr");
   $d->addReference("one2many", "unit_testing_pluginConfigs", "catnr", "1:*", "nr");
   $d->addReference("many2many", "unit_testing_pluginConfigs", "nr", "*:*", "nr");

   $s=new System_Storage_sql($d, "sql");

   //   SELECT nr
   //     FROM notes
   //    ORDER BY many2one.display ASC
   $d->setShowFields();
   $d->clearValues();

   $d->setOperator("one2many", "not in");

   $d->sortOrder("many2one");
   $d->getReferenceData("many2one")->sortOrder("display");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/FROM .?notes.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?many2one.? ON .?notes.?\..?catnr.?=.?many2one.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?many2one.?\..?display.? ASC/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.display ASC\" in ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr
   //     FROM notes
   //    ORDER BY many2one.key DESC one2many.display DESC
   $d->setShowFields();
   $d->clearValues();

   $d->setOperator("one2many", "not in");

   $d->sortOrder("many2one", "one2many");
   $d->getReferenceData("many2one")->sortOrder("key-");
   $d->getReferenceData("one2many")->sortOrder("display-");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/FROM .?notes.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?many2one.? ON .?notes.?\..?catnr.?=.?many2one.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?pluginConfigs.? AS .?many2one\.pluginConfigs.? ON .?many2one.?\..?nr.?=.?many2one\.pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?one2many.? ON .?notes.?\..?nr.?=.?one2many.?\..?catnr.?/", $s->sql))
   {
    $this->e->add("Can't find \"one2many\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?many2one\.pluginConfigs.?\..?key.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs.key DESC\" in ORDER BY clause");
    return UT_FAILED;
   }
   if(!preg_match("/ORDER BY .*.?one2many.?\..?display.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"one2many.display DESC\" in ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr
   //     FROM notes
   //    ORDER BY many2one.plugin DESC, one2many.key ASC, many2many.display ASC
   $d->setShowFields();
   $d->clearValues();

   $d->setOperator("one2many", "not in");

   $d->sortOrder("many2one", "one2many", "many2many");
   $d->getReferenceData("many2one")->sortOrder("plugin-");
   $d->getReferenceData("one2many")->sortOrder("key");
   $d->getReferenceData("many2many")->getReferenceData("unit_testing_pluginConfigs")->sortOrder("display");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/FROM .?notes.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?many2one.? ON .?notes.?\..?catnr.?=.?many2one.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?pluginConfigs.? AS .?many2one\.pluginConfigs.? ON .?many2one.?\..?nr.?=.?many2one\.pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?one2many.? ON .?notes.?\..?nr.?=.?one2many.?\..?catnr.?/", $s->sql))
   {
    $this->e->add("Can't find \"one2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?pluginConfigs.? AS .?one2many\.pluginConfigs.? ON .?one2many.?\..?nr.?=.?one2many\.pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?many2many.? AS .?many2many.? ON .?notes.?\..?nr.?=.?many2many.?\..?notes.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs.? ON .?many2many.?\..?unit_testing_pluginConfigs.?=.?many2many\.unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?many2one\.pluginConfigs.?\..?plugin.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs.plugin DESC\" in ORDER BY clause");
    return UT_FAILED;
   }
   if(!preg_match("/ORDER BY .*.?one2many\.pluginConfigs.?\..?key.? ASC/", $s->sql))
   {
    $this->e->add("Can't find \"one2many.pluginConfigs.key ASC\" in ORDER BY clause");
    return UT_FAILED;
   }
   if(!preg_match("/ORDER BY .*.?many2many\.unit_testing_pluginConfigs.?\..?display.? ASC/", $s->sql))
   {
    $this->e->add("Can't find \"many2many.unit_testing_pluginConfigs.display ASC\" in ORDER BY clause");
    return UT_FAILED;
   }


   //   SELECT nr
   //     FROM notes
   //    ORDER BY many2one.plugin DESC, one2many.key ASC, many2many.key DESC
   $d->setShowFields();
   $d->clearValues();

   $d->setOperator("one2many", "not in");

   $d->sortOrder("many2one", "one2many", "many2many");
   $d->getReferenceData("many2one")->sortOrder("plugin-");
   $d->getReferenceData("one2many")->sortOrder("key");
   $d->getReferenceData("many2many")->getReferenceData("unit_testing_pluginConfigs")->sortOrder("key-");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   if(!preg_match("/FROM .?notes.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?many2one.? ON .?notes.?\..?catnr.?=.?many2one.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?pluginConfigs.? AS .?many2one\.pluginConfigs.? ON .?many2one.?\..?nr.?=.?many2one\.pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?one2many.? ON .?notes.?\..?nr.?=.?one2many.?\..?catnr.?/", $s->sql))
   {
    $this->e->add("Can't find \"one2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?pluginConfigs.? AS .?one2many\.pluginConfigs.? ON .?one2many.?\..?nr.?=.?one2many\.pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?many2many.? AS .?many2many.? ON .?notes.?\..?nr.?=.?many2many.?\..?notes.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs.? ON .?many2many.?\..?unit_testing_pluginConfigs.?=.?many2many\.unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?pluginConfigs.? AS .?many2many\.pluginConfigs.? ON .?many2many\.unit_testing_pluginConfigs.?\..?nr.?=.?many2many\.pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many.pluginConfigs\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/ORDER BY .*.?many2one\.pluginConfigs.?\..?plugin.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs.plugin DESC\" in ORDER BY clause");
    return UT_FAILED;
   }
   if(!preg_match("/ORDER BY .*.?one2many\.pluginConfigs.?\..?key.? ASC/", $s->sql))
   {
    $this->e->add("Can't find \"one2many.pluginConfigs.key ASC\" in ORDER BY clause");
    return UT_FAILED;
   }
   if(!preg_match("/ORDER BY .*.?many2many\.pluginConfigs.?\..?key.? DESC/", $s->sql))
   {
    $this->e->add("Can't find \"many2many.pluginConfigs.key DESC\" in ORDER BY clause");
    return UT_FAILED;
   }

   //   SELECT nr
   //     FROM notes
   //    ORDER BY many2one.plugin DESC, one2many.key ASC, many2many.key DESC
   $d->setShowFields();
   $d->clearValues();

   $d->setOperator("one2many", "not in");

   $d->sortOrder("many2one", "one2many", "many2many");
   $d->getReferenceData("many2one")->sortOrder("plugin-");
   $d->getReferenceData("one2many")->sortOrder("key");
   $d->getReferenceData("many2many")->getReferenceData("unit_testing_pluginConfigs")->sortOrder("key-");

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   //   SELECT nr
   //     FROM notes
   //    WHERE catnr = 1
   //      AND one2many.nr = 2
   //      AND many2many.files = 3
   $d->setShowFields();
   $d->clearValues();

   $d->setValue("catnr", 1);
   $d->setValue("one2many", 2);
   $d->setValue("many2many", 3);

   $d->sortOrder();

   $s->e->clear();

   $_config->su(true);
   $s->get();
   $_config->su();

   foreach(Array("nr") as $k)
   {
    if(!preg_match("/SELECT.*.?notes.?\..?".$k.".?/", $s->sql))
    {
     $this->e->add("Unable to find \"".$k."\" field in SELECT clause");
     return UT_FAILED;
    }
   }

   if(!preg_match("/FROM .?notes.?/", $s->sql))
   {
    $this->e->add("Unable to find FROM clause");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?one2many.? ON .?notes.?\..?nr.?=.?one2many.?\..?catnr.?/", $s->sql))
   {
    $this->e->add("Can't find \"one2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?pluginConfigs.? AS .?one2many\.pluginConfigs.? ON .?one2many.?\..?nr.?=.?one2many\.pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2one.pluginConfigs\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?many2many.? AS .?many2many.? ON .?notes.?\..?nr.?=.?many2many.?\..?notes.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs.? ON .?many2many.?\..?unit_testing_pluginConfigs.?=.?many2many\.unit_testing_pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many\" JOIN");
    return UT_FAILED;
   }
   if(!preg_match("/JOIN .?pluginConfigs.? AS .?many2many\.pluginConfigs.? ON .?many2many\.unit_testing_pluginConfigs.?\..?nr.?=.?many2many\.pluginConfigs.?\..?nr.?/", $s->sql))
   {
    $this->e->add("Can't find \"many2many.pluginConfigs\" JOIN");
    return UT_FAILED;
   }

   if(!preg_match("/WHERE.*.?notes.?\..?catnr.?=.?1.?/", $s->sql))
   {
    $this->e->add("Unable to find \"catnr\" field in WHERE clause");
    return UT_FAILED;
   }
   if(!preg_match("/AND.*.?one2many.?\..?nr.?=.?2.?/", $s->sql))
   {
    $this->e->add("Unable to find \"one2many\" field in WHERE clause");
    return UT_FAILED;
   }
   if(!preg_match("/AND.*.?many2many.?\..?unit_testing_pluginConfigs.?=.?3.?/", $s->sql))
   {
    $this->e->add("Unable to find \"many2many\" field in WHERE clause");
    return UT_FAILED;
   }

   return UT_OK;
  }

  function GetFilter()
  {
   global $_config;

   $d=new Data("notes", "view", "all");
   $d->addField("array", "array", "sql");
   $d->addField("search", "search", "");
   $d->e->clear();

   $s=new System_Storage_sql($d, "sql");

   $d->setValue("nr", 123);
   $d->setValue("catnr", 456);
   $d->setValue("name", "of the game");
   $d->setValue("text", "some awesome text");
   $d->setValue("array", Array("test", "ing", "1.. 2.. 3.."));
   $d->setValue("search", "searching");

   $d->setValue("nr", 1234, 1);
   $d->setValue("catnr", 4567, 1);
   $d->setValue("name", "or \"not\" ", 1);
   $d->setValue("text", "that's now even more awesome", 1);
   $d->setValue("array", Array("more", "tes", "ting"), 1);


   $filters=Array(
    "not nested" => Array(
     "text" => "nr=1",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"1\"",
    ),
    "double nested" => Array(
     "text" => "((
      nr=1
))",
     "from" => Array("notes"),
     "where" => "((`notes`.`nr`=\"1\"))",
    ),
    "not nested and" => Array(
     "text" => "nr=1
and
nr=2",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"1\" AND `notes`.`nr`=\"2\"",
    ),
    "nested and or" => Array(
     "text" => "nr=1 and(nr=2 or nr=3)",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"1\" AND (`notes`.`nr`=\"2\" OR `notes`.`nr`=\"3\")",
    ),
    "nested and or or" => Array(
     "text" => "nr=1 and (nr=2 or nr=3) or nr=4",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"1\" AND (`notes`.`nr`=\"2\" OR `notes`.`nr`=\"3\") OR `notes`.`nr`=\"4\"",
    ),
    "double nested and or or" => Array(
     "text" => "nr=1
     and(
(nr=2 or nr=3)
 or nr=4
)",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"1\" AND ((`notes`.`nr`=\"2\" OR `notes`.`nr`=\"3\") OR `notes`.`nr`=\"4\")",
    ),

    "compare value quoted" => Array(
     "text" => "text != 'testing stuff' ",
     "from" => Array("notes"),
     "where" => "`notes`.`text`<>\"testing stuff\"",
    ),
    "compare variable" => Array(
     "text" => "nr >= @catnr ",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`>=\"456\"",
    ),
    "compare reference" => Array(
     "text" => "catnr = catnr.nr",
     "from" => Array("notes", "categories"),
     "where" => "`notes`.`catnr`=`catnr`.`nr`",
    ),
    "compare reference and" => Array(
     "text" => "catnr=catnr.nr and catnr.catnr=123 ",
     "from" => Array("notes", "categories"),
     "where" => "`notes`.`catnr`=`catnr`.`nr` AND `catnr`.`catnr`=\"123\"",
    ),

    "function start like depth" => Array(
     "text" => "((text START LIKE( 123) ) )",
     "from" => Array("notes"),
     "where" => "((`notes`.`text` LIKE \"123%\"))",
    ),
    "function between variable and value" => Array(
     "text" => "nr between (@nr.0, 765)",
     "from" => Array("notes"),
     "where" => "`notes`.`nr` BETWEEN \"123\" AND \"765\"",
    ),
    "function in variable, value and reference" => Array(
     "text" => "catnr in (@nr, 231, catnr.nr)",
     "from" => Array("notes", "categories"),
     "where" => "`notes`.`catnr` IN (\"123\", \"1234\", \"231\", `catnr`.`nr`)",
    ),

    "value and function like" => Array(
     "text" => "nr=1 or text like ( 123 )",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"1\" OR `notes`.`text` LIKE \"%123%\"",
    ),
    "value and function start like depth" => Array(
     "text" => "nr=1 and (catnr =3 or (text start like ( 'asd')))",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"1\" AND (`notes`.`catnr`=\"3\" OR (`notes`.`text` LIKE \"asd%\"))",
    ),

    "reference compare value" => Array(
     "text" => "catnr.nr|1 ",
     "from" => Array("notes", "categories"),
     "where" => "`catnr`.`nr`|\"1\"",
    ),
    "reference function quoted value, variable and reference " => Array(
     "text" => "catnr.catnr.nr in ('123' , @nr.0 , catnr)",
     "from" => Array("notes",
      "categories.? AS .?catnr.? ON .?notes.?\..?catnr.?=.?catnr.?\..?nr",
      "categories.? AS .?catnr\.catnr.? ON .?catnr.?\..?catnr.?=.?catnr\.catnr.?\..?nr",
     ),
     "where" => "`catnr.catnr`.`nr` IN (\"123\", \"123\", `notes`.`catnr`)",
    ),

    "no values" => Array(
     "text" => "nr = @nr.5",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"\"",
    ),
    "array values" => Array(
     "text" => "text in (@array.0, @array.1)",
     "from" => Array("notes"),
     "where" => "`notes`.`text` IN (\"test\", \"ing\", \"1.. 2.. 3..\", \"more\", \"tes\", \"ting\")",
    ),
    "array array values" => Array(
     "text" => "text in (@array.0, @array.1)",
     "from" => Array("notes"),
     "where" => "`notes`.`text` IN (\"test\", \"ing\", \"1.. 2.. 3..\", \"more\", \"tes\", \"ting\")",
    ),

    "search" => Array(
     "text" => "text like(@search)",
     "from" => Array("notes"),
     "where" => "`notes`.`text` LIKE \"%searching%\"",
    ),

    "search fulltext" => Array(
     "text" => "fulltext like(@search)",
     "from" => Array("notes"),
     "where" => "MATCH (`notes`.`name`, `notes`.`text`) AGAINST (\"searching\")",
    ),
    "value and depth search fulltext" => Array(
     "text" => "nr=@nr and ((fulltext like(@search)))",
     "from" => Array("notes"),
     "where" => "`notes`.`nr`=\"123\" AND ((MATCH (`notes`.`name`, `notes`.`text`) AGAINST (\"searching\")))",
    ),
    "value and search fulltext reference" => Array(
     "text" => "nr=@nr and fulltext like(catnr.text)",
     "from" => Array("notes", "categories"),
     "where" => "`notes`.`nr`=\"123\" AND MATCH (`notes`.`name`, `notes`.`text`) AGAINST (`catnr`.`text`)",
    ),
   );

   foreach($filters as $title => $filter)
   {
    if(!$d->setFilter($filter["text"]))
    {
     $this->e->add("Set filter shouldn't fail for: \"".$title."\"", "error");
     $this->e->add($d->e);
     return UT_FAILED;
    }

    $s->e->clear();

    $_config->su(true);
    $s->get();
    $_config->su();

    foreach($filter["from"] as $from)
    {
     if(!preg_match("/(FROM|JOIN) .?".$from.".?/", $s->sql))
     {
      $this->e->add("Unable to find FROM/JOIN clause for \"".$title."\"", "error");
      return UT_FAILED;
     }
    }

    if(!strstr($s->sql, "WHERE (".$filter["where"].")"))
    {
     $this->e->add("Unable to find WHERE clause for \"".$title."\"", "error");
     return UT_FAILED;
    }
   }

//extended module references
   $d=new Data("notes", "view", "all", Array("format" => "sql"));

   unset($d->reference["catnr"]);
   $d->addReference("many2one", "unit_testing_pluginConfigs", "nr", "*:1", "catnr");
   $d->addReference("one2many", "unit_testing_pluginConfigs", "catnr", "1:*", "nr");
   $d->addReference("many2many", "unit_testing_pluginConfigs", "nr", "*:*", "nr");
   $d->e->clear();

   $filters=Array(
    "references nr fields" => Array(
     "text" => "many2one.nr=1 or (one2many.nr=2 and many2many.unit_testing_pluginConfigs.nr=3)",
     "from" => Array(
      "notes",
      "unit_testing_pluginConfigs.? AS .?many2one", "pluginConfigs.? AS .?many2one\.pluginConfigs",
      "unit_testing_pluginConfigs.? AS .?one2many", "pluginConfigs.? AS .?one2many\.pluginConfigs",
      "many2many.? AS .?many2many", "unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs", "pluginConfigs.? AS .?many2many\.pluginConfigs",
     ),
     "where" => "`many2one.pluginConfigs`.`nr`=\"1\" OR (`one2many.pluginConfigs`.`nr`=\"2\" AND `many2many.pluginConfigs`.`nr`=\"3\")",
    ),
    "references non-extended fields" => Array(
     "text" => "many2one.display=1 or (one2many.display=2 and many2many.unit_testing_pluginConfigs.display=3)",
     "from" => Array(
      "notes",
      "unit_testing_pluginConfigs.? AS .?many2one", "pluginConfigs.? AS .?many2one\.pluginConfigs",
      "unit_testing_pluginConfigs.? AS .?one2many", "pluginConfigs.? AS .?one2many\.pluginConfigs",
      "many2many.? AS .?many2many", "unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs", "pluginConfigs.? AS .?many2many\.pluginConfigs",
     ),
     "where" => "`many2one`.`display`=\"1\" OR (`one2many`.`display`=\"2\" AND `many2many.unit_testing_pluginConfigs`.`display`=\"3\")",
    ),
    "references extend fields" => Array(
     "text" => "many2one.key=1 or (one2many.key=2 and many2many.unit_testing_pluginConfigs.key=3)",
     "from" => Array(
      "notes",
      "unit_testing_pluginConfigs.? AS .?many2one", "pluginConfigs.? AS .?many2one\.pluginConfigs",
      "unit_testing_pluginConfigs.? AS .?one2many", "pluginConfigs.? AS .?one2many\.pluginConfigs",
      "many2many.? AS .?many2many", "unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs", "pluginConfigs.? AS .?many2many\.pluginConfigs",
     ),
     "where" => "`many2one.pluginConfigs`.`key`=\"1\" OR (`one2many.pluginConfigs`.`key`=\"2\" AND `many2many.pluginConfigs`.`key`=\"3\")",
    ),

    "references extended references nr fields" => Array(
     "text" => "many2one.catnr.nr=1 or (one2many.catnr.nr=2 and many2many.unit_testing_pluginConfigs.catnr.nr=3)",
     "from" => Array(
      "notes",
      "unit_testing_pluginConfigs.? AS .?many2one", "pluginConfigs.? AS .?many2one\.pluginConfigs", "categories.? AS .?many2one\.catnr.? ON .?many2one\.pluginConfigs.?\..?catnr.?=.?many2one\.catnr.?\..?nr",
      "unit_testing_pluginConfigs.? AS .?one2many", "pluginConfigs.? AS .?one2many\.pluginConfigs", "categories.? AS .?one2many\.catnr.? ON .?one2many\.pluginConfigs.?\..?catnr.?=.?one2many\.catnr.?\..?nr",
      "many2many.? AS .?many2many", "unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs", "pluginConfigs.? AS .?many2many\.pluginConfigs", "categories.? AS .?many2many\.unit_testing_pluginConfigs\.catnr.? ON .?many2many\.pluginConfigs.?\..?catnr.?=.?many2many\.unit_testing_pluginConfigs\.catnr.?\..?nr",
     ),
     "where" => "`many2one.catnr`.`nr`=\"1\" OR (`one2many.catnr`.`nr`=\"2\" AND `many2many.unit_testing_pluginConfigs.catnr`.`nr`=\"3\")",
    ),

    "non-extend references" => Array(
     "text" => "nr=many2one.display or (nr=one2many.display and nr=many2many.unit_testing_pluginConfigs.display)",
     "from" => Array(
      "notes",
      "unit_testing_pluginConfigs.? AS .?many2one", "pluginConfigs.? AS .?many2one\.pluginConfigs",
      "unit_testing_pluginConfigs.? AS .?one2many", "pluginConfigs.? AS .?one2many\.pluginConfigs",
      "many2many.? AS .?many2many", "unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs", "pluginConfigs.? AS .?many2many\.pluginConfigs",
     ),
     "where" => "`notes`.`nr`=`many2one`.`display` OR (`notes`.`nr`=`one2many`.`display` AND `notes`.`nr`=`many2many.unit_testing_pluginConfigs`.`display`)",
    ),
    "extend references" => Array(
     "text" => "nr=many2one.key or (nr=one2many.key and nr=many2many.unit_testing_pluginConfigs.key)",
     "from" => Array(
      "notes",
      "unit_testing_pluginConfigs.? AS .?many2one", "pluginConfigs.? AS .?many2one\.pluginConfigs",
      "unit_testing_pluginConfigs.? AS .?one2many", "pluginConfigs.? AS .?one2many\.pluginConfigs",
      "many2many.? AS .?many2many", "unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs", "pluginConfigs.? AS .?many2many\.pluginConfigs",
     ),
     "where" => "`notes`.`nr`=`many2one.pluginConfigs`.`key` OR (`notes`.`nr`=`one2many.pluginConfigs`.`key` AND `notes`.`nr`=`many2many.pluginConfigs`.`key`)",
    ),

    "search fulltext non-extend references" => Array(
     "text" => "fulltext=many2one.display or (fulltext=one2many.display and fulltext=many2many.unit_testing_pluginConfigs.display)",
     "from" => Array(
      "notes",
      "unit_testing_pluginConfigs.? AS .?many2one", "pluginConfigs.? AS .?many2one\.pluginConfigs",
      "unit_testing_pluginConfigs.? AS .?one2many", "pluginConfigs.? AS .?one2many\.pluginConfigs",
      "many2many.? AS .?many2many", "unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs", "pluginConfigs.? AS .?many2many\.pluginConfigs",
     ),
     "where" => "MATCH (`notes`.`name`, `notes`.`text`) AGAINST (`many2one`.`display`) OR (MATCH (`notes`.`name`, `notes`.`text`) AGAINST (`one2many`.`display`) AND MATCH (`notes`.`name`, `notes`.`text`) AGAINST (`many2many.unit_testing_pluginConfigs`.`display`))",
    ),
    "search fulltext non-extend references" => Array(
     "text" => "fulltext=many2one.key or (fulltext=one2many.key and fulltext=many2many.unit_testing_pluginConfigs.key)",
     "from" => Array(
      "notes",
      "unit_testing_pluginConfigs.? AS .?many2one", "pluginConfigs.? AS .?many2one\.pluginConfigs",
      "unit_testing_pluginConfigs.? AS .?one2many", "pluginConfigs.? AS .?one2many\.pluginConfigs",
      "many2many.? AS .?many2many", "unit_testing_pluginConfigs.? AS .?many2many\.unit_testing_pluginConfigs", "pluginConfigs.? AS .?many2many\.pluginConfigs",
     ),
     "where" => "MATCH (`notes`.`name`, `notes`.`text`) AGAINST (`many2one.pluginConfigs`.`key`) OR (MATCH (`notes`.`name`, `notes`.`text`) AGAINST (`one2many.pluginConfigs`.`key`) AND MATCH (`notes`.`name`, `notes`.`text`) AGAINST (`many2many.pluginConfigs`.`key`))",
    ),
   );

   foreach($filters as $title => $filter)
   {
    if(!$d->setFilter($filter["text"]))
    {
     $this->e->add("Set filter shouldn't fail for: \"".$title."\"", "error");
     $this->e->add($d->e);
     return UT_FAILED;
    }

    $s->e->clear();

    $_config->su(true);
    $s->get();
    $_config->su();

    foreach($filter["from"] as $from)
    {
     if(!preg_match("/(FROM|JOIN) .?".$from.".?/", $s->sql))
     {
      $this->e->add("Unable to find FROM/JOIN clause for \"".$title."\"", "error");
      return UT_FAILED;
     }
    }

    if(!strstr($s->sql, "WHERE (".$filter["where"].")"))
    {
     $this->e->add("Unable to find WHERE clause for \"".$title."\"", "error");
     return UT_FAILED;
    }
   }

   return UT_OK;
  }


 }

?>
