<?

 /*****************************************************************************
 * Filename: include/script.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."functions.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."errors.php");

 class Script
 {
  function __construct()
  {
   $this->debug=false; //show debug information

   $this->e=new Errors();

   $this->clear();

   $this->markers=Array("|", "/", "!", "`");
   $this->marker("|");

   //default blacklist function block list
   $this->blacklist=Array(
    "__construct", "__destruct", "__clone",
    "clear", "marker",
    "convert", "values",
    "parse", "parseFlow",
   );

   //blacklist this class' constructors
   $tmp=get_class($this);
   array_unshift($this->blacklist, $tmp);
   while($tmp=get_parent_class($tmp)) //get parent's class names
    array_unshift($this->blacklist, $tmp);
  }

  function __clone()
  {
   //clone objects
   $this->e=clone $this->e;
  }

  function clear()
  {
   $this->commands=Array();
   $this->values=Array();
   $this->valuesReadonly=Array();
  }

  function marker($marker)
  {
   if(!in_array($marker, $this->markers))
   {
    $this->e->add("Script marker: \"".$marker."\" not supported", "system");
    return false;
   }

   $this->m=$marker;
   $this->mq=preg_quote($marker, "/");

   return true;
  }

  function convert($value)
  {
   //convert array values
   if(is_array($value))
   {
    foreach($value as $k => $v)
     $value[$k]=$this->convert($v);

    return $value;
   }

   //don't convert non string values
   if(!is_string($value))
    return $value;

   //convert string to int, float or boolean
   if(preg_match("/^[0-9]+$/", $value))
    return (int)$value;
   else if(preg_match("/^[0-9]+\.[0-9]+$/", $value))
    return (float)$value;
   else if($value=="false")
    return false;
   else if($value=="true")
    return true;

   return $value;
  }

  function values($command)
  {
   if($command==="")
    return Array();

   if(!is_array($command)) //normal string
    $command=Array("value" => $command);

   //get arguments
   $values=Array();
   if(preg_match_all("/('([^']+)')|([^\s]+)/", $command["value"], $m))
   {
    for($c=0;$c<sizeof($m[0]);$c++)
    {
     if($m[2][$c]) //quoted
      $values[]=$m[2][$c];
     else //unquoted
      $values[]=$m[3][$c];
    }
   }
   else
    $values=Array($command["value"]);

   //parse each argument
   foreach($values as $k => $v)
   {
    if($v=="''") //fix empty quotes
     $values[$k]=$v="";
    else if($v=="''''") //fix double quotes
     $values[$k]=$v="''";

    //replace each key with corresponding value
    if(preg_match_all("/".$this->mq."(\d+)".$this->mq."/", $v, $m))
    {
     //set normal values
     foreach($m[1] as $c)
     {
      if(isset($command["keys"][$c]))
      {
       $key=$command["keys"][$c];

       if(($value=$this->get($key))!==NULL)
       {
        if(!is_array($value)) //skip arrays for now
         $values[$k]=str_replace($this->m.$c.$this->m, $value, $values[$k]);
       }
       else
        $values[$k]=str_replace($this->m.$c.$this->m, "", $values[$k]);
      }
     }

     //set array values
     $tmp=Array();
     foreach($m[1] as $c)
     {
      if(isset($command["keys"][$c]))
      {
       $key=$command["keys"][$c];

       if(($value=$this->get($key))!==NULL&&is_array($value)) //set array values
       {
        foreach($value as $k1 => $v1)
        {
         if(isset($tmp[$k1])) //entry already exists (previous array values)
          $tmp[$k1]=str_replace($this->m.$c.$this->m, $v1, $tmp[$k1]);
         else //new entry
          $tmp[$k1]=str_replace($this->m.$c.$this->m, $v1, $values[$k]);
        }
       }
      }
     }

     if($tmp) //array entries
     {
      //clear unused keys
      foreach($tmp as $k1 => $v1)
       if(preg_match("/".$this->mq."\d+".$this->mq."/", $v1))
        $tmp[$k1]=preg_replace("/".$this->mq."\d+".$this->mq."/", "", $v1);

      $values[$k]=$tmp;
     }
    }
   }

   return $this->convert($values);
  }


  /*
   * Parser
   */

  function parse($name, $script)
  {
   if(isset($this->commands[$name]))
   {
    $this->e->add("Script parse overwriting script: \"".$name."\"", "debug");
    unset($this->commands[$name]);
   }

   if(!($linebreak=Mutilate::detectLineBreaksS($script)))
    $linebreak="\r\n";


   //parse script
   $fail=false;
   $this->commands[$name]=Array();
   foreach(explode($linebreak, $script) as $line)
   {
    if(preg_match("/^\s*$/", $line)) //skip empty lines
     continue;

    //validate line and grab depth, command, arguments and comments
    if(!preg_match("/^(\s*)(([a-z0-9_\.]+)\.)?([a-z0-9_]+)\s?(.*?)(\s*==\s*(.+?))?(\s*".$this->mq.$this->mq.$this->mq."(.+))?$/i", $line, $m))
    {
     $this->e->add("Script parse incorrect command line: \"".$line."\"", "system");
     $fail=true;
     continue;
    }

    $command=Array(
     "depth" => strlen($m[1]),
     "object" => ($m[3]?$m[3]:false),
     "type" => $m[4],
     "input" => $m[5],
     "output" => (isset($m[6])&&$m[6]?$m[7]:NULL), //== set
     "comments" => (isset($m[9])?$m[9]:false),
     "success" => -1, //on success: exit (by default)
     "failure" => -1, //on failure: exit (by default)
    );

    //parse input
    if(preg_match_all("/".$this->mq."([^".$this->mq."]+)".$this->mq."?([^".$this->mq."\s]*)".$this->mq."/", $command["input"], $m))
    {
     $command["input"]=Array("value" => $command["input"]);

     //replace key|regexp with ID
     for($c=0;$c<sizeof($m[0]);$c++)
      $command["input"]["value"]=str_replace($m[0][$c], $this->m.$c.$this->m, $command["input"]["value"]);

     //store keys
     $command["input"]["keys"]=$m[1];

     //store regexps
     $command["input"]["regexps"]=Array();
     for($c=0;$c<sizeof($m[0]);$c++)
      $command["input"]["regexps"][]=($m[2][$c]?$m[2][$c]:".+"); //no regexp, use .+
    }

    //parse output
    if(preg_match_all("/".$this->mq."([^".$this->mq."]+)".$this->mq."?([^".$this->mq."\s]*)".$this->mq."/", $command["output"], $m))
    {
     $command["output"]=Array("value" => $command["output"]);

     //replace key|regexp with ID
     for($c=0;$c<sizeof($m[0]);$c++)
      $command["output"]["value"]=str_replace($m[0][$c], $this->m.$c.$this->m, $command["output"]["value"]);

     $command["output"]["value"]=preg_quote($command["output"]["value"], "/"); //quote regexp
     $command["output"]["value"]=preg_replace("/".preg_quote($this->mq, "/")."(\d+)".preg_quote($this->mq, "/")."/", $this->m."\\1".$this->m, $command["output"]["value"]); //unquote |ID|

     //store keys
     $command["output"]["keys"]=$m[1];

     //store regexps
     $command["output"]["regexps"]=Array();
     for($c=0;$c<sizeof($m[0]);$c++)
      $command["output"]["regexps"][]=($m[2][$c]?$m[2][$c]:".+"); //no regexp, use .+
    }
    else //convert values
     $command["output"]=$this->convert($command["output"]);


    switch($command["type"])
    {
     case "each":
     {
      if($command["object"])
      {
       $this->e->add("Script parse command: \"".$command["type"]."\" requires no object", "system", $line);
       $fail=true;
       continue; //next command
      }

      if($command["output"])
      {
       $this->e->add("Script parse command: \"".$command["type"]."\" requires no output", "system", $line);
       $fail=true;
       continue; //next command
      }

      if(!($command["input"]=explode(" ", rtrim($command["input"]))))
      {
       $this->e->add("Script parse command: \"".$command["type"]."\" requires input", "system", $line);
       $fail=true;
       continue; //next command
      }

      if(sizeof($command["input"])<2)
      {
       $this->e->add("Script parse command: \"".$command["type"]."\" requires 2 or 3 arguments", "system", $line);
       $fail=true;
       continue; //next command
      }

      unset($command["output"]);
      unset($command["object"]);
      break;
     }


     case "else":
     {
      if($command["object"])
      {
       $this->e->add("Script parse command: \"".$command["type"]."\" requires no object", "system", $line);
       $fail=true;
       continue; //next command
      }

      if(trim($command["input"])||$command["output"])
      {
       $this->e->add("Script parse command: \"".$command["type"]."\" requires no argument", "system", $line);
       $fail=true;
       continue; //next command
      }

      unset($command["input"], $command["output"]);
      unset($command["object"]);
      break;
     }


     case "exit":
     {
      if($command["object"])
      {
       $this->e->add("Script parse command: \"".$command["type"]."\" requires no object", "system", $line);
       $fail=true;
       continue; //next command
      }

      if(!is_array($command["input"])) //static value
      {
       $input=rtrim($command["input"]);

       if($input!=="") //argument
        $command["input"]=$this->convert($input); //convert argument
       else //no argument
        $command["input"]=true; //success by default
      }

      unset($command["object"]);
      break;
     }


     default: //callbacks
     {
      //validate function call
      $function=$command["type"];

      if(!$command["object"]&&in_array($function, $this->blacklist)) //Script functions blacklist
      {
       $this->e->add("Script parse command: \"".$command["type"]."\" blacklisted", "system", $line);
       $fail=true;
       continue;
      }

      break;
     }
    }

    $this->commands[$name][]=$command; //store command
   }


   //check if each/else commands have subsequent (deeper) steps
   $l=sizeof($this->commands[$name]);
   for($step=0;$step<$l;$step++)
   {
    if($this->commands[$name][$step]["type"]=="each"||$this->commands[$name][$step]["type"]=="else")
    {
     if($step+1>=$l) //last step
     {
      $this->e->add("Script parse command: \"".$this->commands[$name][$step]["type"]."\" requires a next step", "system");
      $fail=true;
      continue;
     }

     if($this->commands[$name][$step+1]["depth"]<=$this->commands[$name][$step]["depth"]) //next step should be deeper
     {
      $this->e->add("Script parse command: \"".$this->commands[$name][$step]["type"]."\" requires a sub step", "system");
      $fail=true;
      continue;
     }

     //XXX: check next command isn't a else
    }

    //XXX: exit command may not have subsequent commands on same or deeper level
   }


   //failures?
   if($fail)
   {
    unset($this->commands[$name]);
    return false;
   }


   //set success/failure steps
   $steps=sizeof($this->commands[$name]);
   for($step=0;$step<$steps-1;$step++) //skip last step
   {
    if($this->commands[$name][$step+1]["type"]!="else")
    {
     $this->commands[$name][$step]["success"]=$step+1; //success is always the next step

     if($this->commands[$name][$step]["depth"]>=$this->commands[$name][$step+1]["depth"]) //next step on same or higher level
      $this->commands[$name][$step]["failure"]=$step+1;
     else //next step on deeper level
     {
      //step over success block
      $depth=$this->commands[$name][$step]["depth"];
      for($next=$step+2;$next<$steps;$next++)
      {
       if($depth>=$this->commands[$name][$next]["depth"]) //next on same or higher level
       {
        if($depth>$this->commands[$name][$next]["depth"]&&$this->commands[$name][$next]["type"]=="else") //next step higher else
        {
         $depth=$this->commands[$name][$next]["depth"]; //step over else, using its depth
         continue; //not yet done
        }

        $this->commands[$name][$step]["failure"]=$next;
        break;
       }
      }
     }
    }
    else //next step is else
    {
     if($this->commands[$name][$step]["depth"]<=$this->commands[$name][$step+1]["depth"]) //else step on same or deeper level
     {
      $this->commands[$name][$step]["failure"]=$step+1; //failure is the else step

      //step over sub block of previous step
      $depth=$this->commands[$name][$step+1]["depth"];
      for($next=$step+2;$next<$steps;$next++)
      {
       if($depth>=$this->commands[$name][$next]["depth"]) //next on same or higher level
       {
        if($depth>$this->commands[$name][$next]["depth"]&&$this->commands[$name][$next]["type"]=="else") //next step higher else
        {
         $depth=$this->commands[$name][$next]["depth"]; //step over else, using its depth
         continue; //not yet done
        }

        $this->commands[$name][$step]["success"]=$next;
        break;
       }
      }
     }
     else //else step on higher level
     {
      //step over sub block of previous step
      $depth=$this->commands[$name][$step+1]["depth"];
      for($next=$step+2;$next<$steps;$next++)
      {
       if($depth>=$this->commands[$name][$next]["depth"]) //next on same or higher level
       {
        if($depth>$this->commands[$name][$next]["depth"]&&$this->commands[$name][$next]["type"]=="else") //next step higher else
        {
         $depth=$this->commands[$name][$next]["depth"]; //step over else, using its depth
         continue; //not yet done
        }

        $this->commands[$name][$step]["success"]=$next;
        $this->commands[$name][$step]["failure"]=$next;
        break;
       }
      }
     }
    }
   }


   //point sub each block steps back to each step
   $eachs=Array();
   for($step=0;$step<$steps;$step++)
   {
    if($this->commands[$name][$step]["type"]=="each")
     $eachs[]=$step; //add to search list

    if($eachs) //in sub each block(s)
    {
     foreach($eachs as $key => $each) //check all sub each blocks
     {
      if($each==$step) //current each step
       continue;

      if($this->commands[$name][$each]["failure"]==$step||$this->commands[$name][$each]["depth"]>=$this->commands[$name][$step]["depth"]) //end of sub each block
      {
       unset($eachs[$key]); //remove current each from list
       continue; //continue search
      }

      if($this->commands[$name][$each]["failure"]==$this->commands[$name][$step]["success"]) //success steps at end of current sub each block
       $this->commands[$name][$step]["success"]=$each; //point to current each step
      if($this->commands[$name][$each]["failure"]==$this->commands[$name][$step]["failure"]) //failure steps at end of current sub each block
       $this->commands[$name][$step]["failure"]=$each; //point to current each step
     }
    }
   }

   return true;
  }


  /*
   * Execute
   */

  function execute($name)
  {
   global $_config;

   if(!isset($this->commands[$name]))
   {
    $this->e->add("Script execute failed, script: \"".$name."\" not found", "system");
    return false;
   }

   $script=$this->commands[$name];

   //execute command list
   $step=0;
   $each=Array();
   $l=sizeof($script);
   while(0<=$step&&$step<$l) //keep within execution bounds
   {
    //execute command
    $success=true;
    switch($script[$step]["type"])
    {
     case "each":
     {
      $first=false;
      if(!isset($each[$step])) //first iteration
      {
       $first=true;

       $each[$step]=Array();
       $each[$step]["values"]=$this->get($script[$step]["input"][0]);

       if($each[$step]["values"]===NULL) //no values
       {
        $each[$step]["mode"]="get"; //get values from key/value variable
        $each[$step]["values"]=Array();
       }
       else //values set
       {
        $each[$step]["mode"]="set"; //set values to key/value variable

        if(isset($script[$step]["input"][2])) //keys/values
         $each[$step]["keys"]=array_keys($each[$step]["values"]);
       }
      }

      if($each[$step]["mode"]=="get")
      {
       if(!$first) //subsequent iteration
       {
        if(isset($script[$step]["input"][2])) //key/values
        {
         $key=$this->get($script[$step]["input"][1]);
         $value=$this->get($script[$step]["input"][2]);
        }
        else
         $value=$this->get($script[$step]["input"][1]);

        if($this->debug) //debug enabled
         print str_repeat("&nbsp; ", $script[$step]["depth"])." Each set ".$script[$step]["input"][0]." from ".$script[$step]["input"][1].(isset($script[$step]["input"][2])?" => ".$script[$step]["input"][2]:"")."<br/>\n";

        //add to temporay array
        if(isset($script[$step]["input"][2])&&$key!==NULL&&$value!==NULL) //key/value
         $each[$step]["values"][$key]=$value;
        else if(!isset($script[$step]["input"][2])&&$value!==NULL) //value
         $each[$step]["values"][]=$value;
        else //no new value
        {
         $this->set($script[$step]["input"][0], $each[$step]["values"]); //set values in array

         unset($each[$step]);
         $success=false; //go to end sub each block
        }
       }
      }
      else if($each[$step]["mode"]=="set")
      {
       if($each[$step]["values"]) //values left
       {
        if($this->debug) //debug enabled
         print str_repeat("&nbsp; ", $script[$step]["depth"])." Each set ".$script[$step]["input"][0]." As ".$script[$step]["input"][1].(isset($script[$step]["input"][2])?" => ".$script[$step]["input"][2]:"")."<br/>\n";

        //set first key and remove it from the array
        if(isset($script[$step]["input"][2])) //key/values
        {
         $this->set($script[$step]["input"][1], array_shift($each[$step]["keys"]));
         $this->set($script[$step]["input"][2], array_shift($each[$step]["values"]));
        }
        else //values
         $this->set($script[$step]["input"][1], array_shift($each[$step]["values"]));
       }
       else //no more values
       {
        unset($each[$step]);
        $success=false; //go to end sub each block
       }
      }

      break;
     }


     case "else":
     {
      //else always successful

      if($this->debug) //debug enabled
       print str_repeat("&nbsp; ", $script[$step]["depth"])." Else<br/>\n";

      break;
     }


     case "exit":
     {
      if(!is_array($script[$step]["input"])) //static value
       $return=$script[$step]["input"]; //already converted
      else
       $return=$this->values($script[$step]["input"])[0];

      if($this->debug) //debug enabled
       print str_repeat("&nbsp; ", $script[$step]["depth"])." Exit ".$return."<br/>\n";

      return $return;
     }


     default: //callbacks
     {
      if(isset($script[$step]["object"])) //only call function if object is set
      {
       //prepare input
       $this->values["input"]=$this->values($script[$step]["input"]);

       //get object
       $call=Array();
       if($script[$step]["object"]) //custom object call
       {
        $object=$this->get($script[$step]["object"]);
        if(!$object)
        {
         $this->e->add("Script execute ".$script[$step]["object"].".".$script[$step]["type"].": ".$script[$step]["object"]." not set", "error");
         $success=false;
         break;
        }

        if(!is_object($object))
        {
         $this->e->add("Script execute ".$script[$step]["object"].".".$script[$step]["type"].": ".$script[$step]["object"]." isn't a object", "error");
         $success=false;
         break;
        }

        $call[]=&$object;
       }
       else //Script call
        $call[]=&$this;

       //get function
       $call[]=$script[$step]["type"]; //add function

       //validate function definition
       if(!method_exists($call[0], $call[1]))
       {
        $this->e->add("Script execute ".($script[$step]["object"]?$script[$step]["object"].".":"").$script[$step]["type"].": ".get_class($call[0])." function ".$call[1]." not found", "error");
        $success=false;
        break;
       }

       //validate number of arguments
       if(sizeof($this->values["input"])<sizeof(get_argument_names($call[1], $call[0], true))) //less arguments than required arguments
       {
        $this->e->add("Script execute ".($script[$step]["object"]?$script[$step]["object"].".":"").$script[$step]["type"].": ".get_class($call[0])."::".$call[1]." has too few arguments", "error");
        $success=false;
        break;
       }

       if($this->debug) //debug enabled
        print str_repeat("&nbsp; ", $script[$step]["depth"]).get_class($call[0])."::".$call[1]."(".substr(var_export($this->values["input"], true), 10, -3).")<br/>\n";

       //call function
       $this->values["output"]=call_user_func_array($call, $this->values["input"]);

       //convert output
       $this->values["output"]=$this->convert($this->values["output"]);

       if($this->debug) //debug enabled
        print str_repeat("&nbsp; ", $script[$step]["depth"])."Return: ".var_export($this->values["output"], true)."<br/>\n";
      }
      //else //no class, don't call function, just use previous output


      if($script[$step]["output"]===NULL) //default output checking
      {
       if(!$this->values["output"]) //failure
        $success=false;
      }
      else //custom output checking
      {
       if(is_array($this->values["output"]))
        $output=$this->values["output"];
       else //convert to array
        $output=Array($this->values["output"]);

       if(is_array($script[$step]["output"])) //regexp match
       {
        //unset previously set output values
        foreach($script[$step]["output"]["keys"] as $key)
         $this->set($key);

        //prepare regexp
        $regexp="/^".preg_quote($script[$step]["output"]["value"], "/")."$/s";
        foreach($script[$step]["output"]["regexps"] as $c => $reg)
         $regexp=str_replace($this->mq.$c.$this->mq, "(".$reg.")", $regexp);

        //match output
        $found=false;
        $values=Array();
        foreach($output as $entry) //find at least 1 matching entry
        {
         if(preg_match($regexp, $entry, $m))
         {
          $found=true;

          foreach($script[$step]["output"]["keys"] as $c => $key) //store matched values
          {
           if(!isset($values[$key]))
            $values[$key]=Array();

           $values[$key][]=$m[$c+1];
          }
         }
        }

        //store values
        if($found)
        {
         if(sizeof($output)==1) //set values as non array
         {
          foreach($values as $k => $v)
           $this->set($k, $v[0]);
         }
         else //set values array
         {
          foreach($values as $k => $v)
           $this->set($k, $v);
         }
        }
       }
       else //string match
       {
        $found=false;
        foreach($output as $entry) //find at least 1 matching entry
        {
         if($this->values["output"]===$this->convert($script[$step]["output"])) //match output (strict checking)
          $found=true;
        }
       }

       if(!$found) //no matching entries found
        $success=false;
      }

      break;
     }
    }

    if($success)
     $step=$script[$step]["success"]; //go to success step
    else
     $step=$script[$step]["failure"]; //go to failure step
   }

   if($this->debug) //debug enabled
    print "<br/>\n";

   return true;
  }


  /*
   * Default commands
   */

  function debug()
  {
   $this->debug=(!$this->debug);
   return "Debug ".($this->debug?"on":"off");
  }

  function exist($name)
  {
   if(isset($this->commands[$name]))
    return true;

   return false;
  }

  function set($key, $value=NULL, $readonly=false)
  {
   //get keys
   $keys=explode(".", $key);
   $l=sizeof($keys);

   //lookup keys
   $tmp=&$this->values;
   for($c=0;$c<$l;$c++)
   {
    $path=implode(".", array_slice($keys, 0, $c+1));
    if(in_array($path, $this->valuesReadonly))
    {
     $this->e->add("Set value failed: \"".$path."\" is read only", "error");
     return false;
    }

    if(is_array($tmp))
    {
     if(!isset($tmp[$keys[$c]]))
      $tmp[$keys[$c]]=Array();

     $tmp=&$tmp[$keys[$c]];
    }
    else if(is_object($tmp))
    {
     $tmp2=$keys[$c];

     if(!isset($tmp->$tmp2))
      $tmp->$tmp2=Array();

     $tmp=&$tmp->$tmp2;
    }
    else //non array/object
    {
     $path=implode(".", array_slice($keys, 0, $c+1));
     $this->e->add("Set value failed: \"".$path."\" isn't an array or object", "error");
     return false;
    }
   }

   $tmp=$this->convert($value);

   if($readonly) //add key to readonly list
    $this->valuesReadonly[]=$key;

   return true;
  }

  function get($key, $check=false)
  {
   //get keys
   $keys=explode(".", $key);
   $l=sizeof($keys);

   //lookup keys
   $tmp=&$this->values;
   for($c=0;$c<$l;$c++)
   {
    if(is_array($tmp))
    {
     if(!isset($tmp[$keys[$c]]))
     {
      if($check) //check if variable is set
       return false;

      $path=implode(".", array_slice($keys, 0, $c+1));
      $this->e->add("Get value failed: \"".$path."\" not set", "error");
      return NULL;
     }

     $tmp=&$tmp[$keys[$c]];
    }
    else if(is_object($tmp))
    {
     $tmp2=$keys[$c];

     if(!isset($tmp->$tmp2))
     {
      if($check) //check if variable is set
       return false;

      $path=implode(".", array_slice($keys, 0, $c+1));
      $this->e->add("Get value failed: \"".$path."\" not set", "error");
      return NULL;
     }

     $tmp=&$tmp->$tmp2;
    }
    else //non array/object
    {
     if($check) //check if variable is set
      return false;

     $path=implode(".", array_slice($keys, 0, $c+1));
     $this->e->add("Get value failed: \"".$path."\" isn't an array or object", "error");
     return NULL;
    }
   }

   if($check) //check if variable is set
    return true;

   return $tmp;
  }

  function log($level, $error)
  {
   if(!$this->e->add($error, $level))
    return false;

   return true;
  }

 }

?>
