<?

 /*****************************************************************************
 * Filename: include/storage/mysql.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/sql.php");

 class System_Storage_mysql extends System_Storage_sql
 {
  function __construct(&$d)
  {
   parent::__construct($d, "mysql");

   $this->restype=MYSQLI_ASSOC; //only get associative array indexes

   $this->collation="utf8_general_ci";
   $this->charset="UTF8";
   $this->engine="MyISAM";
   $this->enum_type=true;

   $this->connect();
  }


  function connect($conf=false)
  {
   if(!$conf)
    $conf=$this->data->db;

   if(!($db=$this->getConfig($conf)))
    return false;

   if(!($this->dbcon=$this->_getConnection($db["hash"]))) //get existing connection
   {
    //init connection
    if(!$db["host"])
    {
     $this->e->add("Database configuration: \"".$conf."\" incorrect: hostname not set", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    if(!$db["user"])
    {
     $this->e->add("Database configuration: \"".$conf."\" incorrect: username not set", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    if(!$db["pass"])
    {
     $this->e->add("Database configuration: \"".$conf."\" incorrect: password not set", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    if(!$db["name"])
    {
     $this->e->add("Database configuration: \"".$conf."\" incorrect: database name not set", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    if(!$this->dbcon=@mysqli_connect($db["host"].":".($db["port"]?$db["port"]:3306), $db["user"], $db["pass"]))
    {
     $this->e->add("Unable to connect to database!", "system", "[".mysqli_connect_errno()."] ".mysqli_connect_error(), ($this->data->module!="log")); //don't log
     return false;
    }
    else if(!@mysqli_set_charset($this->dbcon, $this->charset))
    {
     $this->e->add("Unable to set character set to ".$this->charset."!", "system", "[".mysqli_errno($this->dbcon)."] ".mysqli_error($this->dbcon), ($this->data->module!="log")); //don't log
     return false;
    }
    else if(!@mysqli_select_db($this->dbcon, $db["name"]))
    {
     $this->e->add("Unable to select database!", "system", "[".mysqli_errno($this->dbcon)."] ".mysqli_error($this->dbcon), ($this->data->module!="log")); //don't log
     return false;
    }

    $this->_setConnection($db["hash"], $this->dbcon); //store connection for later use
   }

   return true;
  }


  function clear()
  {
   if(isset($this->results)&&is_object($this->results))
    mysqli_free_result($this->results);

   return parent::clear();
  }


  function escape($text, $blob=false)
  {
   if($this->dbcon)
    return mysqli_real_escape_string($this->dbcon, $text);

   return parent::escape($text);
  }


  function exec($sql, $txt=false)
  {
   if(!parent::exec($sql, $txt))
    return false;

   if(!$this->results=mysqli_query($this->dbcon, $this->sql))
   {
    $this->e->add("SQL error: ".$this->txt, "sql", $this->sql."\n\n".mysqli_error($this->dbcon), ($this->data->module!="log")); //don't log
    return false;
   }

   return true;
  }


  function next()
  {
   if($this->type=="fake")
    return parent::next();

   $this->res=NULL;

   if($this->e->any("fatal"))
    return false;

   if(!($this->res=mysqli_fetch_array($this->results, $this->restype)))
   {
    $this->clear(); //cleanup if no more results
    return false;
   }
   else
   {
    $this->resultsCounter++;
    return true;
   }
  }


  function insertID()
  {
   if($this->type!="fake")
    return mysqli_insert_id($this->dbcon);

   return 0;
  }


  function setup($adminconf)
  {
   $this->e->clear();

   $main=clone $this;
   $admin=clone $this; //connect as admin
   if(!$admin->connect($adminconf))
   {
    if($admin->e->getExtra(0))
     $adminError="Admin login failed: ".$admin->e->getExtra(0);
    else
     $adminError=false;

    $admin=false;
   }

   if(!($db=$this->getConfig()))
    return false;

   $last=false;
   while(!$main->connect()) //keep trying to connect
   {
    $errno=preg_replace("/^.*\[(\d+)\].*$/", "\\1", $main->e->getExtra(0));

    if($last==$errno) //last attempt failed in the same way
    {
     $this->e->add("Unable to fix database setup, error code: ".$errno." persists", "error");
     break;
    }

    $last=$errno;
    switch($errno)
    {
     case "2002": //unable to connect
     {
      $this->e->add("Unable to connect, please check your hostname and port", "error");
      break 2;
     }
     case "1044": //no database access
     case "1049": //no database
     {
      if(!$admin)
      {
       $this->e->add("No access to database: \"".$db["name"]."\" for: \"".$db["user"]."@".$db["host"]."\", please provide admin credentials to fix", "error");

       if($adminError)
        $this->e->add($adminError, "warning");

       break 2;
      }

      //get hostname
      if($db["host"]=="localhost")
       $connectFrom="localhost";
      else //get current user's hostname
      {
       if(!$admin->exec("SELECT USER() as user;", "Get current user")||!$admin->next())
       {
        $this->e->add($admin->e);
        break 2;
       }

       $connectFrom=preg_replace("/^.+@/", "", $admin->res["user"]);
      }

      //check if database exists
      if(!$admin->exec("USE ".$this->qt.$db["name"].$this->qt.";", "Select database")) //database doesn't exist
      {
       $admin->e->clear();

       //create database
       if(!$admin->exec("CREATE DATABASE ".$this->qt.$db["name"].$this->qt." DEFAULT CHARACTER SET ".$this->qv.$admin->charset.$this->qv." COLLATE ".$this->qv.$admin->collation.$this->qv.";", "Create database"))
       {
        $this->e->add($admin->e);
        break 2;
       }

       $this->e->add("Database: \"".$db["name"]."\" created", "warning");
      }
      else //database exists, reselect mysql database
       $admin->exec("USE ".$this->qt."mysql".$this->qt.";", "Select database");

      //grant privileges
      if(!$admin->exec("GRANT SELECT, INSERT, UPDATE, DELETE, CREATE, ALTER, INDEX ON ".$this->qt.$db["name"].$this->qt.".* TO ".$this->qv.$admin->escape($db["user"]).$this->qv."@".$this->qv.$admin->escape($connectFrom).$this->qv.";", "Grant privileges"))
      {
       $this->e->add($admin->e);
       break 2;
      }

      $this->e->add("Granted user: \"".$db["user"]."@".$db["host"]."\" access to database: \"".$db["name"]."\"", "warning");

      //update privileges
      if(!$admin->exec("FLUSH PRIVILEGES;", "Flush privileges"))
      {
       $this->e->add($admin->e);
       break 2;
      }

      break;
     }
     case "1045": //access denied
     {
      if(!$admin)
      {
       $this->e->add("Access denied for: \"".$db["user"]."@".$db["host"]."\", please provide admin credentials to fix", "error");

       if($adminError)
        $this->e->add($adminError, "warning");

       break 2;
      }

      //get hostname
      if($db["host"]=="localhost")
       $connectFrom="localhost";
      else //get current user's hostname
      {
       if(!$admin->exec("SELECT USER() as user;", "Get current user")||!$admin->next())
       {
        $this->e->add($admin->e);
        break 2;
       }

       $connectFrom=preg_replace("/^.+@/", "", $admin->res["user"]);
      }

      //check if user exists
      if(!$admin->exec("SELECT ".$this->qf."User".$this->qf." FROM ".$this->qt."user".$this->qt." WHERE ".$this->qf."User".$this->qf."=".$this->qv.$admin->escape($db["user"]).$this->qv." AND ".$this->qf."Host".$this->qf."=".$this->qv.$admin->escape($connectFrom).$this->qv.";", "Get user"))
      {
       $this->e->add($admin->e);
       break 2;
      }

      if(!$admin->next()) //no user
      {
       //create user
       if(!$admin->exec("CREATE USER ".$this->qv.$admin->escape($db["user"]).$this->qv."@".$this->qv.$admin->escape($connectFrom).$this->qv." IDENTIFIED BY ".$this->qv.$admin->escape($db["pass"]).$this->qv.";", "Create user"))
       {
        $this->e->add($admin->e);
        break 2;
       }

       $this->e->add("Database user: \"".$db["user"]."@".$db["host"]."\" created", "warning");
      }
      else //user exists
      {
       //update password
       if(!$admin->exec("SET PASSWORD FOR ".$this->qv.$admin->escape($db["user"]).$this->qv."@".$this->qv.$admin->escape($connectFrom).$this->qv." = PASSWORD(".$this->qv.$admin->escape($db["pass"]).$this->qv.");", "Change password"))
       {
        $this->e->add($admin->e);
        break 2;
       }

       $this->e->add("Database user: \"".$db["user"]."@".$db["host"]."\" password changed", "warning");
      }

      //update privileges
      if(!$admin->exec("FLUSH PRIVILEGES;", "Flush privileges"))
      {
       $this->e->add($admin->e);
       break 2;
      }

      break;
     }
     default:
     {
      $this->e->add("Database setup error: ".$main->e->getExtra(0), "error");
      break 2;
     }
    }

    $main->e->clear(); //clear errors for next run
   }

   //close database connections
   if($admin&&$admin->dbcon)
   {
    mysqli_close($admin->dbcon);
    $admin->dbcon=false;
   }

   if($this->e->any("fatal"))
    return false;

   return true;
  }
 }

?>
