<?

 /*****************************************************************************
 * Filename: include/storage/pgsql.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."storage/sql.php");

 class System_Storage_pgsql extends System_Storage_sql
 {
  function __construct(&$d)
  {
   parent::__construct($d, "pgsql");

   $this->passwordRequired=true;
   $this->collation="en_US.utf8";
   $this->charset="UTF8";

   $this->setQuotes("'", "\"", "\"", "\"");

   $this->connect();
  }


  function connect($conf=false)
  {
   if(!$conf)
    $conf=$this->data->db;

   if(!($db=$this->getConfig($conf)))
    return false;

   if(!($this->dbcon=$this->_getConnection($db["hash"]))) //get existing connection
   {
    //init connection
    if(!$db["host"])
    {
     $this->e->add("Database configuration: \"".$conf."\" incorrect: hostname not set", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    if(!$db["user"])
    {
     $this->e->add("Database configuration: \"".$conf."\" incorrect: username not set", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    if($this->passwordRequired&&!$db["pass"])
    {
     $this->e->add("Database configuration: \"".$conf."\" incorrect: password not set", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    if(!$db["name"])
    {
     $this->e->add("Database configuration: \"".$conf."\" incorrect: database name not set", "system", false, ($this->data->module!="log")); //don't log
     return false;
    }

    $connection="host=".$db["host"];

    if($db["port"])
     $connection.=" port=".$db["port"];

    $connection.=" user=".$db["user"];

    if($db["pass"])
     $connection.=" password=".$db["pass"];

    $connection.=" dbname=".$db["name"];

    if(!$this->dbcon=@pg_connect($connection))
    {
     if(($txt=error_get_last())&&$txt["message"]&&preg_match("/^pg_connect.+:\s+(.+)$/sm", html_entity_decode($txt["message"]), $m))
      $txt=$m[1];
     else
      $txt=false;

     $this->e->add("Unable to connect to database!", "system", $txt, ($this->data->module!="log")); //don't log
     return false;
    }
    else if(pg_set_client_encoding($this->dbcon, $this->charset)!==0)
    {
     $this->e->add("Unable to set character set to ".$this->charset."!", "system", pg_last_error($this->dbcon), ($this->data->module!="log")); //don't log
     return false;
    }

    $this->_setConnection($db["hash"], $this->dbcon); //store connection for later use
   }

   return true;
  }


  function clear()
  {
   if(isset($this->results)&&is_object($this->results))
    pg_free_result($this->results);

   return parent::clear();
  }


  function escape($text, $blob=false)
  {
   if($this->dbcon)
   {
    if($blob)
     return pg_escape_bytea($this->dbcon, $text);
    else
     return pg_escape_string($this->dbcon, $text);
   }

   return parent::escape($text);
  }


  function exec($sql, $txt=false)
  {
   if(!parent::exec($sql, $txt))
    return false;

   if(!$this->results=@pg_query($this->dbcon, $this->sql))
   {
    $this->e->add("SQL error: ".$this->txt, "sql", $this->sql."\n\n".pg_last_error($this->dbcon), ($this->data->module!="log")); //don't log
    return false;
   }

   return true;
  }


  function next()
  {
   if($this->type=="fake")
    return parent::next();

   $this->res=NULL;

   if($this->e->any("fatal"))
    return false;

   if(!($this->res=pg_fetch_assoc($this->results)))
   {
    $this->clear(); //cleanup if no more results
    return false;
   }
   else
   {
    $d=&$this->data;
    foreach(array_keys($this->res) as $k) //decode blob fields
     if($d->check("type", "filedata", $k))
      $this->res[$k]=pg_unescape_bytea($this->res[$k]);

    $this->resultsCounter++;
    return true;
   }
  }


  function insertID()
  {
   if($this->type!="fake"&&$this->next())
   {
    $d=&$this->data;
    return $this->res[$d->getID()];
   }

   return 0;
  }


  function setup($adminconf)
  {
   $this->e->clear();

   $main=clone $this;
   $admin=clone $this; //connect as admin
   $admin->passwordRequired=false; //password optional for admin login
   if(!$admin->connect($adminconf))
   {
    $adminError="Admin login failed: ".$admin->e->getExtra(0);
    $admin=false;
   }

   if(!($db=$this->getConfig()))
    return false;

   $last=false;
   while(!$main->connect()) //keep trying to connect
   {
    $error=$main->e->getExtra(0);

    if($last==$error) //last attempt failed in the same way
    {
     $this->e->add("Unable to fix database setup, error: \"".$error."\" persists", "error");
     break;
    }

    $last=$error;
    if(stripos($error, "Connection refused")!==false) //unable to connect
    {
     $this->e->add("Unable to connect, please check your hostname and port", "error");
     break;
    }
    else if(preg_match("/^database.+does not exist/i", $error)) //no database access
    {
     if(!$admin)
     {
      $this->e->add("No access to database: \"".$db["name"]."\" for: \"".$db["user"]."@".$db["host"]."\", please provide admin credentials to fix", "error");

      if($adminError)
       $this->e->add($adminError, "warning");

      break;
     }

     //check if database exists
     if(!$admin->exec("SELECT ".$this->qf."datname".$this->qf." FROM ".$this->qt."pg_database".$this->qt." WHERE ".$this->qf."datname".$this->qf."=".$this->qv.$admin->escape($db["name"]).$this->qv.";", "Get database"))
     {
      $this->e->add($admin->e);
      break;
     }

     if(!$admin->next()) //database doesn't exist
     {
      //create database
      if(!$admin->exec("CREATE DATABASE ".$this->qt.$db["name"].$this->qt." ENCODING ".$this->qv.$admin->charset.$this->qv." LC_CTYPE ".$this->qv.$admin->collation.$this->qv." TEMPLATE ".$this->qt."template0".$this->qt.";", "Create database"))
      {
       $this->e->add($admin->e);
       break;
      }

      $this->e->add("Database: \"".$db["name"]."\" created", "warning");
     }

     //connect to new database
     if(!$admin->connect(preg_replace("!/[^\/]+/?$!", "/".$db["name"], $adminconf)))
     {
      $this->e->add($admin->e);
      break;
     }

     //grant privileges
     if(
      !$admin->exec("GRANT CONNECT ON DATABASE ".$this->qt.$db["name"].$this->qt." TO ".$this->qt.$admin->escape($db["user"]).$this->qt.";", "Grant database privileges")||
      !$admin->exec("GRANT CREATE, USAGE ON SCHEMA ".$this->qt."public".$this->qt." TO ".$this->qt.$admin->escape($db["user"]).$this->qt.";", "Grant schema privileges")||
      !$admin->exec("GRANT SELECT, USAGE ON ALL SEQUENCES IN SCHEMA ".$this->qt."public".$this->qt." TO ".$this->qt.$admin->escape($db["user"]).$this->qt.";", "Grant sequence privileges")||
      !$admin->exec("GRANT SELECT, INSERT, UPDATE, DELETE ON ALL TABLES IN SCHEMA ".$this->qt."public".$this->qt." TO ".$this->qt.$admin->escape($db["user"]).$this->qt.";", "Grant table privileges")||
      !$admin->exec("ALTER DEFAULT PRIVILEGES IN SCHEMA ".$this->qt."public".$this->qt." GRANT SELECT, USAGE ON SEQUENCES TO ".$this->qt.$admin->escape($db["user"]).$this->qt.";", "Default sequence privileges")||
      !$admin->exec("ALTER DEFAULT PRIVILEGES IN SCHEMA ".$this->qt."public".$this->qt." GRANT SELECT, INSERT, UPDATE, DELETE ON TABLES TO ".$this->qt.$admin->escape($db["user"]).$this->qt.";", "Default table privileges")
     )
     {
      $this->e->add($admin->e);
      break;
     }

     $this->e->add("Granted user: \"".$db["user"]."@".$db["host"]."\" access to database: \"".$db["name"]."\"", "warning");
    }
    else if(stripos($error, "password authentication")!==false||preg_match("/^role.+does not exist/i", $error)) //access denied
    {
     if(!$admin)
     {
      $this->e->add("Access denied for: \"".$db["user"]."@".$db["host"]."\", please provide admin credentials to fix", "error");

      if($adminError)
       $this->e->add($adminError, "warning");

      break;
     }

     //check if user exists
     if(!$admin->exec("SELECT ".$this->qf."usename".$this->qf." FROM ".$this->qt."pg_user".$this->qt." WHERE ".$this->qf."usename".$this->qf."=".$this->qv.$admin->escape($db["user"]).$this->qv.";", "Get user"))
     {
      $this->e->add($admin->e);
      break;
     }

     if(!$admin->next()) //no user
     {
      //create user
      if(!$admin->exec("CREATE USER ".$this->qt.$admin->escape($db["user"]).$this->qt." PASSWORD ".$this->qv.$admin->escape($db["pass"]).$this->qv.";", "Create user"))
      {
       $this->e->add($admin->e);
       break;
      }

      $this->e->add("Database user: \"".$db["user"]."@".$db["host"]."\" created", "warning");
     }
     else //user exists
     {
      //update password
      if(!$admin->exec("ALTER USER ".$this->qt.$admin->escape($db["user"]).$this->qt." PASSWORD ".$this->qv.$admin->escape($db["pass"]).$this->qv.";", "Change password"))
      {
       $this->e->add($admin->e);
       break;
      }

      $this->e->add("Database user: \"".$db["user"]."@".$db["host"]."\" password changed", "warning");
     }
    }
    else
    {
     $this->e->add("Database setup error: ".$error, "error");
     break;
    }

    $main->e->clear(); //clear errors for next run
   }

   //close database connections
   if($admin&&$admin->dbcon)
   {
    pg_close($admin->dbcon);
    $admin->dbcon=false;
   }

   if($this->e->any("fatal"))
    return false;

   return true;
  }


  function _table()
  {
   $d=&$this->data;

   $sql="SELECT ".$this->qf."tablename".$this->qf."
FROM ".$this->qt."pg_tables".$this->qt."
WHERE ".$this->qf."schemaname".$this->qf."=".$this->qv."public".$this->qv."
AND ".$this->qf."tablename".$this->qf."=".$this->qv.$d->module.$this->qv;

   return $sql;
  }

  function info()
  {
   $d=&$this->data;

   //check if table sequence exists 
   $txt="Get auto_increment sequence: ".$d->title;
   $sql="SELECT true FROM ".$this->qt."pg_class".$this->qt." WHERE ".$this->qf."relname".$this->qf."=".$this->qv.$d->module."_auto_increment_seq".$this->qv." AND ".$this->qf."relkind".$this->qf."=".$this->qv."S".$this->qv;
   
   if(!$this->exec($sql, $txt))
    return false;

   $sequence=$this->next();

   //get statistics
   $txt="Get table statistics: ".$d->title;
   $sql="SELECT VERSION() AS ".$this->qf."info_type".$this->qf;
   $sql.=", COUNT(*) AS ".$this->qf."info_rows".$this->qf;

   if($sequence)
    $sql.=", (SELECT CASE WHEN ".$this->qf."last_value".$this->qf." > 1 THEN ".$this->qf."last_value".$this->qf."+1 ELSE 1 END FROM ".$this->qt.$d->module."_auto_increment_seq".$this->qt.") AS ".$this->qf."info_insertID".$this->qf;
   else
    $sql.=", 0 AS ".$this->qf."info_insertID".$this->qf;

   if($d->check("type", "datetime", "updated"))
    $sql.=", MAX(".$this->qf."updated".$this->qf.") AS ".$this->qf."info_updated".$this->qf;
   else
    $sql.=", TIMEZONE(".$this->qv."UTC".$this->qv.", NOW()) AS ".$this->qf."info_updated".$this->qf;

   if($d->check("type", "datetime", "added"))
    $sql.=", MIN(".$this->qf."added".$this->qf.") AS ".$this->qf."info_created".$this->qf;
   else
    $sql.=", '0000-00-00 00:00:00' AS ".$this->qf."info_created".$this->qf;


   $sql.=" FROM ".$this->qt.$d->module.$this->qt;

   if(!$this->exec($sql, $txt))
    return false;

   return true;
  }

  function _iftocase($sql)
  {
   while(($c=stripos($sql, "IF("))!==false) //find all IF statements
   {
    $open=0;
    $start=$c;
    $condition=false;
    $true=false;
    $false=false;
    $end=false;

    //find start, condition, true, false and end positions
    $l=strlen($sql);
    for($c;$c<$l;$c++)
    {
     switch($sql[$c])
     {
      case "(":
      {
       if($open==0) //if opened
        $condition=$c+1;

       $open++;
       break;
      }
      case ")":
      {
       $open--;

       if($open==0) //if closed
       {
        $end=$c+1;
        break 2;
       }

       break;
      }
      case ",":
      {
       if($open==1)
       {
        if($true===false)
         $true=$c+1;
        else if($false===false)
         $false=$c+1;
       }

       break;
      }
      default: break;
     }
    }

    //create case statement
    $case="CASE WHEN ".trim(substr($sql, $condition, $true-$condition-1));
    $case.=" THEN ".trim(substr($sql, $true, $false-$true-1));
    $case.=" ELSE ".trim(substr($sql, $false, $end-$false-1));
    $case.=" END";

    //replace if statement with case
    $sql=substr_replace($sql, $case, $start, $end-$start);
   }

   return $sql;
  }

  function _get($gather)
  {
   $d=&$this->data;

   $sql=parent::_get($gather);

   if(preg_match("/ORDER BY (.+ (DESC|ASC) )/", $sql)) //order by
    $sql=str_replace("SELECT DISTINCT", "SELECT DISTINCT * FROM (SELECT", $sql).") tmp"; //wrap in DISTINCT subquery so order by on foreign columns has sufficient data

   $sql=preg_replace("/(\s+LIMIT )(\d+),\s*(\d+)/", "\\1 \\3 OFFSET \\2", $sql); //replace LIMIT start,total with LIMIT total OFFSET start
   $sql=$this->_iftocase($sql);

   return $sql;
  }


  function _add()
  {
   $d=&$this->data;

   $sql=parent::_add();

   if($id=$d->getID())
    $sql.=" RETURNING ".$this->qf.$id.$this->qf;

   return $sql;
  }


  function _create()
  {
   return $this->_createAlterMods(parent::_create());
  }
  function _alter()
  {
   $d=&$this->data;

   $sqls=parent::_alter();
   $sqls=$this->_createAlterMods($sqls);

   //fixed column changes
   $l=sizeof($sqls);
   for($c=0;$c<$l;$c++)
   {
    if(preg_match_all("/CHANGE ".$this->qf."([^".$this->qf."]+)".$this->qf." ".$this->qf."([^".$this->qf."]+)".$this->qf." ([^\s]+)( COLLATE ".$this->qt."[^".$this->qt."]+".$this->qt.")?( (NOT )?NULL)?( DEFAULT [^,]+)?/", $sqls[$c], $m))
    {
     $l2=sizeof($m[0]);
     for($x=0;$x<$l2;$x++)
     {
      //type / default value changes
      $alter="ALTER COLUMN ".$this->qf.$m[1][$x].$this->qf; //use old name, rename done after this alter query

      $alter.=" TYPE ".$m[3][$x];
      if($m[4][$x]) //collate
       $alter.=$m[4][$x];

      if($m[7][$x]) //default
      {
       $alter.=", ALTER COLUMN ".$this->qf.$m[1][$x].$this->qf; //use old name, rename done after this alter query
       $alter.=" SET".$m[7][$x];
      }

      $sqls[$c]=str_replace($m[0][$x], $alter, $sqls[$c]);

      if($m[1][$x]!=$m[2][$x]) //column renamed
       $sqls[]="ALTER TABLE ".$this->qt.$d->module.$this->qt." RENAME COLUMN ".$this->qf.$m[1][$x].$this->qf." TO ".$this->qf.$m[2][$x].$this->qf; //add a separate query
     }
    }

    if(preg_match_all("/ADD ".$this->qf."([^".$this->qf."]+)".$this->qf." ".$this->qi."(".$d->module."_[^\s]+_enum)".$this->qi." NOT NULL/", $sqls[$c], $m))
    {
     $l2=sizeof($m[0]);
     for($x=0;$x<$l2;$x++)
      $sqls[$c]=str_replace($m[0][$x], $m[0][$x]." DEFAULT ".$this->qv.$d->sqlValue($m[1][$x]).$this->qv, $sqls[$c]);
    }
   }

   return $sqls;
  }
  function _createAlterMods($sqls)
  {
   $d=&$this->data;

   $sqls=preg_replace("/\s*CHARACTER SET ".$this->qv."[^".$this->qv."]+".$this->qv."/", "", $sqls); //remove table+column character set
   $sqls=preg_replace("/\s*COLLATE ".$this->qv."[^".$this->qv."]+".$this->qv."\s*$/", "", $sqls); //remove table collate
   $sqls=preg_replace("/COLLATE ".$this->qv."([^".$this->qv."]+)".$this->qv."/", "COLLATE ".$this->qt."\\1".$this->qt, $sqls); //forge column collate quotes
   $sqls=preg_replace("/(timestamp) NOT NULL DEFAULT ".$this->qv."[^".$this->qv."]+".$this->qv."/", "\\1 NULL", $sqls); //date time NULLable
   $sqls=preg_replace("/(NOT NULL) auto_increment/", "\\1 DEFAULT nextval(".$this->qv.$this->qi.$d->module."_auto_increment_seq".$this->qi.$this->qv.")", $sqls); //auto_increment sequence
   $sqls=preg_replace("/(".$d->module."_[^\s]+_enum) NOT NULL/", $this->qi."\\1".$this->qi." NOT NULL", $sqls); //quote enum types

   return $sqls;
  }


  function _alterSequences()
  {
   $sqls=Array();

   $d=&$this->data;
   $sequence=$d->module."_auto_increment_seq";
   if(($id=$d->getID())&&!$d->extends&&!in_array($sequence, $this->getSequences())) //auto_increment sequence
   {
    $this->e->add("Creating sequence: \"".$sequence."\" for table: \"".$d->module."\"", "note");
    $sqls[]="CREATE SEQUENCE ".$this->qi.$sequence.$this->qi;
   }

   return $sqls;
  }

  function _alterFieldTypes()
  {
   $sqls=Array();

   $d=&$this->data;

   //get current enum types from DB
   if(($current=$this->getEnumTypes())===false)
    return false;

   //get defined enum types
   $enums=Array();
   foreach($d->check("use", "sql") as $field)
   {
    if($d->field[$field]["type"]=="boolean"||$d->field[$field]["type"]=="enum")
    {
     $type=$d->module."_".$field."_enum";

     if($d->field[$field]["type"]=="boolean")
      $values=Array("false", "true");
     else if(array_type($d->listOptions[$field])=="indexed")
      $values=$d->listOptions[$field];
     else
      $values=array_keys($d->listOptions[$field]);

     $enums[$type]=$values;
    }
   }

   //determine unused enum types in DB
   $unusedEnums=array_diff(array_keys($current), array_keys($enums));

   //add/edit/delete enum types in db
   foreach($enums as $type => $values)
   {
    if(isset($current[$type])) //already exists
    {
     $newOptions=array_diff($values, $current[$type]);
     $unusedOptions=array_diff($current[$type], $values);

     if(sizeof($newOptions)) //new options to be added
     {
      $this->e->add("Adding new enum option(s): \"".join("\", \"", $newOptions)."\" for field: \"".$field."\" in table: \"".$d->module."\"", "note");

      $previous=false;
      foreach($values as $v)
      {
       if(in_array($v, $newOptions))
        $sqls[]="ALTER TYPE ".$this->qi.$type.$this->qi." ADD VALUE ".$this->qv.$v.$this->qv.($previous?" AFTER ".$this->qv.$previous.$this->qv:" BEFORE ".$this->qv.reset($current[$type]).$this->qv);

       $previous=$v;
      }
     }

     if(sizeof($unusedOptions)) //old options, leave them in case they're in use
      $this->e->add("Leaving unused enum option(s): \"".join("\", \"", $unusedOptions)."\" for field: \"".$field."\" in table: \"".$d->module."\"", "note");
    }
    else //type doesn't exist yet
    {
     //check whether unused enum type exists with the same options
     $oldtype=false;
     if($unusedEnums)
     {
      foreach($unusedEnums as $unusedtype)
      {
       if(!array_diff($current[$unusedtype], $values)) //options match
       {
        $oldtype=$unusedtype;
        break;
       }
      }
     }

     if($oldtype) //rename old enum type
     {
      $this->e->add("Renaming type: \"".$oldtype."\" to: \"".$type."\" for table: \"".$d->module."\"", "note");
      $sqls[]="ALTER TYPE ".$this->qi.$oldtype.$this->qi." RENAME TO ".$this->qi.$type.$this->qi;

      //dirty hack to ensure field is renamed rather than newly created
      if(!isset($this->enumRename))
       $this->enumRename=Array();

      $this->enumRename[$oldtype]=$type;
     }
     else //create new enum type
     {
      $this->e->add("Creating type: \"".$type."\" for table: \"".$d->module."\"", "note");
      $sqls[]="CREATE TYPE ".$this->qi.$type.$this->qi." AS ENUM (".$this->qv.implode($this->qv.", ".$this->qv, $values).$this->qv.")";
     }
    }
   }

   return $sqls;
  }

  function _alterIndexes()
  {
   $d=&$this->data;

   $sqls=parent::_alterIndexes();
   $sqls=preg_replace("/ INDEX ".$this->qi."([^".$this->qi."]+)".$this->qi."/", " INDEX ".$this->qi.$d->module."_\\1".$this->qi, $sqls); //prefix index names
   return $sqls;
  }


  function getSequences()
  {
   $d=&$this->data;

   $txt="Get sequences for: ".$d->title;
   $sql="SELECT ".$this->qf."relname".$this->qf." AS ".$this->qf."name".$this->qf."
FROM ".$this->qt."pg_class".$this->qt.", ".$this->qt."pg_namespace".$this->qt."
WHERE ".$this->qt."pg_class".$this->qt.".".$this->qf."relkind".$this->qf."=".$this->qv."S".$this->qv."
AND ".$this->qt."pg_class".$this->qt.".".$this->qf."relnamespace".$this->qf."=".$this->qt."pg_namespace".$this->qt.".".$this->qf."oid".$this->qf."
AND ".$this->qt."pg_namespace".$this->qt.".".$this->qf."nspname".$this->qf."=".$this->qv."public".$this->qv;

   if(!$this->exec($sql, $txt))
    return false;

   $sequences=Array();
   while($this->next())
    $sequences[]=$this->res["name"];

   return $sequences;
  }

  function getEnumTypes()
  {
   $d=&$this->data;

   $txt="Get enum types for: ".$d->title;
   $sql="SELECT ".$this->qf."typname".$this->qf." AS ".$this->qf."name".$this->qf.", ".$this->qf."enumlabel".$this->qf." AS ".$this->qf."value".$this->qf."
FROM ".$this->qt."pg_type".$this->qt.", ".$this->qt."pg_enum".$this->qt.", ".$this->qt."pg_namespace".$this->qt."
WHERE ".$this->qt."pg_type".$this->qt.".".$this->qf."typname".$this->qf." LIKE ".$this->qv.$d->module."_%_enum".$this->qv."
AND ".$this->qt."pg_type".$this->qt.".".$this->qf."oid".$this->qf."=".$this->qt."pg_enum".$this->qt.".".$this->qf."enumtypid".$this->qf."
AND ".$this->qt."pg_type".$this->qt.".".$this->qf."typnamespace".$this->qf."=".$this->qt."pg_namespace".$this->qt.".".$this->qf."oid".$this->qf."
AND ".$this->qt."pg_namespace".$this->qt.".".$this->qf."nspname".$this->qf."=".$this->qv."public".$this->qv;

   if(!$this->exec($sql, $txt))
    return false;

   $enums=Array();
   while($this->next())
   {
    if(!isset($enums[$this->res["name"]]))
     $enums[$this->res["name"]]=Array();

    $enums[$this->res["name"]][]=$this->res["value"];
   }

   return $enums;
  }

  function getCollation()
  {
   return $this->collation; //no table collation conversion
  }

  function getFields()
  {
   $d=&$this->data;

   $txt="Get table columns of: ".$d->title;
   $sql="SELECT ".$this->qf."column_name".$this->qf." AS ".$this->qf."name".$this->qf.", ".$this->qf."udt_name".$this->qf." AS ".$this->qf."type".$this->qf.", ".$this->qf."is_nullable".$this->qf." AS ".$this->qf."null".$this->qf.", ".$this->qf."column_default".$this->qf." AS ".$this->qf."default".$this->qf.", ".$this->qf."collation_name".$this->qf." AS ".$this->qf."collation".$this->qf.", ".$this->qf."character_maximum_length".$this->qf." AS ".$this->qf."max".$this->qf."
FROM ".$this->qt."information_schema".$this->qt.".".$this->qt."columns".$this->qt."
WHERE ".$this->qf."table_schema".$this->qf."=".$this->qv."public".$this->qv."
AND ".$this->qf."table_name".$this->qf."=".$this->qv.$d->module.$this->qv;

   if(!$this->exec($sql, $txt))
    return false;

   $fields=Array();
   $sequence=$d->module."_auto_increment_seq";
   while($this->next())
   {
    $fields[$this->res["name"]]=$this->res;

    if(strpos($this->res["default"], "nextval(".$this->qv.$sequence.$this->qv)!==false||strpos($this->res["default"], "nextval(".$this->qv.$this->qi.$sequence.$this->qi.$this->qv)!==false)
     $fields[$this->res["name"]]["extra"]="auto_increment";
    else
     $fields[$this->res["name"]]["extra"]="";

    if(!$fields[$this->res["name"]]["collation"])
     $fields[$this->res["name"]]["collation"]=$this->collation;

    if($fields[$this->res["name"]]["max"])
     $fields[$this->res["name"]]["type"].="(".$fields[$this->res["name"]]["max"].")";
   }

   //dirty hack to ensure field is renamed rather than newly created
   if(isset($this->enumRename))
    foreach($fields as $k => $v) //find fields that match renamed enum type
     if(isset($this->enumRename[$v["type"]]))
      $fields[$k]["type"]=$this->enumRename[$v["type"]]; //set new name to ensure field is renamed

   return $fields;
  }

  function getIndexes()
  {
   $indexes=Array("primary" => Array(), "index" => Array(), "unique" => Array(), "fulltext" => Array());

   if(!$this->table())
    return $indexes;

   $d=&$this->data;
   $txt="Get table indexes of: ".$d->title;
   $sql="SELECT ".$this->qf."indexname".$this->qf." AS ".$this->qf."name".$this->qf.", ".$this->qf."indexdef".$this->qf." AS ".$this->qf."query".$this->qf."
FROM ".$this->qt."pg_indexes".$this->qt."
WHERE ".$this->qf."tablename".$this->qf."=".$this->qv.$d->module.$this->qv."
AND ".$this->qf."indexname".$this->qf." LIKE ".$this->qv.$d->module."_%".$this->qv;

   if(!$this->exec($sql, $txt))
    return false;

   while($this->next())
   {
    $name=str_replace($d->module."_", "", $this->res["name"]); //remove module name prefix

    if($name=="pkey")
     $key="primary";
    else if(stripos($this->res["query"], "create unique index")!==false) //unique index
     $key="unique";
    else //index
     $key="index";

    if(preg_match("/create .+\(([^\)]+)\)$/i", $this->res["query"], $m))
    {
     $columns=explode(", ", $m[1]);
     if($key=="primary")
      $indexes[$key]=$columns;
     else
      $indexes[$key][$name]=$columns;
    }
   }

   return $indexes;
  }



  function getFieldType($field)
  {
   $d=&$this->data;

   if(!($r=parent::getFieldType($field)))
    return false;

   switch(preg_replace("/\(.+\)$/", "", $r["type"]))
   {
    case "int":
    {
     $r["type"]="int4";
     break;
    }
    case "bigint":
    {
     $r["type"]="int8";
     break;
    }
    case "double":
    {
     $r["type"]="float4";
     break;
    }
    case "datetime":
    {
     $r["type"]="timestamp";
     break;
    }
    case "double":
    {
     $r["type"]="float4";
     break;
    }
    case "tinytext":
    case "text":
    case "mediumtext":
    case "longtext":
    {
     $r["type"]="text";
     break;
    }
    case "tinyblob":
    case "blob":
    case "mediumblob":
    case "longblob":
    {
     $r["type"]="bytea";
     break;
    }
    case "enum":
    {
     $r["type"]=$d->module."_".$field."_enum";
     $r["collation"]=false;
     break;
    }
    default:
     break;
   }

   return $r;
  }

 }

?>
