<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/mime.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/base.php");
 require_once($_path["project"]["include"]."backend/cms.php");
 require_once($_path["include"]."fs.php");

 class Backend_Mime extends Backend_Base
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->update=false;
  }

  function main($action=false, $extra=false, $nr=false)
  {
   global $_session, $_config, $_path;

   $_session->frontend();

   //get (and update) mime database noupdate flag
   $fs=new FS();
   $mime=$_path["include"]."mime.xml";

   if(!$fs->exists($mime))
    $this->e->add("Can't find mime-types database: \"".$mime."\"", "error");
   else if(!($content=$fs->read($mime)))
    $this->e->add("Can't read mime-types database: \"".$mime."\"", "error");
   else
   {
    $this->update=!strpos($content, "<data module=\"mimetypes\" noupdate insertID"); //get noupdate flag from mime database

    if(request("update")!==NULL&&!$_config->userRoot())
    {
     $this->e->add("Mime-type updates only permitted for root user", "warning");
     $_session->backend();
     return false;
    }

    if($this->update) //database overwritten with JAS update
    {
     if(request("update")==="0")
     {
      $content=str_replace("<data module=\"mimetypes\" insertID", "<data module=\"mimetypes\" noupdate insertID", $content);

      if(!$fs->write($mime, $content, "overwrite"))
       $this->e->add("Can't update mime-types database: \"".$mime."\"", "error");
      else
       $this->update=!$this->update;
     }
    }
    else //database NOT overwritten with JAS update
    {
     if(request("update")==="1")
     {
      $content=str_replace("<data module=\"mimetypes\" noupdate insertID", "<data module=\"mimetypes\" insertID", $content);

      if(!$fs->write($mime, $content, "overwrite"))
       $this->e->add("Can't update mime-types database: \"".$mime."\"", "error");
      else
       $this->update=!$this->update;
     }
    }
   }


   if($action)
   {
    if(!$_config->userRoot())
    {
     $this->e->add("Mime-type updates only permitted for root user", "warning");
     $_session->backend();
     return false;
    }

    //init data
    $d=Data::cache("mimetypes", $action, $extra, Array("db" => "mime"));

    if(($id=$d->getID())&&$nr!==false&&!$d->check("valuetype", "request", $id))
     $d->setValue($id, $nr);

    //init CMS
    $this->cms=new Backend_Cms($this->backend);

    if(!$this->cms->cmsMain($d))
    {
     $_session->backend();
     return false;
    }

    $this->section="cms";
   }
   else
   {
    $d=Data::cache("mimetypes", false, false, Array("db" => "mime"));

    //set inline search options
    $this->backend->search("normal");

    //search redirect
    if(request("search"))
    {
     $d->setAction("search");
     if($d->fulltextIndex) //full text index exists
      $d->setExtra("index");
     else //normal search
      $d->setExtra("full");

     $d->setShowFields();
     $d->setValue("show_total", 2);
     $d->setRequestValue("search");

     if($d->get()&&$d->vmax==1) //only 1 entry
      Headers::redirectS($this->url("mime/".$d->getValue("extension")."/"));
    }
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $d=Data::cache("log", "view", "all");
   $d=Data::cache("mimetypes", "view", "all", Array("db" => "mime"));

   $this->backend->setTitle($this->icon("mime_dim"),
    $this->link($this->language->t("Mime-types"), Array("mime/")));

   if(request("search"))
    $d->setAction("search");

   $list=$this->viewList($d, Array("get" => 0));

   if($this->backend->prettyurls)
    $list=preg_replace("!href=\"([^\"]*)cms/".$d->module."/!", "href=\"\\1mime/", $list);
   else
    $list=preg_replace("!href=\"([^\"]*)p\[\]=cms&p\[\]=".$d->module."!", "href=\"\\1p[]=mime", $list);

   $r="<div class=\"twothird\">\n";
   $r.=$list;
   $r.="</div>\n";


   //actions
   if($this->update)
    $this->menuItem("actions", $this->language->t("Auto update enabled"), $this->url("mime/", Array("update" => 0)), $this->language->t("JAS upgrade automatically overwrite mime-types database (click to change)"), "enable");
   else
    $this->menuItem("actions", $this->language->t("Auto update disabled"), $this->url("mime/", Array("update" => 1)), $this->language->t("JAS upgrade doesn't update mime-types database (click to change)"), "disable");

   $this->menuItem("actions", $this->language->t("Configuration"), $this->url("configuration/"), $this->language->t("Configuration"), "config");

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";


   $_session->backend();

   return $r;
  }


  function output_cms()
  {
   global $_session;

   $_session->frontend();

   $this->backend->setTitle($this->icon("cms_dim"),
    $this->link($this->language->t("Mime-type"), Array("mime/")),
    $this->link($this->language->t(ucfirst($this->cms->cms->data->action)), $this->url()));

   $r=$this->cms->cmsOutput();

   $_session->backend();
   return $r;
  }
 }
?>
