<?

 /*****************************************************************************
 * Filename: project/jas/include/backend/owner.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["project"]["include"]."backend/modules.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."headers.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."jas.php");

 class Backend_Owner extends Backend_Modules
 {
  function __construct(&$backend)
  {
   parent::__construct($backend);

   $this->nr=false;
  }

  function main($module=false, $nr=false)
  {
   global $_session, $_config;

   $_session->frontend();

   //check module
   if($module)
   {
    if(!in_array($module, JAS::modules())) //invalid module
     $this->e->add("Module: \"".$module."\" not found", "error");
    else //valid module
    {
     //modify quick categories list url
     if($quick=$this->backend->component("backend/categories/quick")) //categories list found
      $quick->path=false; //current page

     $d=Data::cache($module, "owner", "item");

     if(!$d->check("properties", "owner"))
     {
      $d->e->clear();
      $d->e->add($d->title." module doesn't have an owner reference", "error");
     }

     if($d->e->any("fatal"))
      $this->e->add($d->e);
     else
     {
      $id=$d->getID();
      $this->module=$module;

      if(request("confirm"))
      {
       if($d->field[$id]["valuetype"]!="request")
        $this->e->add("No ".$d->title." items selected to change the owner", "warning");
       else
        $this->e->add("Are you sure you wish to change the owner for ".$d->vmax." ".$d->title." item(s)?", "warning");
      }
      else if(request("s")&&$d->field[$id]["valuetype"]=="request") //values submitted
      {
       //set page as CMS in history
       $hs=new History();
       if(!$hs->cms())
        $this->e->add($hs->e);

       $d->submit();
       $this->e->add($d->e);
      }

      if($nr)
      {
       $this->nr=(int)$nr;
       $this->section="item";
      }
      else
      {
       $this->section="list";

       //set inline search options
       if($titles=$d->check("properties", "title"))
        $this->backend->search("data", $this->module, "title", $titles[0]);

       //search redirect
       if(request("search"))
       {
        $d->setAction("search");
        if($d->fulltextIndex) //full text index exists
         $d->setExtra("index");
        else //normal search
         $d->setExtra("full");

        $d->setShowFields();
        $d->setValue("show_total", 2);
        $d->setRequestValue("search");

        if($d->get()&&$d->vmax==1) //only 1 entry
         Headers::redirectS($this->url("owner/".$d->module."/".$d->getValue($d->getID())."/"));
       }
      }
     }
    }
   }

   if(!$this->module)//no module set
   {
    $this->backend->search("modules"); //inline search all modules

    //search
    if($search=request("search"))
    {
     foreach(Backend_Modules::modules(true) as $module => $v) //filter modules list
      if($v["enabled"]&&$v["owner"])
       if(stristr($module, $search)||stristr($v["title"], $search))
        $this->modules[$module]=$v;

     if(sizeof($this->modules)==1) //only one module
      Headers::redirectS($this->url("owner/".key($this->modules)."/"));
    }
    else
    {
     foreach(Backend_Modules::modules(true) as $module => $v) //filter modules list
      if($v["enabled"]&&$v["owner"])
       $this->modules[$module]=$v;
    }
   }

   $_session->backend();

   return true;
  }


  /*
   * Output
   */

  function output_default()
  {
   global $_session, $_config;

   $_session->frontend();

   $this->backend->setTitle($this->icon("owner_dim"),
    $this->link($this->language->t("Owner"), Array("owner/")));

   $this->tableRow("modules", true);
   $this->tableColumn("modules", $this->language->t("Module"), false, false, Array("class" => "module"));
   $this->tableColumn("modules", $this->language->t("Type"), false, false, Array("class" => "type"));

   //modules list
   $enabled=0;
   foreach($this->modules as $module => $v)
   {
    $this->tableRow("modules");
    $this->tableColumn("modules", $v["title"], $this->url("owner/".$module."/"), $v["title"]);

    if($v["type"]=="plugin")
     $this->tableColumn("modules", $v["pluginfo"]["title"], $this->url("plugins/".$v["plugin"]."/"), $v["plugin"]." v".$v["pluginfo"]["version"]." ".$v["pluginfo"]["status"].($v["pluginfo"]["description"]?" - ".$v["pluginfo"]["description"]:""));
    else
     $this->tableColumn("modules", $this->language->t(ucfirst($v["type"])));
   }

   $this->tableRow("modules", false, Array("class" => "total"));

   if(!$this->modules)
    $this->tableColumn("modules", $this->language->t("No modules"), false, false, Array("colspan" => 2));
   else
   {
    $this->tableColumn("modules", $this->language->t("Total"));
    $this->tableColumn("modules", sizeof($this->modules), false, false, Array("class" => "nr"));
   }

   $r="<div class=\"twothird\">\n";
   $r.=$this->table("modules", "owner");
   $r.="</div>\n";

   //actions
   $this->actionsMenu();

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }

  function output_list()
  {
   global $_session, $_config;

   $_session->frontend();

   $d=Data::cache($this->module, "view", "all");
   $owner=$d->check("properties", "owner")[0];

   $this->backend->setTitle($this->icon("owner_dim"),
    $this->link($this->language->t("Owner"), Array("owner/")),
    $this->link($d->l->t($d->title), Array("owner/".$d->module."/")));

   if(request("search"))
    $d->setAction("search");
   else if(($catnr=request("catnr"))&&($field=$d->getRightsReference())) //category filter
   {
    $d->setExtra("list");
    $d->setValue($field, $catnr);
   }

   $list=$this->viewList($d, Array("checkboxes", "nopagerform", "get" => 1, "fields" => $owner));

   if($this->backend->prettyurls)
    $list=preg_replace("!href=\"([^\"]*)data/(".$d->module."/\d+)/\"!", "href=\"\\1owner/\\2/\"", $list);
   else
    $list=preg_replace("!href=\"([^\"]*)p\[\]=data(&p\[\]=".$d->module."&p\[\]=\d+)\"!", "href=\"\\1p[]=owner\\2\"", $list);

   $r="<div class=\"twothird\">\n";
   $r.=$list;
   $r.="</div>\n";

   if($d->e->any("fatal"))
   {
    $_session->backend();
    return $r;
   }

   $r.="<div class=\"third owner\">\n";

   if(request("search"))
    $r.=$this->form->input("hidden", "search", strip_tags(request("search")));
   else if(request("catnr"))
    $r.=$this->form->input("hidden", "catnr", strip_tags(request("catnr")));

   foreach($d->check("type", "stp") as $field)
    $r.=$this->form->input("hidden", $field, $d->getDefaultValue($field));

   //owner submit
   $r.="<h3>".$this->icon("owner_dim", "small")." ".$this->language->t("Owner")."</h3>\n";

   $do=Data::cache($_config->get("usermodule"), "view", "all");
   $do->setShowFields($do->check("properties", "title"));

   $do->clearValues();

   $r.=$this->field($do->l->t($do->title), $this->form->select(false, $owner, $do, request($owner)));

   if(request("confirm")&&request($d->getID()))
    $r.=$this->confirm((!$d->r("owner")?Array("disabled"):false));
   else
    $r.=$this->field($this->language->t("Items"), $this->form->button("confirm", $this->icon("owner", "small")." ".$this->language->t("Owner"), false, false, "submit".($d->r("owner")?"":" disabled")));

   //actions
   $this->actionsMenu($d);

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->menu("actions");

   $r.="</div>\n";

   //make full page part of the form
   $r=$this->form($r);

   $_session->backend();

   return $r;
  }

  function output_item()
  {
   global $_session;

   $_session->frontend();

   $d=Data::cache($this->module, "view", "item");
   $d->setValue($d->getID(), $this->nr);

   //set default title
   $this->backend->setTitle($this->icon("owner_dim"),
    $this->link($this->language->t("Owner"), Array("owner/")),
    $this->link($d->l->t($d->title), Array("owner/".$d->module."/")));

   if(!($r=$this->viewItem($d, Array("show" => "normal", "get" => 1))))
    return false;

   //set title after viewItem got data
   $this->backend->setTitle($this->icon("owner_dim"),
    $this->link($this->language->t("Owner"), Array("owner/")),
    $this->link($d->l->t($d->title), Array("owner/".$d->module."/")),
    $this->link(Mutilate::stripS(strip_tags($d->v("title")), 24), $this->url()));

   $r="<div class=\"twothird\">\n".$r."</div>\n";

   $r.="<div class=\"third\">\n";
   $r.="<h3>".$this->icon("info_dim", "small")." ".$this->language->t("Info")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "default"))."\n";

   $r.="<h3>".$this->icon("link_dim", "small")." ".$this->language->t("Reference")."</h3>\n";
   $r.=$this->viewItem($d, Array("show" => "reference"))."\n";

   $r.="<h3>".$this->icon("execute_dim", "small")." ".$this->language->t("Actions")."</h3>\n";
   $r.=$this->cmsOptions($d, "owner", true)."\n";
   $r.="</div>\n";

   $_session->backend();

   return $r;
  }


 }
?>
